#!/usr/bin/env perl
# Norman Kuring		23-Jun-2000
# Norman Kuring		12-Aug-2002	New webserver and new composites
# Norman Kuring		12-Jun-2003	Changed sharpening program.
# Norman Kuring		13-Jun-2003	Hard-coded path to cat since latest
#					webserver setup seems to be arbitrarily
#					modifying the PATH.
use strict;
use OBPG::Launder qw(launder);

our $imgdir = '/web/data/SeaWiFS/Globes';
our %period = (
#        label string                   base filename
wi => ['Boreal Winter/Austral Summer','winter/S19973552002079.L3m_SCWI_BIOS'],
sp => ['Boreal Spring/Austral Autumn','spring/S19980802002171.L3m_SCSP_BIOS'],
su => ['Boreal Summer/Austral Winter','summer/S19981722001263.L3m_SCSU_BIOS'],
au => ['Boreal Autumn/Austral Spring','autumn/S19972642001354.L3m_SCAU_BIOS'],
);
our $globemask = "$imgdir/globemask.2048x2048.ppm";

our $thisScript = '/cgi/' . (split /\//,$0)[-1];

our $pnmscale = "pnmscale -quiet";
our $pnmarith = "pnmarith -quiet";
our $sharpen  = "pnmnlfilt -quiet -0.7 0.7";
our $convert  = 'convert';
our $cjpeg    = 'cjpeg';
our $cat      = 'cat';

#require '/web/oceancolor/cgi/cgi-lib.pl';
#&ReadParse;

use CGI qw(:all);
#use CGI::Carp qw(fatalsToBrowser);

our $q = new CGI;
our $params = $q->Vars;
our %in = %{$params};
# Filter out shell metacharacters
foreach my $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}


if(defined $in{fmt}){
  send_img($in{lat},$in{lon},$in{per},$in{siz},$in{bgc},$in{fmt});
}
else{
  &page0;
}


sub page0{

  print
  qq@Content-type: text/html\n\n@,
  qq@<html lang="en">\n@,
  qq@<head>\n@,
  qq@<title>SeaWiFS Biosphere Globes</title>\n@,
  qq@</head>\n@,
  qq@<body bgcolor="#000000" text="#FFFFFF" link="#00FFFF" vlink="#FF00FF">\n@,
  qq@<center>\n@,
  qq@<h1>SeaWiFS Biosphere Globes</h1>\n@,
  qq@<table><tr>\n@,
  qq@<td><img src="/seawifs/images/biosphere_seq_left.gif"@,
  qq@ width="200" height="400" alt="a sequence of biospheres"></td>\n@,
  qq@<td align="center">\n@,
  qq@<form method="post" action="$thisScript/$^T">\n@,
  qq@Latitude:\n@,
  qq@<select name="lat" size="1">\n@;

  # Latitude selector
  my $lat;
  for($lat = 90; $lat >= -90; $lat -= 10){
    my $s = sprintf "%d&deg; %s", $lat < 0 ? (-$lat,'South') : ($lat,'North');
    if($lat == 0){
      print qq@<option value="$lat" selected>Equator</option>\n@;
    }
    else{
      print qq@<option value="$lat">$s</option>\n@;
    }
  }

  print
  qq@</select><br />\n@,
  qq@Longitude:\n@,
  qq@<select name="lon" size="1">\n@;

  # Longitude selector
  my $lon;
  for($lon = -170; $lon <= 180; $lon += 10){
    my $s = sprintf "%d&deg; %s", $lon < 0 ? (-$lon,'West') : ($lon,'East');
    $lon == 180 and $s = '180&deg;';
    if($lon == 0){
      print qq@<option value="$lon" selected>0&deg;</option>\n@;
    }
    else{
      print qq@<option value="$lon">$s</option>\n@;
    }
  }

  print
  qq@</select><br />\n@,
  qq@Season:\n@,
  qq@<select name="per" size="1">\n@;

  # Time period selector
  my $per;
  foreach $per (qw/wi sp su au/){
    print qq@<option value="$per">$period{$per}[0]</option>\n@;
  }

  print
  qq@</select><br />\n@,
  qq@Image dimensions:\n@,
  qq@<select name="siz" size="1">\n@;

  # Size selector
  my $siz;
  foreach $siz (64,96,128,192,256,512,768,1024,1536,2048){
    my $s = join ' x ',$siz,$siz;
    print qq@<option value="$siz">$s</option>\n@;
  }

  # Background color selector
  print
  qq@</select><br />\n@,
  qq@Background color:\n@,
  qq@<select name="bgc" size="1">\n@,
  qq@<option>black</option>\n@,
  qq@<option selected>White\n@,
  qq@</select><br />\n@,
  qq@Image format:\n@,
  qq@<select name="fmt" size="1">\n@;

  # Format selector
  my $fmt;
  foreach $fmt (qw/JPEG GIF TIFF PPM PNG/){
    print qq@<option>$fmt</option>\n@;
  }

  print
  qq@</select><br />\n@,
  qq@<input type="submit" value="Fetch image">\n@,
  qq@</form>\n@,
  qq@</td>\n@,
  qq@<td><img src="/seawifs/images/biosphere_seq_right.gif"@,
  qq@ width="200" height="400" alt="a sequence of biospheres"></td>\n@,
  qq@</tr></table>\n@,
  qq@<hr>\n@,
  qq@<a href="/SeaWiFS/"><img border="0" alt=""\n@,
  qq@src="/icons/biosphere_icon.png" width="120"></a>\n@,
  qq@<br />\n@,
  qq@<a href="/SeaWiFS/"><b>SeaWiFS Project Home Page</b></a>\n@,
  qq@<hr>\n@,
  qq@<address>\n@,
  qq@<a href="/staff/gene/"> gene carl feldman</a>\n@,
  qq#(<a href="mailto:gene\@seawifs.gsfc.nasa.gov">#,
  qq#gene\@seawifs.gsfc.nasa.gov</a>)\n#,
  qq@(301) 286-9428\n@,
  qq@</address>\n@,
  qq@</center>\n@,
  qq@</body>\n@,
  qq@</html>\n@;

}

sub send_img{
  my ($lat,$lon,$per,$siz,$bgc,$fmt) = @_;
  my %hfmt = (
     PPM  => 'x-portable-pixmap',
     JPEG => 'jpeg',
     TIFF => 'tiff',
     PNG  => 'png',
     GIF  => 'gif'
  );
  $lon = 0 if $lat == 90 or $lat == -90;
  my $file = sprintf "$imgdir/$period{$per}[1].orth%02d%s%03d%s.ppm",
             ($lat < 0 ? (-$lat,'S') : ($lat,'N')),
             ($lon < 0 ? (-$lon,'W') : ($lon,'E'));

  unless(-s $file){
    print
    qq@Content-type: text/html\n\n@,
    qq@<html lang="en">\n@,
    qq@<head>\n@,
    qq@<title>SeaWiFS Biosphere Globes (Unavailable)</title>\n@,
    qq@</head\n@,
    qq@<body>\n@,
    qq@<h1>SeaWiFS Biosphere Globes</h1>\n@,
    qq@Sorry, but the image you requested is not available yet.\n@,
    qq@</body>\n@,
    qq@</html>\n@;
    exit;
  }
  my $bgmask = '';
  $bgmask = "$pnmarith -a - $globemask |" if $bgc =~ /white/i;
  my $scale = '';
  $scale = "$pnmscale -width $siz |" unless ($siz == 2048);
  my $format;
  if($fmt eq 'PPM'){
    $format = '';
  }
  elsif($fmt eq 'JPEG'){
    $format = "$cjpeg |";
  }
  else{
    $format = "$convert - $fmt:- |";
  }

  my $c = "$cat $file |$bgmask$scale $sharpen |$format";
  open P,$c or die qq@Error opening pipeline, "$c". ($!) Died@;
  local($/) = undef;
  my $img = <P>;
  close P or die qq@Error executing pipeline, "$c". ($!) Died@;
  my $len = length $img;
  print "Content-type: image/$hfmt{$fmt}\nContent-length: $len\n\n$img";
}

