#!/usr/bin/env perl

# This script builds a web page with the contents of the IDL library.
#
# Log:
# Ian T. Durham		 1/99		Created
# Ian T. Durham		 1/99		Altered file read to include 2 lines.
# Sean Bailey            8/2004         Vastly modified

use strict;
use CGI qw(:all);
use CGI::Pretty;
use OBPG::headfoot qw( standard_header standard_footer basic_laundry);
my $style = '';
my $leftside = my $rightside = '';
my $idldir = '/data1/www-data/oceancolor.gsfc.nasa.gov/static/docs/idl/ocidl';

$rightside = h2("IDL Library Introduction");
$rightside .= p("Welcome to the Ocean Color IDL Lilbrary. This library provides an on-line distribution system for IDL routines used by the Ocean Color Discipline Processing Group. These programs were created by the OCDPG, the SeaWiFS Project Team, and other projects. Its original incarnation was as the SIMBIOS IDL Library. With the end of the SIMBIOS program, the library was moved here.");
$rightside .= p("To open a particular category, please click on that category's name on the sidebar to the left. A list of files contained within that category should replace this message. Click on a particular file to view it. If it is a file you would like to save, please go to \"Save Link As...\" under the File menu in your browser and save the file to your computer. These routines are very wide ranging in their application. We hope that they can help you.");
$rightside .= p("The entire library is available for ",a({-href=>'/docs/idl/ocidl.tgz'},"download"),"(gzipped tar file)");
$rightside .= p(h3("Warning!"),"The OCDP Group does not gaurantee that any files contained in this library work as expected.  Use at your own risk.");
my %dirhash = (
    'atmocorr' => 'Atmospheric Correction',
    'device' => 'Device Functions',
    'filter' => 'Filter Fuctions',
    'hdf' => 'HDF Functions',
    'fit' => 'Fitting Functions',
    'math' => 'Math Functions',
    'l3bin' => 'Binning',
    'mos' => 'MOS Instrument',
    'seawifs' => 'SeaWiFS Instrument',
    'octs' => 'OCTS Instrument',
    'navigation' => 'Navigation',
    'simbios' => 'Matching Functions',
    'struct' => 'Structures',
    'time' => 'Time Functions',
    'utils' => 'Miscellaneous Utilites',
    'visual' => 'Visualization',
    'gumley' => 'Liam Gumley Routines',
    'JH' => 'Johns Hopkins Library Routines',
    'dfan' => 'Dave Fanning Routines',
);

opendir IDLIB, "$idldir" || die "Can't read $idldir: $!\n";
#my @cats = grep -d, readdir IDLIB;
my @cats =  grep !/^\.\.?$/,readdir IDLIB;
close IDLIB;

my @leftcells = ();
foreach my $category (@cats){
    next if ($category =~ /\.tgz/);
    next if ($category =~ /\.csh/);
    next if ($category =~ /\.svn/);
    next if ($category =~ /^astro$/);
    next if ($category =~ /^view_hdf$/);
    next if ($category =~ /straylight/);
    my $htmlcat = $category;
    if (exists $dirhash{$category}){$htmlcat = $dirhash{$category}};
   push @leftcells, Tr(td(a({-href=>"/cgi/idllibrary.cgi?dir=$category"},$htmlcat))); 
}
$leftside = table(Tr(td(h3("Categories"))),Tr(td(a({-href=>"/cgi/idllibrary.cgi"},"Introduction"))),@leftcells);

if (param('dir')){
    my $category = param('dir');
    my $subdir = join('/',$idldir,$category);
    my $htmlcat = $category;
    if (exists $dirhash{$category}){$htmlcat = $dirhash{$category}};
    $rightside = h3($htmlcat);
	if ($category eq 'ESRG'){$rightside .= p("IDL code from the Earth Space Research Group at UCSB downloaded from ",a({-href=>"http://www.astro.washington.edu/deutsch/idl/htmlhelp/index.html"},"http://www.astro.washington.edu/deutsch/idl/htmlhelp/index.html"));}
    opendir CAT, "$subdir"  || die "Can't read $subdir: $!\n";
    my @listing = grep /\.pro$/, readdir CAT;
    @listing = sort (@listing);

    foreach my $filename (@listing) {
        open (FILE,'<', "$subdir/$filename") || die "Can't open $subdir/$filename: $!\n";
        my @filarr = <FILE>;
        my $filelist = substr ($filarr[1], 1);
        my @filename = split (/ /, $filelist);
        @filename = reverse (@filename);
        my $length = @filename;
        my $range = $length-6;
        my $endrange = 0;
        my @fildesc = @filename[$endrange..$range];
        @fildesc = reverse (@fildesc);
        if ($filarr[2] ne '') {
            my $filelst2 = substr ($filarr[2], 1);
            my @filenam2 = split (/ /, $filelst2);
            @filenam2 = reverse (@filenam2);
            my $lenth2 = @filenam2;
            my $range2 = $lenth2-5;
            my $endrnge2 = 0;
            my @filedsc2 = @filenam2[$endrnge2..$range2];
            @filedsc2 = reverse (@filedsc2);
            $rightside .= b(a({-href=>"/docs/idl/ocidl/$category/$filename"},$filename),@fildesc,@filedsc2);
        } else {
            $rightside .= b(a({-href=>"/docs/idl/ocidl/$category/$filename"},$filename),@fildesc);
        }
        $rightside .= br;
    } 
}

my %start = (meta=>{author=>"Sean Bailey, Futuretech Corporation"});
standard_header('Ocean Color IDL Library','idllibray.cgi',\%start,$style,0,0,'oceancolor_documents.png');
print (br);
print center(h1("Ocean Color IDL Library"));
print (hr);
print table({-style=>"width: 90%;"},Tr(td({-style=>"width: 30%;"},$leftside),td({-style=>"width: 70%; text-align: left; "},$rightside)));
standard_footer('cgi/idllibrary.cgi',0);
exit;

