#!/usr/bin/env perl
# Norman Kuring		21-Nov-2003	Original development
# Norman Kuring		11-Dec-2003	Added explanatory text
# Norman Kuring		09-Jan-2004	Chdir to /tmp instead of
#					/web/seawifs/tmp to fix a bug
#					discovered on webserver1 where GMT
#					could not write its .gmtcommands
#					file to /web/seawifs/tmp/.
# Norman Kuring		30-Apr-2004	New header image and look compliments
#					of Mandy Johnson
#					<amanda.j.johnson1@jsc.nasa.gov>.
# Norman Kuring		 2-Jul-2004	Additional data brings new naming
#					convention (Lsppprrryyyymmdd...)
# Norman Kuring		 8-Jul-2004	Add IMaRS generated (red-land) images
# Norman Kuring		14-Oct-2004	Replace random default scene with
#					Samoa scene and switch default
#					view from IMaRS quicklook to band 321
#					quasi-natural color.
# Norman Kuring		 4-Jan-2005	New webserver (Sun) and new directory
#					hierarchy.
# Norman Kuring		28-Apr-2005	Remove duplicate path/row combinations
#					when generating a client-side map in
#					the coverage_map subroutine.
# Norman Kuring		 7-Sep-2005	Add support for kelp masks on selected
#					extracts.
# Norman Kuring		29-Mar-2006	Change FTP access from
#					ftp://samoa.gsfc.nasa.gov/landsat/ to
#					ftp://oceans.gsfc.nasa.gov/LANDSAT/ .
# Norman Kuring		16-Feb-2007	Added logo and hyperlink for the 11th
#					International Coral Reef Symposium.
# Norman Kuring		22-Jan-2010	Add link to page describing validated
#					and unvalidated reef classification
#					products.
# Norman Kuring		18-Jul-2012	Change FTP access to HTTP.

use OBPG::Launder qw(launder);

our $wrscoords      = "wrscoords";
our $latlon2pathrow = "latlon2pathrow";

our $coverage_img = '/web/oceancolor/metadata/landsat_coverage_map.ppm';
our $image_map = '/web/oceancolor/metadata/landsat_coverage_map.txt';

our $gmt = 'gmt';
our $pscoast   = "$gmt pscoast -P -X0 -Y0";
our $psxy      = "$gmt psxy -P -X0 -Y0 -L -M";
our $pstext    = "$gmt pstext -P -X0 -Y0 -G255/102/102";
our $gmtset    = "$gmt gmtset BASEMAP_TYPE plain PLOT_DEGREE_FORMAT D" .
                " BASEMAP_FRAME_RGB 102/102/102";

our $pngtopnm = "pngtopnm -quiet";
our $pnmtopng = "pnmtopng -quiet";
our $pnmscale = "pnmscale -quiet";
our $pnmcomp  = "pnmcomp -quiet";
our $ppmline  = "ppmline";

our $gs = 'gs -sDEVICE=ppmraw -sOutputFile=- -q -r100';

our $pi = 4 * atan2 1,1;

# Extract width and height
our ($xwid,$xhgt) = (512,256);

# Thumbnail montage width and height (pixels)
our ($twid,$thgt) = (512,256);

# Thumbnail montage width and height (degrees)
our ($dwid,$dhgt) = (7,3.5);

our $thumbscale = 0.019;

our $thisScript = '/cgi/' . (split /\//, $0)[-1];
our $xScript = '/cgi/landsat_extract.pl';

# Subroutines i.e. types of page layouts
our %sub = (
main_page		=> \&main_page,
new_coverage_img	=> \&new_coverage_img,
coverage_img		=> \&coverage_img,
coverage_map		=> \&coverage_map,
thumb_img		=> \&thumb_img,
thumb_map		=> \&thumb_map,
);

our $thumbdir = '/web/data/LANDSAT';

# Construct a temporary filename prefix.
our $tmp = '/tmpfs01/' . (split /\//,$0)[-1] . ".$^T.$$";

# Go somewhere where I can write the .gmtdefaults and .gmtcommands files.
chdir '/tmp' or die "Could not cd to /tmp ($!)";

use OBPG::CGI qw(ReadParse);
our %in = ReadParse();

# Filter out shell metacharacters
foreach my $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}

$in{t} eq 'IMaRS' or $in{t} = '321';

our @thumbs = thumblist($in{t});

$in{sub} = 'main_page' unless defined $in{sub} and defined $sub{$in{sub}};
unless(defined $in{path}){
  $in{path} = 69;
  $in{row} = 70;
  $in{x} = 2600;
  $in{y} = -2400;
}
#($in{path},$in{row}) = randpathrow(@thumbs) unless defined $in{path};

$in{x} = 0 unless defined $in{x};
$in{y} = 0 unless defined $in{y};
$in{n} = 0 unless defined $in{n};

&{$sub{$in{sub}}};

# Present a clickable map of the world that shows which areas are
# covered by Landsat data in our archive.  The image and client-side
# imagemap that are presented can be generated by this script with
# http://seawifs.gsfc.nasa.gov/cgi/landsat.pl?sub=new_coverage_img
# and
# http://seawifs.gsfc.nasa.gov/cgi/landsat.pl?sub=coverage_map
# but that takes too long, so I just saved the results and have the
# script refer to the saved information.
sub main_page{

  if(defined $in{'i.x'} and defined $in{'i.y'}){

    ($in{path},$in{row},$in{x},$in{y}) =
    tmove($in{path},$in{row},$in{'i.x'},$in{'i.y'},$in{t});

  }
  elsif(defined $in{'f.x'} and defined $in{'f.y'}){

    $in{x} += int($in{'f.x'} - $xwid/2);
    $in{y} += int($in{'f.y'} - $xhgt/2);

  }

  my $sat = prsat($in{path},$in{row},$in{n},$in{t});

  my $page_header = '';
  my $header_switch = '';
  unless(defined $in{nohed}){
    $page_header =
    qq@<tr>\n@ .
    qq@<td align="center"><img border="0" width="700" height="86"\n@ .
    qq@src="/landsat/images/header.jpg"\n@ .
    qq@alt="Millennium Coral Reef Landsat Archive"><p class="expsmall">\n@ .
    qq@A product of\n@ .
    qq@<a href="http://imars.usf.edu/" target="_blank">USF\n@ .
    qq@Millennium Global Coral Reef Mapping Project</a> /\n@ .
    qq@<a href="http://oceancolor.gsfc.nasa.gov/" target="_blank">NASA\n@ .
    qq@SeaWiFS</a> /\n@ .
    qq@<a href="http://eol.jsc.nasa.gov/" target="_blank">NASA\n@ .
    qq@Johnson Space Center</a> &nbsp; &nbsp; \n@ .
    qq@<a href="http://eol.jsc.nasa.gov/Reefs/">Project Overview >></a>\n@ .
    qq@</p></td>\n@ .
    qq@</tr>\n@;

    my $s = state_info(qw/url nohed/);
    if(defined $s){
      substr($s,0,0) = '&';
    }
    else{
      $s = '';
    }

    $header_switch =
    qq@A <a href="$thisScript?nohed=1$s">headerless version</a>\n@ .
    qq@of this page can save you the use of the vertical scrollbars\n@ .
    qq@if you have a large enough display.\n@;
  }
  else{
    my $s = state_info(qw/url nohed/);
    if(defined $s){
      substr($s,0,0) = '?';
    }
    else{
      $s = '';
    }

    $header_switch =
    qq@<a href="$thisScript$s">Show page header</a>.\n@;
  }

  print
  qq@Content-type: text/html\n\n@,
  qq@<html lang="en">\n@,
  qq@<head>\n@,
  qq@<title>Millennium Coral Reefs: Landsat Archive</title>\n@,
  qq@<link rel="stylesheet" type="text/css"\n@,
  qq@href="/landsat/html/style.css">\n@,
  qq@</head>\n@,
  qq@<body bgcolor="#000066"\n@,
  qq@marginheight="0" marginwidth="0" topmargin="0" leftmargin="0">\n@,
  # This is the main table of the page.
  qq@<table width="840" cellpadding="0" cellspacing="0"\n@,
  qq@align="center" border="0">\n@,
  $page_header,
  qq@<tr>\n@,
  qq@<td>\n@,
  qq@<table border="0" cellspacing="5" cellpadding="0">\n@,
  qq@<tr>\n@,
  qq@<td>\n@,
  qq@<img\n@,
  qq@src="$thisScript?sub=coverage_img&@,
  state_info(qw/url sub x y n t/),
  qq@"\nalt="geographical areas covered by Landsat scenes in our archive"\n@,
  qq@width="640" height="224" border="0" usemap="#pathrowmap">\n@,
  qq@</td>\n@,
  qq@<td valign="top">\n@,
  qq@<b>Available Data</b>\n@,
  qq@<p class="exp">\n@,
  qq@Click on any of the tiles in the map at left\n@,
  qq@to reposition the tile mosaic below.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@The open yellow rectangle on the map at left indicates the position\n@,
  qq@of the currently displayed tile mosaic.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  $header_switch,
  qq@</p>\n@,
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,
  qq@<table border="0" cellspacing="5" cellpadding="0">\n@,
  qq@<tr>\n@,
  qq@<td>\n@,
  qq@<form method="post" action="$thisScript">\n@,
  state_info(qw/form sub x y/),
  qq@<input type="image" name="i"\n@,
  qq@src="$thisScript?sub=thumb_img&@,
  state_info(qw/url sub/),
  qq@"\nalt="thumbnails of Landsat scenes around the area of interest"\n@,
  qq@width="$twid" height="$thgt" border="0">\n@,
  qq@</form>\n@,
  qq@</td>\n@,
  qq@<td valign="top">\n@,
  qq@<b>Tile Mosaic</b>\n@,
  qq@<table border="0"><tr>\n@,
  qq@<td>\n@,
  qq@<table border="1" bordercolor="#FFFFFF">\n@,
  qq@<tr>\n@,
  qq@<td>@,
  arrow('thumbnails','nw'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('thumbnails','no'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('thumbnails','ne'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr>\n@,
  qq@<td>@,
  arrow('thumbnails','we'),
  qq@</td>\n@,
  qq@<td><br /></td>\n@,
  qq@<td>@,
  arrow('thumbnails','ea'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr>\n@,
  qq@<td>@,
  arrow('thumbnails','sw'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('thumbnails','so'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('thumbnails','se'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,
  qq@</td>\n@,
  qq@<td>\n@,
  qq@<p class="exp">\n@,
  qq@Click anywhere in the tile mosaic at left to reposition the mosaic\n@,
  qq@and to reposition the detailed view below.\n@,
  qq@Alternatively, you can use the arrows to reposition the mosaic.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@The open red rectangle on the mosaic indicates the position of the\n@,
  qq@detailed view.\n@,
  qq@</p>\n@,
  qq@</td>\n@,
  qq@</tr></table>\n@,
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr>\n@,
  qq@<td valign="top">\n@,
  qq@<form method="post" action="$thisScript">\n@,
  state_info(qw/form sub/),
  qq@<input type="image" name="f"\n@,
  qq@src="$xScript?@,
  state_info(qw/url sub nohed/),
  qq@"\nalt="30 meter resolution extract"\n@,
  qq@width="$xwid" height="$xhgt" border="0">\n@,
  qq@</form>\n@,
  qq@</td>\n@,
  qq@<td>\n@,
  qq@<b>Detailed View <i>(30-meter resolution)</i></b>\n@,
  qq@<br /><br />\n@,
  qq@Mission: <b>Landsat $sat</b>\n@,
  qq@Path: <b>$in{path}</b>\n@,
  qq@Row: <b>$in{row}</b>\n@,
  qq@<br />\n@,
  qq@Date: <b>@,
  &prdate,
  qq@</b>\n@,
  qq@<ul><li>\n@,
  http_access($in{path},$in{row},$in{n},$in{t}),
  qq@</li><li>\n@,
  qq@<a href="/landsat/html/README.html">Access to validated\n@,
  qq@and unvalidated reef classification products</a>\n@,
  qq@</li><li>\n@,
  imars_access($in{path},$in{row},$sat),
  qq@</li></ul>\n@,
  &switch_view,
  qq@<br />\n@,
  qq@<table><tr>\n@,
  qq@<td>\n@,
  qq@<table border="1" bordercolor="#FFFFFF">\n@,
  qq@<tr>\n@,
  qq@<td>@,
  arrow('extract','nw'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('extract','no'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('extract','ne'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr>\n@,
  qq@<td>@,
  arrow('extract','we'),
  qq@</td>\n@,
  qq@<td><br /></td>\n@,
  qq@<td>@,
  arrow('extract','ea'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr align="center">\n@,
  qq@<td>@,
  arrow('extract','sw'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('extract','so'),
  qq@</td>\n@,
  qq@<td>@,
  arrow('extract','se'),
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,
  qq@</td>\n@,
  qq@<td>\n@,
  qq@<p class="exp">\n@,
  qq@Click anywhere in the image at left to choose a new center,\n@,
  qq@or use the arrows to pan around this scene.\n@,
  qq@If this path/row was acquired more than once, then the date\n@,
  qq@label above becomes a hyperlink that allows you to change to\n@,
  qq@other dates.  FTP access to all of the data\n@,
  qq@for this path/row/date combination is also provided.\n@,
  qq@</p>\n@,
  qq@</td>\n@,
  qq@</tr></table>\n@,
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,
  get_image_map(),
  qq@<p>\n@,
  qq@<b>Archive Description</b>\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@In a <a\n@,
  qq@href="http://eol.jsc.nasa.gov/reefs/">NASA-sponsored\n@,
  qq@partnership</a> between remote sensing scientists,\n@,
  qq@international agencies and NGOs, new efforts are being\n@,
  qq@made to (1) develop baseline global reef maps that can\n@,
  qq@be a foundation for future more detailed investigation,\n@,
  qq@(2) assemble the key baseline remote sensing data that is\n@,
  qq@needed for researchers to study coral reef environments,\n@,
  qq@and (3) partner with international organizations to use\n@,
  qq@remote sensing data for applied science problems and\n@,
  qq@improve the management of coral reefs.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@The first challenge for global coral reef mapping\n@,
  qq@with Landsat 7 is in getting global data that\n@,
  qq@is not obscured by cloud cover.  Global data\n@,
  qq@acquisition for coral reef mapping was initiated\n@,
  qq@through the Landsat 7 Long Term Acquisition Plan (<a\n@,
  qq@href="/REFERENCE/IGARSS2000_Gasch_et_al.pdf">Gasch\n@,
  qq@et al. 2000</a>).  Once the instrument began acquiring\n@,
  qq@data over coral reefs, it took several years to get the\n@,
  qq@needed acquisitions because of tropical cloud cover, and to\n@,
  qq@find ways to purchase or trade for the nearly 1000 scenes\n@,
  qq@necessary to complete the global coverage.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@This archive contains standardized data that\n@,
  qq@is tiled, zoomable, and downloadable via FTP .\n@,
  qq@The archive is under construction and images from the <a\n@,
  qq@href="http://imars.marine.usf.edu/MC/index.html">Millennium\n@,
  qq@Coral Reef Mapping Project</a> are still being added.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@<b>Status as of 17 May 2006:</b>\n@,
  qq@<ul>\n@,
  qq@<li>\n@,
  qq@<a\n@,
  qq@href="http://imars.marine.usf.edu/MC/imagedb.html">Images\n@,
  qq@collected</a>: 1724 (1661 with permission to distribute)\n@,
  qq@</li>\n@,
  qq@<li>\n@,
  qq@Images online in archive: 2101 (includes some non-reef scenes\n@,
  qq@which are not counted in the number above)\n@,
  qq@</li>\n@,
  qq@<li>\n@,
  qq@Global coverage is now considered complete.\n@,
  qq@</li>\n@,
  qq@</ul>\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@The collection of this set of\n@,
  qq@global data was a necessity for the <a\n@,
  qq@href="http://imars.marine.usf.edu/MC/index.html">Millennium\n@,
  qq@Coral Reef Mapping</a> project, and was also\n@,
  qq@recognized as a need early in discussions between\n@,
  qq@NASA and NOAA after the establishment of the <a\n@,
  qq@href="http://coralreef.gov/">U.S. Coral Reef Taskforce</a>.\n@,
  qq@By assembling this data in a single freely accessible\n@,
  qq@location, the use of Landsat data in a variety of local,\n@,
  qq@regional, and global mapping projects will become possible.\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@<b>This archive is a joint project of:</b>\n@,
  qq@<ul>\n@,
  qq@<li>\n@,
  qq@<a href="http://oceancolor.gsfc.nasa.gov">SeaWiFS Project</a>,\n@,
  qq@Goddard Space Flight Center\n@,
  qq@</li>\n@,
  qq@<li>\n@,
  qq@<a href="http://imars.marine.usf.edu/MC/index.html">Millennium\n@,
  qq@Global Coral Reef Mapping Project</a>\n@,
  qq@Institute for Marine Remote Sensing, University of South Florida\n@,
  qq@</li>\n@,
  qq@<li>\n@,
  qq@<a href="http://eol.jsc.nasa.gov/">Earth Observations Laboratory</a>,\n@,
  qq@Johnson Space Center\n@,
  qq@</li>\n@,
  qq@</ul>\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@<b>Data contributions by:</b>\n@,
  qq@<ul>\n@,
  qq@<li>\n@,
  qq@<a\n@,
  qq@href="http://www.nos.noaa.gov/topics/oceans/coralreefs/welcome.html">@,
  qq@NOAA,\n@,
  qq@National Ocean Service</a>\n@,
  qq@</li>\n@,
  qq@<li>\n@,
  qq@<a\n@,
  qq@href="http://www.esad.ssc.nasa.gov/datapurchase/">Scientific\n@,
  qq@Data Purchase Program</a>, Stennis Space Center\n@,
  qq@</li>\n@,
  qq@</ul>\n@,
  qq@</p>\n@,
  qq@<p class="exp">\n@,
  qq@<b>11th International Coral Reef Symposium</b><br />\n@,
  qq@<a href="http://www.nova.edu/ncri/11icrs/"><img src=@,
  qq@"/icons/11icrs_header_logo.png"@,
  qq@ width="249" height="86" border="0"></a>\n@,
  qq@</p>\n@,
  qq@<p class="expsmall">\n@,
  qq@Last updated 16 February 2007\n@,
  qq@</p>\n@,
  qq@</tr>\n@,
  qq@</td>\n@,
  qq@</table>\n@,
  qq@<center>\n@,
  qq@<a href="http://eol.jsc.nasa.gov/reefs/"><img\n@,
  qq@src="/icons/reefs_at_NASA.jpg"\n@,
  qq@width="308" height="100" border="0"></a>\n@,
  qq@<br />\n@,
  qq@<font class="footer"><a href="http://eol.jsc.nasa.gov/reefs/">Remote\n@,
  qq@Sensing of Coral Reefs</a></font>\n@,
  qq@</center>\n@,
  qq@</body>\n@,
  qq@</html>\n@;
}

# Generate a client-side imagemap that I can subsequently include in this
# script so that the user doesn't have to wait for it to be generated each
# time a page is loaded.
sub coverage_map{

  # Remove any duplicate path/row combinations.
  my %pr;
  foreach my $f (@thumbs){
    my $pr = unpack "x2a6",$f;
    $pr{$pr}++;
  }
  my @pr = sort keys %pr;

  print
  qq@Content-type: text/html\n\n@,
  qq@<html lang="en">\n@,
  qq@<body>\n@,
  qq@<map name="pathrowmap">\n@;

  foreach my $pr (@pr){
    my ($p,$r) = unpack "a3a3",$pr;
    my @c = map launder($_), split " ",`$wrscoords $p $r`;

    $p += 0;	# Convert to path and row to numeric
    $r += 0;	# form to remove leading zeros.

    my $x = int( ($c[1] + 180)*640/360 + 0.5 );
    my $y = int( (63 -  $c[0])*224/126 + 0.5 );

    print
    qq@<area href="$thisScript?path=$p&row=$r"\n@,
    qq@alt="Path $p  Row $r"\n@,
    qq@shape="circle"\n@,
    qq@coords="$x,$y,2">\n@;
  }
  print
  qq@</map>\n@,
  qq@</body>\n@,
  qq@</html>\n@;
}

# Generate a world map that shows which areas are covered
# by Landsat scenes in our archive.
sub new_coverage_img{
  my $c;

  # Draw the land masses.
  $c =
  "$pscoast -R-180/180/-63/63 -JX6.4d/2.24 -G0/153/0 -S0/0/255 -W1/0/102/0" .
  " -Dl -K >$tmp.ps";
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  # Mark the areas for which we have Landsat scenes.
  $c =
  "$psxy -R-180/180/-63/63 -JX6.4d/2.24" .
  " -G255/0/0 -W1/102/0/0 -O -K >> $tmp.ps";

  open P,"|$c" or die qq@Error opening pipe to command, "$c" ($!)@;

  my $need_separator = 0;
  foreach my $f (@thumbs){
    my ($p,$r) = unpack "x2a3a3",$f;
    $need_separator and print P ">\n";
    $need_separator = 1;
    my @c = map launder($_), split " ",`$wrscoords $p $r`;
    print P "$c[3] $c[2]\n$c[5] $c[4]\n$c[9] $c[8]\n$c[7] $c[6]\n";
  }

  close P or die qq@Error executing command, "$c" ($!)@;

  # Close out the PostScript file.
  $c = "$pstext -R -JX -O < /dev/null >> $tmp.ps";
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  # Convert it to PPM format with GhostScript.
  $c = "$gs -g640x224 - < $tmp.ps > $tmp.ppm";
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  unlink "$tmp.ps";

  $c = "$pnmtopng $tmp.ppm";
  open P,"$c|" or die qq@Error opening pipe from command, "$c" ($!)@;
  local($/) = undef;
  my $png = <P>;
  close P or die qq@Error executing command, "$c" ($!)@;

  unlink "$tmp.ppm";

  my $len = length $png;

  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub coverage_img{

  my ($n,$s,$w,$e) = montage_bounds($in{path},$in{row});

  # Draw a rectangle showing the location of the thumbnail image.

  my $y1 = int( (63 -  $n)*224/126 + 0.5 );
  my $y2 = int( (63 -  $s)*224/126 + 0.5 );
  my $x1 = int( ($w + 180)*640/360 + 0.5 );
  my $x2 = int( ($e + 180)*640/360 + 0.5 );

  my $png = addrect($coverage_img,'rgb:ff/ff/00',$x1,$x2,$y1,$y2);

  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

# Generate a 12 by 6 degree background map with
# the Landsat WRS path and row boundaries marked
# and the Landsat thumnails overlain.
sub thumb_img{
  my ($lat,$lon) = map launder($_),
                   (split " ",`$wrscoords $in{path} $in{row}`)[0,1];

  my $context = sprintf "$thumbdir/context/p%03dr%03d.%s.ppm",
                $in{path},$in{row},$in{t};

  my ($n,$s,$w,$e) = montage_bounds($in{path},$in{row});

  unless(-s $context){

    # If a context image does not exist or is empty...

    my ($w_over_100,$h_over_100) = ($twid/100,$thgt/100);

    my $c;

    # Set up some GMT defaults.
    $c = "$gmtset ANNOT_FONT_PRIMARY Helvetica-Oblique ANNOT_FONT_SIZE 7p" .
         " GRID_CROSS_SIZE_PRIMARY 0.02i ANNOT_OFFSET_PRIMARY -0.075i";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

    # Draw the land masses.
    $c =
    "$pscoast -R$w/$e/$s/$n -JX${w_over_100}d/$h_over_100" .
    " -G204/255/204 -S204/204/255" .
    " -W1/51/102/51 -N1/1/51/102/51 -N2/1/51/102/51 -Di -B1g1 -K > $tmp.ps";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

    # Get the easternmost path and southernmost row in the region.
    my ($epath,$srow) = latlon2pathrow($s,$e);

    # Get the westernmost path and northernmost row in the region.
    my ($wpath,$nrow) = latlon2pathrow($n,$w);

    # Draw the scene boundaries.
    $c = "$psxy -R$w/$e/$s/$n -JX${w_over_100}d/$h_over_100" .
         " -W1/255/102/102 -O -K >> $tmp.ps";

    open P,"|$c" or die qq@Error opening pipe to command, "$c" ($!)@;

    my $need_separator = 0;
    my $p;
    for($p = $epath; $p != $wpath + 1; $p++){
      $p = ($p - 1) % 233 + 1;
      my $r;
      for($r = $nrow; $r <= $srow; $r++){
        $need_separator and print P ">\n";
        $need_separator = 1;
        my @c = map launder($_), split " ",`$wrscoords $p $r`;
        print P "$c[3] $c[2]\n$c[5] $c[4]\n$c[9] $c[8]\n$c[7] $c[6]\n";
      }
    }
    close P or die qq@Error executing command, "$c" ($!)@;

    # Label the scene locations with their path and row number.
    $c = "$pstext -R$w/$e/$s/$n -JX${w_over_100}d/$h_over_100 -O -K >> $tmp.ps";

    open P,"|$c" or die qq@Error opening pipe to command, "$c" ($!)@;
    for($p = $epath; $p != $wpath + 1; $p++){
      $p = ($p - 1) % 233 + 1;
      my $r;
      for($r = $nrow; $r <= $srow; $r++){
        my @c = map launder($_), split " ",`$wrscoords $p $r`;
        print P "$c[1] $c[0] 12 0 1 LB $p\n$c[1] $c[0] 12 0 1 RT $r\n";
      }
    }
    close P or die qq@Error executing command, "$c" ($!)@;

    # Close out the PostScript file.
    $c = "$pstext -R$w/$e/$s/$n -JX${w_over_100}d/$h_over_100 -O" .
         " < /dev/null >> $tmp.ps";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

    # Convert it to PPM format with GhostScript.
    $c = "$gs -g${twid}x$thgt - < $tmp.ps > $tmp.ppm";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

    unlink "$tmp.ps";

    # Place the thumbnails on the background map.
    for($p = $epath; $p != $wpath + 1; $p++){
      $p = ($p - 1) % 233 + 1;
      my $r;
      for($r = $nrow; $r <= $srow; $r++){
        next if $p == $in{path} && $r == $in{row};
        my ($f) = prtofiles($p,$r,$in{t});
        next unless defined $f;
        my ($wid,$hgt) = pngdim("$thumbdir/$f");
        my ($lat,$lon) = map launder($_), (split " ",`$wrscoords $p $r`)[0,1];
        my $new_wid = int($wid/cos($lat*$pi/180) + 0.5);
        my $x = int((deltalon($w,$lon))*$twid/$dwid - $new_wid/2 + 0.5);
        my $y = int(($n - $lat)*$thgt/$dhgt - $hgt/2 + 0.5);
        $c = "$pngtopnm -alpha $thumbdir/$f" .
             " | $pnmscale -width $new_wid -height $hgt > $tmp.pgm";
        system($c) == 0 or die qq@Error executing command, "$c" ($!)@;
        $c = "$pngtopnm $thumbdir/$f " .
             "| $pnmscale -width $new_wid -height $hgt" .
             "| $pnmcomp -alpha $tmp.pgm -xoff $x -yoff $y -" .
             " $tmp.ppm $tmp.1.ppm";
        system($c) == 0 or die qq@Error executing command, "$c" ($!)@;
        rename "$tmp.1.ppm","$tmp.ppm" or die qq@Error renaming $tmp.1.ppm @,
                                              qq@to $tmp.ppm ($!)@;
        unlink "$tmp.pgm";
      }
    }
    $context = "$tmp.ppm";
  }

  # Put the central scene on top so we can see the whole thumbnail.
  my @f = prtofiles($in{path},$in{row},$in{t});
  if(@f){
    $in{n} = 0 if $in{n} >= @f;
    my $f = launder( $f[$in{n}] );
    my ($wid,$hgt) = pngdim("$thumbdir/$f");
    my $new_wid = int($wid/cos($lat*$pi/180) + 0.5);
    my $x = int((deltalon($w,$lon))*$twid/$dwid - $new_wid/2 + 0.5);
    my $y = int(($n - $lat)*$thgt/$dhgt - $hgt/2 + 0.5);
    $c = "$pngtopnm -alpha $thumbdir/$f" .
         " | $pnmscale -width $new_wid -height $hgt > $tmp.pgm";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;
    $c = "$pngtopnm $thumbdir/$f | $pnmscale -width $new_wid -height $hgt" .
         "| $pnmcomp -alpha $tmp.pgm -xoff $x -yoff $y -" .
         " $context $tmp.1.ppm";
    system($c) == 0 or die qq@Error executing command, "$c" ($!)@;
    rename "$tmp.1.ppm","$tmp.ppm" or die qq@Error renaming $tmp.1.ppm @,
                                          qq@to $tmp.ppm ($!)@;
    unlink "$tmp.pgm";

    $context = "$tmp.ppm";
  }

  # Draw a rectangle on the thumbnail image
  # that shows the location of the extract.

  my $hh = $xhgt * $thumbscale;
  my $ww = $xwid * $thumbscale / cos($lat*$pi/180);
  my $x1 = $twid/2 + $in{x} * $thumbscale / cos($lat*$pi/180) - $ww/2;
  my $x2 = $x1 + $ww;
  my $y1 = $thgt/2 + $in{y} * $thumbscale - $hh/2;
  my $y2 = $y1 + $hh;
  $x1 = int($x1 + 0.5);
  $x2 = int($x2 + 0.5);
  $y1 = int($y1 + 0.5);
  $y2 = int($y2 + 0.5);

  my $png = addrect($context,'rgb:ff/00/00',$x1,$x2,$y1,$y2);

  unlink "$tmp.ppm" if -e "$tmp.ppm";

  my $len = length $png;

  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub thumb_map{
  ;
}

# Get a list of all the Landsat thumbnails.
sub thumblist{
  my $t = shift;
  opendir D,$thumbdir or die qq@Could not open directory, "$thumbdir" ($!)@;
  my @thumbs = grep /^L\d{15}\.$t\.png$/, readdir D;
  closedir D;
  @thumbs;
}

# Randomly select a scene from the archive.
# The input list is the one returned by the
# thumblist subroutine.
sub randpathrow{
  my @thumbs = @_;

  my $i = int(rand @thumbs);	# Pick a random thumbnail.
  my ($p,$r) = unpack "x2a3a3",$thumbs[$i];	# Return the path and row.
  $p += 0;
  $r += 0;
  ($p,$r);
}

# Get the path and row that is nearest to the given latitude and longitude.
sub latlon2pathrow{
  my ($lat,$lon) = @_;

  my $text = `$latlon2pathrow $lat $lon 0`;

  my ($p,$r) = $text =~ /Closest Path:\s*(\d+)\s*Row:\s*(\d+)/;
  $r = 0 if $r > 122;

  ($p,$r);
}

# Read the dimensions of a PNG image.
sub pngdim{

  my $f = shift;

  my $pnghdr = "\211PNG\r\n\032\n\0\0\0\015IHDR";

  open F,'<',$f or die qq@Could not open file, "$f" ($!)@;

  my ($hdr,$wh);

  read(F,$hdr,16) == 16
  and $hdr eq $pnghdr
  or die "$f is not a proper PNG image.\n";

  read(F,$wh,8) == 8 or die "Error reading $f ($!)";

  close F;

  map launder($_), unpack "NN",$wh;
}

sub deltalon{
  my ($west,$east) = @_;
  $west += 360 while $west < -180;
  $west -= 360 while $west >  180;
  $east += 360 while $east < -180;
  $east -= 360 while $east >  180;
  $west > $east ? 360 + $east - $west : $east - $west;
}

sub get_image_map{
  open F,'<',$image_map or die qq@Could not open file, "$image_map" ($!)@;
  local($/) = undef;
  my $map = <F>;
  close F;
  $map =~ s#/cgi/landsat.pl#$thisScript#g;
  $map =~ s/\?/?t=$in{t}&/g;
  $map =~ s/\?/?nohed=1&/g if defined $in{nohed};
  $map =~ s/\?/?kelp=1&/g  if defined $in{kelp};
  $map;
}

sub arrow{
  my $i = shift;	# 'thumbnails' or 'extract'
  my $d = shift;	# direction

  my $p = $in{path};	# path
  my $r = $in{row};	# row
  my $x = $in{x};	# x offset from center
  my $y = $in{y};	# y offset from center

  my $poffset = 4;
  my $roffset = 2;

  my $xoffset = int(0.9 * $xwid);
  my $yoffset = int(0.9 * $xhgt);

  my %arrow = (
  no => ['north'    ,30,28],
  so => ['south'    ,30,28],
  we => ['west'     ,28,30],
  ea => ['east'     ,28,30],
  ne => ['northeast',28,28],
  nw => ['northwest',28,28],
  se => ['southeast',28,28],
  sw => ['southwest',28,28],
  );

  if($i eq 'thumbnails'){
    {
      $d eq 'no' and $r -= $roffset, last;
      $d eq 'so' and $r += $roffset, last;
      $d eq 'we' and $p += $poffset, last;
      $d eq 'ea' and $p -= $poffset, last;
      $d eq 'nw' and $p += $poffset, $r -= $roffset, last;
      $d eq 'ne' and $p -= $poffset, $r -= $roffset, last;
      $d eq 'sw' and $p += $poffset, $r += $roffset, last;
      $d eq 'se' and $p -= $poffset, $r += $roffset, last;
    }
    $r =   1 if $r <   1;
    $r = 122 if $r > 122;
    $p = ($p - 1) % 233 + 1;
  }
  else{
    {
      $d eq 'no' and $y -= $yoffset, last;
      $d eq 'so' and $y += $yoffset, last;
      $d eq 'we' and $x -= $xoffset, last;
      $d eq 'ea' and $x += $xoffset, last;
      $d eq 'nw' and $x -= $xoffset, $y -= $yoffset, last;
      $d eq 'ne' and $x += $xoffset, $y -= $yoffset, last;
      $d eq 'sw' and $x -= $xoffset, $y += $yoffset, last;
      $d eq 'se' and $x += $xoffset, $y += $yoffset, last;
    }
  }

  my $s = state_info(qw/url path row x y/);

  qq@<a href="$thisScript?path=$p&row=$r&x=$x&y=$y&$s">@ .
  qq@<img src="/icons/$d.gif" alt="move $arrow{$d}[0]"@ .
  qq@ width="$arrow{$d}[1]" height="$arrow{$d}[2]" border="0"></a>@;
}

sub prdate{
  my $date;

  my @f = prtofiles($in{path},$in{row},$in{t});
  if(@f){
    my @m = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;

    my $n = $in{n};
    $n = 0 if $n >= @f;
    my ($yr,$m,$d) = unpack "x8a4a2a2",$f[$n];
    $m--;
    $d += 0;
    $date = "$d $m[$m] $yr";
    if(@f > 1){
      $n++;
      my $s = state_info(qw/url n/);
      $date =
      qq@<a href="$thisScript?$s&n=$n">$date</a>@;
    }
  }
  else{
    $date = "not available";
  }
  "$date\n";
}

sub prsat{
  my ($p,$r,$n,$t) = @_;

  my $sat;

  my @f = prtofiles($p,$r,$t);
  if(@f){
    $n = 0 if $n >= @f;
    $sat = unpack "xa",$f[$n];
  }
  $sat;
}

sub http_access{
  my ($p,$r,$n,$t) = @_;

  my $href = '';

  my @f = prtofiles($p,$r,$t);
  if(@f){
    $n = 0 if $n >= @f;
    my ($sat,$date) = $f[$n] =~ /^L(\d)\d{6}(\d{8})/;
    my $dir = sprintf "L$sat%03d%03d$date",$p,$r;
    $href =
    qq@<a href="/cgi/landsat_index.cgi/$dir/">HTTP access</a>\n@;
  }
  $href;
}

sub imars_access{
  my ($p,$r,$s) = @_;

  my $href = "http://imars.marine.usf.edu/cgi-bin/crs?path=$p&row=$r&sat=l$s";

  qq@<a href="$href">Go to Millennium Coral Reef Map Status @ .
  qq@for this Path-Row</a>\n@;
}

sub switch_view{
  my $html = '';
  my $s = state_info(qw/url t/);
  if($in{t} eq '321'){
    $html .=
    qq@Quasi-natural color displayed.<br />\n@ .
    qq@<a href="$thisScript?$s&t=IMaRS">Switch to reef-enhanced view.</a>\n@;
  }
  elsif($in{t} eq 'IMaRS'){
    $html .=
    qq@Reef-enhanced view displayed.<br />\n@ .
    qq@<a href="$thisScript?$s&t=321">Switch to quasi-natural color.</a>\n@;
  }
  $html;
}

sub addrect{
  my ($ppm,$color,$x1,$x2,$y1,$y2) = @_;

  my $h = parse_pnm_header($ppm) or die qq@Error parsing header of file, @,
                                        qq@"$ppm"@;

  my $c =
  "$ppmline " .
  "-line $x1,$y1,$x2,$y1 " .
  "-line $x2,$y1,$x2,$y2 " .
  "-line $x2,$y2,$x1,$y2 " .
  "-line $x1,$y2,$x1,$y1 " .
  "$color $ppm" .
  "| $pnmtopng";

  open P,"$c|" or die qq@Error opening pipe from command, "$c" ($!)@;
  local($/) = undef;
  my $png = <P>;
  close P or die qq@Error executing command, "$c" ($!)@;
  $png;
}

sub parse_pnm_header{
  my $pnmfile = shift;
  my $h;
  open F,'<',$pnmfile or return undef;
  my $bytes;
  read F,$bytes,4096 or return undef;
  close F;
  $bytes =~ /^P[1-6][\s#]/ or return undef;
  my $b = $bytes;       # Keep a pristine copy to get the unmunged header from.
  $bytes =~ s/#.*/' ' x length $&/emg;  # Replace comments with whitespace.
  ($h->{magic},$h->{width},$h->{height},$h->{maxval}) = split ' ', $bytes;
  if($h->{magic} eq 'P1' || $h->{magic} eq 'P4'){
    $h->{maxval} = 1;
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s/;
    $h->{hdrlen} = length $&;
  }
  else{
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s+$h->{maxval}\s/;
    $h->{hdrlen} = length $&;
  }
  $h->{header} = substr $b,0,$h->{hdrlen};
  $h->{header} =~ s/\s$/\n/;
  $h;
}

sub state_info{
  my ($style,@omit) = @_;

  my @state = qw/sub path row x y n t nohed kelp/;

  my %omit;
  grep $omit{$_}++, @omit;
  @state = grep !defined $omit{$_}, @state;

  my $state;
  if($style eq 'form'){
    my $s;
    foreach $s (@state){
      if(defined $in{$s} and $in{$s} ne ''){
        $state .= qq@<input type="hidden" name="$s" value="$in{$s}">\n@;
      }
    }
  }
  elsif($style eq 'url'){
    my $s;
    foreach $s (@state){
      if(defined $in{$s} and $in{$s} ne ''){
        $state .= '&' if defined $state;
        $state .= qq@$s=$in{$s}@;
      }
    }
  }
  else{
    die qq@The state_info subroutine must be called with a first argument @,
        qq@of either "form" or "url".  Died@;
  }
  $state;
}

sub tmove{
  my ($p,$r,$x,$y,$t) = @_;

  my ($n,$s,$w,$e) = montage_bounds($p,$r);

  # Determine the lat/lon of the user's click.
  my $lat = $n - $y * $dhgt/$thgt;
  my $lon = $w + $x * $dwid/$twid;
  $lon += 360 while $lon < -180;
  $lon -= 360 while $lon >  180;

  # Determine which path/row includes that lat/lon.
  ($p,$r) = latlon2pathrow($lat,$lon);

  unless(prtofiles($p,$r,$t)){
    # We have no data for the newly selected path/row, so I
    # try to find an overlapping adjacent scene that covers
    # the user's click.
    # This block is more likely to be encountered at higher
    # latitudes where path-to-path overlap is greater and where
    # scene coverage is sparser (not many coral reefs in the high
    # latitudes :-) ).

    my $wp = $p%233 + 1;
    my $ep = ($p - 2)%233 + 1;

    my @pr = ([$wp, $r],[$ep, $r]);
    push @pr, [$wp, $r-1],[$p, $r-1],[$ep, $r-1] if $r >   1;
    push @pr, [$wp, $r+1],[$p, $r+1],[$ep, $r+1] if $r < 122;

    foreach my $pr (@pr){
      my @c = map launder($_), split " ",`$wrscoords $pr->[0] $pr->[1]`;
      my $xp = [@c[3,5,9,7]];
      my $yp = [@c[2,4,8,6]];

      # If the clicked upon longitude is near the dateline
      # then make sure that all of the path/row corner longitudes
      # are numerically in the same hemisphere.
      if($lon < -175){
        grep { $_ > 0 and $_ -= 360 and 0 } @$xp;
      }
      elsif($lon > 175){
        grep { $_ < 0 and $_ += 360 and 0 } @$xp;
      }

      if(pnpoly($xp,$yp,$lon,$lat) and prtofiles($pr->[0],$pr->[1],$t)){
        ($p,$r) = ($pr->[0],$pr->[1]);
        last;
      }
    }
  }

  # Get the center lat/lon of the new path/row.
  ($lat,$lon) = map launder($_), (split " ",`$wrscoords $p $r`)[0,1];

  # Determine the x/y coordinates of the new center lat/lon on the
  # thumbnail montage image.  The xprime/yprime may be off the edge
  # of the image.
  my $xp = deltalon($w,$lon)*$twid/$dwid;
  my $yp = ($n - $lat)*$thgt/$dhgt;

  # Compute the offset from the center of the scene and scale to
  # give the offsets for the full-sized image.
  $x = int( ($x - $xp)*cos($lat * $pi/180)/$thumbscale );
  $y = int( ($y - $yp)/$thumbscale );

  ($p,$r,$x,$y);
}

sub montage_bounds{
  my ($p,$r) = @_;

  # Determine the center coordinates for the specified path/row.
  my ($lat,$lon) = map launder($_), (split " ",`$wrscoords $p $r`)[0,1];

  # Get the map boundaries based on those coordinates.
  my $n = $lat + $dhgt/2;
  my $s = $lat - $dhgt/2;
  my $w = $lon - $dwid/2;
  my $e = $lon + $dwid/2;

  ($n,$s,$w,$e);
}

sub prtofiles{
  my ($p,$r,$t) = @_;

  my $pr = sprintf "%03d%03d",$p,$r;
  my @f = sort grep /^L\d$pr\d{8}\.$t/, @thumbs;
}

# I got this code snippet (originally in C) from
# ftp://rtfm.mit.edu/pub/usenet-by-group/news.answers/graphics/algorithms-faq
# and
# http://www.ecse.rpi.edu/Homepages/wrf/research/geom/pnpoly.html
sub pnpoly{
  my ($xp,$yp,$x,$y) = @_;
  my $npol = @$xp;
  my ($i,$j);
  my $c = 0;
  for($i=0, $j = $npol-1; $i < $npol; $j = $i++){
    if(
    (
    (($yp->[$i]<=$y) && ($y<$yp->[$j])) ||
    (($yp->[$j]<=$y) && ($y<$yp->[$i]))
    ) &&
    (
    $x <
    ($xp->[$j] - $xp->[$i]) * ($y - $yp->[$i]) /
    ($yp->[$j] - $yp->[$i]) +
    $xp->[$i]
    )
    ){
      $c = !$c;
    }
  }
  $c;
}
