#!/usr/bin/env perl
# Norman Kuring		14-Nov-2003	Original development
# Norman Kuring		19-Nov-2003	Return blank image if file not found
# Norman Kuring		 7-Sep-2005	Add a kelp mask to the extract
#					if it's available.
# Norman Kuring		 8-Sep-2008	Launder the input data to foil possible
#					miscreants.
# Norman Kuring		 3-Apr-2012	Remote script from benthos becomes
#					local on main web server.

our $pnmcut       = '/data1/www-data/oceancolor.gsfc.nasa.gov/scripts/pnmcut.pl';
our $pnmtopng     = 'pnmtopng -quiet';
our $ppmcolormask = 'ppmcolormask -quiet';
our $pnmcomp      = 'pnmcomp -quiet -invert';

# The extract width and height
our $xwid = 512;
our $xhgt = 256;

use OBPG::CGI qw(ReadParse);
our %in = ReadParse();

# Filter out shell metacharacters
foreach my $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}

defined $in{path} or $in{path} = 13;
defined $in{row}  or $in{row}  = 42;

defined $in{x} or $in{x} = 0;
defined $in{y} or $in{y} = 0;

defined $in{n} or $in{n} = 0;

$in{t} eq '321' or $in{t} = 'IMaRS';

our $dir = '/web/data/landsat';

our $pfx = sprintf "L[457]%03d%03d",$in{path},$in{row};

our $tmp = "/tmpfs01/" . (split /\//,$0)[-1] . ".$^T.$$";

opendir D,$dir or die qq@Could not open directory, "$dir" ($!)@;
our @d = sort map {/^(L\d+)$/;$1} grep -d "$dir/$_" && /^$pfx/, readdir D;
closedir D;

@d > 0 or &blank; # Send a blank image and exit if a directory is not found.

$in{n} >= @d and $in{n} = 0;

our $f = "$dir/$d[$in{n}]/$d[$in{n}].$in{t}.ppm";

-s $f or &blank; # Send a blank image and exit if the image is not found.

our $h = parse_pnm_header($f) or die qq@Error parsing header of file, "$f".\n@;

our $x = int(($h->{width}  - $xwid)/2 + $in{x});
our $y = int(($h->{height} - $xhgt)/2 + $in{y});

$x = 0 if $x < 0;
$y = 0 if $y < 0;
$x = $h->{width}  - $xwid if $x > $h->{width}  - $xwid;
$y = $h->{height} - $xhgt if $y > $h->{height} - $xhgt;

# Check for the presence of a kelp mask file.
our $kf = "$dir/$d[$in{n}]/$d[$in{n}].kelp.ppm";
our $c;
our @tobedeleted;
if($in{kelp} and -s $kf){
  $c = "$pnmcut $x $y $xwid $xhgt $f > $tmp.0";
  system($c) == 0 or die "Error executing $c ($!)";
  $c = "$pnmcut $x $y $xwid $xhgt $kf > $tmp.1";
  system($c) == 0 or die "Error executing $c ($!)";
  $c = "$ppmcolormask rgb:ff/00/00 $tmp.1 " .
       "| $pnmcomp -alpha=- $tmp.1 $tmp.0 | $pnmtopng";
  push @tobedeleted, "$tmp.0","$tmp.1";
}
else{
  $c = "$pnmcut $x $y $xwid $xhgt $f | $pnmtopng";
}

open P,"$c|" or die qq@Error opening pipe from command, "$c" ($!)@;
local($/) = undef;
our $png = <P>;
close P or die qq@Error executing command, "$c" ($!)@;
@tobedeleted and unlink @tobedeleted;
our $len = length $png;

print "Content-type: image/png\nContent-length: $len\n\n$png";


sub blank{
  # Format a single pixel, blank PNG image.
  my $pix = "789c6360000000020001";	# compressed pixel
  my $pngsignature = chr(137) . "PNG\r\n" . chr(26) . "\n";
  my $ihdr = pack("N",13) . "IHDR" . pack("N2C5H8",1,1,1,3,0,0,0,"25db56ca");
  my $plte = pack("N", 3) . "PLTE" . pack("C3H8"  ,0,0,0        ,"a77a3dda");
  my $idat = pack("N",10) . "IDAT" . pack("H20H8" ,$pix         ,"48afa471");
  my $iend = pack("N", 0) . "IEND" . pack("H8"                  ,"ae426082");
  my $png = join '',$pngsignature,$ihdr,$plte,$idat,$iend;

  # Send it to the client browser.
  print "Content-type: image/png\n\n$png";
  exit;
}

sub parse_pnm_header{
  my $pnmfile = shift;
  my $h;
  open F,'<',$pnmfile or return undef;
  my $bytes;
  read F,$bytes,12288 or return undef;
  close F;
  $bytes =~ /^P[1-6][\s#]/ or return undef;
  my $b = $bytes;       # Keep a pristine copy to get the unmunged header from.
  $bytes =~ s/#.*/' ' x length $&/emg;  # Replace comments with whitespace.
  ($h->{magic},$h->{width},$h->{height},$h->{maxval}) = split ' ', $bytes;
  {
    # Make sure these values aren't tainted.
    $h->{width} =~ /(\d+)/;
    $h->{width} = $1;
    $h->{height} =~ /(\d+)/;
    $h->{height} = $1;
  }
  if($h->{magic} eq 'P1' || $h->{magic} eq 'P4'){
    $h->{maxval} = 1;
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s/;
    $h->{hdrlen} = length $&;
  }
  else{
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s+$h->{maxval}\s/;
    $h->{hdrlen} = length $&;
  }
  $h->{header} = substr $b,0,$h->{hdrlen};
  $h->{header} =~ s/\s$/\n/;
  $h;
}

# This subroutine cleans up variables for use in perl scripts
# where taint checking has been turned on.  Characters allowed through
# are hyphen, at-sign, dot, space, letters, and digits.
#
# Norman Kuring		3-Sep-2008
sub launder{
  # Filter out potentially harmful characters from the outside world.
  my $dirty = shift;
  my $variable = shift || '';
  $variable and $variable .= '=';
  if( $dirty =~ /^([-\@\w. ]*)$/ ){
    return $1;
  }
  else{
    die "[tainted] $variable\"$dirty\"";
  }
}
