#!/usr/bin/env perl
# Make the images generated by /web/oceancolor/scripts/pcgac9000.pl
# available on the web.

# Norman Kuring		27-Jun-2005	Original development
# Norman Kuring		 7-Jul-2005	Added week day string to image caption
# Norman Kuring		 1-Jun-2006	Changed ORBIMAGE reference to GeoEye
# Norman Kuring		 8-Sep-2008	Vet user input via launder() before
#					executing anything that could be
#					dangerous.
# Norman Kuring		 9-Sep-2008	Replace home-grown ppmline program
#					with NetPbm's ppmdraw program.
# Norman Kuring		17-Oct-2008	Allow user to request extract by
#					bounding coordinates.
# Norman Kuring		17-Dec-2008	Launder the $tmp variable.
# Norman Kuring		19-Mar-2009	Allow user to request scaling of
#					coordinate-bounded extract.
# Norman Kuring		20-Mar-2009	Added extra code to remove temporary
#					files before dying with an error.

use POSIX;
use Time::Local;

use OBPG::Launder qw(launder);

our $dir = '/web/data/SeaWiFS/PCGAC9000';

our $pnmtopng = 'pnmtopng -quiet';
our $pnmcat   = 'pnmcat -quiet';
our $pamscale = 'pamscale -quiet';
our $ppmmake  = 'ppmmake -maxval=255 rgb:00/00/00';
our $pnmcut   = '/data1/www-data/oceancolor.gsfc.nasa.gov/scripts/pnmcut.pl';
our $ppmdraw = 'ppmdraw -quiet';

our $thisScript = '/cgi/' . (split /\//,$0)[-1];

our $tmp = '/tmpfs01/' . launder( (split /\//,$0)[-1] . ".$^T.$$" );

use OBPG::CGI qw(ReadParse);
our %in = ReadParse();

# Filter out shell metacharacters
foreach my $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}

&send_image if(defined $ENV{PATH_INFO} and $ENV{PATH_INFO} ne '');

opendir D,$dir or die "Could not open $dir ($!)";
our @small = reverse sort grep /^S\d{7}[WE]\.small\.ppm$/, readdir D;
closedir D;

our $i = 0;
if($in{day} =~ /^\d{7}[WE]/){
  my $s = "S$in{day}";
  for($i=0; $i<@small; $i++){
    last if $small[$i] =~ /^$s/;
  }
  $i = @small - 2 if $i > @small - 2;
}

# Check for a user's click on the map and generate
# appropriate URLs for the global images and the extract
# based on the click location.
our $xclick = (grep /^[LR]\.x$/, keys %in)[0];

our ($lurl,$rurl,$xurl) = translate_click($i,@small);

our ($move_left,$move_right);
if($i > 0){
  my $day = substr($small[$i-1],1,8);
  $move_left =
  qq@<img src="/icons/blank.gif" width="70" height="115"><br />@ .
  qq@<a href="$thisScript?day=$day">@ .
  qq@<img src="$thisScript/arrow_west.png"@ .
  qq@ width="70" height="69" border="0"@ .
  qq@ alt="move westward to more recent swaths">@ .
  qq@</a></br />@ .
  qq@<img src="/icons/blank.gif" width="70" height="116">@;
}
else{
  $move_left = qq@<img src="/icons/blank.gif" width="70" height="300">@;
}
if($i < @small - 2){
  my $day = substr($small[$i+1],1,8);
  $move_right =
  qq@<img src="/icons/blank.gif" width="70" height="115"><br />@ .
  qq@<a href="$thisScript?day=$day">@ .
  qq@<img src="$thisScript/arrow_east.png"@ .
  qq@ width="70" height="69" border="0"@ .
  qq@ alt="move eastward to older swaths">@ .
  qq@</a></br />@ .
  qq@<img src="/icons/blank.gif" width="70" height="116">@;
}
else{
  $move_right = qq@<img src="/icons/blank.gif" width="70" height="300">@;
}

our $left_small  = $small[$i];
our $right_small = $small[$i + 1];

$left_small  =~ s/ppm$/png/;
$right_small =~ s/ppm$/png/;

our $left  = $left_small;
our $right = $right_small;
$left  =~ s/\.small//;
$right =~ s/\.small//;

our ($lefttimes,$righttimes,$daterow,$leftlons,$rightlons);
if($left =~ /W\.png$/){
  $lefttimes  = "west_times.png";
  $righttimes = "east_times.png";
  $leftlons   = "west_lons.png";
  $rightlons  = "east_lons.png";
  my $yyyyddd = substr($left,1,7);
  $daterow =
  qq@<th colspan="2">@ .
  weekday($yyyyddd) . ", " .
  daymonyear($yyyyddd) .
  qq@ <font size="-1">($yyyyddd)</font></th>\n@;
}
else{
  $lefttimes  = "east_times.png";
  $righttimes = "west_times.png";
  $leftlons   = "east_lons.png";
  $rightlons  = "west_lons.png";
  my $yyyyddde = substr($left ,1,7);
  my $yyyydddw = substr($right,1,7);
  $daterow =
  qq@<th>@ .
  weekday($yyyyddde) . ", " .
  daymonyear($yyyyddde) .
  qq@ <font size="-1">($yyyyddde)</font></th>\n@ .
  qq@<th>@ .
  weekday($yyyydddw) . ", " .
  daymonyear($yyyydddw) .
  qq@ <font size="-1">($yyyydddw)</font></th>\n@;
}

our $xtrct = '';
if($xurl){
  my ($w,$h) = (split /\./,$xurl)[-3,-2];
  $xtrct =
  qq@<tr>\n@ .
  qq@<td colspan="4" align="center">@ .
  qq@<img src="$xurl" width="$w" height="$h">@ .
  qq@</td>\n@ .
  qq@</tr>\n@;
}

&head;

print
qq@<table border="1">\n@,
qq@<tr>\n@,
qq@<td>@,
qq@<a href="$thisScript/$left"><font size="-2">@,
qq@Download left half<br /><i>(4500 x 4500 pixels)</i></font></a>@,
qq@</td>\n@,
$daterow,
qq@<td>@,
qq@<a href="$thisScript/$right"><font size="-2">@,
qq@Download right half<br /><i>(4500 x 4500 pixels)</i></font></a>@,
qq@</td>\n@,
qq@</tr>\n@,
qq@<tr>\n@,
qq@<td align="right">@,
qq@<b>Hour</b> <font size="-1"><i>(UTC)</i></font><br />@,
qq@$move_left<br /><b>Longitude</b>@,
qq@</td>\n@,
qq@<td colspan="2">\n@,
qq@<form method="post" action="$thisScript">\n@,
qq@<input type="hidden" name="day" value="$in{day}">\n@,
qq@<img src="$thisScript/$lefttimes"@,
qq@ width=300" height="20" alt="hour annotations">@,
qq@<img src="$thisScript/$righttimes"@,
qq@ width=300" height="20" alt="hour annotations"><br />\n@,
qq@<input type="image" name="L"@,
qq@ src="$lurl"  width=300" height="300" border="0"@,
qq@ alt="$left">@,
qq@<input type="image" name="R"@,
qq@ src="$rurl" width=300" height="300" border="0"@,
qq@ alt="$right"><br />\n@,
qq@<img src="$thisScript/$leftlons"@,
qq@ width=300" height="20" alt="longitude annotations">@,
qq@<img src="$thisScript/$rightlons"@,
qq@ width=300" height="20" alt="longitude annotations">\n@,
qq@</form>\n@,
qq@</td>\n@,
qq@<td>@,
qq@<b>Hour</b> <font size="-1"><i>(UTC)</i></font><br />@,
qq@$move_right<br /><b>Longitude</b>@,
qq@</td>\n@,
qq@</tr>\n@,
$xtrct,
qq@</table>\n@;

&tail;





sub translate_click{
  my ($index,@ppms) = @_;

  my ($lurl,$rurl,$xurl);

  # Set the default URLs for no user click.
  $lurl = $ppms[$index];
  $lurl =~ s/ppm$/png/;
  substr($lurl,0,0) = "$thisScript/";

  $rurl = $ppms[$index + 1];
  $rurl =~ s/ppm$/png/;
  substr($rurl,0,0) = "$thisScript/";

  my $xclick = (grep /^[LR]\.x$/, keys %in)[0];
  if($xclick){
    # If the user clicked on the map...

    # Determine whether click was on left or right side of the map.
    my $isleft  = $xclick =~ /^L/;

    # Get the (x,y) coordinates of the click.
    my $yclick = $xclick;
    $yclick =~ s/x$/y/;
    my ($x,$y) = ($in{$xclick},$in{$yclick});

    # Get the bounds of a 40 x 20 pixel box centered on the click.
    my $x1 = $x - 20;
    my $x2 = $x + 19;
    my $y1 = $y - 10;
    my $y2 = $y +  9;

    # Adjust boundaries that cross an image edge.
    if($y1 < 0){
      $y2 += -$y1;
      $y1 = 0;
    }
    elsif($y2 > 299){
      $y1 -= $y2 - 299;
      $y2 = 299;
    }

    if($isleft){	# User clicked on left half of map.
      $x1 = 0 if $x1 < 0;
      if($x2 > 299){
        # Part of the outline will be drawn on the right image.
        my $xr1 = 0;
        my $xr2 = $x2 - 300;
        $xr2 = "00" if $xr2 == 0;
        $rurl =~ s/small\.png$/$xr1.$y1.$xr2.$y2.small.png/;
      }
      $lurl =~ s/small\.png$/$x1.$y1.$x2.$y2.small.png/;
    }
    else{		# User clicked on right half of map.
      if($x1 < 0){
        # Part of the outline will be drawn on the left image.
        my $xl1 = 300 + $x1;
        my $xl2 = 300;
        $lurl =~ s/small\.png$/$xl1.$y1.$xl2.$y2.small.png/;
        $x1 = 0;
      }
      elsif($x1 == 0){
        $x1 = "00"; # zero but true; a line will be drawn along left edge
      }
      $rurl =~ s/small\.png$/$x1.$y1.$x2.$y2.small.png/;
    }

    # Compose an extract URL.
    my $bigw = 600;
    my $bigh = 300;
    my $bigx = 15 * $x;
    my $bigy = 15 * $y;
    $bigx -= 300;
    $bigy -= 150;
    $bigy = 0 if $bigy < 0;
    if($isleft){	# User clicked on left half of map.
      if($bigx < 0){
        if($index > 0){
          $xurl = $ppms[$index - 1];
          $bigx += 4500;
        }
        else{
          $xurl = $ppms[$index];
          $bigw += $bigx;
          $bigx = 0;
        }
      }
      else{
        $xurl = $ppms[$index];
      }
    }
    else{		# User clicked on right half of map.
      if($bigx < 0){
        $xurl = $ppms[$index];
        $bigx += 4500;
      }
      else{
        $xurl = $ppms[$index + 1];
        if($bigx + $bigw >= 4500){
          if($index + 1 == $#ppms){
            $bigw = 4500 - $bigx;
          }
        }
      }
    }
    $xurl =~ s/small\.ppm/$bigx.$bigy.$bigw.$bigh.png/;
    substr($xurl,0,0) = "$thisScript/";
  }
  ($lurl,$rurl,$xurl);
}

sub send_image{

  my $img = launder( (split /\//,$ENV{PATH_INFO})[-1] );

  if($img =~ /^S\d{7}[WE]\.small\.png$/){
    my $ppm = $img;
    $ppm =~ s/png$/ppm/;
    my $c = "$pnmtopng $dir/$ppm";
    open P,"$c|" or die "Error opening pipe from $c ($!)";
    local($/) = undef;
    my $png = <P>;
    close P or die "Error executing $c ($!)";
    my $len = length $png;
    print "Content-type: image/png\nContent-length: $len\n\n$png";
  }
  elsif(
  $img =~ /^(?:west|east)_(?:times|lons)\.png$/ or
  $img =~ /^^arrow_(?:west|east)\.png$/
  ){
    my $f = "$dir/$img";
    open F,'<',$f or die "Could not open $f ($!)";
    local($/) = undef;
    my $png = <F>;
    close F;
    my $len = length $png;
    print "Content-type: image/png\nContent-length: $len\n\n$png";
  }
  elsif($img =~ /^S\d{7}[WE]\.png$/){
    my $ppm = $img;
    $ppm =~ s/png$/ppm/;
    my $c = "$pnmtopng -tran =rgb:00/00/00 $dir/$ppm";
    open P,"$c|" or die "Error opening pipe from $c ($!)";
    local($/) = undef;
    my $png = <P>;
    close P or die "Error executing $c ($!)";
    my $len = length $png;
    print "Content-type: image/png\nContent-length: $len\n\n$png";
  }
  elsif($img =~ /^S(\d{7})([WE])\.(\d+)\.(\d+)\.(\d+)\.(\d+)\.png$/){
    my ($yyyyddd,$hemi,$x,$y,$wid,$hgt) = ($1,$2,$3,$4,$5,$6);
    my $ppm = $img;
    $ppm =~ s/\d+\.\d+\.\d+\.\d+\.png$/ppm/;
    $hgt = 4500 if $hgt > 4500;
    $y = 4500 - $hgt if $y + $hgt >= 4500;
    $wid = 4500 if $wid > 4500;
    if($x + $wid < 4500){
      my $c = "$pnmcut $x $y $wid $hgt $dir/$ppm | $pnmtopng";
      open P,"$c|" or die "Error opening pipe from $c ($!)";
      local($/) = undef;
      my $png = <P>;
      close P or die "Error executing $c ($!)";
      my $len = length $png;
      print "Content-type: image/png\nContent-length: $len\n\n$png";
    }
    else{
      my $w1 = 4500 - $x;
      my $w2 = $wid - $w1;
      my $ppm2;
      if($hemi eq 'W'){
        $ppm2 = "S${yyyyddd}E.ppm";
      }
      else{
        my $prevday = yyyyddd(yd2day($yyyyddd) - 1);
        $ppm2 = "S${prevday}W.ppm";
      }
      my $c = "$pnmcut $x $y $w1 $hgt $dir/$ppm > $tmp.left";
      run_with_cleanup($c,"$tmp.left");
      if(-s "$dir/$ppm2"){
        $c = "$pnmcut 0 $y  $w2 $hgt $dir/$ppm2 > $tmp.right";
        run_with_cleanup($c,"$tmp.right");
        $c = "$pnmcat -lr $tmp.left $tmp.right|$pnmtopng";
      }
      else{
        $c = "$pnmtopng $tmp.left";
      }
      unless( open P,"$c|" ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.right";
        die "Error opening pipe from $c ($errstr)";
      }
      local($/) = undef;
      my $png = <P>;
      unless( close P ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.right";
        die "Error executing $c ($errstr)";
      }
      unlink "$tmp.left","$tmp.right";
      my $len = length $png;
      print "Content-type: image/png\nContent-length: $len\n\n$png";
    }
  }
  elsif($img =~ /^S(\d{7})([WE])\.(\d+)\.(\d+)\.(\d+)\.(\d+)\.small\.png$/){
    my ($yyyyddd,$hemi,$x1,$y1,$x2,$y2) = ($1,$2,$3,$4,$5,$6);

    $y1 = 299 if $y1 > 299;
    $y2 = 299 if $y2 > 299;

    my $ppm = $img;
    $ppm =~ s/\d+\.\d+\.\d+\.\d+\.small\.png$/small.ppm/;

    my $lines = 'setcolor rgb:00/fe/00;';
    $lines .= "line $x1 $y1 $x2 $y1;";
    $lines .= "line $x2 $y1 $x2 $y2;" if $x2 < 300;
    $lines .= "line $x2 $y2 $x1 $y2;";
    $lines .= "line $x1 $y2 $x1 $y1;" if $x1;

    my $c = "$ppmdraw -script='$lines' $dir/$ppm|$pnmtopng";
    open P,"$c|" or die "Error opening pipe from $c ($!)";
    local($/) = undef;
    my $png = <P>;
    close P or die "Error executing $c ($!)";
    my $len = length $png;
    print "Content-type: image/png\nContent-length: $len\n\n$png";
  }
  elsif(
  $img =~
  /^S(\d{7})\.([\d.]+)([NS])\.([\d.]+)([NS])\.([\d.]+)([WE])\.([\d.]+)([WE])\.png$/
  ){
    my ($yyyyddd,$n,$nh,$s,$sh,$w,$wh,$e,$eh) = ($1,$2,$3,$4,$5,$6,$7,$8,$9);
    $nh eq 'S' and $n = -$n;
    $sh eq 'S' and $s = -$s;
    $wh eq 'W' and $w = -$w;
    $eh eq 'W' and $e = -$e;

    $n <= $s and message("Incorrect URL",<<"    --");
    Northern boundary (first coordinate after date) must be north of
    the southern boundary (second coordinate after date).
    --

    ($n > 90 or $s < -90) and message("Incorrect URL",<<"    --");
    The northern and southern coordinates (first and second after
    the date) must be in the range of 90 South to 90 North.
    --

    $w -= 360 while $w >= 180;
    $w += 360 while $w < -180;
    $e -= 360 while $e >= 180;
    $e += 360 while $e < -180;

    $w == $e and message("Incorrect URL",<<"    --");
    The western and eastern coordinates (third and fourth after
    the date) must not be equal.
    --

    my $scale = '';
    if(defined $in{wid} and $in{wid} =~ /^\d{2,4}$/){
      $scale = "|$pamscale -width $in{wid}";
    }

    if( $w < $e and ($w < 0 and $e < 0 or $w >= 0 and $e >= 0) ){
      my ($hs,$ho) = $w < 0 ? ('W',180) : ('E',0);
      my $ppm = "$dir/S$yyyyddd$hs.ppm";

      -s $ppm or message("Data unavailable",<<"      --");
      There is no imagery available for the day you requested.
      --

      my $x = floor( 25*($w + $ho) );
      my $y = floor( 25*(90 -  $n) );
      my $wid = ceil( 25*($e - $w) );
      my $hgt = ceil( 25*($n - $s) );
      my $c = "$pnmcut $x $y $wid $hgt $ppm $scale| $pnmtopng";
      open P,"$c|" or die "Error opening pipe from $c ($!)";
      local($/) = undef;
      my $png = <P>;
      close P or die "Error executing $c ($!)";
      my $len = length $png;
      print "Content-type: image/png\nContent-length: $len\n\n$png";
    }
    elsif( $w < 0 and $e >= 0 or $w >= 0 and $e < 0 ){
      my ($wf,$ef,$ho,$da);

      if($w < 0){
        ($wf,$ef) = map "$dir/S$yyyyddd$_.ppm", qw/W E/;
        $ho = 180;
        $da = 0;
      }
      else{
        my $prev = yyyyddd(yd2day($yyyyddd) - 1);
        $wf = "$dir/S${yyyyddd}E.ppm";
        $ef = "$dir/S${prev}W.ppm";
        $ho = 0;
        $da = 360;
      }

      my ($swf,$sef) = (-s $wf, -s $ef);

      $swf or $sef or message("Data unavailable",<<"      --");
      There is no imagery available for the day you requested.
      --

      my $x = floor( 25*($w + $ho) );
      my $y = floor( 25*(90 -  $n) );
      my $wid = 4500 - $x;
      my $hgt = ceil( 25*($n - $s) );
      my $wid2 = ceil( 25*($e - $w + $da) ) - $wid;

      my $c;
      if($swf){
        $c = "$pnmcut $x $y $wid $hgt $wf > $tmp.left";
      }
      else{
        $c = "$ppmmake $wid $hgt > $tmp.left";
      }
      run_with_cleanup($c,"$tmp.left");

      if($sef){
        $c = "$pnmcut 0 $y $wid2 $hgt $ef > $tmp.right";
      }
      else{
        $c = "$ppmmake $wid2 $hgt > $tmp.right";
      }
      run_with_cleanup($c,"$tmp.right");

      $c = "$pnmcat -lr $tmp.left $tmp.right$scale|$pnmtopng";
      unless( open P,"$c|" ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.right";
        die "Error opening pipe from $c ($errstr)";
      }
      local($/) = undef;
      my $png = <P>;
      unless( close P ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.right";
        die "Error executing $c ($errstr)";
      }
      unlink "$tmp.left","$tmp.right";
      my $len = length $png;
      print "Content-type: image/png\nContent-length: $len\n\n$png";
    }
    elsif( ($w > $e) and ($w < 0 and $e < 0 or $w >= 0 and $e >= 0) ){
      my ($wf,$mf,$ef,$ho,$da);

      if($w < 0){
        my $prev = yyyyddd(yd2day($yyyyddd) - 1);
        $wf = "$dir/S${yyyyddd}W.ppm";
        $mf = "$dir/S${yyyyddd}E.ppm";
        $ef = "$dir/S${prev}W.ppm";
        $ho = 180;
        $da = 180;
      }
      else{
        my $next = yyyyddd(yd2day($yyyyddd) + 1);
        $wf = "$dir/S${next}E.ppm";
        $mf = "$dir/S${yyyyddd}W.ppm";
        $ef = "$dir/S${yyyyddd}E.ppm";
        $ho = 0;
        $da = 180;
      }

      my ($swf,$smf,$sef) = (-s $wf, -s $mf, -s $ef);

      $swf or $smf or $sef or message("Data unavailable",<<"      --");
      There is no imagery available for the day you requested.
      --

      my $x = floor( 25*($w + $ho) );
      my $y = floor( 25*(90 -  $n) );
      my $wid = 4500 - $x;
      my $hgt = ceil( 25*($n - $s) );
      my $wid2 = ceil( 25*($e - $w + $da) ) - $wid;

      my $c;
      if($swf){
        $c = "$pnmcut $x $y $wid $hgt $wf > $tmp.left";
      }
      else{
        $c = "$ppmmake $wid $hgt > $tmp.left";
      }
      run_with_cleanup($c,"$tmp.left");

      if($smf){
        $c = "$pnmcut 0 $y 4500 $hgt $mf > $tmp.mid";
      }
      else{
        $c = "$ppmmake 4500 $hgt > $tmp.mid";
      }
      run_with_cleanup($c,"$tmp.mid");

      if($sef){
        $c = "$pnmcut 0 $y $wid2 $hgt $ef > $tmp.right";
      }
      else{
        $c = "$ppmmake $wid2 $hgt > $tmp.right";
      }
      run_with_cleanup($c,"$tmp.right");

      $c = "$pnmcat -lr $tmp.left $tmp.mid $tmp.right$scale|$pnmtopng";
      unless( open P,"$c|" ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.mid","$tmp.right";
        die "Error opening pipe from $c ($errstr)";
      }
      local($/) = undef;
      my $png = <P>;
      unless( close P ){
        my $errstr = "$!";
        unlink "$tmp.left","$tmp.mid","$tmp.right";
        die "Error executing $c ($errstr)";
      }
      unlink "$tmp.left","$tmp.mid","$tmp.right";
      my $len = length $png;
      print "Content-type: image/png\nContent-length: $len\n\n$png";
    }
  }
  else{
    print
    qq@Content-type: text/html\n\n@,
    qq@<html>\n@,
    qq@<head><title>Unrecognized URL</title></head>\n@,
    qq@<body>\n@,
    qq@Sorry, I don't know how to interpret this URL.\n@,
    qq@</body>\n@,
    qq@</html>\n@;
  }
  exit;
}

sub run_with_cleanup{
  my ($cmd,$tmpfile) = @_;
  unless( system($cmd) == 0 ){
    my $errstr = "$!";
    unlink $tmpfile;
    die "Error executing $cmd ($errstr)";
  }
}

sub daymonyear{
  my $yyyyddd = shift;
  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
  my ($y,$d) = unpack "a4a3",$yyyyddd;
  my ($day,$mon) = (gmtime(timegm(0,0,0,1,0,$y-1900) + 86400*($d - 1)))[3,4];
  "$day $mon[$mon] $y";
}

sub weekday{
  my $yyyyddd = shift;
  my @wday = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
  my ($y,$d) = unpack "a4a3",$yyyyddd;
  my $wday = (gmtime(timegm(0,0,0,1,0,$y-1900) + 86400*($d - 1)))[6];
  $wday[$wday];
}

sub yyyyddd{
  my $day = shift;
  my @t = gmtime(86400*$day);
  sprintf "%04d%03d",$t[5]+1900,$t[7]+1;
}

sub yd2day{
  my $yyyyddd = shift;
  my ($y,$d) = unpack "a4a3",$yyyyddd;
  timegm(0,0,0,1,0,$y-1900)/86400 + $d - 1;
}

sub message{
  my ($title,$message) = @_;
  print
  qq@Content-type: text/html\n\n@,
  qq@<html>\n@,
  qq@<head><title>$title</title></head>\n@,
  qq@<body>\n<p>\n@,
  $message,
  qq@</p>\n</body>\n@,
  qq@</html>\n@;
  exit;
}

sub head{
print <<"--";
Content-type: text/html

<html lang="en">
<head>
<title>SeaWiFS Global True Color Maps</title>
</head>
<body>
--
}

sub tail{
print <<"--";
<b>
<p>
Provided by the SeaWiFS Project,
NASA/Goddard Space Flight Center, and GeoEye
</p>

<p>
NOTE:<br />
All SeaWiFS images and data presented or referred to on
this web site are for research and educational use only.<br />
All commercial use of SeaWiFS data must be coordinated with
GeoEye (<a href="http://geoeye.com/"
target="_blank">http://geoeye.com/</a>).
</p>
</b>
</body>
</html>
--
}
