#!/usr/bin/env perl
#
# Norman Kuring		28-Oct-1999

# Display the six most recent hotspot links instead of just
# the most recent.
# Norman Kuring		20-Oct-2000

# Norman Kuring		22-Oct-2002	New style depth images
# Norman Kuring		01-Nov-2002	Allow user to turn on and off
#					the "other images" overlays.
# Norman Kuring		08-Nov-2002	Draw reefs from vector instead of
#					raster data.
# Norman Kuring		25-Nov-2002	Ability to draw Ashley Holt's moved
#					reefs.
# Norman Kuring		15-Dec-2003	Added text as suggested by Julie
#					Robinson.
# Norman Kuring		 9-Jan-2004	Chdir to /tmp instead of $tmproot
#					to fix a bug discovered on webserver1
#					where GMT could not write its
#					.gmtcommands file to $tmproot.
# Norman Kuring		 6-Jan-2005	New webserver (Sun)
# Norman Kuring		19-Aug-2005	Fix some hyperlinks and add FTP pointer
#					to SeaWiFS bathymetry data set.
# Norman Kuring		13-Apr-2010	Updated the NOAA Hotspots hyperlink.

use POSIX qw(asin tan);
use OBPG::Launder qw(launder);

our $pi = 4 * atan2(1,1);

# my $i1file_default = 'SeaWiFS.depths.mode.small.35N.35S.180W.180E.ppm';
# my $i2file_default = 'SeaWiFS.depths.mode.35N.35S.180W.180E.pgm';
our $i1file_default = 'SeaWiFS_median_depth.small.35N.35S.180W.180E.ppm';
our $i2file_default = 'SeaWiFS_median_depth.35N.35S.180W.180E.pgm';
our $i2cmap_default = 'depth256.ppm';
our $hscmap = 'hotspot.ppm';
our $hscbar = 'hotspot_colorbar.ppm';
our $i2wid_default = 500;
our $i2hgt_default = 500;
our $imgdir = '/web/data/Reefs';
our $tmproot = '/tmpfs01';	# Used by the
our $tmpname_counter = 0;		# tmpname subroutine.
our $smallreefbase =
   'Reefbase.coral.mangrove.PlateCarree.35N.35S.180W.180E.720x140.ppm';
our $basemap = 'map.small.35N.35S.180W.180E.pgm';
our $locator = 'locator.small.35N.35S.180W.180E.ppm';
# my $rbimage  = "$imgdir/ReefBase.coral.90N.90S.180W.180E.pbm";
our $thisScript = '/cgi/' . (split /\//,$0)[-1];

our $coast_color = "0,0,0";
our $reef_color  = "255,0,0";
our $creef_color = "255,255,0";
our $wrs_color   = "255,0,0";
our $rect_color  = "255,0,255";

our @colors = qw/
             0,255,255
             255,0,0
             255,102,0
             255,0,255
             255,255,255
             0,0,0
             255,255,0
             255,153,153
             0,255,0
             102,51,0
             204,153,102
             51,0,102
             102,0,0
             /;

our $maxwid = 500;
our $maxhgt = 500;

# Programs executed by this script.
our $combine    = "composite";
our $quant      = "convert -colors 256";
our $cjpeg      = 'cjpeg -sample 1x1';
our $pnmcut     = '/data1/www-data/oceancolor.gsfc.nasa.gov/scripts/pnmcut.pl';
our $ppmtogif   = "ppmtogif -quiet";
our $pnmcat     = "pnmcat -quiet";
our $pnmpad     = "pnmpad -quiet -black";
our $pnmpaste   = "pnmpaste -quiet";
our $pnmscale   = "pnmscale -quiet";
our $pnmenlarge = "pnmenlarge -quiet";
our $pnminvert  = "pnminvert -quiet";
our $pnmdepth   = "pnmdepth -quiet 255";
our $pgmtoppm   = "pgmtoppm -quiet";
our $ppmnorm    = "ppmnorm -quiet";
our $pnmtopng   = "pnmtopng -quiet";
our $sharpen    = "pnmnlfilt -0.7 0.8";
our $gmt        = 'gmt';
our $pscoast    = "$gmt pscoast";
our $gmtset     = "$gmt gmtset ANNOT_FONT_SIZE 12 PLOT_DEGREE_FORMAT DF" .
                 " FRAME_WIDTH 0.06 TICK_LENGTH 0.1 BASEMAP_FRAME_RGB 0/0/0" .
                 " FRAME_PEN 1.25p ANNOT_FONT_PRIMARY Helvetica" .
                 " GRID_PEN_PRIMARY 0.25p ANNOT_OFFSET_PRIMARY 0.03i";
our $psbasemap  = "$gmt psbasemap";
our $gs         = 'gs -sDEVICE=pbmraw -sOutputFile=- -q -r100';
our $line       = 'ppmline -quiet';
our $text       = 'ppmtext -quiet';
our $wrscoords  = 'wrscoords';
our $draw_reefs = '/web/oceancolor/metadata/reefs/draw_reefs.pl';
our $draw_changes = '/web/oceancolor/metadata/reefs/draw_changes.pl';

use OBPG::CGI qw(ReadParse);
our %in = ReadParse();

# Filter out shell metacharacters
foreach my $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}

if($ENV{PATH_INFO}){
  # If PATH_INFO is defined, then we should
  # send image data to the client browser.
  $ENV{PATH_INFO} = '/' . launder( substr($ENV{PATH_INFO}, 1) );
  &send_img;
} elsif($in{COLOR}){
  # If COLOR is defined, just send back a single-pixel image
  # having the specified color.
  &send_color;
} else{
  #print STDERR "\n\n###################################################\n";
  #print STDERR "vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv\n";
  #while(($n,$v) = each %in){
    #print STDERR "$n=$v\n";
  #}
  #print STDERR "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n";
  #print STDERR "###################################################\n\n\n";
  &send_hypertext;
}

exit;




sub send_hypertext{

  ###########################################################################
  # First, figure out what to display based on the set of CGI parameters    #
  # that we were given.                                                     #
  ###########################################################################

  my $i1file  = $in{I1FILE}  || $i1file_default;
  my $i1aoi   = $in{I1AOI}   || '';
  my $i1scale = $in{I1SCALE} || '';
  my $i2file  = $in{I2FILE}  || $i2file_default;
  my $i2aoi   = $in{I2AOI}   || '';
  my $i2scale = $in{I2SCALE} || '';

  # Handle a reconfiguration request if one was submitted.
  my @reconfignames = grep /^NEW_/, keys %in;
  my $name;
  foreach $name (@reconfignames){
    my $iname = "$name.x";
    $iname =~ s/^NEW_/NEW/;
    if(defined $in{$iname}){
      ($i1file,$i1aoi,$i1scale,$i2file) = split /\+/, $in{$name};
      $i2scale = '';
      $i2aoi = '';
      delete $in{I2AOI};
      last;
    }
  }

  my $is_default_i1 = 0;
  if($i1file eq $i1file_default){
    $is_default_i1 = 1;
    $i1aoi   = '';	# The default image is always displayed whole
    $i1scale = '';	# and without rescaling.
  }

  my ($i1cmap,$i2cmap) = ('','');
  $i1file eq $i2file_default and $i1cmap = $i2cmap_default;
  $i2file eq $i2file_default and $i2cmap = $i2cmap_default;

  my ($showi2,$direction);

  if(defined $in{'I1.x'}){			# User clicked on "I1" image.
    my $i1x = $in{'I1.x'};
    my $i1y = $in{'I1.y'};
    $i2aoi = point_to_aoi($i1x,$i1y,$i1file,$i1aoi,$i1scale,
                          $i2file,$i2scale,$maxwid,$maxhgt  );
    $showi2 = 1;
  }
  elsif(defined $in{'I1SW.x'}){			# User clicked on "I1SW" image.
    my $i1x = $in{'I1SW.x'};
    my $i1y = $in{'I1SW.y'};
    $i2aoi = point_to_aoi($i1x,$i1y,$i1file,$i1aoi,$i1scale,
                          $i2file,$i2scale,$maxwid,$maxhgt  );
    $in{DGSWM} = 'D';
    $in{DGRBM} = 'R';
    $in{DGDSM} = 'R';
    $showi2 = 1;
  }
  elsif(defined $in{'I1RB.x'}){			# User clicked on "I1RB" image.
    my $i1x = $in{'I1RB.x'};
    my $i1y = $in{'I1RB.y'};
    $i2aoi = point_to_aoi($i1x,$i1y,$i1file,$i1aoi,$i1scale,
                          $i2file,$i2scale,$maxwid,$maxhgt  );
    $in{DGSWM} = 'R';
    $in{DGRBM} = 'D';
    $in{DGDSM} = 'R';
    $in{REEF} = $reef_color;	# Turn on the ReefBase overlay.
    $showi2 = 1;
  }
  elsif(defined $in{'I1DS.x'}){			# User clicked on "I1DS" image.
    my $i1x = $in{'I1DS.x'};
    my $i1y = $in{'I1DS.y'};
    $i2aoi = point_to_aoi($i1x,$i1y,$i1file,$i1aoi,$i1scale,
                          $i2file,$i2scale,$maxwid,$maxhgt  );
    $in{DGSWM} = 'R';
    $in{DGRBM} = 'R';
    $in{DGDSM} = 'D';
    $in{OIMGS} = 1;		# Turn on the "Other images" overlay.
    $showi2 = 1;
  }
  elsif(defined $in{OVL}){
    $showi2 = 1;
  }
  elsif($direction = &read_direction){		# User clicked on an "I2"
    $i2aoi = translate_aoi($i2aoi,$direction);	# image-panning arrow.
    $showi2 = 1;
  }
  elsif(defined $in{ZOOM} and $in{ZFAC} > 0 and $in{ZFAC} != 1){
    my $native_scale = (imginfo($i2file))[6];	# User requested a scale change
    $i2scale or $i2scale = $native_scale;	# for "I2" image.
    $i2scale *= $in{ZFAC};
    $i2aoi = adjust_boundaries($i2aoi,$i2scale,$maxwid,$maxhgt);
    $i2scale == $native_scale and $i2scale = '';
    $showi2 = 1;
  }
  elsif(defined $in{I2AOI}){
    $showi2 = 1;
  }
  else{
    $showi2 = 0;
  }

  # Set $dgrbm to 1 if the global reefbase map is to be displayed.
  # The first line here indicates that it should be displayed on the
  # default webpage produced by this script (i.e. the page that results
  # when no CGI parameters are specified as part of the URL or part
  # of a form's content -- namely, when $in contains no information).
  # The second line allows on/off information to be passed from page
  # to page as the user interacts with the script.
  my $dgrbm = %in ? 0 : 1;
  defined $in{DGRBM} and $in{DGRBM} =~ /^D/ and $dgrbm = 1;

  # Same as above except that these two lines control display
  # of a data set locator image.
  my $dgdsm = %in ? 0 : 1;
  defined $in{DGDSM} and $in{DGDSM} =~ /^D/ and $dgdsm = 1;

  # Once again, but now for the SeaWiFS depth classification image.
  my $dgswm = %in ? 0 : 1;
  defined $in{DGSWM} and $in{DGSWM} =~ /^D/ and $dgswm = 1;

  my $reef  = $in{REEF}  ? $in{REEF}  : '';
  my $creef = $in{CREEF} ? $in{CREEF} : '';
  my $coast = $in{COAST} ? $in{COAST} : '';
  my $wrs   = $in{WRS}   ? $in{WRS}   : '';
  my $frame = $in{FRAME} ? $in{FRAME} : '';
  my $oimgs = $in{OIMGS} ? $in{OIMGS} : '';

  ###########################################################################
  # Next, construct the various different parts of the webpage.             #
  ###########################################################################

  my $webpage = webpage($i1file,$i1aoi,$i1scale,$i1cmap,
                        $i2file,$i2aoi,$i2scale,$i2cmap,
                        $reef,$creef,$coast,$wrs,$frame,$oimgs,
                        $is_default_i1,$dgswm,$dgrbm,$dgdsm,$showi2);

  ###########################################################################
  # Finally, send the HTML to the client browser.                           #
  ###########################################################################

  &head;
  print $webpage;
  &tail;
}

sub webpage{

  my ($i1file,$i1aoi,$i1scale,$i1cmap,
      $i2file,$i2aoi,$i2scale,$i2cmap,
      $reef,$creef,$coast,$wrs,$frame,$oimgs,
      $is_default_i1,$dgswm,$dgrbm,$dgdsm,$showi2   ) = @_;

  my $rect = $showi2 ? "&RECT=$i2aoi,$rect_color" : '';

  my ($i1wid,$i1hgt) = imgdims($i1file,$i1aoi,$i1scale);

  # Make an informative filename in case the user wants to
  # save the image to her local disk.
  my $i1usrfile = $i1file;
  $i1usrfile =~ s/\.[^.]*$/.jpg/;

  my ($i1img,$i2img,$ovl,$gdata,$others);

  if($is_default_i1){

    # The default "I1" images are not scaled or framed.  Nor do they get
    # any sort of overlay other than a possible rectangle to show the
    # area of interest represented by the "I2" image.

    # Construct the image elements of the form along with some controls
    # as to whether or not to display some of the images.
    $i1img = "";

    my ($display_sw,$display_rb,$display_ds);
    if($dgswm){
      # Display global SeaWiFS depth map with a button to remove it.

      $i1img .=
      qq@<table cellpadding="0" cellspacing="0" border="0">\n@ .
      qq@<tr valign="middle">\n@ .
      qq@<td><input type="image" name="I1SW"@ .
      qq@ alt="global composite of SeaWiFS depth classification"@ .
      qq@ src="$thisScript/$i1usrfile?FILE=$i1file$rect"@ .
      qq@ width="$i1wid" height="$i1hgt"></td>\n@ .
      qq@<th>SeaWiFS Bathymetry Composite</th>\n@ .
      qq@</tr></table>\n@;

      $display_sw = 0;
    }
    else{
      $display_sw = 1;
    }

    if($dgrbm){
      # Display global ReefBase map with a button to remove it.

      $i1img .=
      qq@<table cellpadding="0" cellspacing="0" border="0">\n@ .
      qq@<tr valign="middle">\n@ .
      qq@<td><input type="image" name="I1RB"@ .
      qq@ alt="global coral reef locations"@ .
      qq@ src="$thisScript/reefbase.gif?FILE=$smallreefbase$rect"@ .
      qq@ width="$i1wid" height="$i1hgt"></td>\n@ .
      qq@<th>UNEP-WCMC Coral Reef Map</th>\n@ .
      qq@</tr></table>\n@;

      $display_rb = 0;
    }
    else{
      $display_rb = 1;
    }

    if($dgdsm){
      # Display global data set map with a button to remove it.

      $i1img .=
      qq@<table cellpadding="0" cellspacing="0" border="0">\n@ .
      qq@<tr valign="middle">\n@ .
      qq@<td><input type="image" name="I1DS"@ .
      qq@ alt="locations of other earth images"@ .
      qq@ src="$thisScript/datasets.gif?FILE=$locator$rect"@ .
      qq@ width="$i1wid" height="$i1hgt"></td>\n@ .
      qq@<th>Data Locator</th>\n@ .
      qq@</tr></table>\n@;

      $display_ds = 0;
    }
    else{
      $display_ds = 1;
    }

    if($display_sw){
      # Present a button to display the global SeaWiFS depth image.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGSWM"@ .
                qq@ VALUE="Display SeaWiFS depth classification">\n@;
    }
    else{
      # Present a button to remove it.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGSWM"@ .
                qq@ VALUE="Remove SeaWiFS depth classification">\n@ .
                qq@<INPUT TYPE="hidden" NAME="DGSWM" VALUE="D">\n@;
    }
    if($display_rb){
      # Present a button to display the global ReefBase image.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGRBM"@ .
                qq@ VALUE="Display ReefBase map">\n@;
    }
    else{
      # Present a button to remove it.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGRBM"@ .
                qq@ VALUE="Remove ReefBase map">\n@ .
                qq@<INPUT TYPE="hidden" NAME="DGRBM" VALUE="D">\n@;
    }
    if($display_ds){
      # Present a button to display the global data set locator image.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGDSM"@ .
                qq@ VALUE="Display data set locator">\n@;
    }
    else{
      # Present a button to remove it.
      $i1img .= qq@<INPUT TYPE="submit" NAME="DGDSM"@ .
                qq@ VALUE="Remove data set locator">\n@ .
                qq@<INPUT TYPE="hidden" NAME="DGDSM" VALUE="D">\n@;
    }

    %in or $i1img .= <<"    --";
    <p>
    <b>
    Instructions
    </b>
    </p>

    <p>
    The first of the three images above is a scaled-down
    version of the most recent global composite of SeaWiFS
    one-kilometer data that have been passed through
    a depth-classification algorithm. The second image
    shows coral reef locations in red. This image was
    created by the <a href="http://www.unep-wcmc.org/">World
    Conservation Monitoring Centre</a>
    using data from a variety of navigational charts
    and other maps, and is available online through
    the <a href="http://www.reefbase.org/">Reefbase
    Interactive Map Server</a>, or in the published <a
    href="http://www.unep-wcmc.org/marine/coralatlas/">World
    Atlas of Coral Reefs (Spalding et al. 2001)</a>. The
    third image shows the locations of images from a variety
    of sensors which can be viewed with this interface.
    </p>

    <p>
    Click on any of the three images above to view the
    full-resolution SeaWiFS bathymetry data for the location
    clicked upon. You can remove any of the images above by
    clicking on the appropriate buttons.
    </p>

    <ul>
    <li>
    Clicking on the depth image will result in extracts that,
    by default, have no overlays turned on.
    </li>
    <li>
    Clicking on the ReefBase image causes reef locations to
    be overlaid by default.
    </li>
    <li>
    Clicking on the data set locator image causes the outlines
    of other images to be drawn by default.
    </li>
    <li>
    Overlays can still be turned on and off using controls on
    the extract page regardless of which image you originally
    click on this page.
    </li>
    </ul>

    <p>
    <b>
    SeaWiFS Bathymetry Description
    </b>
    </p>

    <p>
    Characterizing water depth in tropical oceans is an
    important first step for improving the identification
    and characterization of coral reefs.  NOAA scientists
    have developed a new bathymetry algorithm,
    originally applied to 4-meter IKONOS imagery (<a
    href="/REFERENCE/LimnolOceanogr2003_Stumpf_et_al.pdf">Stumpf
    et al. 2003b</a>), that could be applied
    to other instruments with blue and green
    bands. The algorithm was used to produce a global
    composite 1-km bathymetry map from using SeaWiFS data (<a
    href="http://eol.jsc.nasa.gov/newsletter/DataFusionInCoralReefMapping/">Robinson
    et al. 2000</a>, Stumpf et al. <a
    href="/REFERENCE/Seawifsspies.pdf">1999</a>, <a
    href="http://eol.jsc.nasa.gov/newsletter/CoastalZone/default.htm">2003a</a>).
    </p>

    <p>
    The value of each pixel in the full-resolution SeaWiFS
    depth classification image is derived from depths computed
    from different SeaWiFS overflights so that cloud pixels
    can be removed and best depth estimates developed that
    eliminate transient effects of sediment, chlorophyll,
    etc. We have used the median measurement (after cloud
    pixels have been removed) to represent each 0.01 by 0.01
    degree location on the map.
    </p>

    <p>
    The most current SeaWiFS-derived depth map was updated on 6
    November 2002 and represents the contribution of 28,937 GAC
    resolution files (all five years of the SeaWiFS mission),
    25,682 recorded LAC-resolution files and 1,804 HRPT station
    files from all around the world. The image below shows how
    many separate SeaWiFS pixels (including cloudy pixels)
    were used to determine each pixel on the final map. As
    of 13 November 2002, no map pixel had fewer than 4 input
    pixels, and many had more than 200.
    </p>

    <p>
    The source data for the bathymetric images displayed
    at this website are available
    <a href="ftp://ocdist5.sci.gsfc.nasa.gov/pub/seawifs_bathymetry/README">here</a>.
    </p>

    <img src="/seawifs/images/depth_bin_tallies.jpg"
    alt="Image showing density of SeaWiFS pixels"
    width="724" height="231">

    <p>
    The global SeaWiFS bathymetry map has been used to
    identify and correct errors in the UNEP-WCMC Reef Map.
    Combining the SeaWiFS bathymetry product with the WCMC
    map has increased map accuracy for use in evaluating the
    global distribution of marine protected areas (Green et
    al. in review).
    </p>

    <p>
    <b>
    Data Archive Proof-of-Concept
    </b>
    </p>

    <p>
    This web interface was also developed as a
    proof of concept for the use of remotely sensed
    earth observations from various platforms in
    the mapping of coral reefs around the globe. A <a
    href="http://eol.jsc.nasa.gov/reefs/">NASA-sponsored
    partnership</a> between remote sensing scientists,
    international agencies and NGOs has developed from this
    prototype, and a full <a href="/cgi/landsat.pl">Landsat
    Coral Reef Data Archive</a> is now available.
    </p>

    <p>
    <b>
    Other data sets displayed by this interface include: 
    </b>

    <ul>
    <li>
    <a href="http://www.vtt.fi/tte/research/tte1/tte14/proj/aisa/">AISA</a>
    </li>
    <li>
    <a href="http://aviris.jpl.nasa.gov/">AVIRIS</a>
    <blockquote>
    AVIRIS Hyperspectral data collection was conducted with a
    NASA high altitude aircraft over the Hawaiian islands in
    April 2000. Preliminary data processing has been completed
    at NASA's Jet Propulsion Laboratory and tapes containing
    <a href="/AVIRIS/aviris_flight_lines.html">104 flight
    lines</a> are being sent to NASA/Goddard Space
    Flight Center for further processing and distribution.
    </blockquote>
    </li>
    <li>
    <a href="http://edcsgs9.cr.usgs.gov/glis/hyper/guide/etopo5">ETOPO5</a>
    </li>
    <li>
    <a href="http://geo.arc.nasa.gov/sge/landsat/landsat.html">Landsat</a>
    </li>
    <li>
    <a href="/cgi/mos_browse.pl">MOS</a>
    </li>
    <li>
    <a href="http://eol.jsc.nasa.gov/">NASA Orbital Photography</a>
    </li>
    <li>
    <a href="http://coralreefwatch.noaa.gov/satellite/">NOAA
    Hotspot maps</a>
    </li>
    <li>
    <a href="http://www.reefbase.org/">ReefBase</a>
    </li>
    <li>
    <a href="http://terraserver-usa.com/">USGS Digital Orthophoto Quadrangles</a> 
    </li>
    </ul>

    </p>

    <p>
    <b>
    This work is a joint project of:
    </b>

    <ul>
    <li>
    Gene Feldman, Norman Kuring, and Bryan Franz
    <br />
    <a href="/seawifs/">SeaWiFS Project</a>
    <br />
    Goddard Space Flight Center
    </li>
    <li>
    Richard P. Stumpf
    <br />
    <a
    href="http://www.nos.noaa.gov/topics/oceans/coralreefs/welcome.html">NOAA
    National Ocean Service</a>
    </li>
    <li>
    Julie A. Robinson
    <br />
    <a href="http://eol.jsc.nasa.gov/reefs/">Earth Observations Laboratory</a>
    <br />
    Johnson Space Center
    </li>
    <li>
    Edmund P. Green
    <br />
    <a href="http://www.unep-wcmc.org/">UNEP-World
    Conservation Monitoring Centre</a>
    </li>
    <li>
    Marco Noordeloos
    <br />
    <a href="http://www.reefbase.org/">ReefBase</a>
    <br />
    <a href="http://www.worldfishcenter.org/">World Fish Center</a>
    </li>
    </ul>

    </p>

    <p>
    <b>
    References:
    </b>

    <ul>

    <li>
    Robinson, J. A., G. C. Feldman, N. Kuring, B. Franz,
    E. Green, M. Noordeloos, and R. P. Stumpf. 2000. Data
    fusion in coral reef mapping: working at multiple scales
    with SeaWiFS and astronaut photography. Proceedings of
    the 6th International Conference on Remote Sensing for
    Marine and Coastal Environments, Vol. 2, pp. 473-483. <a
    href="http://eol.jsc.nasa.gov/newsletter/DataFusionInCoralReefMapping/">http://eol.jsc.nasa.gov/newsletter/DataFusionInCoralReefMapping/</a>
    </li>

    <li>
    Spalding, M. D., C. Ravilious, and E. P. Green.  World
    Atlas of Coral Reefs.  University of California Press,
    Berkeley.  424 pp. <a
    href="http://www.unep-wcmc.org/marine/coralatlas/">http://www.unep-wcmc.org/marine/coralatlas/</a>
    </li>

    <li>
    Stumpf, R.P., G.C. Feldman, N. Kuring, B. Franz,
    E. Green, and J. Robinson. 1999.  SeaWiFS
    spies reefs.  Reef Encounter 26:29-30.  <a
    href="/REFERENCE/Seawifsspies.pdf">http://oceancolor.gsfc.nasa.gov/REFERENCE/Seawifsspies.pdf</a>
    </li>

    <li>
    Stumpf, R. P., K. Holderied, J. A. Robinson,
    G. Feldman, N. Kuring. 2003a. Mapping water
    depths in clear water from space. Coastal Zone
    03, Baltimore, Maryland, 13-17 July 2003. <a
    href="http://eol.jsc.nasa.gov/newsletter/CoastalZone/default.htm">http://eol.jsc.nasa.gov/newsletter/CoastalZone/default.htm</a>
    </li>

    <li>
    Stumpf, R.P. K. Holderied, and
    M. Sinclair. 2003b. Determination
    of water depth with high-resolution
    satellite imagery over variable bottom types. <a
    href="/REFERENCE/LimnolOceanogr2003_Stumpf_et_al.pdf">Limnology
    and Oceanography 48(1, part 2):547-556</a>.
    </li>

    </ul>

    </p>

    <p>
    <b>
    Press Releases
    </b>

    <ul>

    <li>
    <a
    href="http://pao.gsfc.nasa.gov/gsfc/earth/cr/creef_im.html">Landsat
    7 Shows Global Trove of Coral Images/Movies</a> (Goddard
    Space Flight Center, 10/25/2000
    <br />
    Makes available a large number of recent satellite
    images of coral reefs along with several animations and
    flybys. Also, a recent poster called Remote Sensing of
    Coral Reefs is available online at the same website.
    </li>

    <li>
    <a
    href="http://eol.jsc.nasa.gov/newsletter/CoralReefs/">NASA
    Astronaut Photos Contribute to New Coral Reef Atlas</a>
    (Johnson Space Center, 10/22/2001)
    </li>

    </ul>

    </p>

    Last updated 12/4/2003
    --
  }
  else{

    # An image other than the default one for "I1" has been specified in
    # the CGI parameters.

    # Make the filename that the user sees reflect the area of interest.
    $i1aoi and $i1usrfile = replace_boundary_string($i1usrfile,$i1aoi);
    
    $i1img = qq@<CENTER><INPUT TYPE="image" NAME="I1"@ .
             qq@ ALT="$i1usrfile"@ .
             qq@ SRC="$thisScript/$i1usrfile?FILE=$i1file$rect@;
    $i1aoi   and $i1img .= qq@&AOI=$i1aoi@;
    $i1scale and $i1img .= qq@&SCALE=$i1scale@;
    $i1cmap  and $i1img .= qq@&CMAP=$i1cmap@;

    $i1img .= qq@" WIDTH="$i1wid" HEIGHT="$i1hgt">\n@;

    my ($sensor,$date) = (get_name_info($i2file))[0,1];
    if(defined $sensor){
      $i1img .= qq@<BR>\n<B>$sensor@;
      if(defined $date){
        $i1img .= qq@ : $date@;
      }
      $i1img .= qq@</B><BR>\n@;
      if($sensor eq "NASA Space Shuttle"){
        my ($m,$r,$f) = mission_roll_frame($i2file);
        my $url = "http://eol.jsc.nasa.gov/scripts/sseop/photo.pl" .
                  "?mission=$m&roll=$r&frame=$f";
        $i1img .= qq@<A HREF="$url" TARGET="_blank">Detailed photo@ .
                  qq@ information</A> from Johnson Space Center<BR>\n@;
      }
    }
    $i1img .= qq@</CENTER>\n@;
  }

  if($showi2){

    my ($i2wid,$i2hgt) = imgdims($i2file,$i2aoi,$i2scale);

    # Make a name for "I2" image that reflects the displayed area of interest
    # in case the user wants to save the image to their local disk.
    my $i2usrfile = $i2file;
    $i2usrfile =~ s/\.[^.]*$/.jpg/;
    $i2aoi and $i2usrfile = replace_boundary_string($i2usrfile,$i2aoi);

    # Allow the user to view related global data sets in the same
    # area of interest as the "I2" image.
    my $usrfile = 'ETOPO5.' . aoi_to_string($i2aoi) . '.gif';
    my $gscale = $i2scale || (imginfo($i2file))[6];
    $gdata = <<"    --";
    <BR><CENTER><IMG SRC="/icons/line_invert.gif" ALT="separator"></CENTER><BR>
    The following hyperlinks are to data that are global in their scope.
    Clicking on one of these links will download an extract from the
    corresponding data set.  The extract is presented as an image having
    the same scale and covering the same area of interest as the extract
    image above.  A coastline and coordinate frame are always overlaid on
    the downloaded image. The ReefBase and WRS overlays are applied only
    if they have already been applied to the above image.<BR>
    --
    $gdata .= qq@<A HREF="$thisScript/$usrfile?@ .
              qq@FILE=ETOPO5_.90N.90S.180W.180E.ppm&AOI=$i2aoi&SCALE=$gscale@;
    $reef  and $gdata .= qq@&REEF=$reef@;
    $creef and $gdata .= qq@&CREEF=$creef@;
    $wrs   and $gdata .= qq@&WRS=$wrs@;
    $gdata .= qq@&COAST=$coast_color@;	# Always draw the coastline
    $gdata .= qq@&FRAME=1@;		# and lat/lon frame for etopo5 data.
    $gdata .= qq@">Bathymetry (ETOPO5/NOAA)</A><BR>\n@;

    my @hs = sort &get_hotspot_scenes;
    my $hs;
    foreach $hs (@hs[$#hs-5 .. $#hs]){
      my $yyyyddd = substr $hs,2,7;
      my $text = 'NOAA Hotspot Data: ' . datestr($yyyyddd);
      $usrfile = 'HotSpot.'
               . short_datestr($yyyyddd)
               . '.'
               . aoi_to_string($i2aoi)
               . '.gif';
      $gdata .= qq@<A HREF="$thisScript/$usrfile?FILE=$hs@ .
                qq@&AOI=$i2aoi&SCALE=$gscale&CMAP=$hscmap@ .
                qq@&BOT=$hscbar@;
      $reef  and $gdata .= qq@&REEF=$reef@;
      $creef and $gdata .= qq@&CREEF=$creef@;
      $wrs   and $gdata .= qq@&WRS=$wrs@;
      $gdata .= qq@&COAST=$coast_color@;# Always draw the coastline
      $gdata .= qq@&FRAME=1@;		# and lat/lon frame for hotspot data.
      $gdata .= qq@">$text</A> <A HREF="http://www.osdpd.noaa.gov@ .
                qq@/PSB/EPS/SST/climohot.html" @ .
                qq@TARGET="_blank"><FONT SIZE="-1">more info</FONT></A><BR>\n@;
    }
    $gdata .=
    qq@<BR><CENTER><IMG SRC="/icons/line_invert.gif" ALT="separator">@ .
    qq@</CENTER><BR>\n@;

    my $rect = '';
    if($in{OIMGS}){
      # Get a list of other images of the selected area of interest, and
      # draw colored rectangles on the "I2" image indicating where the
      # other scenes' boundaries are.
      my @others = get_aoi_scenes($i2aoi);
      my $f;
      my $num = 0;
      foreach $f (sort @others){
        $f eq $i2file and next;	# Skip the file if it's already displayed.
        my ($sensor,$date,$no,$so,$we,$ea) = get_name_info($f) or next;
        #my $newi1file  = $i2file_default;
        my $newi1file  = $f;
        my $newi1aoi   = "$no,$so,$we,$ea";
        my $newi1scale = choose_scale($newi1aoi,$maxwid,$maxhgt);
        my $newi2file  = $f;
        my $newvalues = join '+',$newi1file,$newi1aoi,$newi1scale,$newi2file;
        my $color = &next_color;
        $rect .= "&RECT=$no,$so,$we,$ea,$color";
        my $str = $sensor;
        $str .= " : $date" if defined $date;
        $others .= qq@<INPUT TYPE="hidden"@ .
                   qq@ NAME="NEW_$num" VALUE="$newvalues">\n@ .
                   qq@<INPUT TYPE="image" SRC="$thisScript?COLOR=$color"@ .
                   qq@ ALT="COLOR=$color"@ .
                   qq@ NAME="NEW$num" WIDTH="24" HEIGHT="8"> $str<BR>\n@;
        $num++;
      }
    }

    if(defined $others){
      $others = <<"      --";
      The following data sets cover the same area or some part of it as the
      image displayed above.  The color of the rectangular bullet next to
      each data set's name corresponds to the color of the rectangle drawn
      (in whole or in part) on the image above.  Click on the colored bullet
      of your choice to view that data set.<BR>
      $others
      <BR><CENTER><IMG SRC="/icons/line_invert.gif" ALT="separator">
      </CENTER><BR>
      --
    }

    my $default_cbar = '';
    $i2file eq $i2file_default and
    $default_cbar = qq@<TD ROWSPAN="3"><IMG@ .
                    qq@ SRC="/seawifs/images/depth_scale.gif"@ .
                    qq@ ALT="depth color scale"@ .
                    qq@ WIDTH="914" HEIGHT="267"></TD>\n@;

    $i2img = qq@<IMG SRC="$thisScript/$i2usrfile?FILE=$i2file$rect@;
    $i2aoi   and $i2img .= qq@&AOI=$i2aoi@;
    $i2scale and $i2img .= qq@&SCALE=$i2scale@;
    $i2cmap  and $i2img .= qq@&CMAP=$i2cmap@;
    $reef    and $i2img .= qq@&REEF=$reef@;
    $creef   and $i2img .= qq@&CREEF=$creef@;
    $coast   and $i2img .= qq@&COAST=$coast@;
    $wrs     and $i2img .= qq@&WRS=$wrs@;
    $frame   and $i2img .= qq@&FRAME=1@;
    $oimgs   and $i2img .= qq@&OIMGS=1@;
    if($frame){
      $i2wid += 130;
      $i2hgt +=  70;
    }
    $i2img .= qq@" WIDTH="$i2wid" HEIGHT="$i2hgt" ALT="$i2usrfile">\n@;

    my $dpp = $i2scale || (imginfo($i2file))[6];
    $dpp = 1/$dpp/100;			# degrees per pixel
    my $sclwid = 15;
    my $sclhgt = 31;
    my $sclimg = qq@<IMG SRC="/icons/scalebar.gif"@ .
                 qq@ ALT="scale bar" WIDTH="$sclwid" HEIGHT="$sclhgt">@;
    my $sclkm = sprintf "%.2g", $dpp * 60 * 1.852 * $sclhgt;
    $dpp = sprintf "%.2g", $dpp;
    my $scltbl = qq@<TABLE>\n@ .
                 qq@<TR><TD>$sclimg</TD>@ .
                 qq@<TD><FONT SIZE="-1">$sclkm km</FONT></TD>@ .
                 qq@<TD><FONT SIZE="-2">@ .
                 qq@(1 pixel = $dpp degrees of latitude)</FONT></TD></TR>@ .
                 qq@</TABLE>@;

    my $explanation = '';

    my ($sensor,$date) = (get_name_info($i2file))[0,1];
    if(defined $sensor){
      $i2img .= qq@<BR>\n$sensor@;
      if(defined $date){
        $i2img .= qq@ : $date@;
      }
      $i2img .= qq@$scltbl\n@;
      $explanation .= <<"      --";
      The image above is a blow-up of the rectangle-enclosed area in
      the image above it.
      --
    }
    elsif($i2file eq $i2file_default){
      $i2img .= qq@<BR>\nSeaWiFS depth classification$scltbl\n@;
      $explanation .= <<"      --";
      <p>
      The image above has been extracted from our most recent composite
      of radiometrically-derived depth classes.  The interpretation of
      the colors in this false-color SeaWiFS product is given in the
      colorbar to the right of the image.
      </p>
      --
    }

    $explanation .= <<"    --";
    <P>
    Colored rectangles (or portions thereof) drawn on the extract indicate
    the presence of other data sets for the same area.  You can click on
    one of the eight black arrows around the extract to move the field
    of view in the indicated direction.  To turn the various overlays
    on or off, just check off the appropriate checkboxes and press "Go".
    To zoom in or out on the extract, select the desired zoom factor
    and press "Zoom".
    --

    $explanation .= qq@<BR><CENTER>@ .
                    qq@<IMG SRC="/icons/line_invert.gif" ALT="separator">@ .
                    qq@</CENTER><BR>\n@;

    $i2img = qq@<TABLE>\n@ .
             qq@<TR>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="NW" SRC="/icons/nw.gif"@ .
             qq@ ALT="move northwest" BORDER="0">@ .
             qq@</TH>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="NO" SRC="/icons/no.gif"@ .
             qq@ ALT="move north" BORDER="0">@ .
             qq@</TH>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="NE" SRC="/icons/ne.gif"@ .
             qq@ ALT="move northeast" BORDER="0">@ .
             qq@</TH>\n@ .
             qq@$default_cbar@ .
             qq@</TR>\n<TR>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="WE" SRC="/icons/we.gif"@ .
             qq@ ALT="move west" BORDER="0">@ .
             qq@</TH>\n@ .
             qq@<TH>$i2img</TH>\n@ .
             qq@<TH>\n@ .
             qq@<INPUT TYPE="image" NAME="EA" SRC="/icons/ea.gif"@ .
             qq@ ALT="move east" BORDER="0">@ .
             qq@</TH>\n</TR>\n<TR>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="SW" SRC="/icons/sw.gif"@ .
             qq@ ALT="move southwest" BORDER="0">@ .
             qq@</TH>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="SO" SRC="/icons/so.gif"@ .
             qq@ ALT="move south" BORDER="0">@ .
             qq@</TH>\n<TH>@ .
             qq@<INPUT TYPE="image" NAME="SE" SRC="/icons/se.gif"@ .
             qq@ ALT="move southeast" BORDER="0">@ .
             qq@</TH>\n</TR>\n</TABLE>\n@ .
             $explanation;

    $ovl = qq@<TABLE WIDTH="100%"><TR><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="COAST" VALUE="$coast_color">@ .
           qq@Coastlines\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="REEF" VALUE="$reef_color">@ .
           qq@ReefBase\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="CREEF" VALUE="$creef_color">@ .
           qq@Corrected Reefs\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="WRS" VALUE="$wrs_color">@ .
           qq@Landsat WRS\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="FRAME" VALUE="1">Frame\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="checkbox" NAME="OIMGS" VALUE="1">Other images\n@ .
           qq@</TD><TD>\n@ .
           qq@<INPUT TYPE="submit" NAME="OVL" VALUE="Go">\n@ .
           qq@</TD></TR></TABLE>\n@;

    if($coast){
      $ovl =~ s/ NAME="COAST" / NAME="COAST" CHECKED /;
      $ovl =~ s/Coastlines/Coastlines <IMG
              SRC="$thisScript?COLOR=$coast_color"
              ALT="color chip ($coast_color)"
              WIDTH="24" HEIGHT="8" BORDER="0">/x;
    }
    if($reef){
      $ovl =~ s/ NAME="REEF" / NAME="REEF" CHECKED /;
      $ovl =~ s/ReefBase/ReefBase <IMG
              SRC="$thisScript?COLOR=$reef_color"
              ALT="color chip ($reef_color)"
              WIDTH="24" HEIGHT="8" BORDER="0">/x;
    }
    if($creef){
      $ovl =~ s/ NAME="CREEF" / NAME="CREEF" CHECKED /;
      $ovl =~ s/Corrected\ Reefs/Corrected Reefs <IMG
              SRC="$thisScript?COLOR=$creef_color"
              ALT="color chip ($creef_color)"
              WIDTH="24" HEIGHT="8" BORDER="0">/x;
    }
    if($wrs){
      $ovl =~ s/ NAME="WRS" / NAME="WRS" CHECKED /;
      $ovl =~ s/Landsat\ WRS/Landsat WRS <IMG
              SRC="$thisScript?COLOR=$wrs_color"
              ALT="color chip ($wrs_color)"
              WIDTH="24" HEIGHT="8" BORDER="0">/x;
    }
    if($frame){
      $ovl =~ s/ NAME="FRAME" / NAME="FRAME" CHECKED /;
    }
    if($oimgs){
      $ovl =~ s/ NAME="OIMGS" / NAME="OIMGS" CHECKED /;
    }

    $zoom = qq@<SELECT NAME="ZFAC" SIZE="1">\n@ .
            qq@<OPTION>0.25</OPTION>\n@ .
            qq@<OPTION>0.33</OPTION>\n@ .
            qq@<OPTION>0.5</OPTION>\n@ .
            qq@<OPTION SELECTED>2</OPTION>\n@ .
            qq@<OPTION>3</OPTION>\n@ .
            qq@<OPTION>4</OPTION>\n@ .
            qq@</SELECT>\n@ .
            qq@<INPUT TYPE="submit" NAME="ZOOM" VALUE="Zoom">\n<P>\n@;

  }
  else{
    $coast and $ovl .= qq@<INPUT TYPE="hidden" NAME="COAST" VALUE="$coast">\n@;
    $reef  and $ovl .= qq@<INPUT TYPE="hidden" NAME="REEF" VALUE="$reef">\n@;
    $creef and $ovl .= qq@<INPUT TYPE="hidden" NAME="CREEF" VALUE="$creef">\n@;
    $wrs   and $ovl .= qq@<INPUT TYPE="hidden" NAME="WRS"  VALUE="$wrs">\n@;
    $frame and $ovl .= qq@<INPUT TYPE="hidden" NAME="FRAME" VALUE="$frame">\n@;
    $oimgs and $ovl .= qq@<INPUT TYPE="hidden" NAME="OIMGS" VALUE="$oimgs">\n@;
  }

  my $html =
  qq@<FORM METHOD="post" ACTION="$thisScript">\n@ .
  qq@<INPUT TYPE="hidden" NAME="I1FILE" VALUE="$i1file">\n@ .
  qq@<INPUT TYPE="hidden" NAME="I2FILE" VALUE="$i2file">\n@;

  $i1aoi   and $html .= qq@<INPUT TYPE="hidden" NAME="I1AOI" VALUE="$i1aoi">\n@;
  $i1scale and $html .= qq@<INPUT TYPE="hidden"@ .
                        qq@ NAME="I1SCALE" VALUE="$i1scale">\n@;
  $i2aoi   and $html .= qq@<INPUT TYPE="hidden" NAME="I2AOI" VALUE="$i2aoi">\n@;
  $i2scale and $html .= qq@<INPUT TYPE="hidden"@ .
                        qq@ NAME="I2SCALE" VALUE="$i2scale">\n@;

  $html .= qq@$i1img$i2img$ovl$zoom$gdata$others</FORM>\n@;
  $in and $html .= qq@<CENTER>\n@ .
                   qq@<A HREF="$thisScript">Return</A> to top-level page.\n@ .
                   qq@</CENTER>\n@;
  $html;
}

sub point_to_aoi{
  my ($x,$y,$i1file,$i1aoi,$i1scale,$i2file,$i2scale,$maxwid,$maxhgt) = @_;

  # Get the native AOI and scale from the "I1" image file.
  my ($n,$s,$w,$e,$wid,$hgt,$scale,$iscolor) = imginfo($i1file);

  # Override the native AOI and/or scale if new ones are provided.
  $i1aoi and ($n,$s,$w,$e) = split /,/, $i1aoi;
  $i1scale or $i1scale = $scale;

  # Use this AOI and scale to compute the latitude and longitude
  # that correspond to $y and $x.
  my ($lat,$lon) = unproject($x,$y,$n,$s,$w,$e,$i1scale);

  # If no scale is provided for the "I2" image, then get the native scale
  # from the "I2" image file.
  $i2scale or $i2scale = (imginfo($i2file))[6];

  # Determine, in degrees, the distance
  # from the center of the "I2" AOI to its edge.
  my $halfhgt = $maxhgt/2/100/$i2scale;
  my $halfwid = $maxwid/2/100/$i2scale;

  # Make sure that the above two numbers fit on our globe.
  $halfhgt =  90 if $halfhgt >  90;
  $halfwid = 180 if $halfwid > 180;

  # Compute the four boundaries.
  $n = $lat + $halfhgt;
  $s = $lat - $halfhgt;
  $w = $lon - $halfwid;
  $e = $lon + $halfwid;

  ($n,$s,$w,$e) = constrain_aoi_bounds($n,$s,$w,$e);

  # Return the boundaries as a comma-delimited string.
  "$n,$s,$w,$e";
}

sub translate_aoi{
  my ($aoi,$direction) = @_;
  my ($n,$s,$w,$e) = split /,/, $aoi;
  my $half_dlat = ($n - $s)/2;
  my $half_dlon = deltalon($w,$e)/2;
  $direction =~ /n/ and $n += $half_dlat, $s += $half_dlat;
  $direction =~ /s/ and $n -= $half_dlat, $s -= $half_dlat;
  $direction =~ /w/ and $w -= $half_dlon, $e -= $half_dlon;
  $direction =~ /e/ and $w += $half_dlon, $e += $half_dlon;
  ($n,$s,$w,$e) = constrain_aoi_bounds($n,$s,$w,$e);
  "$n,$s,$w,$e";
}



sub send_img{
  my $srcimage = "$imgdir/$in{FILE}";
  my $curimage = $srcimage;

  # Get some boundary and scale information from files and global variables.
  my ($n_i,$s_i,$w_i,$e_i,
      $n_o,$s_o,$w_o,$e_o,
      $scale_i,$scale_o,
      $iscolor,$format    ) = &init_params;

  if($n_o != $n_i or $s_o != $s_i or $w_o != $w_i or $e_o != $e_i){
    # Cut out and assemble the desired portions of the source file.
    $curimage = compose_aoi($srcimage,
                            $n_i,$s_i,$w_i,$e_i,
                            $n_o,$s_o,$w_o,$e_o,
                            $scale_i);
  }

  if(!$iscolor and defined $in{CMAP}){
    # Color the image if so requested.
    my $oldimage = $curimage;
    $curimage = color_img($curimage,"$imgdir/$in{CMAP}");
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($scale_i != $scale_o){
    # Scale the image to the desired output scale.
    my $oldimage = $curimage;
    $curimage = scale_img($curimage,$scale_o/$scale_i);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if((get_name_info($in{FILE}))[0] eq 'SeaWiFS'){
    # Normalize and sharpen the SeaWiFS true-color image.
    my $oldimage = $curimage;
    $curimage = enhance_img($curimage);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{WHERE}){
    # Send an image that shows where the various data sets known
    # to this script are located.
    my $oldimage = $curimage;
    my ($r,$g,$b) = split /,/,$in{WHERE};
    my @others = get_aoi_scenes("35,-35,-180,180");
    my $f;
    foreach $f (@others){
      my ($n,$s,$w,$e) = string_to_bounds($f);
      my $oldimage = $curimage;
      $curimage = draw_rect($curimage,
                            $n_o,$s_o,$w_o,$e_o,
                            $n,$s,$w,$e,
                            $scale_o,
                            $r,$g,$b);
      # Delete the old image if it's in the $tmproot directory.
      unlink $oldimage if $oldimage =~ /^$tmproot\// and $curimage ne $oldimage;
    }
  }

  if($in{COAST}){
    # Generate and overlay a coastline on the image.
    my $oldimage = $curimage;
    my ($r,$g,$b) = split /,/,$in{COAST};
    $curimage = draw_coast($curimage,$n_o,$s_o,$w_o,$e_o,$scale_o,$r,$g,$b);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{REEF}){
    # Cut, assemble, and scale Reefbase image and then
    # overlay it on the output image.
    my $oldimage = $curimage;
    my ($r,$g,$b) = split /,/,$in{REEF};
    $curimage = draw_reefs($curimage,$n_o,$s_o,$w_o,$e_o,$scale_o,$r,$g,$b);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{CREEF}){
    # Cut, assemble, and scale changed Reefbase image and then
    # overlay it on the output image.
    my $oldimage = $curimage;
    my ($r,$g,$b) = split /,/,$in{CREEF};
    $curimage = draw_changes($curimage,$n_o,$s_o,$w_o,$e_o,$scale_o,$r,$g,$b);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{WRS}){
    # Identify LANDSAT WRS-2 path and row coordinates on the image.
    my $oldimage = $curimage;
    my ($r,$g,$b) = split /,/,$in{WRS};
    $curimage = draw_wrs($curimage,$n_o,$s_o,$w_o,$e_o,$scale_o,$r,$g,$b);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{RECT}){
    # Draw one or more rectangles on the image
    # around the specified areas of interest.
    my @rects = split /(?:\\|%5C)0/, $in{RECT};
    my $rect;
    foreach $rect (@rects){
      my ($n_r,$s_r,$w_r,$e_r,$r,$g,$b) = split /,/,$rect;
      my $oldimage = $curimage;
      $curimage = draw_rect($curimage,
                            $n_o,$s_o,$w_o,$e_o,
                            $n_r,$s_r,$w_r,$e_r,
                            $scale_o,
                            $r,$g,$b);
      # Delete the old image if it's in the $tmproot directory.
      unlink $oldimage if $oldimage =~ /^$tmproot\// and $curimage ne $oldimage;
    }
  }

  if($in{FRAME}){
    # Put a frame around the image showing latitudes and longitudes.
    my $oldimage = $curimage;
    $curimage = add_frame($curimage,$n_o,$s_o,$w_o,$e_o,$scale_o);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  if($in{BOT}){
    # Tack the specified image onto the bottom of the image we are building.
    my $oldimage = $curimage;
    my $bot = "$imgdir/$in{BOT}";
    $curimage = add_under($curimage,$bot);
    # Delete the old image if it's in the $tmproot directory.
    unlink $oldimage if $oldimage =~ /^$tmproot\//;
  }

  # Convert image to output format and send it to the client browser.
  format_img($curimage,$format);

  # Delete the old image if it's in the $tmproot directory.
  unlink $curimage if $curimage =~ /^$tmproot\//;
}

sub send_color{
  my ($r,$g,$b) = split /,/, $in{COLOR};
  my $color = pack "CCC", $r,$g,$b;
  my $f1 = &tmpname;
  open F,'>',"$f1" or die qq@Could not create file, "$f1". Died@;
  print F "P6\n1 1\n255\n$color";
  close F;
  my $pipe = "$ppmtogif $f1|";
  open P,  $pipe or die qq@Error opening pipe from "$ppmtogif $f1". Died@;
  local($/) = undef;
  my $gif = <P>;
  close P or die qq@Error executing pipe, "$ppmtogif $f1". Died@;
  unlink $f1;
  print "Content-type: image/gif\n\n$gif";
}

sub next_color{
  push @colors, shift @colors;
  $colors[0];
}

sub init_params{
  my ($n_i,$s_i,$w_i,$e_i,$wid,$hgt,$scale_i,$iscolor) = imginfo($in{FILE});
  my ($n_o,$s_o,$w_o,$e_o) = defined $in{AOI} ? (split /,/,$in{AOI})
                                              : ($n_i,$s_i,$w_i,$e_i);
  my $scale_o = $in{SCALE} || $scale_i;
  my $fmt;
  if($ENV{PATH_INFO} =~ /\.gif$/i){
    $fmt = 'gif';
  }
  elsif($ENV{PATH_INFO} =~ /\.jpe?g$/i){
    $fmt = 'jpeg';
  }
  else{
    die qq@Unrecognized image type extension in PATH_INFO variable. @,
        qq@PATH_INFO = "$ENV{PATH_INFO}". Died@;
  }
  ($n_i,$s_i,$w_i,$e_i,$n_o,$s_o,$w_o,$e_o,$scale_i,$scale_o,$iscolor,$fmt);
}

sub compose_aoi{
  my ($f,$n_i,$s_i,$w_i,$e_i,$n_o,$s_o,$w_o,$e_o,$scale) = @_;

  # Start with the longitudes which are more complicated because the globe
  # is unbounded (i.e. periodic) in longitude.
  my ($x1,$x2,$wid1,$wid2,$lp,$rp,$mp);
  if(islonbetween($w_i,$e_i,$w_o)){
    # The western edge of the extract is between the edges of
    # the source image.
    $x1 = deltalon($w_i,$w_o);
    if(islonbetween($w_i,$e_i,$e_o)){
      # Both the western and the eastern edges of the extract lie between
      # the western and eastern edges of the source image.
      if(deltalon($w_i,$e_o) < $x1){
        # The requested extract region wraps from the eastern edge of the
        # source region into the western edge of the source region.
        $x2 = 0;
        $wid1 = deltalon($w_o,$e_i);
        $wid2 = deltalon($w_i,$e_o);
        if(deltalon($w_o,$e_o) > $wid1 + $wid2){
          # The source region does not extend all the way around the globe,
          # so the extract will have a gap in the middle.
          $mp = deltalon($w_o,$e_o) - $wid1 - $wid2;
        }
      }
      else{
        # The region to be extracted does not wrap across the edges of
        # the input region.
        $wid1 = deltalon($w_o,$e_o);
      }
    }
    else{
      # The western edge (but not the eastern) of the extract is between the
      # edges of the source image.
      $wid1 = deltalon($w_o,$e_i);
      $rp = deltalon($e_i,$e_o);
    }
  }
  elsif(islonbetween($w_i,$e_i,$e_o)){
    # The eastern edge (but not the western) of the extract is between the
    # edges of the source image.
    $x1 = 0;
    $wid1 = deltalon($w_i,$e_o);
    $lp = deltalon($w_o,$w_i);
  }
  elsif(islonbetween($w_o,$e_o,$w_i)){
    # The requested extract extends beyond both the western and eastern edges
    # of the source image.
    $x1 = 0;
    $wid1 = deltalon($w_i,$e_i);
    $lp = deltalon($w_o,$w_i);
    $rp = deltalon($e_i,$e_o);
  }
  else{
    # The two regions do not overlap.
    die qq@The requested extract longitudes ($w_o,$e_o) do not overlap the @,
        qq@source image longitudes ($w_i,$e_i). Died@;
  }

  # Next treat the latitudes of the source and extract regions.
  my ($y,$hgt,$tp,$bp);
  if($n_i >= $n_o and $n_o >= $s_i){
    # The northern edge of the extract is between the northern and southern
    # edges of the source region.
    $y = $n_i - $n_o;
    if($n_i >= $s_o and $s_o >= $s_i){
      # Both the northern and southern edges of the extract fall between
      # the northern and southern edges of the source region.
      $hgt = $n_o - $s_o;
    }
    else{
      # Only the northern edge (not the southern) of the extract falls
      # between the northern and southern edges of the source region.
      $hgt = $n_o - $s_i;
      $bp = $s_i - $s_o;
    }
  }
  elsif($n_i >= $s_o and $s_o >= $s_i){
    # Only the southern edge (not the northern) of the extract falls between
    # the northern and southern edges of the source region.
    $y = 0;
    $hgt = $n_i - $s_o;
    $tp = $n_o - $n_i;
  }
  elsif($n_o >= $n_i and $n_i >= $s_o){
    # Both the northern and southern edges of the source region fall between
    # the northern and southern edges of the extract region.
    $y = 0;
    $hgt = $n_i - $s_i;
    $tp = $n_o - $n_i;
    $bp = $s_i - $s_o;
  }
  else{
    # The two regions do not overlap.
    die qq@The requested extract llatitudes ($n_o,$s_o) do not overlap the @,
        qq@source image latitudes ($n_i,$s_i). Died@;
  }

  # Now convert all dimensions from degrees to integral pixels
  # using the provided scale.
  $x1   = int($x1   * 100 * $scale + 0.5);
  $y    = int($y    * 100 * $scale + 0.5);
  $wid1 = int($wid1 * 100 * $scale + 0.5);
  $hgt  = int($hgt  * 100 * $scale + 0.5);
  $x2   = int($x2   * 100 * $scale + 0.5) if defined $x2;
  $wid2 = int($wid2 * 100 * $scale + 0.5) if defined $wid2;
  $lp   = int($lp   * 100 * $scale + 0.5) if defined $lp;
  $rp   = int($rp   * 100 * $scale + 0.5) if defined $rp;
  $mp   = int($mp   * 100 * $scale + 0.5) if defined $mp;
  $tp   = int($tp   * 100 * $scale + 0.5) if defined $tp;
  $bp   = int($bp   * 100 * $scale + 0.5) if defined $bp;

  # Force widths and heights to be at least 1.
  $wid1 == 0 and $wid1 = 1;
  $wid2 == 0 and $wid2 = 1 if defined $wid2;
  $hgt  == 0 and $hgt  = 1;

  # Build the extract a piece at a time if need be.
  my $f1 = &tmpname;
  my $f2 = &tmpname;
  my $f3 = &tmpname;
  my $command = "$pnmcut $x1 $y $wid1 $hgt $f > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  if(defined $x2){
    my $command = "$pnmcut $x2 $y $wid2 $hgt $f > $f2";
    system($command) == 0 or die qq@Error executing command, @,
                                 qq@"$command". Died@;
    if(defined $mp){
      my $command = "$pnmpad -l$mp $f2 > $f3";
      system($command) == 0 or die qq@Error executing command, @,
                                   qq@"$command". Died@;
      rename $f3, $f2 or die qq@Error renaming "$f3" to "$f2". Died@;
    }
    $command = "$pnmcat -lr $f1 $f2 > $f3";
    system($command) == 0 or die qq@Error executing command, @,
                                 qq@"$command". Died@;
    rename $f3, $f1 or die qq@Error renaming "$f3" to "$f1". Died@;
    unlink $f2;
  }

  # Add any necessary padding around the edges of the image.
  my $padargs;
  $padargs .= " -l$lp" if defined $lp;
  $padargs .= " -r$rp" if defined $rp;
  $padargs .= " -t$tp" if defined $tp;
  $padargs .= " -b$bp" if defined $bp;
  if(defined $padargs){
    my $command = "$pnmpad $padargs $f1 > $f2";
    system($command) == 0 or die qq@Error executing command, @,
                                 qq@"$command". Died@;
    rename $f2, $f1 or die qq@Error renaming "$f2" to "$f1". Died@;
  }
  $f1;          # Return the name of the resulting image file.
}

sub color_img{
  my ($f,$cm) = @_;
  my $f1 = &tmpname;
  my $command = "$pgmtoppm -map $cm $f > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub scale_img{
  my ($f,$s) = @_;
  my $f1 = &tmpname;
  my $sc = $s == int($s) ? $pnmenlarge : $pnmscale;
  my $command = "$sc $s $f | $pnmdepth > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub enhance_img{
  my $f = shift;
  my $f1 = &tmpname;
  my $command = "$ppmnorm $f | $sharpen > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub draw_coast{
  my ($f,$n,$s,$w,$e,$scale,$r,$g,$b) = @_;
  my $f1 = &tmpname;

  # Pick a GMT database appropriate to the scale.
  if($scale > 5){
    $gmtD = "-Df";      # full resolution
  }
  elsif($scale > 0.5){
    $gmtD = "-Dh";      # high resolution
  }
  elsif($scale > 0.1){
    $gmtD = "-Di";      # intermediate resolution
  }
  elsif($scale > 0.05){
    $gmtD = "-Dl";      # low resolution
  }
  else{
    $gmtD = "-Dc";      # crude resolution
  }

  # Get rid of any dateline discontinuity in the bounding longitudes.
  # GMT doesn't like those.
  $e += 360 if $w > $e;

  # Compute the center longitude.
  my $clon = ($w + $e)/2;

  # Compute the dimensions of the output image.
  my $wid = int(deltalon($w,$e) * 100 * $scale + 0.5);
  my $hgt = int(      ($n - $s) * 100 * $scale + 0.5);
  my $gmtw = $wid/100;

  # Compose and run the command that generates the coastline image and
  # overlays it in the specified color on the input image.
  my $command;
  my $color_the_overlay;
  if($r != 0 or $g != 0 or $b != 0){
    # Compose an X11-style hexadecimal color specifier
    # from the input red, green, and blue values.
    my $color = sprintf "rgb:%02x/%02x/%02x", $r,$g,$b;
    $color_the_overlay = "| $pnminvert | $pgmtoppm $color";
    $command = "$pscoast -R$w/$e/$s/$n -JQ$clon/$gmtw -P -X0 -Y0 -W $gmtD --PROJ_LENGTH_UNIT=i |" .
               "$gs -g${wid}x$hgt - | $pnmdepth $color_the_overlay |" .
               "$pnmtopng -tran rgb:00/00/00 | $combine - $f $f1";
  }
  else{
    $command = "$pscoast -R$w/$e/$s/$n -JQ$clon/$gmtw -P -X0 -Y0 -W $gmtD --PROJ_LENGTH_UNIT=i |" .
               "$gs -g${wid}x$hgt - | $pnmdepth |" .
               "$pnmtopng -tran rgb:ff/ff/ff | $combine - $f $f1";
  }

  # The following two lines prevent pscoast from writing warnings to
  # standard error.
#  $ENV{HOME} = $tmproot;# Prevents warning:
                        # "GMT Warning: Could not determine home directory!"
  chdir $tmproot;	# Prevents warning:
                        # "GMT Fatal Error: Could not update .gmtcommands file"

  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub draw_reefs{
  my ($f,$n,$s,$w,$e,$scale,$r,$g,$b) = @_;

  my ($br,$bg,$bb) = (255,255,255);
  ($br,$bg,$bb) = (0,0,0) if($r==255 and $g==255 and $b==255);

  my $f1 = &tmpname;

  # Make the Reefbase data have the same boundaries and dimensions as
  # the input image.
  my $command = "$draw_reefs $n $s $w $e $scale $r $g $b $br $bg $bb > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;

  # Overlay the Reefbase data on the input image using the specified color.
  my $f2 = &tmpname;
  $command =
  sprintf "$pnmtopng -tran rgb:%02x/%02x/%02x $f1 | $combine - $f $f2",
  $br,$bg,$bb;
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  unlink $f1 if $f1 =~ /^$tmproot\//;
  $f2;
}

sub draw_changes{
  my ($f,$n,$s,$w,$e,$scale,$r,$g,$b) = @_;

  my ($br,$bg,$bb) = (255,255,255);
  ($br,$bg,$bb) = (0,0,0) if($r==255 and $g==255 and $b==255);

  my $f1 = &tmpname;

  # Make the changed Reefbase data have the same boundaries and dimensions as
  # the input image.
  my $command = "$draw_changes $n $s $w $e $scale $r $g $b $br $bg $bb > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;

  # Overlay the changed Reefbase data on the input image
  # using the specified color.
  my $f2 = &tmpname;
  $command =
  sprintf "$pnmtopng -tran rgb:%02x/%02x/%02x $f1 | $combine - $f $f2",
  $br,$bg,$bb;
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  unlink $f1 if $f1 =~ /^$tmproot\//;
  $f2;
}

sub draw_wrs{
  my ($f,$n,$s,$w,$e,$scale,$r,$g,$b) = @_;
  my $f1 = &tmpname;;
  my ($p1,$p2,$r1,$r2) = path_row_ranges($n,$s,$w,$e);
  my ($path,$row);
  my %wrs;
  my $color = sprintf "rgb:%02x/%02x/%02x", $r,$g,$b;
  my $pipe = "|$line $color $f > $f1";
  open P,$pipe or die qq@Error opening pipe, "$pipe". Died@;
  for($path = $p1; ; $path++){
    $path = 1 if $path > 233;
    for($row = $r1; $row <= $r2; $row++){
      my( $clat, $clon,
         $ullat,$ullon,
         $urlat,$urlon,
         $lllat,$lllon,
         $lrlat,$lrlon ) = wrs_coords($path, $row);

      # Draw the nominal scene outline by connecting the four corners
      # with straight lines.
      my ($ulx,$uly) = project($ullat,$ullon,$n,$s,$w,$e,$scale);
      my ($urx,$ury) = project($urlat,$urlon,$n,$s,$w,$e,$scale);
      my ($lrx,$lry) = project($lrlat,$lrlon,$n,$s,$w,$e,$scale);
      my ($llx,$lly) = project($lllat,$lllon,$n,$s,$w,$e,$scale);
      print P "$ulx,$uly,$urx,$ury\n",
              "$urx,$ury,$lrx,$lry\n",
              "$lrx,$lry,$llx,$lly\n",
              "$llx,$lly,$ulx,$uly\n";

      # Save the center coordinates and path/row designator
      # for each scene, so that we can label the boxes we are
      # drawing.
      $wrs{"$path/$row"} = [$clat,$clon];
    }
    last if $path == $p2;
  }
  close P or die qq@Error executing pipe, "$pipe". Died@;

  # Draw the path/row labels on the image.
  my $h = parse_pnm_header($f1);
  my ($wid,$hgt) = ($h->{width},$h->{height});
  my $f2 = &tmpname;
  $pipe = "|$text $color $f1 > $f2";
  open P,$pipe or die qq@Error opening pipe, "$pipe". Died@;
  my $islabelled = 0;
  my $label;
  foreach $label (keys %wrs){
    my ($lat,$lon) = @{$wrs{$label}};
    my ($x,$y) = project($lat,$lon,$n,$s,$w,$e,$scale);
    $x >= 0 and $x < $wid and $y >= 0 and $y < $hgt and $islabelled = 1;
    print P "$x,$y,$label\n";
  }
  unless($islabelled){
    my ($clat,$clon) = center_coords("$n,$s,$w,$e");
    my $row = lat_to_wrs_row($clat);
    my $path = lon_to_wrs_path($clon,$row);
    my $label = "$path/$row";
    my $x = int($wid/2 + 0.5);
    my $y = int($hgt/2 + 0.5);
    print P "$x,$y,$label\n";
  }
  close P or die qq@Error executing pipe, "$pipe". Died@;
  unlink $f1;
  $f2;
}

sub draw_rect{
  
  my ($f,$n,$s,$w,$e,$nr,$sr,$wr,$er,$scale,$r,$g,$b) = @_;

  # Determine the Y coordinates of the rectangle by comparing latitudes.
  if($nr < $s or $sr > $n){
    # The desired rectangle does not overlap the image.
    return $f;
  }
  my ($x1,$x2,$x3,$x4,$y1,$y2);
  my ($north_edge,$south_edge) = (1,1);
  if($nr < $n){
    # The northern edge of the rectangle is between the northern and southern
    # edges of the image.
    $y1 = $n - $nr;
  }
  else{
    # The northern edge of the rectangle is north of the northern
    # edge of the image.
    $y1 = 0;
    $north_edge = 0;
  }
  if($sr > $s){
    # The southern edge of the rectangle is between the northern and southern
    # edges of the image.
    $y2 = $n - $sr;
  }
  else{
    # The southern edge of the rectangle is south of the southern
    # edge of the image.
    $y2 = $n - $s;
    $south_edge = 0;
  }

  # Determine the X coordinates of the rectangle by comparing longitudes.
  if(islonbetween($w,$e,$wr)){
    # The western edge of the rectangle is between the western and eastern
    # edges of the image.
    $x1 = deltalon($w,$wr);
    if(islonbetween($w,$e,$er)){
      # Both western and eastern edges of the recgtangle are between
      # the edges of the image.
      if(deltalon($w,$er) < $x1){
        # The specified rectangle wraps across the dateline.
        $x2 = deltalon($w,$e);
        $x3 = 0;
        $x4 = deltalon($w,$er);
      }
      else{
        # The specified rectangle does not wrap across the dateline.
        $x2 = deltalon($w,$er);
      }
    }
    else{
      # Only the western edge of the recgtangle falls within the image.
      $x2 = deltalon($w,$e);
    }
  }
  elsif(islonbetween($w,$e,$er)){
    # Only the eastern edge of the rectangle is between the western and eastern
    # edges of the image.
    $x1 = 0;
    $x2 = deltalon($w,$er);
  }
  elsif(islonbetween($wr,$er,$w)){
    # The entire image lies between the desired eastern and western edges of
    # the rectangle.
    $x1 = 0;
    $x2 = deltalon($w,$e);
  }
  else{
    # The desired rectangle does not overlap the image.
    return $f;
  }

  # Convert from degrees to numbers of pixels.
  $y1 = int($y1 * 100 * $scale + 0.5);
  $y2 = int($y2 * 100 * $scale + 0.5) - 1;
  $x1 = int($x1 * 100 * $scale + 0.5);
  $x2 = int($x2 * 100 * $scale + 0.5) - 1;
  $x3 = int($x3 * 100 * $scale + 0.5)     if defined $x3;
  $x4 = int($x4 * 100 * $scale + 0.5) - 1 if defined $x4;

  # Draw the sides of the rectangle.
  my $color = sprintf "rgb:%02x/%02x/%02x", $r,$g,$b;
  my $f1 = &tmpname;
  my $command = "$line -line $x1,$y1,$x1,$y2";
  $command   .= " -line $x1,$y1,$x2,$y1" if $north_edge;
  $command   .= " -line $x1,$y2,$x2,$y2" if $south_edge;
  if(defined $x3){
    $command .= " -line $x4,$y1,$x4,$y2";
    $command .= " -line $x3,$y1,$x4,$y1" if $north_edge;
    $command .= " -line $x3,$y2,$x4,$y2" if $south_edge;
  }
  else{
    $command .= " -line $x2,$y1,$x2,$y2";
  }
  $command .= " $color $f > $f1";
  system($command) == 0 or die qq@Error executing command, "$command" ($?)@;
  $f1;
}

sub add_frame{
  my ($f,$n,$s,$w,$e,$scale) = @_;

  # Get rid of any dateline discontinuity in the bounding longitudes.
  # GMT doesn't like those.
  $e += 360 if $w > $e;

  # Compute the center longitude.
  my $clon = ($w + $e)/2;

  # Compute the dimensions of the output image.
  my $wid = int(deltalon($w,$e) * 100 * $scale + 0.5);
  my $hgt = int(      ($n - $s) * 100 * $scale + 0.5);
  my $gmtw = $wid/100;

  # Choose an annotation interval that is an even multiple of a minute of arc.
  # The bitmap representations of the latitudes or longitudes get to be too
  # wide if they must include degrees, minutes, *and* seconds.
  my $ann = int(60/$scale) || 1;
  my $tick = $ann/10;
  $wid += 130;
  $hgt += 70;

  # Compose and run the command that generates the coastline image and
  # overlays it in the specified color on the input image.
  my $f1 = &tmpname;
  my $command = "$gmtset; $psbasemap -R$w/$e/$s/$n -JQ$clon/$gmtw " .
                "-P -X0.65 -Y0.35 -B${ann}mf${tick}m --PROJ_LENGTH_UNIT=i |" .
                "$gs -g${wid}x$hgt - |" .
                "$pnmpaste $f 65 35 - > $f1";

  # The following two lines prevent pscoast from writing warnings to
  # standard error.
#  $ENV{HOME} = $tmproot;# Prevents warning:
                        # "GMT Warning: Could not determine home directory!"
  chdir $tmproot;	# Prevents warning:
                        # "GMT Fatal Error: Could not update .gmtcommands file"

  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub add_under{
  my ($f,$bot) = @_;
  my $f1 = &tmpname;
  my $command = "$pnmcat -tb $f $bot > $f1";
  system($command) == 0 or die qq@Error executing command, "$command". Died@;
  $f1;
}

sub format_img{
  my ($f,$format) = @_;
  my $pipe;

  if($format eq 'gif'){
    $pipe = "$quant $f -|$ppmtogif|";
  }
  elsif($format eq 'jpeg'){
    $pipe = "$cjpeg $f|";
  }
  else{
    die qq@Unrecognized image format ($format). Died@;
  }
  open P,$pipe or die qq@Could not open pipe, "$pipe". Died@;
  local($/) = undef;
  my $img = <P>;
  close P or die qq@Error executing pipe, "$pipe". Died@;
  my $len = length $img;
  print "Content-type: image/$format\nContent-length: $len\n\n$img";
}

# Construct a temporary filename.  This function returns a different
# name each time it is called.
sub tmpname{
  my $time = (split /\//,$0)[-1] . ".$^T.$$";
  my $tmpbase = "$tmproot/" . $time;
  # Increment a global counter.
  $tmpname_counter++;
  "$tmpbase.$tmpname_counter.pnm";
}

sub imgdims{
  my($file,$aoi,$scale) = @_;
  my ($wid,$hgt);
  if($aoi and $scale){
    my ($n,$s,$w,$e) = split /,/, $aoi;
    $wid = int(deltalon($w,$e) * 100 * $scale + 0.5);
    $hgt = int(      ($n - $s) * 100 * $scale + 0.5);
  }
  else{
    # If either $aoi or $scale are not provided, then they must
    # be determined by reading the file.
    my ($n,$s,$w,$e,$wi,$ht,$scl,$iscolor) = imginfo($file);
    if($aoi or $scale){
      $scale or $scale = $scl;
      $aoi and ($n,$s,$w,$e) = split /,/, $aoi;
      $wid = int(deltalon($w,$e) * 100 * $scale + 0.5);
      $hgt = int(      ($n - $s) * 100 * $scale + 0.5);
    }
    else{
      # Use the dimensions of the input file unchanged.
      ($wid,$hgt) = ($wi,$ht);
    }
  }
  ($wid,$hgt);
}

sub choose_scale{
  my ($aoi,$maxwid,$maxhgt) = @_;
  my ($n,$s,$w,$e) = split /,/, $aoi;
  my $xscale = $maxwid/deltalon($w,$e)/100;
  my $yscale = $maxhgt/      ($n - $s)/100;
  $xscale < $yscale ? $xscale : $yscale;
}

sub imginfo{
  my $file = shift;
  my $path = fullpath($file);
  my ($n,$s,$w,$e) = string_to_bounds($file) or die
                     qq@Could not parse boundary coordinates from @,
                     qq@filename, "$file". Died@;
  my $h = parse_pnm_header($path) or die qq@Error parsing header of file, @,
                                         qq@"$path". Died@;
  my ($wid,$hgt) = map launder($_), ($h->{width},$h->{height});
  my $scale = $hgt/($n - $s)/100;
  my $iscolor = $h->{magic} =~ /^P[36]$/;
  ($n,$s,$w,$e,$wid,$hgt,$scale,$iscolor);
}

sub get_name_info{
  my $f = shift;
  my ($sensor,$date,$n,$s,$w,$e);
  if($f =~ /^S\d{13}/){
    $sensor = "SeaWiFS";
    $date = substr $f,1,7;
    $date = datestr($date);
  }
  elsif($f =~ /^M\d{7}/){
    $sensor = "MOS";
    $date = substr $f,1,7;
    $date = datestr($date);
  }
  elsif($f =~ /^TM\d{7}/){
    $sensor = "LANDSAT Thematic Mapper";
    $date = substr $f,2,7;
    $date = datestr($date);
  }
  elsif($f =~ /^STS\d{7}/){
    $sensor = "NASA Space Shuttle";
    $date = substr $f,3,7;
    $date = datestr($date);
  }
  elsif($f =~ /^BH/){
    $sensor = "NOAA benthic habitats map";
    $date = undef;
  }
  elsif($f =~ /^NA/){
    $sensor = "NOAA airborne imager";
    $date = substr $f,2,7;
    $date = datestr($date);
  }
  elsif($f =~ /^AV/){
    $sensor = "AVIRIS";
    $date = substr $f,2,7;
    $date = datestr($date);
  }
  elsif($f =~ /^USGS/){
    $sensor = "USGS Aircraft";
    $date = substr $f,4,7;
    $date = datestr($date);
  }
  else{
    return ();
  }
  ($n,$s,$w,$e) = string_to_bounds($f);
  ($sensor,$date,$n,$s,$w,$e);
}

sub mission_roll_frame{
  my $file = shift;
  my ($mission,$roll,$frame) = (0,0,0);
  if($file =~ /^STS\d{7}/){
    my ($m,$r,$f) = $file =~ /^STS\d{7}\.([^-]*)-([^-]*)-([^.]*)/;
    $mission = sprintf "STS%03d", $m;
    $roll = $r;
    ($frame = $f) =~ s/x|_$//;
  }
  ($mission,$roll,$frame);
}

sub fullpath{
  my $file = shift;
  "$imgdir/$file";
}

sub datestr{
  my $yyyyddd = shift;
  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
  my @wday = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
  my ($y,$d) = unpack "a4a3",$yyyyddd;
  use Time::Local;
  my $time = timegm(0,0,0,1,0,$y-1900) + ($d-1)*86400;
  my ($mday,$mon,$wday) = (gmtime($time))[3,4,6];
  "$wday[$wday], $mday $mon[$mon] $y";
}

sub short_datestr{
  my $yyyyddd = shift;
  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
  my ($y,$d) = $yyyyddd =~ /^(\d{4})(\d{3})/;
  use Time::Local;
  my $time = timegm(0,0,0,1,0,$y-1900) + ($d-1)*86400;
  my ($mday,$mon) = (gmtime($time))[3,4];
  $mday = $1 if $mday =~ /^(\d+)$/;
  $mon  = $1 if $mon  =~ /^(\d+)$/;
  sprintf "%02d$mon[$mon]$y", $mday;
}

# Parse geographical boundaries from a specially-formatted string.
sub string_to_bounds{
  my $string = shift;
  my ($n,$nh,$s,$sh,$w,$wh,$e,$eh) = $string =~
     /(?:[^\d]|\d{4,})
     \.(\d+(?:\.\d+)?)([NS])
     \.(\d+(?:\.\d+)?)([NS])
     \.(\d+(?:\.\d+)?)([EW])
     \.(\d+(?:\.\d+)?)([EW])
     /x;
  defined $n or return ();
  $n = $nh eq "N" ? $n : -$n;
  $s = $sh eq "N" ? $s : -$s;
  $w = $wh eq "E" ? $w : -$w;
  $e = $eh eq "E" ? $e : -$e;
  $n > 90 || $n < -90 || $s > 90 || $s < -90 and return ();
  $w += 360 while($w < -180);
  $w -= 360 while($w >  180);
  $e += 360 while($e < -180);
  $e -= 360 while($e >  180);
  ($n,$s,$w,$e);
}

sub aoi_to_string{
  my $aoi = shift;
  bounds_to_string(split /,/, $aoi);
}

sub bounds_to_string{
  my ($n,$s,$w,$e) = @_;
  $w += 360 while($w < -180);
  $w -= 360 while($w >  180);
  $e += 360 while($e < -180);
  $e -= 360 while($e >  180);
  my ($nh,$sh,$wh,$eh);
  ($n,$nh) = $n < 0 ? (-$n, 'S') : ($n, 'N');
  ($s,$sh) = $s < 0 ? (-$s, 'S') : ($s, 'N');
  ($w,$wh) = $w < 0 ? (-$w, 'W') : ($w, 'E');
  ($e,$eh) = $e < 0 ? (-$e, 'W') : ($e, 'E');
  "$n$nh.$s$sh.$w$wh.$e$eh";
}

sub replace_boundary_string{
  my ($string,$aoi) = @_;
  my ($n,$s,$w,$e) = split /,/, $aoi;
  my $bstr = bounds_to_string($n,$s,$w,$e);
  $string =~ s/([^\d]|\d{4,})
              \.\d+(?:\.\d+)?[NS]
              \.\d+(?:\.\d+)?[NS]
              \.\d+(?:\.\d+)?[EW]
              \.\d+(?:\.\d+)?[EW]
              /$1.$bstr/x;
  $string;
}

sub get_aoi_scenes{
  my $aoi = shift;
  my ($n,$s,$w,$e) = split /,/, $aoi;
  opendir D,$imgdir or die qq@Could not open directory, "$imgdir". Died@;
  my @files = grep /\.p[gp]m$/, readdir D;
  closedir D;
  my ($f,@scenes);
  foreach $f (@files){
    my ($fn,$fs,$fw,$fe) = string_to_bounds($f) or next;
    push @scenes,$f if isovrlap($n,$s,$w,$e,$fn,$fs,$fw,$fe);
  }
  @scenes;
}

sub get_hotspot_scenes{
  opendir D,$imgdir or die qq@Could not open directory, "$imgdir". Died@;
  my @files = grep s/^(HS\d{7}[.NSWE0-9]*\.pgm)$/$1/, readdir D;
  closedir D;
  @files;
}

sub isovrlap{
  my ($n1,$s1,$w1,$e1,$n2,$s2,$w2,$e2) = @_;
  $n1 < $s2 and return 0;
  $n2 < $s1 and return 0;
  islonbetween($w1,$e1,$w2) and return 1;
  islonbetween($w1,$e1,$e2) and return 1;
  islonbetween($w2,$e2,$w1) and return 1;
  return 0;
}

sub islonbetween{
  my ($west,$east,$lon) = @_;
  deltalon($west,$east) >= deltalon($lon,$east);
}

sub deltalon{
  my ($west,$east) = @_;
  $west += 360 while $west < -180;
  $west -= 360 while $west >  180;
  $east += 360 while $east < -180;
  $east -= 360 while $east >  180;
  $west > $east ? 360 + $east - $west : $east - $west;
}

sub constrain_aoi_bounds{
  my ($n,$s,$w,$e) = @_;
  # Make sure that latitudes and longitudes of the boundaries
  # of the area of interest are in the proper ranges.
  if($n > 90){
    $s -= $n - 90;
    $n = 90;
  }
  elsif($s < -90){
    $n += -90 - $s;
    $s = -90;
  }
  $w += 360 while $w < -180;
  $w -= 360 while $w >  180;
  $e += 360 while $e < -180;
  $e -= 360 while $e >  180;
  ($n,$s,$w,$e);
}

sub path_row_ranges{
  my ($n,$s,$w,$e) = @_;
  my($p1,$p2,$r1,$r2);
  $r1 = lat_to_wrs_row($n);
  $r2 = lat_to_wrs_row($s);
  $p1 = lon_to_wrs_path($e,$r2);
  $p2 = lon_to_wrs_path($w,$r1);
  ($p1,$p2,$r1,$r2);
}

sub lat_to_wrs_row{
  my $lat = shift;
  my $row;
  if($lat > 80.7833){
    $row = 1;
  }
  elsif($lat < -81.85){
    $row = 122;
  }
  else{
    # I got the following relationship by reverse engineering
    # the code in wrscoords.c .
    my $a = -$pi/124;
    my $b = -15 * $pi / 31;
    my $c = sin(98.2098 * $pi / 180.0);
    my $d = 6378.137 * 6378.137 * $c / (6356.752314 * 6356.752314);
    my $e = tan($lat * $pi / 180.0);
    $row = int((asin($e/sqrt($d*$d + $c*$c*$e*$e)) + $b) / $a + 0.5);
  }
  $row;
}

sub lon_to_wrs_path{
  my ($lon,$row) = @_;
  my $path;
  my $gamma = ($row - 184) * $pi / 124;
  my $theta = atan2(-cos(98.2098 * $pi/180.0) * sin($gamma), cos($gamma));
  $path = ((103.03948 - $lon) * $pi/180.0 - $theta
          - $gamma*0.25/3.640625) * 233/(2*$pi) + 1;
  $path = $path < 0 ? int($path - 0.5) : int($path + 0.5);
  $path += 233 while($path <   1);
  $path -= 233 while($path > 233);
  $path;
}

sub wrs_coords{
  my ($path,$row) = @_;
  map launder($_), split " ",`$wrscoords $path $row`;
}

sub project{
  # Compute the Plate Carree projection.
  my ($lat,$lon,$n,$s,$w,$e,$scale) = @_;

  my $y = ($n - $lat)*100*$scale;
  # Round to the nearest integer.
  $y = $y >= 0 ? int($y + 0.5) : int($y - 0.5);

  my $x;
  if(islonbetween($w,$e,$lon)){
    $x = int(deltalon($w,$lon)*100*$scale + 0.5);
  }
  else{
    my $left  = deltalon($lon,$w);
    my $right = deltalon($e,$lon);
    if($left <= $right){
      $x = int(-$left*100*$scale - 0.5);
    }
    else{
      $x = int(deltalon($w,$lon)*100*$scale + 0.5);
    }
  }
  ($x,$y);
}

sub unproject{
  # Compute the inverse Plate Carree projection.
  my ($x,$y,$n,$s,$w,$e,$scale) = @_;
  my $lon = $w + 1/200/$scale + $x/100/$scale;
  $lon += 360 while $lon < -180;
  $lon -= 360 while $lon >  180;
  my $lat = $n - 1/200/$scale - $y/100/$scale;
  ($lat,$lon);
}

sub read_direction{
  defined $in{'NW.x'} and return 'nw';
  defined $in{'NO.x'} and return 'n';
  defined $in{'NE.x'} and return 'ne';
  defined $in{'WE.x'} and return 'w';
  defined $in{'EA.x'} and return 'e';
  defined $in{'SW.x'} and return 'sw';
  defined $in{'SO.x'} and return 's';
  defined $in{'SE.x'} and return 'se';
  undef;
}

sub adjust_boundaries{
  my ($aoi,$scale,$wid,$hgt) = @_;
  my ($n,$s,$w,$e);
  my ($clat,$clon) = center_coords($aoi);
  my $dlat = $hgt/2/$scale/100;
  my $dlon = $wid/2/$scale/100;
  $n = $clat + $dlat;
  $s = $clat - $dlat;
  $w = $clon - $dlon;
  $e = $clon + $dlon;
  $w += 360 while($w < -180);
  $w -= 360 while($w >  180);
  $e += 360 while($e < -180);
  $e -= 360 while($e >  180);
  "$n,$s,$w,$e";
}

sub center_coords{
  my $aoi = shift;
  my ($n,$s,$w,$e) = split /,/, $aoi;
  my $clat = ($n + $s)/2;
  $e += 360 if $w > $e;
  my $clon = ($w + $e)/2;
  ($clat,$clon);
}

sub parse_pnm_header{
  my $pnmfile = shift;
  my $h;
  open F,'<',$pnmfile or return undef;
  my $bytes;
  read F,$bytes,4096 or return undef;
  close F;
  $bytes =~ /^P[1-6][\s#]/ or return undef;
  my $b = $bytes;       # Keep a pristine copy to get the unmunged header from.
  $bytes =~ s/#.*/' ' x length $&/emg;  # Replace comments with whitespace.
  ($h->{magic},$h->{width},$h->{height},$h->{maxval}) = split ' ', $bytes;
  if($h->{magic} eq 'P1' || $h->{magic} eq 'P4'){
    $h->{maxval} = 1;
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s/;
    $h->{hdrlen} = length $&;
  }
  else{
    $bytes =~ /$h->{magic}\s+$h->{width}\s+$h->{height}\s+$h->{maxval}\s/;
    $h->{hdrlen} = length $&;
  }
  $h->{header} = substr $b,0,$h->{hdrlen};
  $h->{header} =~ s/\s$/\n/;
  $h;
}

sub head{
  my $title = 'Remote Sensing of Coral Reefs';
  my $subtitle = 'SeaWiFS Bathymetry and Data Archive Proof-of-Concept';

  print
  qq@Content-type: text/html\n\n@,
  qq@<html lang="en">\n@,
  qq@<head>\n@,
  qq@<title>$title</title>\n@,
  qq@</head>\n@,
  qq@<body bgcolor="#FFFFFF">\n@,
  qq@<center>\n@,
  qq@<b>$title :</b>\n@,
  qq@<br />\n@,
  qq@<b>$subtitle</b>\n@,
  qq@</center>\n@;
}

sub tail{
  print << "  --";
  <center>
  <table width="100%">
  <tr align="center">
  <td><a href="http://eol.jsc.nasa.gov/reefs/"><img
  src="/icons/reefs_at_NASA.png" width="308 height="100" border="0"
  alt=""></td>
  <td><a href="/seawifs/"><img
  src="/icons/biosphere_icon.png" width="120 height="60" border="0" 
  alt=""></td>
  </tr>
  <tr align="center">
  <th><a href="http://eol.jsc.nasa.gov/reefs/">Remote Sensing
  of Coral Reefs</a></th>
  <th><a href="/seawifs/">SeaWiFS Project Home Page</a></th>
  </tr>
  </table>
  <img src="/icons/line_invert.gif" alt=""><br />
  <a href="/staff/gene/">gene carl feldman</a>
  (<a href="mailto:gene\@seawifs.gsfc.nasa.gov">gene\@seawifs.gsfc.nasa.gov</a>)
  (301) 286-9428<br />
  Norman Kuring
  (<a href="mailto:norman\@seawifs.gsfc.nasa.gov">norman\@seawifs.gsfc.nasa.gov</a>)
  (301) 286-2264
  </center>
  --
  print qq@</body>\n@,
        qq@</html>\n@;
}
