#!/usr/bin/env perl
# This little script will take images from a directory and create pretty web pages...
# Assumes there are thumbnails in the same directory as the images with the
# format of fullsizeimage.small.(gif|jpg)
#
#	query options:
#		TYP = (global,ratio,difference,diffhist,ratiohist,comphist,scatter,gainloss)
#		CASE = (pertest,perproduct)
#		
#
#
# Norman Kuring		15-Aug-2002	This is Sean Bailey's script.  I just
#					moved it to the new Linux webserver
#					which entailed a few minor pathname
#					changes.

use strict;
use CGI qw(-no_xhtml :all);
#use CGI::Carp qw/fatalsToBrowser/;

our %tests = (11 => 'Operational',
	1 => 'Add MOBY Straylight Fix',
	2 => 'Add Exponential Time-Dependence',
	3 => 'Change Out-of-Band Correction',
	4 => 'Add Miscellaneous Repro #4 Change',
	5 => 'Switch to Arnone NIR Correction',
	6 => 'Eliminate Glint Masking',
	7 => 'Change Cloud Flag Algorithm',
	8 => 'Reduce Straylight Masking',
	#9 => 'Add Epsilon-Targeted Smoothing of Lt(765)',
	10 => 'Cumulative Change Repro #4 versus Repro #3',
	9 => 'Fresnel Reflectance Correction',);
	
our %dtypes = ('MO'=>'Monthly',
		'8D'=>'Weekly',
		'DAY'=>'Daily');

our @testorder = (0,1,2,9,3,4,5,6,7,17);

our %scaling = (chlor => {global => '0.01 - 64 mg/m<sup>3</sup>',
			difference => '-0.2 - 0.2 mg/m<sup>3</sup>'},
	       angstrom => {global => '-0.1 - 1.48',
	       		difference => '-0.15 - 0.15'},
	       tau => {global => '0.0 - 0.5',
	       		difference => '0.05 - 0.05'},
	       nLw_412 => {global => '0.0 - 5.0 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.4 - 0.4 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_443 => {global => '0.0 - 3.8 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.3 - 0.3 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_490 => {global => '0.0 - 2.5 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.2 - 0.2 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_510 => {global => '0.0 - 2.5 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.15 - 0.15 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_555 => {global => '0.0 - 1.3 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.1 - 0.1 &micro;W/cm<sup>2</sup>/nm/sr'},
	       );

our %prodlst = (
	chlor => 'Chlorophyll',
	nLw_412 => 'L<sub>wn</sub><br>(412 nm)<br>',
	nLw_443 => 'L<sub>wn</sub><br>(443 nm)<br>',
	nLw_490 => 'L<sub>wn</sub><br>(490 nm)<br>',
	nLw_510 => 'L<sub>wn</sub><br>(510 nm)<br>',
	nLw_555 => 'L<sub>wn</sub><br>(555 nm)<br>',
	angstrom => 'Angstrom<br> (510:865)<br>',
	tau => 'Aerosol Optical Thickness<br> at 870 nm<br>',
);
our %caselabels = ('PERTEST' => 'By Change', 'PERPROD' => 'By Product','TESTPROD' => 'By Product and Change');
our %analysislabels = (
	'global' => 'Product Images',
	'ratio' => 'Ratio Images',
	'difference' => 'Difference Images',
	'ratiohist' => 'Ratio Histograms',
	'diffhist' => 'Difference Histograms',
	'scatter' => 'Scatter Plots',
	'comphist' => 'Comparison Histograms',
);
our %LAC = ('HNSG'=>'North East Coast U.S.',
	'HUSF' => 'Gulf of Mexico',
	'LAC' => 'Hawaii',
	'HCHL' => 'Patagonian Shelf',);
	
our $baseimgdir = '/reprocessing/r2002/seawifs';
our $base = '/data1/www-data/oceancolor.gsfc.nasa.gov/static/reprocessing/r2002/seawifs';             
our $img_query = $ENV{'QUERY_STRING'};

our @query_arr = split /\&/,$img_query;
our %query = ();
$query{LEVEL} = 'L3BIN';
foreach our $element (@query_arr){
	our ($key,$value) = split /=/,$element;
	$query{$key} = $value;
}
our $product = $query{PROD};
our $testnum = $query{T};
our $imgdir = join('/',$baseimgdir,$query{TYP});
our $basedir = join('/',$base,$query{TYP});
our (@files,@ratfiles);
our @subdirs = ('global','ratio','difference','ratiohist','diffhist','scatter','comphist');
our %case = ('global'=>'ALGTEST',
	'ratio' => 'ratio\.',
	'difference' => 'diff\.',
	'ratiohist' => 'ratio_hist',
	'diffhist' => 'diff_hist',
	'scatter' => 'scatt',
	'comphist' => 'comp_hist',
	'gainloss' => 'gain_loss',
	'SMP_gainloss' => 'smp_gain_loss',
	);
if ($query{LEVEL} eq 'L2'){ 
	$basedir .= '/LAC';
	$imgdir .= '/LAC';
@testorder = (0,1,2,9,3,4,5,6,6,17,17);
%scaling = (chlor => {global => '0.01 - 64 mg/m<sup>3</sup>',
			difference => '-0.2 - 0.2 mg/m<sup>3</sup>'},
	       angstrom => {global => '-0.2 - 1.5',
	       		difference => '-0.15 - 0.15'},
	       tau => {global => '0.0 - 0.3',
	       		difference => '0.05 - 0.05'},
	       nLw_412 => {global => '0.0 - 3.0 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.4 - 0.4 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_443 => {global => '0.0 - 3 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.3 - 0.3 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_490 => {global => '0.0 - 2 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.2 - 0.2 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_510 => {global => '0.0 - 1.5 &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.15 - 0.15 &micro;W/cm<sup>2</sup>/nm/sr'},
	       nLw_555 => {global => '0.0 - 1. &micro;W/cm<sup>2</sup>/nm/sr',
	       		difference => '-0.1 - 0.1 &micro;W/cm<sup>2</sup>/nm/sr'},
	       );
}
our $teststr = &teststring($query{TYP},$testnum,$query{LEVEL});
if ($query{CASE} eq 'PERTEST'){
	opendir DIR, "$basedir";
	@files = grep /(gif|jpg)$/i, readdir DIR;
	@files = grep /$teststr/i,@files;
	@files = grep /small/,@files unless ($query{TYP} =~ /(ratio|diff)hist/);
	close DIR;
}
elsif ($query{CASE} eq 'TESTPROD'){
	foreach our $subdir (@subdirs){
		$teststr = &teststring($subdir,$testnum,$query{LEVEL});
		
		if ($query{LEVEL} eq 'L2'){$subdir .= '/LAC'}
		opendir DIR, "$base/$subdir";
		@ratfiles = grep /(gif|jpg)$/i, readdir DIR;
		close DIR;
		@ratfiles = grep /$product/,@ratfiles;
		@ratfiles = grep /$teststr/,@ratfiles;
		@ratfiles = grep /small/,@ratfiles unless ($subdir =~ /(ratio|diff)hist/);
		push @files, @ratfiles;
	}
}
else{
	opendir DIR, "$basedir";
	@files = grep /(gif|jpg)$/i, readdir DIR;
	@files = grep /small/,@files unless ($query{TYP} =~ /(ratio|diff)hist/);
	close DIR;
}
our $titlestr = "Effects of reprocessing #4 changes";

&standard_header($titlestr,'repro4_images.cgi');
our @qkeys = keys %query;
if ($#qkeys < 1) {&frontend}
else{
	if ($query{LEVEL} eq 'L2'){
		if ($query{CASE} eq 'PERTEST') {&make_table_pertest_lac($imgdir,$testnum,@files)}
		elsif ($query{CASE} eq 'TESTPROD') {&make_table_testprod_lac($baseimgdir,$testnum,$product,@files)}
		else {&make_table_perproduct_lac($imgdir,$product,$testnum,@files)}	
	}
	else {
		if ($query{CASE} eq 'PERTEST') {&make_table_pertest($imgdir,$testnum,@files)}
		elsif ($query{CASE} eq 'TESTPROD') {&make_table_testprod($baseimgdir,$testnum,$product,@files)}
		else {&make_table_perproduct($imgdir,$product,$testnum,@files)}
	}
}
&standard_footer('August',15,2002);
exit;

################################################################################
sub frontend {
	my @tests = sort {$a <=> $b} (keys %tests);
	my @products = sort (keys %prodlst);
	push @products, shift @products;
	my %prodlabels = %prodlst;
	my @prodstr;
	my %levels = ('L3BIN' => 'Level 3 Bin','L2' => 'Level 2 Scenes');
	foreach my $prod (@products){
		$prodlabels{$prod} =~ s/<[^>]*>//g;
		push @prodstr,$prodlabels{$prod};
	}
	my $prodstr = join(', ',@prodstr);
	#shift @tests;
	print h1("Reprocessing #4 Results Query");
	print p("In order to assess the impact of the various changes proposed for implementation in the SeaWiFS reprocessing #4, ",
		"a series of test runs on a full month's worth of data (May 1999) were performed.  The results of each test run ",
		"were compared to the previous run in the sequence, with the final run incorporating all the proposed changes for ",
		"reprocessing #4.  The results of this last, full reprocessing #4 run were also compared to the reprocessing #3 results.");
	print p("The test runs were as follows: Add MOBY Straylight Fix, Add Exponential Time-Dependence, Change Out-of-Band Correction, ",
		"Add Miscellaneous Repro #4 Change, Switch to Arnone NIR Correction, Eliminate Glint Masking, Change Cloud Flag Algorithm, ",
		"Reduce Straylight Masking, Add Epsilon-Targeted Smoothing of Lt(765)");
	print p("For each test the following analyses were performed:");
	print ul(['Global Product Images',
	'Ratio Images',
	'Difference Images',
	'Ratio Histograms',
	'Difference Histograms',
	'Scatter Plots',
	'Comparison Histograms']);
	print p("These analyses were produced for the following products: $prodstr");
	#print ul([@products]);
	print p("Using this form you can return the results of the reprocessing #4 analyses in three different views:",br);
	print ul(['By Change: Results of a given change on all products relative to the previous change in the sequence',
		'By Product: Results of the various changes for a selected product',
		'By Product and Change: All analyses for a selected product and selected change']);
	print p("Many of the analyses were performed on LAC resolution Level 2 scenes as well as the Global bin products.  You ",
		"may choose to view the Level 3 bin results or the Level 2 scene results.");
	print h2("Select a change"),popup_menu(-name=>'T',
                   -values=>[@tests],
		   -labels=>\%tests,
                   -default=>1);
         print h2("Select a product"),popup_menu(-name=>'PROD',
                   -values=>[@products],
                   -default=>'chlor',
		   -labels=>\%prodlabels);
         print h2("Select an analysis"),popup_menu(-name=>'TYP',
                   -values=>[@subdirs],
                   -default=>'global',
		   -labels=>\%analysislabels);
	print h2("Select a method to display the results"),popup_menu(-name=>'CASE',
                   -values=>['PERTEST','PERPROD','TESTPROD'],
		   -labels=>\%caselabels,
                   -default=>'PERTEST');
	print h2("Global or Local Analyses"),popup_menu(-name=>'LEVEL',
		   -values=>['L3BIN','L2'],
		   -default=>'L3BIN',
		   -labels=>\%levels);
    
    print center(p(submit()));
}
################################################################################
sub oops {
	print h2("The combination of options you selected is not valid, sorry :(");
}
################################################################################
sub teststring {
	my ($type,$testnum,$level) = @_;
	my $teststring = join('',"R",$testorder[$testnum-1],"_R",$testorder[$testnum],"_");
	if ($testnum == 10) {
		$teststring = join('',"R",$testorder[0],"_R",$testorder[9],"_");
	}
#	if ($testnum > 10) {
#		$teststring = join('',"R",$testorder[$testnum-2],"_R",$testorder[$testnum-1],"_");
#	}
	#$teststring =~ s/_R10/_10/ if $type =~ /(ratio|diff)hist/;
	my $lvlstr = 'ALGTEST';
	if ($level eq 'L2'){$lvlstr = 'test'}
	if ($type eq 'global'){
		$teststring = "$lvlstr";
		if ($level eq 'L3BIN'){$teststring .= '0'  unless ( ($testorder[$testnum] >= 10) || ($testorder[$testnum] == 0)) }
		#$teststring = "ALGTEST" if ( ($testorder[$testnum] >= 10) || ($testorder[$testnum] == 0)) ;
		$teststring .= $testorder[$testnum];
		#if ($testnum == 10){$teststring = "ALGTEST10"}
		if ($level eq 'L2'){$teststring .= '\.'}
		
	}
	return $teststring;
}

################################################################################
sub make_table_testprod {
	my $basehttp = shift;
	my $test = shift;
	my $product = shift;
	if ($test == 0 ){
		&oops;
		return;
	}
	my @images = @_;
	my @datatype = ('MO');
	my @thead = ('Analysis',$dtypes{'MO'});
	my @monthly = grep /MO/,@images;
	my @weekly = grep /8D/,@images;
	my @daily = grep /DAY/,@images;
	
	#print p(@weekly,@daily);
	my @showimg;
	push @datatype,'8D' if ($#weekly >= 0);
	push @thead,$dtypes{'8D'} if ($#weekly >= 0);
	push @datatype,'DAY' if ($#daily >= 0);
	push @thead,$dtypes{'DAY'} if ($#daily >= 0);
	my @rows;
	my $width = 90./($#datatype+1);
	$width = "$width%";
	foreach my $analysis (@subdirs){
		my $http = join('/',$basehttp,$analysis);
		my $altstr = "$prodlst{$product} $analysis image";
		$altstr =~ s/<[^>]*>//g;
		my $casestr = $case{$analysis};
		my @link = my @showing = grep /$casestr/,@monthly,@weekly,@daily;
		foreach my $link (@link){$link =~ s/\.small//}		
		my $scalestr = '0.8 - 1.2';
		my @td;
		
		if ($analysis =~ /(ratio|diff)hist/){
			$scalestr = '';
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"})) if ($#showing > 0);
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"})) if ($#showing > 1);
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"}))) if ($#showing > 0);
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"}))) if ($#showing > 1);
		}
		$scalestr = '' if ($analysis =~ /(comphist|scatter)/);
		$scalestr = $scaling{$product}{$analysis} if ($analysis =~ /(difference|global)/);
		push @rows, Tr(th({width=>'10%'},"$analysis",br,"$scalestr"),@td);
		shift @monthly;
		shift @weekly;
		shift @daily;
		#print "$#showing\n";
	}		
	my $http = join('/',$basehttp,'gainloss');
	$teststr = &teststring('gainloss',$test,$query{LEVEL});
	$teststr =~ s/_$//;
	$teststr = "$teststr\.small.gif";
	#if ($test != 10) {
	opendir DIR, "$base/gainloss";
	my @gfiles = grep /(gif|jpg)$/i, readdir DIR;
	close DIR;
	@monthly = grep /MO/,@gfiles;
	@weekly = grep /8D/,@gfiles;
	@daily = grep /DAY/,@gfiles;
	my @glink = my @glfiles = grep /$teststr/,@monthly,@weekly,@daily;
	
	foreach my $glink (@glink){$glink =~ s/\.small//}
	my @smp = grep /^smp/,@glfiles;
	my @smplink = grep /^smp/,@glink;
	my @td = td({width=>$width},a({href=>"$http/$smplink[0]"},(img{-src=>"$http/$smp[0]",-alt=>"Monthly Sample Gain/Loss"})));
	push @td,td({width=>$width},a({href=>"$http/$smplink[1]"},(img{-src=>"$http/$smp[1]",-alt=>"Weekly Sample Gain/Loss"}))) if ($#weekly >= 0);
	push @td,td({width=>$width},a({href=>"$http/$smplink[2]"},(img{-src=>"$http/$smp[2]",-alt=>"Daily Sample Gain/Loss"}))) if ($#daily >= 0);
	push @rows, Tr(th({width=>'10%'},"Sample Gain/Loss"),@td);
	my @bin = grep /^gain/,@glfiles;
	my @binlink = grep /^gain/,@glink;
	@td = td({width=>$width},a({href=>"$http/$binlink[0]"},(img{-src=>"$http/$bin[0]",-alt=>"Monthly Bin Gain/Loss"})));
	push @td,td({width=>$width},a({href=>"$http/$binlink[1]"},(img{-src=>"$http/$bin[1]",-alt=>"Weekly Bin Gain/Loss"}))) if ($#weekly >= 0);
	push @td,td({width=>$width},a({href=>"$http/$binlink[2]"},(img{-src=>"$http/$bin[2]",-alt=>"Daily Bin Gain/Loss"}))) if ($#daily >= 0);
	push @rows, Tr(th({width=>'10%'},"Bin Gain/Loss"),@td);
	#}
	my $prodstr = $prodlst{$product};
	$prodstr =~ s/<[^>]*>//g;

	print h1("Effects of $tests{$test} on $prodstr");
	print hr;
	print center(img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change"));
	print center(p("Results are relative to the previous test in the sequence:",br,"$tests{$test-1}")) unless $test == 11;
	print center(p("Results are relative to the operational (reprocessing #3) results")) if $test == 11;
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);

}

################################################################################
sub make_table_pertest {
	my $http = shift;
	my $test = shift;
	my @images = @_;
	if ($test == 0 && $query{TYP} ne 'global'){
		&oops;
		return;
	}
	my @datatype = ('MO');
	my @thead = ('Product',$dtypes{'MO'});
	my @monthly = grep /MO/,@images;
	my @weekly = grep /8D/,@images;
	my @daily = grep /DAY/,@images;
	my @showimg;
	push @datatype,'8D' if ($#weekly >= 0);
	push @thead,$dtypes{'8D'} if ($#weekly >= 0);
	push @datatype,'DAY' if ($#daily >= 0);
	push @thead,$dtypes{'DAY'} if ($#daily >= 0);
	my @rows;
	my $width = 90./($#datatype+1);
	$width = "$width%";
	my @products = sort (keys %prodlst);
	push @products, shift @products;
	foreach $product (@products){
		my $altstr = "$prodlst{$product} image";
		$altstr =~ s/<[^>]*>//g;
		my @link = my @showing = grep /$product/,@monthly,@weekly,@daily;
		foreach my $link (@link){$link =~ s/\.small//}		
		my $scalestr = '<br>0.8 - 1.2';
		my @td;
		if ($query{TYP} =~ /(ratio|diff)hist/){
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"})) if ($#weekly >= 0);
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"})) if ($#daily >= 0);
			$scalestr = '<br>';
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"}))) if ($#weekly >= 0);
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"}))) if ($#daily >= 0);
		}
		$scalestr = '' if ($query{TYP} =~ /(comphist|scatter)/);
		$scalestr = $scaling{$product}{$query{TYP}} if ($query{TYP} =~ /(global|difference)/);
		push @rows, Tr(th({width=>'10%'},"$prodlst{$product} $scalestr"),@td);
	}
	print h1("Effects of $tests{$test}") unless ($query{TYP} eq 'global');
	print h1("$tests{$test}") if ($query{TYP} eq 'global');
	print h1("Reprocessing #4") if ($query{TYP} eq 'global' && $test == 10);
	print h1("Reprocessing #3") if ($query{TYP} eq 'global' && $test == 0);
	print center(h3("$analysislabels{$query{TYP}}"));
	if ($query{TYP} !~ /(hist|scatter)/){
		print hr;
		print center(img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change")) unless  ($query{TYP} eq 'global');
		if ($query{TYP} eq 'global'){	
			print center(img{-src=>"$baseimgdir/colorbar.jpg",-alt=>'Colorbar'});
		}
	}		
	print center(p("Results are relative to the previous test in the sequence:",br,"$tests{$test-1}")) unless ($test == 11 || $query{TYP} eq 'global');
	print center(p("Results are relative to the operational (reprocessing #3) results")) if ($test == 10 && $query{TYP} ne 'global');
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);

}

################################################################################
sub make_table_perproduct {
	my $http = shift;
	my $prod = shift;
	my @images = grep /$prod/i,@_;
	my @datatype = ('MO');
	my @thead = ('Test Case',$dtypes{'MO'});
	my @monthly = grep /MO/,@images;
	my @weekly = grep /8D/,@images;
	my @daily = grep /DAY/,@images;
	my @showimg;
	push @datatype,'8D' if ($#weekly >= 0);
	push @thead,$dtypes{'8D'} if ($#weekly >= 0);
	push @datatype,'DAY' if ($#daily >= 0);
	push @thead,$dtypes{'DAY'} if ($#daily >= 0);
	my @rows;
	my $width = 90./($#datatype+1);
	$width = "$width%";
	my $last = 10;
	$last = 9 if ($query{TYP} eq 'global');
	for (my $i = 0;$i < $last; $i++){
		my $altstr = "$prodlst{$prod} $query{TYP} image for test: $tests{$i+1}";
		$altstr =~ s/<[^>]*>//g;
		my $teststr = &teststring($query{TYP},$i+1,$query{LEVEL});
		my @link = my @showing = grep /$teststr/,@monthly,@weekly,@daily;
		foreach my $link (@link){$link =~ s/\.small//}
		my @td;
		if ($query{TYP} =~ /(ratio|diff)hist/){
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"})) if ($#weekly >= 0);
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"})) if ($#daily >= 0);
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"$dtypes{'MO'} $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"$dtypes{'8D'} $altstr"}))) if ($#weekly >= 0);
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"$dtypes{'DAY'} $altstr"}))) if ($#daily >= 0);
		}
		push @rows, Tr(th({width=>'10%'},$tests{$i+1}),@td);
	}
	my $scalestr = '0.8 - 1.2';
	$scalestr = $scaling{$prod}{$query{TYP}} if ($query{TYP} =~ /(global|difference)/);
	my $prodstr = $prodlst{$product};
	$prodstr =~ s/<[^>]*>//g;
	print h1("Effects on $prodstr");
	print center(h3("$analysislabels{$query{TYP}}"));
	if ($query{TYP} !~ /(scatter|hist|global)/){
		print hr;
		print center(h3($scalestr),img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change"));
	print center(p("Results are relative to the previous test in the sequence."));
	}
	if ($query{TYP} eq 'global'){	
		print hr;
		print center($scalestr,br,img{-src=>"$baseimgdir/colorbar.jpg",-alt=>'Colorbar'});
	}
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);
}

################################################################################
sub make_table_testprod_lac {
	my $basehttp = shift;
	my $test = shift;
	my $product = shift;
	if ($test == 0 || $test == 8){
		&oops;
		return;
	}
	my @images = @_;
	my @showimg;
	my @rows;
	my $width = 90./4.;
	$width = "$width%";
	#print p(@images);
	my @datatype = ('MO');
	my @thead = ('Analysis','North East US','Gulf of Mexico','Hawaii','Patagonia');
	my @HNSG = grep /HNSG/,@images;
	my @HUSF = grep /HUSF/,@images;
	my @LAC = grep /LAC/,@images;
	my @HCHL = grep /HCHL/,@images;
	#print p(@HNSG);
	opendir TC,"$base/truecolor";
	my @TCimages = readdir TC;
	closedir TC;
	my @TCHUSF = grep /HUSF/,@TCimages;
	my @TCLAC = grep /LAC/,@TCimages;
	my @TCHCHL = grep /HCHL/,@TCimages;
	my @TCHNSG =  grep /HNSG/,@TCimages;
	my @TClink = my @TC = grep /small/, @TCHNSG,@TCHUSF,@TCLAC,@TCHCHL;
	foreach my $TClink (@TClink){$TClink =~ s/\.small//}
	my @td = td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[0]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[0]",-alt=>"North East U.S. True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[1]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[1]",-alt=>"Gulf of Mexico True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[2]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[2]",-alt=>"Hawaii True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[3]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[3]",-alt=>"Patagonia True Color"})));
	push @rows, Tr(th({width=>'10%'},'True Color',@td));
#	my @lacanaylses = ('global','ratiohist','diffhist','comphist');
	my @lacanaylses = ('global','ratio','difference','ratiohist','diffhist');
#	my @lacanaylses = ('ratio','difference','ratiohist','diffhist');
	foreach my $analysis (@lacanaylses){
		next if ($product eq 'tau' && $analysis eq 'ratio');
		my $http = join('/',$basehttp,$analysis,'LAC');
		my $altstr = "$prodlst{$product} $analysis image";
		$altstr =~ s/<[^>]*>//g;
		my $casestr = $case{$analysis};
		if ($analysis eq 'global'){$casestr = 'test'}
		my @link = my @showing = grep /$casestr/,@HNSG,@HUSF,@LAC,@HCHL;
		next if $#link == -1;
		foreach my $link (@link){$link =~ s/\.small//}		
		my $scalestr = '0.8 - 1.2';
		my @td;
		
		if ($analysis =~ /(ratio|diff)hist/){
			$scalestr = '';
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"}));
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[3]"},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"})));
		}
		$scalestr = '' if ($analysis =~ /(comphist|scatter)/);
		$scalestr = $scaling{$product}{$analysis} if ($analysis =~ /(difference|global)/);
		push @rows, Tr(th({width=>'10%'},"$analysis",br,"$scalestr"),@td);
		shift @HNSG;
		shift @HUSF;
		shift @HCHL;
		shift @LAC;
	}		
	my $prodstr = $prodlst{$product};
	$prodstr =~ s/<[^>]*>//g;

	print h1("Effects of $tests{$test} on $prodstr");
	print hr;
	print center(img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change"));
	print center(p("Results are relative to the previous test in the sequence:",br,"$tests{$test-1}")) unless $test == 11;
	print center(p("Results are relative to the operational (reprocessing #3) results")) if $test == 11;
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);

}

################################################################################
sub make_table_pertest_lac {
	my $http = shift;
	my $test = shift;
	my @images = @_;
	if (($test == 0 && $query{TYP} ne 'global') || ($test ==8)){
		&oops;
		return;
	}
	my @showimg;
	my @rows;
	my $width = 90./4.;
	$width = "$width%";
	my @products = sort (keys %prodlst);
	push @products, shift @products;
	my @thead = ('Product','North East US','Gulf of Mexico','Hawaii','Patagonia');
	my @HNSG = grep /HNSG/,@images;
	my @HUSF = grep /HUSF/,@images;
	my @LAC = grep /LAC/,@images;
	my @HCHL = grep /HCHL/,@images;
	opendir TC,"$base/truecolor";
	my @TCimages = readdir TC;
	closedir TC;
	my @TCHUSF = grep /HUSF/,@TCimages;
	my @TCLAC = grep /LAC/,@TCimages;
	my @TCHCHL = grep /HCHL/,@TCimages;
	my @TCHNSG =  grep /HNSG/,@TCimages;
	my @TClink = my @TC = grep /small/, @TCHNSG,@TCHUSF,@TCLAC,@TCHCHL;
	foreach my $TClink (@TClink){$TClink =~ s/\.small//}
	my @td = td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[0]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[0]",-alt=>"North East U.S. True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[1]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[1]",-alt=>"Gulf of Mexico True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[2]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[2]",-alt=>"Hawaii True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[3]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[3]",-alt=>"Patagonia True Color"})));
	push @rows, Tr(th({width=>'10%'},'True Color',@td));
	#print p($teststr);
	#print p(@images);
	foreach $product (@products){
		next if ($product eq 'tau');
		my $altstr = "$prodlst{$product} image";
		$altstr =~ s/<[^>]*>//g;
		my @link = my @showing = grep /$product/,@HNSG,@HUSF,@LAC,@HCHL;
		foreach my $link (@link){$link =~ s/\.small//}		
		my $scalestr = '<br>0.8 - 1.2';
		my @td;
		if ($query{TYP} =~ /(ratio|diff)hist/){
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S. $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"}));
			$scalestr = '<br>';
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S. $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[3]"},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"})));
		}
		$scalestr = '' if ($query{TYP} =~ /(comphist|scatter)/);
		$scalestr = $scaling{$product}{$query{TYP}} if ($query{TYP} =~ /(global|difference)/);
		push @rows, Tr(th({width=>'10%'},"$prodlst{$product} $scalestr"),@td);
	}
	print h1("Effects of $tests{$test}") unless ($query{TYP} eq 'global');
	print h1("$tests{$test}") if ($query{TYP} eq 'global');
	print h1("Reprocessing #4") if ($query{TYP} eq 'global' && $test == 10);
	print h1("Reprocessing #3") if ($query{TYP} eq 'global' && $test == 0);
	print center(h3("$analysislabels{$query{TYP}}"));
	if ($query{TYP} !~ /(hist|scatter)/){
		print hr;
		print center(img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change")) unless  ($query{TYP} eq 'global');
		if ($query{TYP} eq 'global'){	
			print center(img{-src=>"$baseimgdir/colorbar.jpg",-alt=>'Colorbar'});
		}
	}		
	print center(p("Results are relative to the previous test in the sequence:",br,"$tests{$test-1}")) unless ($test == 11 || $query{TYP} eq 'global');
	print center(p("Results are relative to the operational (reprocessing #3) results")) if ($test == 10 && $query{TYP} ne 'global');
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);

}

################################################################################
sub make_table_perproduct_lac {
	my $http = shift;
	my $prod = shift;
	my @images = grep /$prod/i,@_;
	my @showimg;
	my @rows;
	my $width = 90./4.;
	$width = "$width%";
	my $last = 10;
	$last = 9 if ($query{TYP} eq 'global');
	if ($prod eq 'tau'){
		&oops;
		return;
	}
	my @datatype = ('MO');
	my @thead = ('Test Case','North East US','Gulf of Mexico','Hawaii','Patagonia');
	my @HNSG = grep /HNSG/,@images;
	my @HUSF = grep /HUSF/,@images;
	my @LAC = grep /LAC/,@images;
	my @HCHL = grep /HCHL/,@images;
	opendir TC,"$base/truecolor";
	my @TCimages = readdir TC;
	closedir TC;
	my @TCHUSF = grep /HUSF/,@TCimages;
	my @TCLAC = grep /LAC/,@TCimages;
	my @TCHCHL = grep /HCHL/,@TCimages;
	my @TCHNSG =  grep /HNSG/,@TCimages;
	my @TClink = my @TC = grep /small/, @TCHNSG,@TCHUSF,@TCLAC,@TCHCHL;
	foreach my $TClink (@TClink){$TClink =~ s/\.small//}
	my @td = td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[0]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[0]",-alt=>"North East U.S. True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[1]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[1]",-alt=>"Gulf of Mexico True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[2]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[2]",-alt=>"Hawaii True Color"})));
	push @td,td({width=>$width},a({href=>"$baseimgdir/TRUECOLOR/$TClink[3]"},(img{-src=>"$baseimgdir/TRUECOLOR/$TC[3]",-alt=>"Patagonia True Color"})));
	push @rows, Tr(th({width=>'10%'},'True Color',@td));
	for (my $i = 0;$i < $last; $i++){
	next if $i == 7;
	
		my $altstr = "$prodlst{$prod} $query{TYP} image for test: $tests{$i+1}";
		$altstr =~ s/<[^>]*>//g;
		my $teststr = &teststring($query{TYP},$i+1,$query{LEVEL});
		my @link = my @showing = grep /$teststr/,@HNSG,@HUSF,@LAC,@HCHL;
		foreach my $link (@link){$link =~ s/\.small//}
		my @td;
		if ($query{TYP} =~ /(ratio|diff)hist/){
			@td = td({width=>$width},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S. $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"}));
			push @td,td({width=>$width},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"}));
			
		}
		else{
			@td = td({width=>$width},a({href=>"$http/$link[0]"},(img{-src=>"$http/$showing[0]",-alt=>"North East U.S. $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[1]"},(img{-src=>"$http/$showing[1]",-alt=>"Gulf of Mexico $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[2]"},(img{-src=>"$http/$showing[2]",-alt=>"Hawaii $altstr"})));
			push @td,td({width=>$width},a({href=>"$http/$link[3]"},(img{-src=>"$http/$showing[3]",-alt=>"Patagonia $altstr"})));
		}
		push @rows, Tr(th({width=>'10%'},$tests{$i+1}),@td);
	}
	my $scalestr = '0.8 - 1.2';
	$scalestr = $scaling{$prod}{$query{TYP}} if ($query{TYP} =~ /(global|difference)/);
	my $prodstr = $prodlst{$product};
	$prodstr =~ s/<[^>]*>//g;
	print h1("Effects on $prodstr");
	print center(h3("$analysislabels{$query{TYP}}"));
	if ($query{TYP} !~ /(scatter|hist|global)/){
		print hr;
		print center(h3($scalestr),img{-src=>"$baseimgdir/purple_orange.jpg",-alt=>'Colorbar'},p("light grey indicates zero-change"));
	print center(p("Results are relative to the previous test in the sequence."));
	}
	if ($query{TYP} eq 'global'){	
		print hr;
		print center($scalestr,br,img{-src=>"$baseimgdir/colorbar.jpg",-alt=>'Colorbar'});
	}
	print hr;
	print table({-cellpadding=>1,width=>'90%'}, 
		Tr(th([@thead])),@rows);
}

################################################################################
sub standard_header { 

	my ($title,$url) = @_;
	
#	my $http = "http://seawifs.gsfc.nasa.gov/SEAWIFS/RECAL/Repro4";
	my $style = << "END";
BODY{ background-color: white; color: black; margin-left: 2%; margin-right: 2%; padding-bottom: 20px; padding-top: 10px;  }
H1{ font-family: Times, serif; font-weight: bold; font-size: 18pt; align:center;}
H2{ font-family: Times, serif; font-weight: bold; font-size: 14pt; }
H3{ font-family: Times, serif; font-weight: bold; font-size: 12pt; }
P{ font-family: Helvetica, Arial, sans-serif; font-style: normal; font-weight: normal; font-size: 10pt; }
TD{ font-family: Helvetica, Arial, sans-serif; font-style: normal; font-weight: normal; font-size: 10pt; align:center; }
UL{ font-family: Helvetica, Arial, sans-serif; font-style: normal; font-weight: normal; font-size: 10pt; }
LI{ list-style-type: circle; }
EM{ font-weight: bold; font-style: normal; }
FONT{ font-family: Helvetica, Arial, sans-serif; }
BLOCKQUOTE{ font-family: Helvetica, Arial, sans-serif; font-size: 10pt; }

.width { width:700px; }
.minus1 { font-size: 10pt; }
.minus2 { font-size: 8pt; }
.align { text-align: justify; }
END

	print header;
	print start_html(
		-title=>$title, 
		-style=>{-code=>$style},
		-dtd=>'-//W3C//DTD HTML 4.01 Transitional//EN'
	);
    print start_form(-action=>$url,-method=>'GET');
	print "<div class=width>\n";
}

################################################################################
sub standard_footer {

    my ($month,$day,$year) = @_;

    print (hr);
    print center(
		p(
        	font({-class=>'minus1'}, "Author/Site Curator: ", a({-href=>"mailto:sean\@seawifs.gsfc.nasa.gov"}, "Sean Bailey")),
   		br, 
		font({-class=>'minus1'}, "Responsible NASA Official:  Gene Carl Feldman"),
    		br, 
		font({-class=>'minus1'}, "Last revised $day $month $year"))
    );
	print "</div>\n";
	print end_form();
	print end_html;
}


################################################################################
sub sort_images {

	my @images = @_;

	my @sorted = (-1,-1,-1,-1,-1,-1);

	foreach my $image (@images) {
		if ($image =~ /DAY/) { if ($image =~ /chlor_a/) { $sorted[0] = $image; } else { $sorted[1] = $image; } }
		elsif ($image =~ /8D/) { if ($image =~ /chlor_a/) { $sorted[2] = $image; } else { $sorted[3] = $image; } }
		elsif ($image =~ /MO/) { if ($image =~ /chlor_a/) { $sorted[4] = $image; } else { $sorted[5] = $image; } }
	}
	return @sorted;
}

################################################################################
sub find_unique {

	my @array = @_;
	my %seen = ();
	my @unique = grep {! $seen{$_} ++} @array;
	return @unique;
}
