#!/usr/bin/env perl
#
#use warnings FATAL => 'all';
use Data::Dumper;
# This CGI script lets a user browse SeaWiFS GAC, LAC, and HRPT data.
#
# Norman Kuring		22-May-2002
# Norman Kuring		 2-Aug-2002	Move to new web server.
# Norman Kuring		20-Nov-2002	Corrected ordinal function to give
#					proper 'th' suffix to teen numbers.
# Norman Kuring		17-Dec-2002	Allow display of list of filenames
#					instead of thumbnails.
# Norman Kuring		17-Jan-2003	Use the level-3 quicklook images
#					that I have started generating.
# Norman Kuring		15-May-2003	Change function of up arrow on
#					level3 pages.
# Norman Kuring		29-May-2003	Replaced IPC::Open2 with IPC::Run
#					and replaced nph-no_response.pl
#					with CGI header call.  Thanks Sean
#					Bailey for figuring this out.
# Norman Kuring		18-Jun-2003	Add support for Terra/Aqua MODIS.
# Norman Kuring		30-Jun-2003	Added extra, anchored link to the
#					Help page concerning the compressed,
#					downloadable MODIS data.
# Norman Kuring		16-Jul-2003	Added link to documentation for
#					downloadable MODIS files.
# Norman Kuring		18-Jul-2003	Add ordering capability for MODIS data.
# Norman Kuring		 6-Aug-2003	Add ordering capability for SeaWiFS
#					data. (If it's > 5 years old, it's in
#					the public domain.)
# Norman Kuring		21-Aug-2003	Add support for SST browse images.
# Norman Kuring		12-Sep-2003	Up the order limit from 2x10^9 to
#					1x10^12 bytes.
# Norman Kuring		 8-Oct-2003	Added ability to select a subset
#					of the "most interesting" scenes
#					from the list of search results.
# Norman Kuring		21-Oct-2003	Bug fixes associated with selectability
# Norman Kuring		 6-Jan-2004	Up the order limit to 1x10^14 bytes.
# Norman Kuring		28-Jan-2004	Add SeaWiFS MLAC searches to browser.
# Norman Kuring		30-Jan-2004	Add ancillary data ordering capability.
#					Remove "thb" parameter and filelist
#					subroutine.
# Norman Kuring		 5-Feb-2004	Add ancillary ordering capability plus
#					a few extra directions to the SDPS
#					order manager.
# Norman Kuring		 6-Feb-2004	Use smaller menu buttons.
# Norman Kuring		 9-Feb-2004	Update Help information.
# Norman Kuring		10-Feb-2004	Don't allow the submission of orders
#					that have zero allowable files.
# Norman Kuring		19-Feb-2004	Give special download and order
#					privileges to authorized SeaWiFS users.
# Norman Kuring		08-Apr-2004	Add map of user's area of interest.
# Norman Kuring		19-Apr-2004	Add ticker tape applet.
# Norman Kuring		21-May-2004	Force extract coordinates to numeric
#					form to get rid of any inapposite
#					characters entered by user.
# Norman Kuring		 1-Jun-2004	Allow user to order parameter extracts.
# Norman Kuring		10-Jun-2004	Fix bug that prevented user from
#					ordering ancillary data without the
#					corresponding level 1 or 2 files.
# Norman Kuring		 2-Jul-2004	Allow user to specify custom date
#					ranges for data searches.
# Norman Kuring		 9-Jul-2004	Add an advisory about the non-
#					availability of SeaWiFS HRPT and LAC
#					data for download.
# Norman Kuring		15-Jul-2004	Fix bug that was allowing users to
#					order SeaWiFS HRPT data.
# Norman Kuring		22-Jul-2004	Move help text out of this script
#					and make the help slightly more
#					context sensitive.
# Norman Kuring		22-Nov-2004	New webserver (Sun) and new directory
#					hierarchies.
#
# Sean Bailey		08-Dec-2004	Modified to use DB metadata with
#					quadspehre bin level 6
#
# Norman Kuring		15-Feb-2005	Added translate_fname subroutine to
#					clean up filename parsing a bit.
# Norman Kuring		29-Apr-2005	Location of SeaWiFS authorized user
#					list changed.
# Norman Kuring		29-Apr-2005	Add restricted_scene_found subroutine
#					to permit display of advisory message
#					to user about to order restricted
#					SeaWiFS data.
# Norman Kuring		17-Jun-2005	Add pointers to OCTS data.
# Norman Kuring		19-Jul-2005	Add pointers to CZCS data.
# Norman Kuring		28-Jul-2005	Check for existence of browse file
#					before displaying swath locator image
#					(i.e. sub=locimage).
# Sean Bailey		28-Jul-2005	Added test on hide value to prevent
#					broken images in thumbtable when record
#					exists in DB, but BRS image is not there
#					(e.g. during reprocessings)
# Sean Bailey		01-Aug-2005	Added extra logic for SeaWiFS embargo'd
#					data to allow displaying in thumbtable
#					even with a hide of 3
# Norman Kuring		20-Sep-2005	Added link to SeaWiFS processing
#					documentation.
# Norman Kuring		11-Jan-2006	Override SST browse palette with
#					external palette.
# Norman Kuring		 2-Feb-2006	Adding nighttime SST capability...
# Norman Kuring		16-Feb-2006	SST browse files now have new -2 to 45
#					scaling and a new palette, so I stopped
#					using the external palette mentioned
#					above.
# Norman Kuring		 2-Mar-2006	The Start_Node metadatum is sometimes
#					incorrect in the browse files, so I
#					am instead switching to using
#					Start_Center_Latitude and
#					End_Center_Latitude to determine which
#					way is up in the browse images.
# Norman Kuring		30-Mar-2006	Fix bug that caused overly wide search
#					boxes to generate server errors.
# Norman Kuring		11-Apr-2006	SeaWiFS MLAC now has a separate set of
#					mission dates from the rest of the
#					SeaWiFS data types.
# Norman Kuring		11-Apr-2006	Enable display and search of Terra
#					MODIS scenes.
# Norman Kuring		20-Apr-2006	Removed advisory about availability of
#					SeaWiFS HRPT and LAC data.
# Norman Kuring		 8-May-2006	Fixed bug when determining mission_gap.
# Norman Kuring		 6-Jul-2006	Add order confirmation URL to email
#					message sent to user.
# Norman Kuring		10-Jul-2006	Removed the "select all of this time
#					period's scenes" checkbox.  This is
#					now the default if no other geographic
#					constraints have been specified.
# Norman Kuring		28-Jul-2006	Added capability to highlight search
#					areas on browse images.
# Norman Kuring		10-Aug-2006	Added "browse image not available"
#					thumbnail to avoid broken thumbnail
#					icons that occur when searches result
#					in both day and night scenes. Day scenes
#					will have no sst4 images and night
#					scenes will have no chl or tc images.
# Norman Kuring		12-Sep-2006	Disallow highlighting when the entire
#					globe is the search area.
#
# Sean Bailey		02-Oct-2006	Minor mod to eliminate duplicate 'calls'
#					to quadregion, as user defined region
#					includes 'pre-defined' regions as well
#
# Sean Bailey		12-Oct-2006	Added L0 hyperlinks ... oof.
#
# Norman Kuring		 3-Nov-2006	Commented out cup_o_joe call.  See
#					inline comments.
# Norman Kuring		12-Jan-2007	Allow display of Terra chlorophyll
#					browse images in preparation for
#					initial forward-stream Terra ocean
#					color products.
# Norman Kuring		 5-Feb-2007	Fixed bug that was causing browse
#					state information to accumulate
#					redundant wid and hgt values which
#					were showing up in our access logs
#					as increasingly long URLs.
#
# Sean Bailey		23-Feb-2007	Made a tweek to the regex for the call
#					to locate_file to ensure only files we
#					want to distribute are selected
#
# Sean Bailey		13-Mar-2007	Modified queries for qbins2list function
#
# Norman Kuring		 6-Jun-2007	Added searches for scenes with in situ
#					matchups.
# Norman Kuring		 7-Jun-2007	The getfile URL now needs to begin
#					with restrict instead of cgi for
#					privileged users.
# Norman Kuring		12-Dec-2007	Allow users to request a list of
#					filenames.
# Norman Kuring		13-Dec-2007	Our server is adding Expires headers to
#					the output of this script.  This script
#					now sets its own Expires header to a
#					date in the past to avoid having its
#					dynamic output cached.
# Norman Kuring		11-Feb-2008	Fix bug in limit_volume subroutine
#					that was failing to differentiate
#					between L2 and L2SST files.
# Norman Kuring		12-Feb-2008	Yesterday's bug fix introduced a new
#					one.  I must allow for both ".bz2"
#					and ".Z" compression in filenames.
# Norman Kuring		 4-Mar-2008	Reinstate level-0 order capability.
#					Get rid of "L1B" and "CHLOR_A" portions
#					of browse filenames when displaying
#					the names with the thumbnails.
#					Add title attributes to thumbnails.
# Norman Kuring		10-Mar-2008	Direct data orderers to Dan Knowles'
#					new order confirming and tracking
#					web page.
# Norman Kuring		14-Mar-2008	Changed some ordering verbiage per
#					Dan's suggestion.
# Norman Kuring		 4-Jun-2008	Comment button still pointed to ocforum
#					web server.  It now goes to the forum
#					on oceancolor.
# Norman Kuring		 3-Sep-2008	Make script work under taint mode.
# Norman Kuring		10-Sep-2008	Replace "$#{@$selected}"
#					with    "@$selected - 1" to fix a
#					"Bizarre copy of ARRAY in leave"
#					error that started cropping up when
#					we upgraded our web server software.
# Norman Kuring		25-Nov-2008	Make order form prompts red with black
#					background if user forgets to fill in a
#					required field.
# Norman Kuring		25-Nov-2008	Change message sent to users who have
#					ordered data.  Message text supplied by
#					Dan Knowles.
# Norman Kuring		26-Nov-2008	Tweak the presentation of the order-
#					submitted page a bit.
# Norman Kuring		15-Jan-2009	Admonish registered users to use
#					their registered email address.
# Norman Kuring		 3-Feb-2009	Add further clarification to the
#					aforementioned admonishment.
# Norman Kuring		 6-Mar-2009	Add support for MERIS data, but only to
#					a privileged few.
# Norman Kuring		29-May-2009	Allow the user to pass a filename in
#					the URL to display a specific
#					individual scene page.
# Norman Kuring		23-Jul-2009	Trim leading and trailing whitespace
#					from entered email addresses.
# Norman Kuring		 8-Sep-2009	Remove references to ancillary data
#					from the order form and reword the
#					level-2 parameter extract checklist.
# Norman Kuring		 9-Sep-2009	New directory structure and file-naming
#					convention for SeaWiFS level-2 files.
# Norman Kuring		14-Sep-2009	Change getfile URL to refer to
#					oceandata.sci.gsfc.nasa.gov .
# Norman Kuring		14-Sep-2009	Remove SeaWiFS LAC from the list of
#					browsable file types.
# Norman Kuring		15-Sep-2009	Make MERIS data browsable by all.
# Norman Kuring		15-Sep-2009	Fix filenames generated in limit_volume
#					subroutine.
# Norman Kuring		 5-Nov-2009	Make all thumbnails 70 pixels wide.
# Norman Kuring		18-Feb-2010	Changes associated with the latest
#					Aqua-MODIS reprocessing.
# Norman Kuring		23-Mar-2010	Changes to some of the in situ
#					parameters that are displayed.
# Norman Kuring		 5-Oct-2010	Changes associated with the latest
#					OCTS reprocessing.
# Norman Kuring		 4-Nov-2010	Changes associated with the latest
#					Terra-MODIS reprocessing.
# Norman Kuring		 8-Dec-2010	Stop checking the return value of
#					IPC::run when it invokes brsrevnav
#					because that program has started
#					returning a non-zero status for some
#					reason even though it still works.
# Norman Kuring		 9-Feb-2011	It seems that different versions of
#					ncdump render space-containing
#					attributes with or without an
#					underscore -- presumably depending on
#					the ncdump programmer's mood.  I now
#					allow either a space or an underscore.
# Sean Bailey		17-Feb-2011	Perl 5.10 now complains if tainted data
#					are used in (s)printf format strings,
#					so...added laundering to CTSM date range
#					results prior to creating the string
# Norman Kuring		 1-Mar-2011	Add support for (simulated) VIRRS data.
# Sean Bailey		11-May-2011	Modified to use the new webDB.
# Sean Bailey		11-Jul-2011	Modified to create url listing for non-
#					extract orders
# Norman Kuring		21-Nov-2011	Allow for NPPS string in browse file
#					names.
# Norman Kuring		16-Mar-2012	Make swathdays return datadays in
#					chronological order so that locator
#					maps get arranged in such a way that
#					dateline-spanning swath sections are
#					placed next to each other.
# Norman Kuring		16-Apr-2012	And now the NPPS string goes away again,
#					and I add a hyperlink to the L1B_NPP.tar
#					file.
# Norman Kuring		18-Apr-2012	Replace L1A with L1B in MERIS and VIIRS
#					file names.
# Norman Kuring		 5-Jul-2012	Add note that VIIRS L1 data are not
#					extractable.
# Norman Kuring		 7-Sep-2012	The pamdice program changed how it
#					named its output files when we upgraded
#					to Ubuntu 12.04.
# Sean Bailey           17-Oct-2012	Modified to handle MERIS RR data and the#					N1 filename format
#
# Norman Kuring		 4-Mar-2013	Add HICO support.
# Norman Kuring		21-Mar-2013	Disallow data extraction for any order
#					that includes Level-0 data.
# Norman Kuring		19-Feb-2014	Disallow data extraction for any order
#					that includes HICO data.
# Norman Kuring		27-Aug-2014	Handle new netCDF (.nc) products
#					including L2_IOP for OCTS.
# Norman Kuring		 8-Sep-2014	Further tweaks to make new netCDF
#					products work in data orders.
# Norman Kuring		 9-Oct-2014	Tweak some web pages to reflect 2014
#					reprocessing.
# Norman Kuring		 9-Dec-2014	Remove access to old-style, HDF4 VIIRS
#					level-2 files.
# Norman Kuring		18-Mar-2015	Fix bug that caused VIIRS L2 thumbnails
#					not to display.
# Norman Kuring		29-May-2015	Add Aqua-MODIS to the set of
#					reprocessed missions.
# Norman Kuring		 8-Jul-2015	Need to handle newly reprocessed SST
#					file-naming conventions.
# Norman Kuring		15-Oct-2015	Forum poster pointed out bug in
#					filenamelist subroutine.  SST files
#					were being reset to OC files in the
#					list.
# Norman Kuring		 4-Nov-2015	SeaWiFS (2014.0) has now been published
# Norman Kuring		16-Dec-2015	And now CZCS joins the ".nc" crowd.
# Norman Kuring		22-Dec-2015	Get a URS username from the user
#					instead of an email address.  The
#					email address will be derived from
#					the username via the HTTP environment
#					variable, URS_EMAIL.
# Norman Kuring		25-Mar-2016	Tweaks to handle new 6-minute VIIRS
#					granules (SNPP)
# Norman Kuring		 5-Apr-2016	Add hyperlinks for GEO-M_SNPP.nc files.
# Norman Kuring		 6-May-2016	Add GOCI data to the browser.
# Norman Kuring		16-Jun-2016	Add VIIRS SST data.
# Norman Kuring		 3-Feb-2017	Add OLCI access.
# Norman Kuring		25-Aug-2017	Add "CONFIRMATION BY WEB" to the email
#					message.
# Norman Kuring		 3-Oct-2018	Switch to reprocessed (.nc) HICO data
#					and add JPSS1/VIIRS data to browser.
# Norman Kuring		 4-Oct-2018	Add HICO CHL browse images.
# Norman Kuring		20-May-2019	HICO parent_table was missing L1B file
#					link when CHL was selected.  Fixed.
# Norman Kuring		11-Jul-2019	Add Google Tag Manager code for the
#					bean counters.
# Norman Kuring		29-Aug-2019	Added a few return statements to the
#					fullpath subroutine for consistency's
#					sake.
# Norman Kuring		 6-Sep-2019	Was still looking for insitu push-pin
#					images under .../htdocs/icons/ .  The
#					code has been fixed to look under
#					.../static/icons/ instead.
# Norman Kuring		 7-Nov-2019	Replace the filename, translate_fname,
#					and full_path subroutines with
#					fname2msntime and msntimetyp2name
#					in hopes that those will be easier to
#					maintain for however much longer we
#					still need this steaming pile of code.
# Norman Kuring		 8-Nov-2019	Add three more types of Sentinel-3
#					data to the mission selector and start
#					using the new naming convention for
#					SNPP/VIIRS SST files.
# Norman Kuring		 9-Dec-2019	Make the swathdays subroutine use the
#					fname2msntime subroutine.
# Norman Kuring		16-Jan-2020	At Sean's direction, changed the default
#					prefix for getfile from cgi to ob.
# Norman Kuring		 6-Apr-2020	Adding new data-type IDs to recover
#					hyperlinks to VIIRS GEO files that seem
#					to have gone missing at some point.
# Norman Kuring		20-Jul-2020	Add a notice about reprocessed MERIS
#					data available from ESA.
# Norman Kuring		15-Jan-2021	Fix some improper form/table nesting.
# Norman Kuring		16-Apr-2021	Adding HawkEye data to the browser.
# Norman Kuring		21-Apr-2021	Hard code path to programs in
#					/web/oceancolor/bin/ .
# Norman Kuring		23-Apr-2021	Use a larger scale locator map for
#					HawkEye scenes.
# Norman Kuring		16-Jun-2021	L1 OLCI and MERIS data can now be
#					extracted.

our $invocation_time = time;

use Time::Local;
use IPC::Run3 qw();
use Storable qw(retrieve);
use List::MoreUtils qw(uniq);

sub run3 {
	# This STDOUT junk is a workaround for running run/run3 in an mod_perl/CGI::Compile context
	#save off existing stdout
	my ($save_stdin,$save_stdout);
	open $save_stdin, '>&STDIN';
	open $save_stdout, '>&STDOUT';
	#open it again as the "normal things"
	open STDIN, '>&=0';
	open STDOUT, '>&=1';

	my $ret = IPC::Run3::run3(@_);
		
	#clean up after yourself
	close(STDIN);
	close(STDOUT);
	open STDIN, '>&', $save_stdin;	
	open STDOUT, '>&', $save_stdout;	
	# End mod_perl/CGI::Compile rigging
	return $ret;
}

use CGI qw(header start_html);
use Math::Round qw( nearest );
use OBPG::Launder qw(launder);
use OBPG::LocateFile qw(locate_file rename_L0 exclude date2dataday);
use File::Path;

File::Path::make_path($ENV{GMT_TMPDIR} = "/tmpfs01/gmt_tmp/$$");

my ($plack_session) = ($ENV{HTTP_COOKIE} =~ /plack_session2=([\da-f]+)/);

our $session;
my $session_file = "$ENV{session_dir}/$plack_session";
if (-f $session_file){
	$session = retrieve($session_file);
} else {
	$session = {profile => {}};
}

# Subroutines i.e. types of page layouts
our %sub = (
level3        => \&level3,
level1or2list => \&level1or2list,
level1or2     => \&level1or2,
order         => \&order,
l3image       => \&l3image,
l12image      => \&l12image,
l12thumb      => \&l12thumb,
locimage      => \&locimage,
mapimg        => \&mapimg,
aoimap        => \&aoimap,
xtrctmap      => \&xtrctmap,
date_ranges   => \&date_ranges,
filenamelist  => \&filenamelist,
help          => \&help
);

# Sensors
our %sen = (
#                                   quicklook             file
#                                   stacking    database  name
#       name              1st char  sort order  sen_id    type
hico => ['HICO(ISS)'        , 'H' ,     1,          25,   'ISS'  ],
swml => ['SeaWiFS(MLAC)'    , 'S' ,     2,           6,   'MLAC' ],
swga => ['SeaWiFS(GAC)'     , 'S' ,     3,           6,   'GAC'  ],
amod => ['Aqua'             , 'A' ,     4,           7,   'LAC'  ],
tmod => ['Terra'            , 'T' ,     5,           8,   'LAC'  ],
octs => ['OCTS(ADEOS)'      , 'O' ,     6,           9,   'GAC'  ],
czcs => ['CZCS(Nimbus-7)'   , 'C' ,     7,          11,   'MLAC' ],
mefr => ['MERIS(FRS)'       , 'M' ,     8,          19,   'FRS'  ],
merr => ['MERIS(RR)'        , 'M' ,     9,          19,   'RR'   ],
vrsn => ['Suomi-NPP'        , 'V' ,    10,          14,   'SNPP' ],
vrj1 => ['NOAA-20'          , 'V' ,    11,          33,   'JPSS1'],
s3ar => ['Sentinel3A(ERR)'  , 'L' ,    12,          29,   'ERR'  ],
s3af => ['Sentinel3A(EFR)'  , 'L' ,    13,          29,   'EFR'  ],
s3br => ['Sentinel3B(ERR)'  , 'LS3B' , 14,          36,   'ERR'  ],
s3bf => ['Sentinel3B(EFR)'  , 'LS3B' , 15,          36,   'EFR'  ],
goci => ['GOCI(COMS)'       , 'G' ,    16,          27,   'COMS' ],
hawk => ['HawkEye(SeaHawk)' , 'K' ,    17,          35,   'LAC'  ],
);

our %dtids = (
hico => [1381,1382],
swga => [1120,1121,1124],
swml => [1119,1121,1124],
amod => [7,46,1040,1041,1047,1050,1053,1059,1062,1353,1044,1056,1356],
tmod => [4,45,1080,1081,1087,1090,1093,1099,1102,1362,1084,1096,1365],
octs => [1140,1141,1144],
czcs => [1151,1152],
mefr => [36,1180,1183,1178],
merr => [36,1180,1183,1179],
vrsn => [1000,1001,1002,1003,1005,1011,1014,1017,1023,1026,1008,1020],
vrj1 => [1190,1191,1192,1193,1195,1198,1201,1204,1207,1210,1213,1216],
s3ar => [1261,1263,1265,1267,1269],
s3af => [1260,1262,1264,1266,1268],
s3br => [1461,1463,1465,1467,1469],
s3bf => [1460,1462,1464,1466,1468],
goci => [71,1250],
hawk => [1281,1282],
);

our $senchars = join '', map $_->[1], values %sen;

# Some significant dates.
our $czcs_end      = daynumber("22Jun1986");
our $octs_start    = daynumber("01Nov1996");
our $octs_end      = daynumber("30Jun1997");
our $seawifs_start = daynumber("04Sep1997");

# Parameters
our %prm = (
#	 parameter name                        1st      2nd  filename fragments
CHL => ['Chlorophyll'                        , 'L2'    ,'_CHLOR_A'    ],
TC  => ['Quasi True Color'                   , 'L1A'   ,''            ],
SST => ['Sea Surface Temperature (11 &#956;)', 'L2'    ,'_SST'        ],
SST4=> ['Sea Surface Temperature (4 &#956;)' , 'L2'    ,'_SST4'       ],
L0  => ['Level-0'                            , 'L0'    ,''            ],
);

our %brs2parent = (
TC   => 'L1',
CHL  => 'L2OC',
SST  => 'L2SST',
SST4 => 'L2SST4'
);

# Day, night, or mixed
our %dnm = (
D => 'Day',
N => 'Night',
M => 'Mixed',
);

# Time Periods (for level 3 composites)
our %per = (
DAY => 'Day',
'8D'=> 'Octave',
 MO => 'Month',
 YR => 'Year',
 CU => 'Mission',
CSTM=> 'Custom'
);

# Level-2 parameter subset groups and their opt_flag values.
our %l2prm = (
65536   => 'chlorophyll <u>a</u>',
# 131072  => 'sea surface temperature (MODIS only)',
262144  => 'Kd490 (K490)',
524288  => 'remote sensing reflectances (normalized, water-leaving radiances)',
1048576 => 'aerosol products',
8388608 => 'particulate inorganic carbon',
16777216 => 'particulate organic carbon',
#33554432 => 'colored dissolved organic matter index (no longer available after <a href="/reprocessing/r2014/">Reprocessing 2014.0</a>)',
67108864 => 'fluorescence line height and instantaneous PAR (MODIS only)',
134217728 => 'photosynthetically available radiation',
);

our $thisScript = $ENV{SCRIPT_NAME};
if( $thisScript =~ /^(\/(?:cgi|restrict|echo)\/browse\.pl)$/ ){
  $thisScript = $1;
}
else{
  die "Unexpected SCRIPT_NAME=\"$thisScript\"";
}

our $is_privileged = defined($session->{profile}{email_address});

# Quicklook directories
our %qdir;

$qdir{swga}{TC}{GAC}    = '/web/oceancolor/ql/SeaWiFS/TC/GAC';
$qdir{swga}{CHL}{GAC}   = '/web/oceancolor/ql/SeaWiFS/CHL/GAC';

$qdir{swml}{TC}{MLAC}   = '/web/oceancolor/ql/SeaWiFS/TC/MLAC';
$qdir{swml}{CHL}{MLAC}  = '/web/oceancolor/ql/SeaWiFS/CHL/MLAC';

$qdir{tmod}{TC}{LAC}  = '/web/oceancolor/ql/MODIST/TC/LAC';
$qdir{tmod}{CHL}{LAC} = '/web/oceancolor/ql/MODIST/CHL/LAC';
$qdir{tmod}{SST}{LAC} = '/web/oceancolor/ql/MODIST/SST/LAC';
$qdir{tmod}{SST4}{LAC}= '/web/oceancolor/ql/MODIST/SST4/LAC';

$qdir{amod}{TC}{LAC}  = '/web/oceancolor/ql/MODISA/TC/LAC';
$qdir{amod}{CHL}{LAC} = '/web/oceancolor/ql/MODISA/CHL/LAC';
$qdir{amod}{SST}{LAC} = '/web/oceancolor/ql/MODISA/SST/LAC';
$qdir{amod}{SST4}{LAC}= '/web/oceancolor/ql/MODISA/SST4/LAC';

$qdir{mefr}{TC}{FRS}  = '/web/oceancolor/ql/MERIS/TC/FRS';
$qdir{mefr}{CHL}{FRS} = '/web/oceancolor/ql/MERIS/CHL/FRS';
$qdir{merr}{TC}{RR}  = '/web/oceancolor/ql/MERIS/TC/RR';
$qdir{merr}{CHL}{RR} = '/web/oceancolor/ql/MERIS/CHL/RR';

$qdir{octs}{TC}{GAC}  = '/web/oceancolor/ql/OCTS/TC/GAC';
$qdir{octs}{CHL}{GAC} = '/web/oceancolor/ql/OCTS/CHL/GAC';

$qdir{czcs}{TC}{MLAC}  = '/web/oceancolor/ql/CZCS/TC/MLAC';
$qdir{czcs}{CHL}{MLAC} = '/web/oceancolor/ql/CZCS/CHL/MLAC';

$qdir{vrsn}{TC}{SNPP}  = '/web/oceancolor/ql/VIIRS/TC/SNPP';
$qdir{vrsn}{CHL}{SNPP} = '/web/oceancolor/ql/VIIRS/CHL/SNPP';
$qdir{vrsn}{SST}{SNPP} = '/web/oceancolor/ql/VIIRS/SST/SNPP';

$qdir{vrj1}{TC}{JPSS1} = '/web/oceancolor/ql/VIIRSJ1/TC/JPSS1';
$qdir{vrj1}{CHL}{JPSS1}= '/web/oceancolor/ql/VIIRSJ1/CHL/JPSS1';

$qdir{hico}{TC}{ISS}  = '/web/oceancolor/ql/HICO/TC/ISS';
$qdir{hico}{CHL}{ISS}  = '/web/oceancolor/ql/HICO/CHL/ISS';

$qdir{goci}{TC}{COMS}   = '/web/oceancolor/ql/GOCI/TC/COMS';
$qdir{goci}{CHL}{COMS}  = '/web/oceancolor/ql/GOCI/CHL/COMS';

$qdir{s3ar}{TC}{ERR}  = '/web/oceancolor/ql/S3OLCI/TC/ERR';
$qdir{s3ar}{CHL}{ERR}  = '/web/oceancolor/ql/S3OLCI/CHL/ERR';
$qdir{s3af}{TC}{EFR}  = '/web/oceancolor/ql/S3OLCI/TC/EFR';
$qdir{s3af}{CHL}{EFR}  = '/web/oceancolor/ql/S3OLCI/CHL/EFR';

$qdir{s3br}{TC}{ERR}  = '/web/oceancolor/ql/S3BOLCI/TC/ERR';
$qdir{s3br}{CHL}{ERR}  = '/web/oceancolor/ql/S3BOLCI/CHL/ERR';
$qdir{s3bf}{TC}{EFR}  = '/web/oceancolor/ql/S3BOLCI/TC/EFR';
$qdir{s3bf}{CHL}{EFR}  = '/web/oceancolor/ql/S3BOLCI/CHL/EFR';

$qdir{hawk}{TC}{LAC} = '/web/oceancolor/ql/SE1/TC/LAC';
$qdir{hawk}{CHL}{LAC} = '/web/oceancolor/ql/SE1/CHL/LAC';

our $qbinsetdir       = '/web/oceancolor/metadata/qbin6sets';
our $coast512x256_day = '/web/oceancolor/metadata/world_outline.png';
our $coast512x256_ngt = '/web/oceancolor/metadata/world_outline.0-360.png';
our $not_avail_img = '/web/oceancolor/resources/browse_image_not_available.ppm';

# Temporary files get written here.
our $tmpdir = '/tmpfs01';
our $logdir = '/web/oceancolor/logfiles';

# Other programs used by this script.
our $ncdump	      = 'ncdump-hdf -h';
our $hdfrasdims        = '/web/oceancolor/bin/hdfrasdims';
our $hdfrastopnm       = '/web/oceancolor/bin/hdfrastopnm';
our $pnmscale          = "pnmscale -quiet";
our $pnmpad            = "pnmpad -quiet";
our $pnmflip           = "pnmflip -quiet";
our $pnmtopng          = "pnmtopng -quiet";
our $pngtopnm          = "pngtopnm -quiet -mix -background black";
our $pamdice           = "pamdice -quiet";
our $pnmcat            = "pnmcat -quiet";
our $composite         = 'composite';
our $convert           = 'convert';
our $latlon2qbin       = '/web/oceancolor/bin/latlon2qbin';
our $outline2bins      = '/web/oceancolor/bin/outline2bins';
our $circle            = '/web/oceancolor/bin/circle_coords';
our $qbins2outlines    = '/web/oceancolor/bin/qbins2outlines';
our $composite_browse  = '/web/oceancolor/bin/cmpbrs512';
our $gmt               = 'gmt';
our $sun               = '/web/oceancolor/bin/subsolarpoint';
our $rnav              = '/web/oceancolor/bin/brsrevnav';
our $gs                = 'gs -sDEVICE=ppmraw -SOutputFile=- -q -r100';
our $mapbrsscene       = '/data1/www-data/oceancolor.gsfc.nasa.gov/scripts/mapbrsscene.pl';

# For sending email regarding data orders ...
our $return_addr = 'dataorders@seawifs.gsfc.nasa.gov';
our $sendmail = "sendmail -f$return_addr -t";

# The url of the OC Forum.
our $comment_url = '/forum/oceancolor/forum_show.pl';

# Menu buttons.
our %button = (
up_on      => menu_button('/icons/button_active_ua.png'),
up_off     => menu_button('/icons/button_inactive_ua.png'),
left_on    => menu_button('/icons/button_active_la.png'),
left_off   => menu_button('/icons/button_inactive_la.png'),
right_on   => menu_button('/icons/button_active_ra.png'),
right_off  => menu_button('/icons/button_inactive_ra.png'),
hlt_on     => menu_button('/icons/button_flashlight_on.png'),
hlt_off    => menu_button('/icons/button_flashlight_off.png'),
hlt_na     => menu_button('/icons/button_flashlight_na.png'),
eye_open   => menu_button('/icons/button_eye_open.png'),
eye_closed => menu_button('/icons/button_eye_closed.png'),
eye_off    => menu_button('/icons/button_eye_inactive.png'),
TC_on      => menu_button('/icons/button_active_TC.png'),
TC_off     => menu_button('/icons/button_inactive_TC.png'),
CHL_on     => menu_button('/icons/button_active_Chl.png'),
CHL_off    => menu_button('/icons/button_inactive_Chl.png'),
SST_on     => menu_button('/icons/button_active_SST.png'),
SST_off    => menu_button('/icons/button_inactive_SST.png'),
SST4_on    => menu_button('/icons/button_active_SST4.png'),
SST4_off   => menu_button('/icons/button_inactive_SST4.png'),
cbar       => menu_button('/icons/button_active_cbar.png'),
order      => menu_button('/icons/button_active_Order.png'),
comment    => menu_button('/icons/button_active_Comment.png'),
help       => menu_button('/icons/button_active_Help.png'),
);

# Add the alt tags that are invariant.
$button{hlt_on}     =~ s/alt=""/alt="Area of interest is highlighted. Turn highlights off."/;
$button{hlt_off}    =~ s/alt=""/alt="Area of interest is not highlighted. Turn highlights on."/;
$button{eye_open}   =~ s/alt=""/alt="Viewing only the user's selections. Click to view all the search results."/;
$button{eye_closed} =~ s/alt=""/alt="Viewing the complete list of search results. Click to restrict to user's selections."/;
$button{TC_on}      =~ s/alt=""/alt="Switch to viewing of quasi-true-color images"/;
$button{CHL_on}     =~ s/alt=""/alt="Switch to viewing of chlorophyll images"/;
$button{SST_on}     =~ s/alt=""/alt="Switch to viewing of 11 micron sea-surface-temperature images"/;
$button{SST4_on}    =~ s/alt=""/alt="Switch to viewing of 4 micron sea-surface-temperature images"/;
$button{cbar}       =~ s/alt=""/alt="View color scales"/;
$button{order}      =~ s/alt=""/alt="Order selected scenes"/;
$button{comment}    =~ s/alt=""/alt="Make a comment or ask a question"/;
$button{help}       =~ s/alt=""/alt="Help"/;

our $l3wid = 512;
our $l3hgt = $l3wid/2;

our $xtrctmap_w = 360;
our $xtrctmap_h = 300;

use OBPG::CGI qw(ReadParse);
our %in = ReadParse();

# Filter out shell metacharacters
foreach our $key ( keys %in ){
  $in{$key} = launder( $in{$key}, "\$in{$key}" );
}

# Open connection to MySQL database.
our $dbh = DBI->connect("DBI:mysql:database=webstuff;host=$ENV{webDB}", 'web_user', 'gimmedata') or die "Couldn't connect!\n".DBI->errstr;
$dbh->do("SET tx_isolation='READ-COMMITTED'");
$dbh->do("SET time_zone = '+0:00';");
# Specify default parameters.
if(defined $in{file} and keys(%in) == 1){
  # This is for the special case where a user
  # passes just a filename in the QUERY_STRING.
  foreach our $p (qw/TC CHL SST SST4/){
    my $f = msntimetyp2name(fname2msntime($in{file}),$p);
    if($f){
      $in{file} = (split /\//,$f)[-1];
      $in{prm} = $p;
      last;
    }
  }
  my ($daynight,@days) = swathdays($in{file});
  $in{dnm} = $daynight;
  $in{day} = $days[0];
  $in{sub} = 'level1or2list';
}
$in{sub} = 'level3' unless defined $in{sub} and defined $sub{$in{sub}};
$in{prm} = 'CHL'    unless defined $in{prm} and defined $prm{$in{prm}};
$in{set} = 10       unless defined $in{set} and $in{set} > 1;
$in{ndx} = 0        unless defined $in{ndx} and $in{ndx} > 0;
$in{qbt} = 0        unless defined $in{qbt};
$in{dnm} = 'D@M'    unless defined $in{dnm};
$in{dnm} = 'D@N'    if $in{sub} eq 'filenamelist';
$in{rad} = 0        unless defined $in{rad};
$in{frc} = 0        unless defined $in{frc};

# Certain products are not available for nighttime-only searches.
# The same is true for daytime-only searches.  Change the displayed
# parameter to SST in those cases since it is available for both
# daytime and nighttime.
if( $in{dnm} !~ /D/ and $in{prm} eq 'CHL' || $in{prm} eq 'TC' or $in{dnm} !~ /N/ and $in{prm} eq 'SST4'){
  $in{prm} = 'SST';
  if(defined $in{sen} and $in{sen} and $in{sen} !~ /[at]mod|vrsn/){
    # Add Aqua-MODIS to the list of sensors if a MODIS or VIIRS is not already
    # there because (at the time of this writing [10-Jun-2016]) only MODIS
    # and VIIRS offer SST data.
    $in{sen} .= '@amod';
  }
}
# If the user selected night, and prm was set to SST, but mission is NOT Aqua or
# Terra or VIIRS, set prm to CHL
if (defined $in{sen} and $in{sen} and $in{sen} !~ /[at]mod|vrsn/ and $in{prm} eq 'SST'){
  $in{prm} = 'CHL';
}

# Put a border around the currently selected parameter.
our $current_param_button = "$in{prm}_off";
$button{$current_param_button} =~ s/border="0"/border="3"/;

# Earliest and most recent days of mission.
our ($mission_start,$mission_end) = mission_dates($in{sen});
&{$sub{$in{sub}}};

$dbh->disconnect();


sub level3{

  $in{per} = 'CU' unless defined $in{per} and defined $per{$in{per}};
  $in{day} = $mission_end unless defined $in{day};

  # Three consecutive months are displayed in calendar form and
  # side by side on the web page.  The month displayed in the middle
  # calendar is controlled by the "mon" parameter (i.e. $in{mon}).

  if($in{per} eq 'MO' or $in{per} eq '8D' or $in{per} eq 'DAY'){
    # If the user clicks on any time period that is a month or
    # shorter, then set the "mon" parameter to that month.
    $in{mon} = (period_bounds('MO',$in{day}))[0];
  }

  if(!defined $in{mon}){
    # If a month has not been selected, then choose the month that
    # precedes the most recent month of the mission as the default.
    my $s = (period_bounds('MO',$mission_end))[0];
    $in{mon} = (period_bounds('MO',$s - 1))[0];
  }
  elsif($in{per} eq 'YR'){
    # If a "mon" parameter has been passed through but a different
    # year was clicked on, then attempt to display the same months
    # from the newly selected year.
    my $yr = (gmtime(86400*$in{day}))[5];
    my $mon = (gmtime(86400*$in{mon}))[4];
    $in{mon} = timegm(0,0,0,1,$mon,$yr)/86400;
  }

  # Restrict the three calendars displayed to the mission time period.
  my $mission_start_month_end = (period_bounds('MO',$mission_start))[1];
  my $mission_end_month_start = (period_bounds('MO',$mission_end))[0];
  if( $in{mon} <= $mission_start_month_end
  and $mission_end_month_start - $mission_start_month_end > 1){
    $in{mon} = $mission_start_month_end + 1;
  }
  elsif($in{mon} >= $mission_end_month_start){
    $in{mon} = (period_bounds('MO',$mission_end_month_start - 1))[0];
  }

  my $mission_table = mission_table($in{sen});

  my $cal1day = $in{mon} - 1;
  my $cal2day = $in{mon};
  my $cal3day = $in{mon} + 31;

  my $cal1 = calendar($cal1day);
  my $cal2 = calendar($cal2day);
  my $cal3 = calendar($cal3day);

  my ($prev,$next,$up) = prev_next_up($in{per},$in{day},$in{mon});

  my $period_string = period2str($in{per},$in{day},1);

  # Make the period string a hyperlink to the custom date range page.
  $period_string =
  qq@<a href="$thisScript?sub=date_ranges&@ .
  state_info(qw/url sub/) .
  qq@">$period_string</a>@;

  my $jdaystr = '';
  if($in{per} eq 'DAY'){
    $jdaystr = sprintf
    qq@<br />(<font color="#330000">%s</font><font color="#003300">%s</font>)@,
    unpack "a4a3", yyyyddd($in{day});
  }

  # Construct a filename for show only.
  # This filename will start with 1 to 3 letters (S,T,A) to indicate
  # the sensor combination.  If the combination includes SeaWiFS (S),
  # then the name will also indicate which data types (LAC,MLAC,GAC)
  # are included.  This is the filename that should come up in a file
  # save dialog if the user wants to save the map image to her disk.
  my $name;
  my @files = map {(split /\//)[-1]}
              qlpaths($in{sen},$in{prm},$in{per},$in{day},$in{dnm});
  if(@files){
    my %c;
    my %sortord;
    foreach my $sen (keys %sen){
      $sortord{ $sen{$sen}[1] } = $sen{$sen}[2];
    }
    grep $c{substr($_,0,1)}++ && 0,@files;
    my $c = join '', sort {$sortord{$a} <=> $sortord{$b}} keys %c;
    my @types = map {/^S\d+\.[A-Z]+\.([A-Z]+)/; $1;}
                grep /^S\d+\.[A-Z]+\.[A-Z]+/, @files;
    my $types = '';
    $types = '.' . join('_', @types) if @types;
    $name = $files[0];
    $name =~ s/^.(\d+\.[A-Z]+).*(\.[A-Z8]+\.png)$/$c$1$types$2/;
  }
  else{
    $name = (split /\//,$coast512x256)[-1];
  }

  my ($tc,$chl,$sst,$sst4) = tc_chl_sst($in{prm},$in{sen},$in{dnm});

  my $ticker = &ticker;

  my $region_select = &region_select;

  head("Specify Search Parameters");

  # Layout buttons along the top of the page. ###############################
  print
  qq@<table width="100%"><tr>\n@,

  # Left hand buttons
  qq@<td align="left">\n@,
  qq@<table border="0" cellpadding="0" cellspacing="0"><tr>\n@,
  qq@<td>$up</td>\n@,
  qq@<td>$prev</td>\n@,
  qq@<td>$next</td>\n@,
  qq@<td>$tc</td>\n@,
  qq@<td>$chl</td>\n@,
  qq@<td>$sst</td>\n@,
  qq@<td>$sst4</td>\n@,
  qq@<td><a href="/SeaWiFS/IMAGES/chl_sst_colorbar.png">@,
  qq@$button{cbar}</a></td>\n@,
  qq@</tr></table></td>\n@,

  # Ticker, comment button, and Help button
  qq@<td>$ticker</td>\n@,
  qq@<td align="right"><a href="$comment_url"\n@,
  qq@target="_blank">$button{comment}</a></td>\n@,
  qq@<td align="right">@,
  qq@<a href="javascript:helpwin('top')">$button{help}</a>@,
  qq@<noscript><a href="$thisScript?sub=help" target="OCBrowseHelp">@,
  qq@Help</a></noscript>@,
  qq@</td>\n@,

  qq@</tr></table>\n@,

  # Layout the search form. #################################################
  qq@<form method="post" action="$thisScript">\n@,
  qq@<input type="hidden" name="sub" value="level1or2list">\n@,
  state_info(qw/form sub file sen typ rad frc
                n w e s all set rgn id cul dnm is/),

  qq@<table >\n@,
  qq@<tr>\n@,
  qq@<td>\n@,
  qq@<table width="100%" border="1" cellpadding="0" cellspacing="0">\n@,	# start mission/day/night table
  qq@<tr>\n@,

  # SeaWiFS data type check boxes
  qq@<td>\n@,
  qq@<table width="100%" cellspacing="0">\n@;	# start SeaWiFS table

  foreach (qw/swga swml/){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<tr align="center">\n@,
    qq@<td>@,
    qq@<input type="checkbox" name="sen" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label>@,
    qq@</td>\n</tr>\n@;
  }

  print
  qq@</table>\n@,	# end SeaWiFS table
  qq@</td>\n@,

  # MODIS mission check boxes
  qq@<td>\n@,
  qq@<table width="100%" cellspacing="0">\n@,	# start MODIS table
  qq@<tr align="center">\n@,
  qq@<th>MODIS</th>\n@,
  qq@</tr>\n@;

  foreach (qw[ amod tmod ]){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<tr align="center">\n@,
    qq@<td>@,
    qq@<input type="checkbox" name="sen" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label>@,
    qq@</td>\n</tr>\n@;
  }


  print
  qq@</table>\n@,	# end MODIS table
  qq@</td>\n@,

  # VIIRS mission check boxes
  qq@<td>\n@,
  qq@<table width="100%" cellspacing="0">\n@,	# start VIIRS table
  qq@<tr align="center">\n@,
  qq@<th>VIIRS</th>\n@,
  qq@</tr>\n@;

  foreach (qw[ vrsn vrj1 ]){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<tr align="center">\n@,
    qq@<td>@,
    qq@<input type="checkbox" name="sen" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label>@,
    qq@</td>\n</tr>\n@;
  }
  print
  qq@</table>\n@,	# end VIIRS table
  qq@</td>\n@,

  # MERIS data type check boxes
  qq@<td>\n@,
  qq@<table width="100%" cellspacing="0">\n@;	# start MERIS table

  foreach (qw/merr mefr/){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<tr align="center">\n@,
    qq@<td>@,
    qq@<input type="checkbox" name="sen" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label>@,
    qq@</td>\n</tr>\n@;
  }
  print
  qq@</table>\n@,	# end MERIS table
  qq@</td>\n@;

  # Day, mixed, night check boxes
  print
  qq@<td rowspan=2 width="50px">\n@,
  qq@<table width="100%" cellspacing="0">\n@,	# start day/night table
  qq@<tr align="center">\n@,
  qq@<th>Select</th>\n@,
  qq@</tr>\n@;

  foreach (sort keys %dnm){
    /M/ and next;	# Don't present the mixed option for now.
    my $checked = '';
    defined $in{dnm} and $in{dnm} =~ /$_/ and $checked = ' checked';
    my $id="dnm_$_";
    print
    qq@<tr align="center">\n@,
    qq@<td>@,
    qq@<input type="checkbox" name="dnm" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$dnm{$_}</label></td>\n@,
    qq@</tr>\n@;
  }

  print
  qq@</table>\n@,	# end day/night table
  qq@</td>\n</tr>\n<tr>\n@,

  # S3A OLCI, S3B OLCI check boxes
  qq@<td>\n@,
  qq@<table width="100%" cellspacing="0">\n@,	# start OLCI table
  qq@<tr align="center">\n@,
  qq@<th colspan="2">OLCI</th>\n@,
  qq@</tr>\n<tr align="center">\n@;

  foreach (qw/s3ar s3af/){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<td><input type="checkbox" name="sen" id="$id"@,
    qq@ value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label></td>\n@;
  }
  print
  qq@</tr>\n<tr align="center">\n@;
  foreach (qw/s3br s3bf/){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<td><input type="checkbox" name="sen" id="$id"@,
    qq@ value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label></td>\n@;
  }
  print
  qq@</tr>\n@,
  qq@</table>\n@,	# end OLCI table
  qq@</td>\n@,

  # ... other sensors.
  qq@<td colspan="3">\n@,
  qq@<table width="100%" cellspacing="0">\n@,	# start other sensors table
  qq@<tr align="center">\n@;

  foreach (qw/octs hico goci czcs hawk/){
    my $checked = '';
    $in{sen} =~ /$_/ and $checked = ' checked';
    my $s = $sen{$_}[0];
    $s =~ s#\(#<br />(#;
    my $id = "sensor_$_";
    print
    qq@<td width="20%">@,
    qq@<input type="checkbox" name="sen" id="$id" value="$_"$checked>@,
    qq@<br />\n@,
    qq@<label for="$id">$s</label>@,
    qq@</td>\n@;
  }

  print
  qq@</tr>\n@,
  qq@</table>\n@,	# end other sensors table
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,	# end mission/day/night table

  # Search radii.
  qq@<table>\n@,
  qq@<tr align="center">\n@,
  qq@<td>\n@;

  print
  qq@<p>\n@,
  qq@Radius (<i>km</i>) about map click or about typed-in location:\n@,
  qq@<table border="1">\n@;

  foreach (qw/0 400 800 1200 1500/){
    my $checked = '';
    defined $in{rad} and $in{rad} eq $_ and $checked = ' checked';
    my $label = $_ || "72";
    print
    qq@<tr>\n@,
    qq@<td><input type="radio" name="rad" value="$_"$checked></td>\n@,
    qq@<td>$label</td>\n@,
    qq@</tr>\n@;
  }

  print
  qq@</table>\n@,
  qq@</p>\n@,
  qq@\n@,
  qq@</td>\n<td>\n@,
  qq@\n@,
  qq@<p>\n@,
  qq@Select swaths containing <i>(at least)</i>:<br />\n@,
  qq@<table border="1"><tr>\n@;

  my @numbr = (        0 ,  0.25 ,   0.5 ,  0.75 ,    1 );
  my @label = ('any part', '25 %', '50 %', '75 %', 'all');
  foreach (@numbr){
    my $checked = '';
    defined $in{frc} and $in{frc} == $_ and $checked = ' checked';
    my $label = shift @label;
    print
    qq@<tr>\n@,
    qq@<td><input type="radio" name="frc" value="$_"$checked></td>\n@,
    qq@<td>$label</td>\n@,
    qq@</tr>\n@;
  }

  # Pass through boundary coordinates if specified.
  my ($vn,$vw,$ve,$vs) = ('','','','');
  defined $in{n} and $vn = qq@ value="$in{n}"@;
  defined $in{w} and $vw = qq@ value="$in{w}"@;
  defined $in{e} and $ve = qq@ value="$in{e}"@;
  defined $in{s} and $vs = qq@ value="$in{s}"@;

  my $checked = '';
  defined $in{is} and $in{is} eq 'yes' and $checked = ' checked';

  print
  qq@</table>\n@,
  qq@of the area of interest.\n@,
  qq@</p>\n@,
  qq@</td>\n@,

  qq@<td>\n@,
  qq@Select<br />only<br />scenes<br />having<br />in situ<br />matchups.\n@,
  qq@<br />\n@,
  qq@<input type="checkbox" name="is" value="yes"$checked><br />\n@,
  qq@<img src="/icons/button_insitu.png" width="24" height="20" border="0">@,
  qq@</td>\n@,

  qq@</tr>\n@,
  qq@</table>\n@,
  qq@</td><td align="center">\n@,

  # Global map
  qq@<b>$period_string</b>$jdaystr<br />\n@,
  qq@<input type="image" name="i" width="$l3wid" height="$l3hgt"\n@,
  qq@border="0" alt="$name"\n@,
  qq@src="$thisScript/$name?sub=l3image&@,
  state_info(qw/url sub rad frc n w e s all set rgn id/),
  qq@"><br />\n@,
  qq@<b>$prm{$in{prm}}[0]</b>\n@,

  qq@<table width="100%">\n@,
  qq@<tr align="center">\n@,
  qq@<td>\n@,
  qq@Display results@,
  qq@<input type="text" name="set" value="$in{set}" size="3">@,
  qq@at a time.\n@,
  qq@</td>\n@,
  qq@<td>\n@,
  qq@<input type="submit" name="sub" value="Reconfigure page">\n@,
  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,

  qq@</td>\n@,
  qq@<td align="center">\n@,

  # Predefined regions
  qq@Select one or more regions:<br />\n@,
  $region_select,
  qq@<br />\n@,

  # Bounding box
  qq@or specify boundary coordinates or a single location:<br />\n@,
  qq@N:\n@,
  qq@<input type="text" name="n"$vn size="6">\n@,
  qq@<br />\n@,
  qq@W:<input type="text" name="w"$vw size="6">@,
  qq@<input type="text" name="e"$ve size="6">:E\n@,
  qq@<br />\n@,
  qq@S:\n@,
  qq@<input type="text" name="s"$vs size="6">\n@,
  qq@<br />\n@,

  # Find swaths button
  qq@<input type="submit" value="Find swaths">\n@,

  qq@</td>\n@,
  qq@</tr></table>\n@,
  qq@</form>\n@,

  # Layout the calendars. ###################################################
  qq@<table><tr valign="top">\n@,
  qq@<td>\n@,
  $mission_table,
  qq@</td><td>\n@,
  qq@<table>\n@,
  qq@<tr>\n@,
  qq@<td>\n$cal1</td>\n<td>\n$cal2</td>\n<td>\n$cal3</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,
  qq@</td>\n</tr></table>\n@;
  &tail;
}

sub level1or2list{
  my @list;
  my $n;
  my %qbins;
  $dbh->do("call temp_qbin(-1)");

  if(defined $in{file}){
    # This is for the special case where a user
    # passes just a filename in the QUERY_STRING.
    my ($msn,$time) = fname2msntime($in{file});
    my $yyyydddhhmmss = yyyydddhhmmss($time);
    my ($senchar,$sta) = @{$sen{$msn}}[1,4];
    $list[0] = [$senchar,$yyyydddhhmmss,$sta,0,0,0];
    $in{all} = 1;
    $in{per} = 'DAY';
    $in{sen} = $msn;
    $in{id} = "$invocation_time.$$";
    $in{prm} = 'TC' if $in{sen} eq 'hico';
    write_list($in{id},@list);
    &level1or2;
    return;
  }
  elsif(defined $in{id}){
    @list = read_list($in{id});
    $n = @list;
    if(defined $in{tog} and defined $in{ndx}){
      if(defined $list[$in{ndx}][6]){
        undef $list[$in{ndx}][6];
      }
      else{
        $list[$in{ndx}][6] = $in{ndx};
      }
      write_list($in{id},@list);
    }
  }
  elsif(
  defined $in{'i.x'} and defined $in{'i.y'}                               or
  ($in{n} ne '' xor $in{s} ne '') and ($in{w} ne '' xor $in{e} ne '')     or
  $in{n} ne '' and $in{s} == $in{n} and $in{w} ne '' and $in{e} == $in{w} or
  defined $in{rgn}                                                        or
  $in{n} ne '' and $in{s} ne '' and $in{w} ne '' and $in{e} ne ''
  ){
    undef $in{all};

    if(
    defined $in{'i.x'} and defined $in{'i.y'}                             or
    ($in{n} ne '' xor $in{s} ne '') and ($in{w} ne '' xor $in{e} ne '')   or
    $in{n} ne '' and $in{s} == $in{n} and $in{w} ne '' and $in{e} == $in{w}
    ){

      my ($lat,$lon) = &get_lat_lon;

      my @bins;

      if(defined $in{rad} and $in{rad} > 0){
        my $space = $in{rad}/10;
        my $c = "$circle $lat $lon $in{rad} $space | $outline2bins 6";
        open P,"$c|" or die qq@Error opening pipeline from command, "$c".@,
                            qq@ ($!) Died@;
        @bins = <P>;
        close P or die qq@Error executing command, "$c". ($!) Died@;
      }
      else{
        @bins = (`$latlon2qbin $lat $lon 6`);
      }
      chop @bins;
      grep $qbins{$_}++ && 0, @bins;
      foreach my $bin (@bins){$dbh->do("call temp_qbin($bin)")}
    }


    if(defined $in{rgn}
    or    $in{n} ne '' and $in{s} ne ''
    and   $in{w} ne '' and $in{e} ne ''){
      if(defined $in{rgn}){
        # The user selected a predefined region of interest.
        my @regions = split /@/, $in{rgn};
        my $r;
        foreach $r (@regions){
	    $dbh->do("call rgn2qbins(\'$r\')");

	    my $getregionscmd = <<"--";
SELECT quad_id from webstuff.quadregion
where region = \'$r\'
--
	    my $qid;
	    my @bins= ();
	    my $sth = $dbh->prepare("$getregionscmd");
		#$sth->{"mysql_use_result"} = 1;
	    $sth->execute;
	    $sth->bind_columns(\$qid);
	    while ($sth->fetch){push @bins,$qid}
	    $sth->finish;
	    grep $qbins{$_}++ && 0, @bins;
        }
      }
      if( $in{n} ne '' and $in{s} ne ''
      and $in{w} ne '' and $in{e} ne ''){
        # The user specified a bounding box for the search.
        my @cmd = ($outline2bins,6);
        my $input = "$in{n} $in{w}\n$in{n} $in{e}\n$in{s} $in{e}\n$in{s} $in{w}\n";

        my $delta = deltalon($in{w},$in{e});
        if($delta > 90){
          # Add more points to the outline when the left and right
          # sides of the box are more than 90 degrees apart.  The
          # outline2bins program does not work in those cases. (30-Mar-2006)
          my @lons = ();
          my $lon;
          for($lon=$in{w}; $lon<$in{w}+$delta; $lon+=30){
            my $l = $lon;
            $l += 360 while $l < -180;
            $l -= 360 while $l >= 180;
            push @lons,$l;
          }
          push @lons,$in{e};
          $input = '';
          my $i;
          for($i=0; $i<@lons; $i++){
            $input .= "$in{n} $lons[$i]\n";
          }
          for(--$i; $i>=0; $i--){
            $input .= "$in{s} $lons[$i]\n";
          }
        }

		my ($output,$err);
		run3(\@cmd,\$input,\$output,\$err) or die qq@Error running $outline2bins ($!)@;

        my @bins = split(' ',$output);
        grep $qbins{$_}++ && 0, @bins;
        foreach my $bin (@bins){$dbh->do("call temp_qbin($bin)")}
      }
    }

    my @qbins = keys %qbins;

    write_aoi_map("$invocation_time.$$",@qbins);

    @list = qbins2list($in{sen},$in{per},$in{day},
                       $in{frc},$in{rgn},$in{dnm},$in{is}, "$invocation_time.$$");

    $n = @list;
    if($n == 0){
      print header(-status=>204);
      return;
    }
    elsif($n == 1){
      $in{file} =
      (split /\//,msntimetyp2name(gran2msntime($list[0]),$in{prm}))[-1];
      $in{wid} = $list[0][3];
      $in{hgt} = $list[0][4];
      $in{id} = "$invocation_time.$$";
      write_list($in{id},@list);
      &level1or2;
      return;
    }
    else{
      $in{id} = "$invocation_time.$$";
      write_list($in{id},@list);
    }
  }
  else{
    $in{all} = 1;
    @list = datadays2list($in{sen},$in{per},$in{dnm},$in{is},$in{day});
    $n = @list;
    if($n == 0){
      print header(-status=>204);
      return;
    } elsif($n == 1){
      $in{file} =
      (split /\//,msntimetyp2name(gran2msntime($list[0]),$in{prm}))[-1];
      $in{wid} = $list[0][3];
      $in{hgt} = $list[0][4];
      $in{id} = "$invocation_time.$$";
      write_list($in{id},@list);
      &level1or2;
      return;
    } else{
      $in{id} = "$invocation_time.$$";
      write_list($in{id},@list);

      # Pass along boundary coordinates for
      # drawing the default extraction map.
      $in{cn} =   90;
      $in{cs} =  -90;
      $in{cw} = -180;
      $in{ce} =  180;
    }
  }
  if ($in{sen} eq 'ol'){
	for (@list){
		$_->[1] =~ s/(\d{6})$/T$1/;
	}
  }
  my $thumbtable = thumbtable(@list);
  if ($in{qbt} == 0){head("Thumbnails");}
  print $thumbtable;
  &tail;

  $dbh->do("call temp_qbin(-1)");
}

sub level1or2{

  # Compose the URLs for the left, right, and up arrows.
  # Also make the toggle button for switching between the culled
  # and the complete scene list if this image is one of multiple
  # search results.

  my $culled_complete;
  my $up;

  my $number = "1 swath found";

  my $prev = $button{left_off};
  my $next = $button{right_off};

  my @list = read_list($in{id});

  if(@list > 1){

    if(defined $in{tog} and defined $in{ndx}){
      if(defined $list[$in{ndx}][6]){
        undef $list[$in{ndx}][6];
      } else {
        $list[$in{ndx}][6] = $in{ndx};
      }
      write_list($in{id}, @list);
    }

    my $state = state_info(qw/url ndx file wid hgt/);

    my $n = @list;

    $number = ordinal($in{ndx} + 1) . " of $n swaths";

    my @selected = grep defined $_->[6], @list;

    unless(@selected){
      # No scenes have been singled out by the user yet, so it is useless
      # to offer a choice between a culled and complete list of scenes.
      $culled_complete = $button{eye_off};

      undef $in{cul} if defined $in{cul};
    }
    elsif(defined $in{cul}){

      # The user wants to see only the scenes she has interactively selected.

      my $i = find_nearest($in{ndx},\@selected);

      if($selected[$i][6] != $in{ndx}){
        # The ndx value passed to this script does not point to one of the
        # selected files, so I need to reset it (and file) so that it points
        # to the nearest selected file.
        $in{ndx} = $selected[$i][6];
        $in{file} =
        (split /\//,msntimetyp2name(gran2msntime($selected[$i]),$in{prm}))[-1];
        $in{wid} = $selected[$i][3];
        $in{hgt} = $selected[$i][4];
      }


      $number =
      sprintf "%s of %d swaths interactively selected from %d swaths",
      ordinal($i + 1),scalar(@selected),scalar(@list);

      if($i > 0){
        # Generate the hyperlink for the previous scene.
        my $file =
        (split /\//,msntimetyp2name(gran2msntime($selected[$i-1]),$in{prm}))[-1];
        my $wid  = $selected[$i - 1][3];
        my $hgt  = $selected[$i - 1][4];
        my $ndx  = $selected[$i - 1][6];

        $prev =~ s/inactive/active/;
        $prev =~ s/alt=""/alt="Later swath"/;

        $prev =
        qq@<a href="$thisScript?$state&ndx=$ndx&file=$file@ .
        qq@&wid=$wid&hgt=$hgt">$prev</a>\n@;
      }

      if($i < $#selected){
        # Generate the hyperlink for the next scene.
        my $file =
        (split /\//,msntimetyp2name(gran2msntime($selected[$i+1]),$in{prm}))[-1];
        my $wid  = $selected[$i + 1][3];
        my $hgt  = $selected[$i + 1][4];
        my $ndx  = $selected[$i + 1][6];
        $next =~ s/inactive/active/;
        $next =~ s/alt=""/alt="Earlier swath"/;

        $next =
        qq@<a href="$thisScript?$state&ndx=$ndx&file=$file@ .
        qq@&wid=$wid&hgt=$hgt">$next</a>\n@;
      }

      $culled_complete =
      qq@<a href="$thisScript?@ .
      state_info(qw/url cul/) .
      qq@">$button{eye_open}</a>@;
    } else{

      my $i = find_nearest($in{ndx},\@selected);

      my $file =
      (split /\//,msntimetyp2name(gran2msntime($selected[$i]),$in{prm}))[-1];
      my $wid  = $selected[$i][3];
      my $hgt  = $selected[$i][4];
      my $ndx  = $selected[$i][6];


      $culled_complete =
      qq@<a href="$thisScript?@ .
      state_info(qw/url file ndx wid hgt cul/) .
      qq@&cul=1&ndx=$ndx&file=$file&wid=$wid&hgt=$hgt">@ .
      qq@$button{eye_closed}</a>@;
    }

    unless(defined $in{cul}){
      if($in{ndx} > 0){
        my $i = $in{ndx} - 1;

        my $file =
        (split /\//,msntimetyp2name(gran2msntime($list[$i]),$in{prm}))[-1];

        $prev =~ s/inactive/active/;
        $prev =~ s/alt=""/alt="Later swath"/;
        $prev = qq@<a href="$thisScript?$state&ndx=$i&file=$file@ .
                qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$prev</a>\n@;
      }

      if($in{ndx} < $#list){
        my $i = $in{ndx} + 1;

        my $file =
        (split /\//,msntimetyp2name(gran2msntime($list[$i]),$in{prm}))[-1];

        $next =~ s/inactive/active/;
        $next =~ s/alt=""/alt="Earlier swath"/;
        $next = qq@<a href="$thisScript?$state&ndx=$i&file=$file@ .
                qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$next</a>\n@;
      }
    }

    $up = $button{up_on};
    $up =~ s/alt=""/alt="Up to list of thumbnails"/;

    $up =
    qq@<a href="$thisScript?sub=level1or2list&@ .
    state_info(qw/url sub file/) .
    qq@">$up</a>\n@;
  } else{
    $in{sub} = 'level1or2';

    $up = $button{up_on};
    $up =~ s/alt=""/alt="Up to Search Page"/;

    $up =
    qq@<a href="$thisScript?sub=level3&@ .
    state_info(qw/url sub file/) .
    qq@">$up</a>\n@;

    $culled_complete = $button{eye_off};
  }

  my $insitu_map = '';
  my $insitu_table = '';
  if($in{is} eq 'yes'){
    my @insitu;
    foreach my $prm (keys %prm){
      my $path = msntimetyp2name(fname2msntime($in{file}),$prm) or next;
      my $id = $in{all} ? undef : $in{id};
      @insitu = brs2insitu($path,$id) and last;
    }
    $insitu_table = insitu_table(@insitu);
  }

  my ($tc,$chl,$sst,$sst4) = tc_chl_sst($in{prm},$in{sen},$in{dnm});

  my $days = '';
  my ($daynight,@days) = swathdays($in{file});

  if($days[0] ne '---'){
    $days = qq@<table border="1">\n<tr align="center">\n@;
    my $d;
    foreach $d (reverse @days){
      my $str = period2str('DAY',$d,1);
      $days .= qq@<td>$str</td>\n@;
    }
    $days .= qq@</tr><tr align="center">\n@;
    foreach $d (reverse @days){
      my $str = yyyyddd($d);
      $days .= qq@<td>$str</td>\n@;
    }
    $days .= qq@</tr><tr align="center">\n@;
    my $fp = msntimetyp2name(fname2msntime($in{file}),$in{prm});
    my $f = (split /\//,$fp)[-1];
    if(-s $fp){
      # Only request the locator image if the browse file exists
      # and has a nonzero size.

      foreach $d (reverse @days){
        $days .=
        qq@<td><img\nsrc="$thisScript/$in{file}.map.png@ .
        qq@?sub=locimage&file=$f&day=$d&dnm=$dnm2{$daynight}"\n@ .
        qq@alt="swath locator image" width="256" height="128"></td>\n@;
      }

      # Add a different type of locator image.
      $days .=
      qq@</tr>\n<tr>\n<td><img\nsrc="$thisScript/$in{file}.map2.png@ .
      qq@?sub=mapimg&file=$f"\n@ .
      qq@alt="mapped browse image" width="256" height="256"></td>\n@;
    }
    else{
      my $nd = @days;
      my $prm = $prm{$in{prm}}[0];
      $days .=
      qq@<td colspan="$nd">There is currently no browse file available @ .
      qq@for the $prm parameter.</td>\n@;
    }
    $days .= qq@</tr>\n</table>\n@;
  }

  my $parent_table = parent_table($in{file});
  my $swath_table = swath_table($in{file},defined $list[$in{ndx}][6]);
  my $search_criteria = &present_criteria;

  my $highlight = highlight($in{hlt});

  head("Individual Swath");
  print
  qq@<table width="100%"><tr>\n@,
  qq@<td align="left">\n@,
  qq@<table border="0" cellpadding="0" cellspacing="0"><tr>\n@,
  qq@<td>$up</td>\n@,
  qq@<td>$prev</td>\n@,
  qq@<td>$next</td>\n@,
  qq@<td>$highlight</td>\n@,
  qq@<td>$culled_complete</td>\n@,
  qq@<td>$tc</td>\n@,
  qq@<td>$chl</td>\n@,
  qq@<td>$sst</td>\n@,
  qq@<td>$sst4</td>\n@,
  qq@<td><a href="/SeaWiFS/IMAGES/chl_sst_colorbar.png">@,
  qq@$button{cbar}</a></td>\n@,
  qq@</tr></table></td>\n@,
  qq@<td align="right"><a href="$comment_url"\n@,
  qq@target="_blank">$button{comment}</a></td>\n@,
  qq@<td align="right">@,
  qq@<a href="javascript:helpwin('scene')">$button{help}</a>@,
  qq@<noscript><a href="$thisScript?sub=help&hlp=scene">@,
  qq@Help</a></noscript>@,
  qq@</td>\n@,
  qq@</tr></table>\n@,

  qq@<table>\n@,
  qq@<tr>\n@,
  qq@<td valign="top">\n@,

  $parent_table,
  $swath_table,

  qq@</td>\n@,
  qq@<td valign="top">\n@,

  qq@<table>\n@,
  qq@<tr>\n@,
  qq@<td>\n@,

  $days,

  qq@</td>\n@,
  qq@</tr>\n@,
  qq@<tr>\n@,
  qq@<td>\n@,

  $search_criteria,

  qq@<p>\n@,
  qq@<b>Number of swaths:</b> $number\n@,
  qq@</p>\n@,

  $insitu_table,

  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,

  qq@</td>\n@,
  qq@</tr>\n@,
  qq@</table>\n@,

  $insitu_map;

  &tail;
}

sub order{
   unless ($session->{profile}{uid}){
      return level3(@_);
   }
   # Read the list of search results.
   my @list = read_list($in{id});

   # If any of the scenes in the search results have been
   # specially selected, then reduce the list of scenes
   # so that only the specially selected ones get considered.
   my @reduced = grep defined $_->[6], @list;
   @list = @reduced if @reduced > 0;

   # Determine whether there are any MODIS scenes in the search results.
   my $modis_scenes = grep $_->[0] =~ /^[TA]$/, @list;

   # Determine whether there are any VIIRS scenes in the search results.
   my $viirs_scenes = grep $_->[0] =~ /^V$/, @list;

   # Are there any scenes from Reprocessing 2014 in the list (that have IOPs)?
   my $reproc2014 = grep $_->[0] =~ /^[OVATS]$/, @list;

   # Determine whether there are any HICO L1B scenes in the search results.
   my $hicol1b_scenes= grep $_->[0] =~ /^H$/, @list;

   # Determine whether there are any HICO L1B scenes in the search results.
   my $gocil1b_scenes= grep $_->[0] =~ /^G$/, @list;

   # The flh and ipar extracts are only available for Aqua-MODIS.
   $modis_scenes or delete $l2prm{67108864};

   # Store the users order preferences in the @ord array and %ord hash.
   my @ord = split /@/, $in{ord};
   my %ord;
   grep $ord{$_}++ && 0, @ord;

   # Count the number of level-2 parameters
   # that the user selected for extraction.
   my @l2prm = sort {$a <=> $b} keys %l2prm;
   my $prmcount = 0;
   foreach (@l2prm){
      $prmcount++ if defined $ord{$_};
   }

   if($prmcount > 0){

      # Make sure that the level-2 opt_flag bit is set if any of
      # the level-2 parameters have been selected for extraction.
      $ord{2} = 1;

      if($prmcount == @l2prm){
         # If a user selects all the possible parameter extracts, it is the
         # same as not having selected any at all i.e. the file need not be
         # extracted.  Below, I remove the parameter extract flags if they
         # have all been specified.
         foreach (@l2prm){
            delete $ord{$_};
         }
         $prmcount = 0;
      }
   }

   # Reconstitute the @ord array.
   @ord = sort {$a <=> $b} keys %ord;

   # Generate the HTML for the buttons across the top of the page.
   my $state = state_info(qw/url sub file/);
   my $up = $button{up_on};
   $up =~ s/alt=""/alt="Up to Thumbnails Page"/;

   my $menubar =
   qq@<table width="100%"><tr>\n@ .
   qq@<td align="left">\n@ .
   qq@<table border="0" cellpadding="0" cellspacing="0"><tr>\n@ .
   qq@<td><a href="$thisScript?sub=level1or2list&$state">@ .
   qq@$up</a></td>\n@ .
   qq@</tr></table></td>\n@ .
   qq@<td align="right"><a href="$comment_url"\n@ .
   qq@target="_blank">$button{comment}</a></td>\n@ .
   qq@<td align="right">@ .
   qq@<a href="javascript:helpwin('order')">$button{help}</a>@ .
   qq@<noscript><a href="$thisScript?sub=help&hlp=order" @ .
   qq@target="OCBrowseHelp">@ .
   qq@Help</a></noscript>@ .
   qq@</td>\n@ .
   qq@</tr></table>\n@;

   #Do not add any bits.  Contact John Wilding or Tommy Owens.
   #DO NOT USE THE FOLLOWING
   #ORDERS_OPT_STAGE_BIN=268435456
   #ORDERS_OPT_STAGE_MAP=536870912
   #ORDERS_OPT_STAGE_PNG=1073741824

   if(!grep /^(1|2|1024|2097152|4194304|131072)$/, @ord
  or !defined $in{xtr} or $in{xtr} eq ''){

      # Pass through coordinates of box that circumscribes
      # the user's area of interest.
      my ($vn,$vw,$ve,$vs) = ('','','','');
      defined $in{cn} and $vn = qq@ value="$in{cn}"@;
      defined $in{cw} and $vw = qq@ value="$in{cw}"@;
      defined $in{ce} and $ve = qq@ value="$in{ce}"@;
      defined $in{cs} and $vs = qq@ value="$in{cs}"@;

      my $lev0 = $modis_scenes == 0 ? '' :
      qq@<input type="checkbox" name="ord" value="1024"@ .
      ord_checked(1024,@ord) .
      qq@>\n@ .
      qq@Level 0 <font size="-1"><i>(MODIS only)\n@ .
      qq@Note that we currently do not extract level-0 data\n@ .
      qq@regardless of what you may have selected for other data\n@ .
      qq@in your order.</i></font><br />\n@;

      my $lev0b = ($modis_scenes || $hicol1b_scenes || $gocil1b_scenes) == 0 ? '' :
      qq@<p><b>\n@ .
      qq@If you select Level-0 data below or if any SEAHAWK or HICO L1 scenes\n@ .
      qq@are included in your order, then none of the files you\n@ .
      qq@order will be extracted regardless of which other boxes you checked\n@ .
      qq@below.</p><p> If you want to get Level-0 or SEAHAWK/HICO L1 data and other\n@ .
      qq@extracted data, then you must place separate orders.\n@ .
      qq@</b></p>\n@;

      my $lev2sst = $modis_scenes == 0 && $viirs_scenes == 0 ? '' :
      qq@<input type="checkbox" name="ord" value="4194304"@ .
      ord_checked(4194304,@ord) .
      qq@>\n@ .
      qq@Level 2 SST (11 &#956;) (MODIS and VIIRS only)<br />\n@;

      if($modis_scenes){
         $lev2sst .=
         qq@<input type="checkbox" name="ord" value="2097152"@ .
         ord_checked(2097152,@ord) .
         qq@>\n@ .
         qq@Level 2 SST (4 &#956;) (MODIS nighttime only)<br />\n@;
      }

      #if($viirs_scenes){
      #  $lev2sst .=
      #  qq@<input type="checkbox" name="ord" value="536870912"@ .
      #  ord_checked(536870912,@ord) .
      #  qq@>\n@ .
      #  qq@Level 2 SST (triple window) (VIIRS nighttime only)<br />\n@;
      #}

      my $lev2iop = $reproc2014 == 0 ? '' :
      qq@<input type="checkbox" name="ord" value="131072"@ .
      ord_checked(131072,@ord) .
      qq@>\n@ .
      qq@Level 2 (IOP) (Only available for missions\n@ .
      qq@<a href="/reprocessing/r2014/">reprocessed @ .
      qq@since 2014</a>)\n@ .
      qq@<br />\n@;

      head("Scene Order Form");

      print $menubar;

      my $ord_reqired = '';
      if( defined $in{rvw}
       and !grep /^(1|2|4|8|1024|2097152|4194304)$/, @ord){
         $ord_reqired =
         qq@\n<b style="color: #ff0000; background-color: black; @ .
         qq@font-size: 150%">You must check at least one\n@ .
         qq@of the following product choices.</b><br />\n@;
      }

      my @b = ('','');
      if(defined $in{rvw} and (!defined $in{xtr} or $in{xtr} eq '')){
         @b = (
         '<b style="color: #ff0000; background-color: black; font-size: 150%">',
         '</b>'
         );
      }

      print
      qq@</center>\n@,
      qq@<form action="$thisScript" method="post">\n@,
      qq@<input type="hidden" name="rvw" value="1">\n@,

      state_info(qw/form ord xn xs xw xe xtr/),

      qq@<p>\n@,
      qq@All correspondence regarding this particular\n@,
      qq@order will be sent to the following email address.<br />\n@,
      qq@<strong>$session->{profile}{email_address}</strong>\n@,
      qq@</p>\n@,

      qq@<table border="1">\n@,
      qq@<tr>\n@,
      qq@<td width="50%">\n@,
      $lev0b,
      qq@<p>\n@,
      qq@In order to reduce the volume of data that you have to deal with,\n@,
      qq@we can extract the geographical area indicated at right from the\n@,
      qq@swaths you ordered before we place the data in our download area.\n@,
      qq@</p>\n@,
      qq@<p>\n@,
      $b[0],
      qq@Please choose one of the following options.<br />\n@,
      $b[1],
      qq@<strong>Do</strong><input type="radio" name="xtr" value="1"@,
      xtr_checked(1),
      qq@>\n@,
      qq@<strong>Do not</strong><input type="radio" name="xtr" value="0"@,
      xtr_checked(0),
      qq@>\n@,
      qq@extract my order for me.\n@,
      qq@</p>\n@,
      qq@<p>\n@,
      qq@You may adjust the extraction region by altering the coordinates\n@,
      qq@at right.\n@,
      qq@</p>\n@,
      qq@<p>\n@,
      qq@The default coordinates are the ones which circumscribe the area or\n@,
      qq@areas of interest that you used to do your search.  If you started\n@,
      qq@your search by just clicking on the world map without specifying\n@,
      qq@a larger search radius, then you may want to increase the size\n@,
      qq@of your extract region since the default search radius is 72\n@,
      qq@kilometers.\n@,
      qq@</p>\n@,
      qq@<p>\n@,
      qq@All four coordinates are expected to be in decimal degrees.\n@,
      qq@Degrees north of the\n@,
      qq@equator and east of the Greenwich meridian should be\n@,
      qq@positive, and degrees south of the equator and west\n@,
      qq@of the Greenwich meridian should be negative.\n@,
      qq@</p>\n@,
      qq@<p>\n@,
      qq@Extracted L1 data are processible with\n@,
      qq@<a href="/seadas/" target="_blank">SeaDAS</a>.\n@,
      qq@</p>\n@,
      qq@</td>\n@,
      qq@<td width="50%">\n@,
      qq@<table>\n@,
      qq@<tr>\n@,
      qq@<td></td>\n@,
      qq@<th>North<br /><input type="text" name="xn"$vn size="6"></th>\n@,
      qq@<td></td>\n@,
      qq@</tr><tr>\n@,
      qq@<th>West<br /><input type="text" name="xw"$vw size="7"></th>\n@,
      qq@<td><img src="$thisScript?sub=xtrctmap&@,
      state_info(qw/url sub per day prm set ndx mon rgn id rad n s w e/),
      qq@" width="$xtrctmap_w" height="$xtrctmap_h"></td>\n@,
      qq@<th>East<br /><input type="text" name="xe"$ve size="7"></th>\n@,
      qq@</tr><tr>\n@,
      qq@<td></td>\n@,
      qq@<th>South<br /><input type="text" name="xs"$vs size="6"></th>\n@,
      qq@<td></td>\n@,
      qq@</tr>\n@,
      qq@</table>\n@,
      qq@</td>\n@,
      qq@</tr>\n@,
      qq@</table>\n@,

      qq@<p>\n@,
      qq@Pick which data products you want for your selected scenes.<br />\n@,
      $ord_reqired,
      $lev0,
      qq@<input type="checkbox" name="ord" value="1"@,
      ord_checked(1,@ord),
      qq@>\n@,
      qq@Level 1 <i>(GOCI, HICO, and HawkEye data cannot be extracted at level-1.  If you want those data sets plus extracts of other missions, please consider placing separate orders for the extractable and the non-extractable missions.)<br />\n@,
      qq@<input type="checkbox" name="ord" value="2"@,
      ord_checked(2,@ord),
      qq@>\n@,
      qq@Level 2 (OC)<br />\n@,
      qq@<blockquote>\n@,
      qq@<font size="-1"><i>\n@,

      qq@You may elect to receive only a subset of the products that\n@,
      qq@would normally be stored in the level-2 files you order by\n@,
      qq@checking one or more of the groupings listed below.  If you\n@,
      qq@select none of the product groups below and simply check\n@,
      qq@&quot;Level 2&quot; above, then you will receive all of the\n@,
      qq@products stored in our standard level-2 files.\n@,
      qq@<br /><br />\n@,
      qq@The <a href="/reprocessing/r2009/">2009</a> and\n@,
      qq@<a href="/reprocessing/r2014/">2014</a>\n@,
      qq@reprocessings expanded and changed the\n@,
      qq@<a href="/reprocessing/r2009/l2prod/">list</a> of\n@,
      qq@<a href="/atbd/">standard level-2 products</a> that we distribute.\n@,
      qq@Again, if you check none of the subsets below and only check\n@,
      qq@&quot;Level 2&quot; above, then you will receive all of the\n@,
      qq@products in each ordered standard level-2 file -- be it pre- or\n@,
      qq@post-reprocessing.</i>\n@,
      qq@<br />\n@;

      foreach (@l2prm){
         print
         qq@<input type="checkbox" name="ord" value="$_"@,
         ord_checked($_,@ord),
         qq@>\n@,
         qq@$l2prm{$_}<br />\n@;
      }

      print
      qq@</font></blockquote>\n@,
      $lev2iop,
      $lev2sst,
      qq@</p>\n@,

      qq@<p>\n@,
      qq@<font size="-1">\n@,
      qq@<input type="checkbox" name="ord" value="16" checked>\n@,
      qq@Remind me when my order is about to expire.<br />\n@,
      qq@<input type="checkbox" name="ord" value="32" checked>\n@,
      qq@Require my email confirmation for early file deletion.<br />\n@,
      qq@<input type="checkbox" name="ord" value="256" checked>\n@,
      qq@Notify me when my data have been deleted from the staging area.\n@,
      qq@</font>\n@,
      qq@</p>\n@,

      qq@<p>\n@,
      qq@<input type="submit" value="Review order">\n@,
      qq@</p>\n@,

      qq@</form>\n@,
      qq@<center>\n@;
   }
   else 
   {

      # Make a levels array for the limit_volume subroutine.
      my @levels = ();
      push @levels,'L1A'    if grep /^1$/        , @ord;
      push @levels,'L2'     if grep /^2$/        , @ord;
      push @levels,'L0'     if grep /^1024$/     , @ord;
      push @levels,'L2SST'  if grep /^4194304$/  , @ord;
      #push @levels,'L2SST3' if grep /^536870912$/, @ord;
      push @levels,'L2SST4' if grep /^2097152$/  , @ord;
      push @levels,'L2IOP'  if grep /^131072$/, @ord;

      # This is how large of an order (in bytes) that we are willing to stage.
      my $threshold = 100_000_000_000_000;
      my $readablethresh = add_commas($threshold);

      my (@unavail,@allowed,@restricted,@denied,$avol,$rvol,$dvol);
      limit_volume(\@levels,$threshold,\@list,
                 \@unavail,\@allowed,\@restricted,\@denied,
                 \$avol,\$rvol,\$dvol);

      if (defined $in{xn} and $in{xn} == 90. and 
      defined $in{xs} and $in{xs} == -90 and
      defined $in{xw} and $in{xw} == -180 and
      defined $in{xe} and $in{xe} == 180
      )
      {
	      $in{xtr} = 0;
      }

      if( grep /^1024$/, @ord or grep /^(H|SEAHAWK)/, @allowed ){
         # Disallow extracts if the user asked for any L0 data
         # or if any SEAHAWK/HICO scenes are in the allowed list.
         $in{xtr} = 0;
         $prmcount = 0;
      }

      if (defined $in{place_order} && $in{xtr} == 0 && $prmcount == 0){
	      &orderfilelist(@allowed,@denied);
      }
      elsif (defined $in{place_order} && ($in{xtr} != 0 or $prmcount > 0)){
         # Submit the order.
         # Generate an order ID.
         $orderid = &orderid;
         # Build the opt_flag.
         my $opt_flag = 0;
         # $opt_flag = 32768;	# Set the test bit while trying out new code.

         grep { $opt_flag += $_ } @ord;

         # Pass along the extract coordinates if they have been specified.
         my $xcoords = '';
         if( defined $in{xn} and $in{xn} ne '' and defined $in{xs} and $in{xs} ne '' and defined $in{xw} and $in{xw} ne '' and defined $in{xe} and $in{xe} ne '')
         {
            # Make sure that these are numbers in case
            # the user has entered non-number-related characters.
            $in{xn} += 0;
            $in{xs} += 0;
            $in{xw} += 0;
            $in{xe} += 0;

            $xcoords =
            "north=$in{xn}\n" .
            "south=$in{xs}\n" .
            "west=$in{xw}\n" .
            "east=$in{xe}\n";
         }

         # The stage_extracts flag gets set for both geographical
         # and parameter extracts.
         if($xcoords ne '' or $prmcount > 0){
            $opt_flag += 64;
         }

         # A URL for checking the status of the order
         my $status_url ="https://oceandata.sci.gsfc.nasa.gov/odps/";

         # Send order to the data system.
         $numfiles = @allowed;
         open P,"|$sendmail" or die qq@Could not open pipe to command, @,
                                 qq@"$sendmail". ($!)@;
         print P
         "From: $return_addr\n",
         "To: $return_addr\n",
         "Errors-To: norman\@seawifs.gsfc.nasa.gov\n",
         "Subject: DATA REQUEST [$orderid]\n",
         "\n",
         "email_address=$session->{profile}{email_address}\n",
         "volume=$avol\n",
         "opt_flag=$opt_flag\n",
         "number_of_files=$numfiles\n",
         $xcoords,
         "START_FILE_LIST\n";

         my @orderlist;
         if( @allowed == 0 and grep /^(4|8)$/,@ord and !grep /^(1|2|2097152|4194304)$/,@ord){
            # The user wishes to order ancillary data without the
            # corresponding level 1 or 2 files.
            @orderlist = list2names(\@list);
         }
         else{
            @orderlist = @allowed;
         }

         foreach my $f (@orderlist){
            print P "$f\n";
         }

         print P
         "END_FILE_LIST\n";

         close P or die qq@Error executing command, "$sendmail". ($!)@;

         # Send request for response to data requestor.

      my $request = <<"      --";
      From: $return_addr
      To: $session->{profile}{email_address}
      Errors-To: sdps\@seawifs.gsfc.nasa.gov
      Subject: Your order [$orderid]

      We have received your request for Ocean Color data.

      There is still ONE MORE thing needed to be done in order to receive
      your order.  The id of your requested order is $orderid.  You will
      now need to confirm this order.  Please use one of the following
      methods to confirm your order.

      CONFIRMATION VIA EMAIL
      You can simply respond to this email without modifying the email,
      making sure that you are replying from the same email address which
      you indicated in the order form;  then your order will be automatically
      confirmed and a subsequent email will be sent telling you where your
      files are staged.

      CONFIRMATION BY WEB
      If you are an Ocean Color Support Services registered user
      you may conveniently confirm and monitor all your orders
      using the following web page:
      https://oceandata.sci.gsfc.nasa.gov/odps/orders/ .
      If you are not a registered user, you may become one by going here:
      https://urs.earthdata.nasa.gov/users/new .

      If you experience any problems with your order, you may report
      them to data\@seawifs.gsfc.nasa.gov.

      Thank you for your request.
      Share and enjoy.

      John, and Tommy's Better Bytes by the Billions
      --

         $request =~ s/^ +//mg;

         open P,"|$sendmail" or die qq@Could not open pipe to command, @,
                                 qq@"$sendmail". ($!)@;
         print P $request;

         close P or die qq@Error executing command, "$sendmail". ($!)@;

         # Acknowledge the order with a web page.

         head("DATA Order Submitted");

      print <<"      --";
      $menubar
      </center>
      <p>
      There is still <strong>ONE MORE</strong> thing needed to
      be done in order to receive your order.  The id of your
      requested order is $orderid.  You will now need to confirm
      this order.  Please use one of the following methods to
      confirm your order.
      </p>

      <dl>
      <dt>CONFIRMATION BY EMAIL</dt>
      <dd>
      You will shortly be receiving an automated email from us
      regarding this order.  If you simply respond to this email
      without modifying the email, making sure that you are
      replying from the same email address which you indicated
      in the order form then your order will be automatically
      confirmed and a subsequent email will be sent telling you
      where your files are staged.
      </dd>

      <dt>CONFIRMATION BY WEB</dt>
      <dd>
      If you are an Ocean Color Support Services registered user
      you may conveniently confirm and monitor all your orders
      using the following web page:
      <a href="https://oceandata.sci.gsfc.nasa.gov/odps/orders/">https://oceandata.sci.gsfc.nasa.gov/odps/orders/</a>
      If you are not a registered user, you may become one by going here:
      <a href="https://urs.earthdata.nasa.gov/users/new">https://urs.earthdata.nasa.gov/users/new</a>
      </dd>
      </dl>

      <p>
      If you experience any problems with your order, you may report them to
      <a href="mailto:data\@seawifs.gsfc.nasa.gov">data\@seawifs.gsfc.nasa.gov</a>.
      </p>
      <p>
      <i>
      Please do not reload this page, since that will cause a duplicate
      order to be submitted.
      </i>
      </p>
      <center>
      --
    }
    else{
      # Display details of the user's order and ask for confirmation.

      head("Order Review");

      print
      $menubar,
      qq@</center>\n@;

      if(@allowed){

        my $do_extracts = (defined $in{xtr} and $in{xtr});

        print
        qq@<p>\n@,
        qq#You are about to order the following @{[scalar @allowed]} files\n#,
        qq@from the Ocean Color Data Processing System.\n@,
        qq@</p>\n@;
	if ($do_extracts == 0 && $prmcount == 0){
	    print
	    qq@<p>\n@,
	    qq@Since you did not request extraction or parameter subsetting,\n@,
            qq@or since those services were denied for this order (because\n@,
            qq@it contains Level-0 data, SEAHAWK or HICO data), when you click\n@,
            qq@'Submit order' you will recieve a listing of file URLs which\n@,
            qq@you may download immediately.\n@,
        qq@</p>\n@,
	    qq@<p>\n@,
	    qq@You may wish to save the next page as a text file and use it as\n@,
	    qq@input to a <a href="https://oceancolor.gsfc.nasa.gov/forum/oceancolor/topic_show.pl?tid=3081">web downloading program</a> (e.g. wget, cURL)\n@,
	    qq@</p>\n@;
        }

        print qq@<p>\n@;
        # List the files that the user is allowed to order.
        print tabulate(\@allowed,5,'col',-1);

        $avol = add_commas($avol);

        my $be = $do_extracts ? 'before extraction ' : '';

        print
        qq@</p>\n@,
        qq@<p>\n@,
        qq@The total volume of the above files $be<i>(in the compressed\n@,
        qq@form in which they are stored in our archive)</i> is\n@,
        qq@<b>$avol</b> bytes.\n@,
        qq@</p>\n@;

        if($do_extracts){
          # Constrain coordinates to legal values.
          $in{xn} += 0;
          $in{xs} += 0;
          $in{xw} += 0;
          $in{xe} += 0;
          $in{xn} =  90 if $in{xn} >  90;
          $in{xn} = -90 if $in{xn} < -90;
          $in{xs} =  90 if $in{xs} >  90;
          $in{xs} = -90 if $in{xs} < -90;
          if($in{xn} < $in{xs}){
            my $tmp = $in{xn};
            $in{xn} = $in{xs};
            $in{xs} = $tmp;
          }

         if($in{xn} eq $in{xs})
         {
            $in{xn} = $in{xn} + 1;
         }

         if($in{xe} eq $in{xw})
         {
            $in{xe} = $in{xe} + 1;
         }

          $in{xw} -= 360 while $in{xw} >  180;
          $in{xw} += 360 while $in{xw} < -180;
          $in{xe} -= 360 while $in{xe} >  180;
          $in{xe} += 360 while $in{xe} < -180;

          print
          qq@<p>\n@,
          qq@Files in your order will be trimmed using\n@,
          qq@the following coordinates.\n@,
          qq@<blockquote>\n@,
          qq@<tt>\n@,
          qq@Northernmost latitude: $in{xn}<br />\n@,
          qq@Southernmost latitude: $in{xs}<br />\n@,
          qq@Westernmost longitude: $in{xw}<br />\n@,
          qq@Easternmost longitude: $in{xe}\n@,
          qq@</tt>\n@,
          qq@</blockquote>\n@,
          qq@</p>\n@,
          qq@<img src="$thisScript?sub=xtrctmap&@,
          state_info(qw/url sub/),
          qq@" width="$xtrctmap_w" height="$xtrctmap_h">@;
        }
        else{
          undef $in{cn};
          undef $in{cs};
          undef $in{cw};
          undef $in{ce};
          undef $in{xn};
          undef $in{xs};
          undef $in{xw};
          undef $in{xe};
        }

        print qq@<hr />\n@;

      }
      else{
        undef $in{cn};
        undef $in{cs};
        undef $in{cw};
        undef $in{ce};
        undef $in{xn};
        undef $in{xs};
        undef $in{xw};
        undef $in{xe};
      }

      if($prmcount > 0){
        print
        qq@<p>\n@,
        qq@The ocean color level-2 files in your order will contain only\n@,
        qq@the following products.\n@,
        qq@<ul>\n@;

        foreach my $n (@l2prm){
          print qq@<li><b>$l2prm{$n}</b></li>\n@ if(defined $ord{$n});
        }

        print
        qq@</ul>\n@,
        qq@</p>\n@;
      }

      if(@denied){
        # List the files (if any) that were excluded from the order
        # because of order volume restrictions.

        print
        qq@<font size="-1">\n@,
        qq@<p>\n@,
        qq#The following @{[scalar @denied]} files will not be included\n#,
        qq@in your order because they would cause your order to exceed\n@,
        qq@our current order size limit of $readablethresh bytes.\n@,
        qq@</p>\n@,
        qq@<p>\n@,
        qq@You can still download these files individually using the\n@,
        qq@hyperlinks above the browse images if you wish.\n@,
        qq@</p>\n@,
        qq@</font>\n@,
        qq@<p>\n@;

        print tabulate(\@denied,5,'col',-2);

        $dvol = add_commas($dvol);
        print
        qq@</p>\n@,
        qq@<p>\n@,
        qq@<font size="-1">\n@,
        qq@Total volume of files not included because of volume\n@,
        qq@restriction = $dvol bytes\n@,
        qq@</font>\n@,
        qq@</p>\n@,
        qq@<hr />\n@;
      }

      if(@restricted){
        # List the files (if any) that were excluded from the order
        # because they are only available to authorized users.

        print
        qq@<font size="-1">\n@,
        qq@<p>\n@,
        qq#The following @{[scalar @restricted]} files will not be included\n#,
        qq@in your order because you have not $logged_in as an authorized\n@,
        qq@SeaWiFS user.  You may\n@,
        qq@<a href="/cgi/apply.pl?page=du">apply here</a>\n@,
        qq@for the necessary authorization, or, if you are already\n@,
        qq@authorized, you may\n@,
        qq@<a href="/cgi/new_passwd.pl">request a new password</a>\n@,
        qq@for this website.\n@,
        qq@</p>\n@,
        qq@</font>\n@,
        qq@<p>\n@;

        print tabulate(\@restricted,5,'col',-2);

        $rvol = add_commas($rvol);
        print
        qq@</p>\n@,
        qq@<p>\n@,
        qq@<font size="-1">\n@,
        qq@Total volume of SeaWiFS files not included because you are\n@,
        qq@not logged in as an authorized user = $rvol bytes\n@,
        qq@</font>\n@,
        qq@</p>\n@,
        qq@<hr />\n@;
      }

      if(@allowed){
         my $state = state_info(qw/form/);
         my $do_extracts = (defined $in{xtr} and $in{xtr});
	      if ($do_extracts != 0 || $prmcount != 0){
            my $remind  = grep(/^16$/,@ord) ? 'do' : 'do not';
            my $require = grep(/^32$/,@ord) ? 'do' : 'do not';
            my $notify  = grep(/^256$/,@ord) ? 'do' : 'do not';

            print
            qq@<p>\n@,
            qq@You <strong>$remind</strong> wish to be reminded by email when\n@,
            qq@your order is about to expire, and<br />\n@,
            qq@you <strong>$require</strong>\n@,
            qq@require email confirmation when you use the Web to request early\n@,
            qq@deletion of your staged order, and<br />\n@,
            qq@you <strong>$notify</strong> wish to be notified when your\n@,
            qq@order has been deleted from our staging area.\n@,
            qq@</p>\n@,
            qq@<p>\n@,
            qq@The email address associated with your account is\n@,
            qq@<font size="+2"><b>$session->{profile}{email_address}</b></font> .\n@,
            qq@</p>\n@,
            qq@<p>\n@,
            qq@If all this information is correct and you wish to submit\n@,
            qq@your order to be filled, then click the &quot;<tt>Submit\n@,
            qq@order</tt>&quot; button below.\n@,
            qq@</p>\n@;
         }
	      print
         qq@<form method="post" action="$thisScript">\n@,
         $state,
         qq@<input type="hidden" name="place_order" value="1">\n@,
         qq@<input type="submit" value="Submit order">\n@,
         qq@</form>\n@;
      }
      else{

            print
            qq@<p>\n@,
            qq@We're sorry, but none of the scenes in your search results\n@,
            qq@are currently available for ordering from our archive.\n@,
            qq@</p>\n@;
         }
      }
   }

   &tail;
}

sub orderfilelist{

  my @list = @_;

 # Generate an order ID for filename
 my $orderid = &orderid;
 my $save_filename = "obpg_order_results_" . $orderid .".txt";

  print "Content-type: text/plain\nContent-disposition: inline; filename=$save_filename\n\n";
  my $pfx = "ob";

    foreach my $l (sort {$a->[1] cmp $b->[1]} @list){
      my %lf = (pattern=>"$l");
      my %urlsiz = locate_file(\%lf);
      my $url = (grep /$l(?:\.(bz2|Z|gz|zip))?/, keys %urlsiz)[0];

      if ($l =~ /^S3[AB]_/){
		  $url = $l;
         $pfx = 'sentinel';
      } elsif ($url =~ /^M/) {
         $pfx = 'echo';
      } elsif ($url =~ /^H/) {
         $pfx = 'hico';
      } elsif ($url =~ /^COMS_GOCI/) {
         $pfx = 'goci';
      }
      print "https://oceandata.sci.gsfc.nasa.gov/$pfx/getfile/$url\n";
  }

  exit;
}

sub filenamelist{

  my @list = read_list($in{id});
  # If any of the scenes in the search results have been
  # specially selected, then reduce the list of scenes
  # so that only the specially selected ones get considered.
  my @reduced = grep defined $_->[6], @list;
  @list = @reduced if @reduced > 0;

  print "Content-type: text/plain\n\n";

  my $typ = $in{prm};
  $typ =~ s/^SST4/L2SST4/;
  $typ =~ s/^SST/L2SST/;
  $typ =~ s/^CHL$/L2OC/;
  $typ =~ s/^TC$/L1/;

  foreach my $l (sort {$a->[1] cmp $b->[1]} @list){
    my $f = msntimetyp2name(gran2msntime($l),$typ);
    $f or next;
    print "$f\n";
  }

  exit;
}

sub l3image{
  my $png = qlstack(qlpaths($in{sen},$in{prm},$in{per},$in{day},$in{dnm}));

  # Send the image to the client.
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub l12image{
  my $path = msntimetyp2name(fname2msntime($in{file}),fname2type($in{file}));

  my $rot = '';
  my $pin = 1;
  # Try to present images in a north-is-up orientation.
  if($in{file} =~ /^C\d/){		# CZCS
    $rot = "$pnmflip -tb|";
    $pin = 2;
  }
  else{
    my ($start_lat,$end_lat)
    = `$ncdump $path` =~
       / Start(?:\s|_)Center(?:\s|_)Latitude\s*=\s*([-0-9.]+).*
         End(?:\s|_)Center(?:\s|_)Latitude  \s*=\s*([-0-9.]+)  /sx;
    if($start_lat < $end_lat){
      $rot = "$pnmflip -r180|";
      $pin = 3;
    }
  }

  my $hilight = '';
  my $overlay = '';
  if($in{hlt}){
    # Highlight area of interest on thumbnail.
    $overlay = aoi_overlay($in{id},$path);
    $hilight = "|$convert - $overlay -compose SrcOver -composite PPM:-";
  }

  my $insitu = '';
  if($in{is}){
    # Put a cursor on the scene for each matching in situ record.
    my $cdir = '/data1/www-data/oceancolor.gsfc.nasa.gov/static/icons';
    my $id = $in{all} ? undef : $in{id};
    my @insitu = brs2insitu($path,$id);

    # Only keep one in situ record for each dataid since all records
    # having the same dataid should also have the same coordinates.
    my %dataid;
    @insitu = grep $dataid{$_->{dataid}}++ == 0, @insitu;

    # The orientation of the four push pin icons is as follows
    # (before rotation/transformation).
    #   pin0.png - pin head leans toward upper left
    #   pin1.png - pin head leans toward upper right
    #   pin2.png - pin head leans toward lower right
    #   pin3.png - pin head leans toward lower left

    foreach (@insitu){
      # I subtract 25 because the cursor is 50 pixels wide and high, and
      # I want the center of the cursor to overlay the point of interest.
      my $geom = sprintf "%+d%+d", $_->{x} - 25, $_->{y} - 25;

      # Make sure we choose a pin that is visible
      # for stations near the edge of a scene.
      my %p = qw/0 1 1 1 2 1 3 1/;

      if ($_->{x} <              25
      and $_->{x} >= $_->{wid} - 25){
        if($_->{x} > $_->{wid}/2){ delete $p{'1'}; delete $p{'2'}; }
        else                     { delete $p{'0'}; delete $p{'3'}; }
      }
      else{
        if($_->{x} <              25){ delete $p{'0'}; delete $p{'3'}; }
        if($_->{x} >= $_->{wid} - 25){ delete $p{'1'}; delete $p{'2'}; }
      }
      if ($_->{y} <              25
      and    $_->{y} >= $_->{hgt} - 25){
        if($_->{y} > $_->{hgt}/2){ delete $p{'2'}; delete $p{'3'}; }
        else                     { delete $p{'0'}; delete $p{'1'}; }
      }
      else{
        if($_->{y} <              25){ delete $p{'0'}; delete $p{'1'}; }
        if($_->{y} >= $_->{hgt} - 25){ delete $p{'2'}; delete $p{'3'}; }
      }

      my $p = $p{$pin} ? $pin : (sort keys %p)[0];

      $insitu .= "|$composite -geometry $geom $cdir/pin$p.png - -";
    }
  }

  my $c = "$hdfrastopnm $path$hilight$insitu|$rot$pnmtopng";
  open P,"$c|" or die qq@Error opening pipe from command, "$c". ($!) Died@;
  local($/) = undef;
  my $png = <P>;
  close P or die qq@Command, "$c", failed. ($!) Died@;
  if($in{hlt}){
    unlink $overlay;
  }
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub l12thumb{
  my $path = msntimetyp2name(fname2msntime($in{file}),fname2type($in{file}));

  my $png;
  if(-s $path){

    my $rot = '';
    # Try to present images in a north-is-up orientation.
    if($in{file} =~ /^C\d/){		# CZCS
      $rot = "$pnmflip -tb|";
    }
    else{
      my ($start_lat,$end_lat)
      = `$ncdump $path` =~
         / Start(?:\s|_)Center(?:\s|_)Latitude\s*=\s*([-0-9.]+).*
           End(?:\s|_)Center(?:\s|_)Latitude  \s*=\s*([-0-9.]+)  /sx;
      $rot = "$pnmflip -r180|" if $start_lat < $end_lat;
    }

    my $hilight = '';
    my $overlay = '';
    if($in{hlt}){
      # Highlight area of interest on thumbnail.
      $overlay = aoi_overlay($in{id},$path);
      $hilight = "|$convert - $overlay -compose SrcOver -composite PPM:-";
    }

    my $c = "$hdfrastopnm $path$hilight|" .
            "$pnmscale -width $in{wid} -height $in{hgt}|" .
            "$rot$pnmtopng";
    open P,"$c|" or die qq@Error opening pipe from command, "$c". ($!) Died@;
    local($/) = undef;
    $png = <P>;
    close P or die qq@Command, "$c", failed. ($!) Died@;
    if($in{hlt}){
      unlink $overlay;
    }
  }
  else{
    my $h = int(1.5 * $in{wid});
    my $pad = '';
    if($h > $in{hgt}){
      $h = $in{hgt};
    }
    elsif($h < $in{hgt}){
      my $b = $in{hgt} - $h;
      $pad = "|$pnmpad -white -b $b";
    }
    my $c = "$pnmscale -width $in{wid} -height $h $not_avail_img$pad|$pnmtopng";
    open P,"$c|" or die qq@Error opening pipe from command, "$c". ($!) Died@;
    local($/) = undef;
    $png = <P>;
    close P or die qq@Command, "$c", failed. ($!) Died@;
  }
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

# Make an area-of-interest overlay and alpha mask that can be used
# to highlight the user's search area on the given browse file.
sub aoi_overlay{
  my ($id,$path) = @_;

  my $f = (split /\//,$path)[-1];

  my $hilight_color = 'red';
  $hilight_color = 'magenta' if($f =~ /CHLOR/);
  $hilight_color = 'white'   if($f =~ /SST/  );

  # Read the previously written level-6 bin numbers and convert
  # them to level 7.
  my $binsfile = "$tmpdir/browse_qbins.$in{id}";
  open(F,'<',$binsfile) or die "Could not open $binsfile ($!)";
  my @bins = map {
    chop;
    my $bin = 4*$_;   # Convert from level-6 to level-7.
    ($bin,$bin+1,$bin+2,$bin+3);
  } <F>;
  close F;

  # Convert the bin numbers into a PGM mask image.
  my @cmd = ($rnav,$path);
  local($") = "\n";
  my $input = "@bins\n";
  my ($output,$err);
  run3 \@cmd,\$input,\$output,\$err;
  my ($wid,$hgt,@indices) = split ' ',$output;
  my $mask = chr(0) x ($wid*$hgt);
  my $white = chr(255);
  foreach (@indices){
    substr($mask,$_,1) = $white;
  }
  my $maskfile = "$tmpdir/$f.$invocation_time.$$.pre.pgm";
  open(F,'>',$maskfile) or die "Could not create $maskfile ($!)";
  print F "P5\n$wid $hgt\n255\n$mask";
  close F;

  # Make a 50% transparent overlay out of the mask file using the
  # selected highlight color.  The "Erode" method below causes the
  # overlay to have an opaque outline.
  my $overlay = "$tmpdir/$f.$invocation_time.$$.overlay.png";
  my $c = join ' ', split ' ', <<"  --";
  $convert $maskfile
   \\( +clone -morphology Erode Square -fill gray -opaque white
      -transparent black -clone 0 +swap -composite \\)
   \\( -clone 0 -fill $hilight_color -opaque white -clone 1
      -compose CopyOpacity -composite \\)
   -delete 0,1 $overlay
  --
  system($c) == 0 or die "$c failed ($!)";
  unlink $maskfile;
  $overlay;
}

sub locimage{
  my $path = msntimetyp2name(fname2msntime($in{file}),fname2type($in{file}));
#open FH,">> /tmp/getfile_log" or die $!;
#print FH "$path\n";
#close FH;
  my $yyyyddd = yyyyddd($in{day});
  my $meth = 1;
  my $ign = '';
  if($in{file} =~ /L2/){
    $meth = 2;
  }

  # SeaWiFS crosses the equator at noon which
  # is the default for the compositing program.
  my $eqx = '';

  # Terra and MERIS and SEAHAWK cross the equator at 10:30.
  $eqx = ' -t 10.5' if $in{file} =~ /^(?:[MT]|SEAHAWK)/;

  # Aqua and VIIRS cross the equator at 13:30.
  $eqx = ' -t 13.5' if $in{file} =~ /^[AV]/;

  # Pass along a day or night specification if needed.
  my $dorn = '';
  if($in{dnm} =~ /D/ and $in{dnm} !~ /N/){
    $dorn = ' -n 0';
  }
  elsif($in{dnm} =~ /N/ and $in{dnm} !~ /D/){
    $dorn = ' -n 1';
  }

  my $dayovl = make_day_overlay($in{file},$in{dnm});

  my $c = join ' ', split ' ', <<"  --";
  $composite_browse$eqx$dorn -m $meth -d $yyyyddd -e 255,0,0 $path |
  $convert - $dayovl -compose blend -define compose:args=35 -composite
  -resize 50% PNG:-
  --
  open P,"$c|" or die qq@Error opening pipe from command, "$c". ($!) Died@;
  local($/) = undef;
  my $png = <P>;
  close P or die qq@Error executing command, "$c". ($!) Died@;
  unlink $dayovl;
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub mapimg{
  my $path = msntimetyp2name(fname2msntime($in{file}),fname2type($in{file}));
  $in{file} =~ /^(SEAHAWK)/;
  my $sen = $1 || substr $in{file},0,1;
  my %hor = (
     S => 120,
     A => 40,
     T => 40,
     O => 80,
     C => 40,
     M => 80,
     V => 40,
     H =>  3,
     G => 40,
     SEAHAWK => 8,
  );
  my $png = `$mapbrsscene $path $hor{$sen} 256`;
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub make_day_overlay{
  my ($f,$dnm) = @_;

  my $pi = 4*atan2(1,1);

  my $tmp = "$tmpdir/day_overlay.$invocation_time.$$.ppm";

  my $d = 'd';
  my $cen = 0;
  if($dnm =~ /N/ and $dnm !~ /D/){
    $d = 'g';
    $cen = 180;
  }

  # Get the solar position vector at the start time of the scene.
  my ($year,$dy,$mo,$doy,$hh,$mm,$ss);
  if ($f =~ /^MER/){
    my $tstr = substr $f,14,15;

    ( $year,  $mo,  $dy,   $hh,  $mm,  $ss) = $tstr =~
    /(\d{4})(\d\d)(\d\d).(\d\d)(\d\d)(\d\d)/;

    $tstr = substr ymdhms2yd(join('',$year,$mo,$dy,$hh,$mm,$ss)),4,3;
    ($doy) = $tstr =~/(\d{3})/;

  } elsif ($f =~ /^SEAHAWK/){
    my $tstr = substr $f,17,15;

    ( $year,  $mo,  $dy,   $hh,  $mm,  $ss) = $tstr =~
    /(\d{4})(\d\d)(\d\d).(\d\d)(\d\d)(\d\d)/;

    $tstr = substr ymdhms2yd(join('',$year,$mo,$dy,$hh,$mm,$ss)),4,3;
    ($doy) = $tstr =~/(\d{3})/;

  } elsif ($f =~ /^S3(A|B)/) {
    my $tstr = substr $f,3,17;
    ( $year,  $mo,  $dy,   $hh,  $mm,  $ss) = $tstr =~
    /(\d{4})(\d\d)(\d\d).(\d\d)(\d\d)(\d\d)/;

    $tstr = substr ymdhms2yd(join('',$year,$mo,$dy,$hh,$mm,$ss)),4,3;
    ($doy) = $tstr =~/(\d{3})/;

  } else {
    my $tstr = substr $f,1,13;

    ( $year,  $doy,  $hh,  $mm,  $ss) = $tstr =~
    /(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)/;
  }
  my $sec = 3600*$hh + 60*$mm + $ss;
  my @sun = map launder($_), (split ' ',`$sun $year $doy $sec`)[0..2];

  # Compute the lat/lon of the sub-solar point.
  my $sslat = atan2($sun[2], sqrt($sun[0]*$sun[0] + $sun[1]*$sun[1]))*180/$pi;
  my $sslon = atan2($sun[1], $sun[0])*180/$pi;

  # Choose a latitude that is 90 degrees from the sub-solar latitude.
  my $sslat90 = $sslat > 0 ? $sslat - 90 : $sslat + 90;

  # Get coordinates of spherical triangles with their apex at the sub-solar
  # point and their 250-kilometer-long bases along the terminator.
  my $c = "$gmt project -T$sslon/$sslat -C$sslon/$sslat90 -G250 -Q -L0/40000";
  open P,"$c|" or die "Error opening pipe from $c ($!)";
  my @coords = map {
                 my ($x,$y)=split ' ';
                 "$x $y\n","$sslon $sslat\n>\n","$x $y\n";
               } <P>;
  unshift @coords,pop @coords;

  # Draw the trangles in yellow on a Plate Carree map
  # to highlight the daylit portion of the Earth.
  $c = join ' ', split ' ', <<"  --";
  $gmt psxy -P -X0 -Y0 -R$d -JQ$cen/5.12 -W1,255/255/0
  --PS_PAGE_COLOR=0/0/0 --PROJ_LENGTH_UNIT=i
  |$gs -g512x256 -dGraphicsAlphaBits=4 ->$tmp
  --
  open P,"|$c" or die "Error opening pipe to $c ($!)";
  print P @coords;
  close P or die "Error executing $c ($!)";

  $tmp;
}

sub aoimap{
  my $f = "$tmpdir/browse_aoi.$in{id}.png";
  open F,'<',$f or die qq@Could not open file, "$f" ($!)@;
  local($/) = undef;
  my $png = <F>;
  close F;
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub xtrctmap{
  my ($n,$s,$w,$e);

  if(
  defined $in{xn} and $in{xn} ne '' and
  defined $in{xs} and $in{xs} ne '' and
  defined $in{xw} and $in{xw} ne '' and
  defined $in{xe} and $in{xe} ne ''
  ){
    ($n,$s,$w,$e) = ($in{xn},$in{xs},$in{xw},$in{xe});
  }
  else{
    ($n,$s,$w,$e) = ($in{cn},$in{cs},$in{cw},$in{ce});
  }

  $e += 360 if $e < $w;
  my $dlon = $e - $w;
  my $dlat = $n - $s;

  my $xbord = 80;	# padding on left and right for latitude  annotations
  my $ybord = 50;	# padding on top and bottom for longitude annotations

  my $wid = $xtrctmap_w - 2*$xbord;
  my $hgt = $xtrctmap_h - 2*$ybord;

  my $ppd;		# pixels per degree
  if($dlat > $dlon){
    $ppd = $hgt/$dlat;
    $wid = int($ppd*$dlon + 0.5);
    $wid = 1 if $wid == 0;
    $xbord = int(($xtrctmap_w - $wid)/2 + 0.5);
  }
  else{
    $ppd = $wid/$dlon;
    $hgt = int($ppd*$dlat + 0.5);
    $hgt = 1 if $hgt == 0;
    $ybord = int(($xtrctmap_h - $hgt)/2 + 0.5);
  }

  my $grd = sprintf "%e",60/$ppd;
  my ($mantissa,$exponent) = split /e/,$grd;
  $mantissa = int($mantissa + 0.5);
  $grd = join 'e',$mantissa,$exponent;

  my $res;
  if   ($ppd > 500){
    $res = 'f';		# full resolution
  }
  elsif($ppd > 100){
    $res = 'h';		# high resolution
  }
  elsif($ppd > 20){
    $res = 'i';		# intermediate resolution
  }
  elsif($ppd > 5){
    $res = 'l';		# low resolution
  }
  else{
    $res = 'c';		# crude resolution
  }

  # I need to CD here so GMT is able to write out its .gmtcommands file.
  chdir $tmpdir or die qq@Could not chdir to $tmpdir ($!)@;

  $wid   /= 100;
  $hgt   /= 100;
  $xbord /= 100;
  $ybord /= 100;

  my $c =
  "$gmt set FORMAT_GEO_OUT F " .
  "MAP_FRAME_TYPE plain MAP_ANOT_OFFSET 0.075i FONT_ANNOT Helvetica-Bold PROJ_LENGTH_UNIT in;" .
  "$gmt pscoast -P -X$xbord -Y$ybord -R$w/$e/$s/$n -JX${wid}d/${hgt}d " .
  "-D$res -G153/51/0 -S0/153/255 -B${grd}g$grd --PROJ_LENGTH_UNIT=i" .
  "|$gs -g${xtrctmap_w}x$xtrctmap_h -" .
  "|$pnmtopng";
  open P,"$c|" or die qq@Error opening pipe from command, "$c" ($!)@;
  local($/) = undef;
  my $png = <P>;
  close P or die qq@Error executing command, "$c" ($!)@;
  my $len = length $png;
  print "Content-type: image/png\nContent-length: $len\n\n$png";
}

sub swathdays{
  my $file = shift;

  my ($msn,$time) = fname2msntime($file);

  my $swathdayscmd = join ' ',split ' ',<<"  --";
  SELECT distinct g.data_day, g.daynight
  from webstuff.granules g, webstuff.scene_info s
  where g.scene_id = s.scene_id
  and s.hide = 0
  and s.sen_id = $sen{$msn}[3]
  and s.start_time between @{[$time-120]} and @{[$time+120]};
  --
  my $swathday;
  my $daynight;
  my @days = ();
  my %days = ();
  my $sth = $dbh->prepare("$swathdayscmd");
  $sth->execute;
  $sth->bind_columns(\$swathday,\$daynight);
  while ($sth->fetch){
    push @{$days{$swathday}},$daynight;
  }
  $sth->finish;
  @days = sort {$a <=> $b} keys %days;
  foreach my $d (@days){
    $daynight = 2 if ($#{$days{$d}} > 0);
  }
  # Translate the day/night information from the database.
  my @dnm = qw/D N D@N/;

  ($dnm[$daynight], @days);
}

sub date_ranges{

  my $ranges_string;

  $ranges_string  =  rangestostring(perdaytoranges($in{per},$in{day}));
  $ranges_string and $ranges_string .= ' ';
  $ranges_string .=  &process_periodics;
  $ranges_string and $ranges_string .= ' ';
  $ranges_string .=  &process_ranges;

  my %ranges = consolidate_ranges(split " ",$ranges_string);

  ($in{per},$in{day}) = rangestoperday(%ranges);

  if($in{cdr} eq 'Finished'){
    $in{sub} = 'level3';
    if($in{per} eq 'CSTM'){
      if($in{day} eq ''){
        $in{per} = 'CU';
        $in{day} = $mission_end;
      }
      else{
        foreach my $s (sort {$a <=> $b} keys %ranges){
          $s < $mission_start and next;
          $in{mon} = (period_bounds('MO',$s))[1] + 1;
          last;
        }
      }
    }
    &{$sub{$in{sub}}};
    exit;
  }

  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;

  my $start_year = (gmtime($mission_start*86400))[5] + 1900;
  my   $end_year = (gmtime($mission_end  *86400))[5] + 1900;

  ######################################################################
  my $ranges_table =
  qq@<form action="$thisScript" method="post">\n@ .
  state_info(qw/form per day/) .
  qq@<table>\n@;

  my $d0 =   day_selector("rd.0.0",'');
  my $m0 = month_selector("rm.0.0",'');
  my $y0 =  year_selector("ry.0.0",$start_year,$end_year,'');
  my $d1 =   day_selector("rd.0.1",'');
  my $m1 = month_selector("rm.0.1",'');
  my $y1 =  year_selector("ry.0.1",$start_year,$end_year,'');

  my $through = qq@<font size="-1">through</font>@;

  my $i = 1;
  foreach my $start (sort {$a <=> $b} keys %ranges){
    my $end = $start + $ranges{$start} - 1;
    my ($sd,$sm,$sy) = (gmtime($start*86400))[3..5];
    my ($ed,$em,$ey) = (gmtime($end  *86400))[3..5];
    $sy += 1900;
    $ey += 1900;
    my $d0 =   day_selector("rd.$i.0",$sd);
    my $m0 = month_selector("rm.$i.0",$mon[$sm]);
    my $y0 =  year_selector("ry.$i.0",$start_year,$end_year,$sy);
    my $d1 =   day_selector("rd.$i.1",$ed);
    my $m1 = month_selector("rm.$i.1",$mon[$em]);
    my $y1 =  year_selector("ry.$i.1",$start_year,$end_year,$ey);
    my $rs = qq@<input type="checkbox" name="rr.$i" checked>\n@;

    $ranges_table .=
    qq@<tr><td>$d0$m0$y0 $through $d1$m1$y1$rs</td></tr>\n@;

    $i++;
  }

  $ranges_table .=
  qq@<tr><td>$d0$m0$y0 $through $d1$m1$y1</td></tr>\n@ .
  qq@</table>\n@ .
  qq@<p>\n@ .
  qq@<input type="submit" name="cdr" value="Continue">\n@ .
  qq@</p>\n@ .
  qq@<p>\n@ .
  qq@<input type="submit" name="cdr" value="Finished">\n@ .
  qq@</p>\n@ .
  qq@</form>\n@ .
  qq@<table><tr><td>\n@ .
  qq@<p>\n@ .
  qq@To choose a date range other than the ones offered on the main\n@ .
  qq@browser page:\n@ .
  qq@<ul>\n@ .
  qq@<li>\n@ .
  qq@modify or add date ranges using the menus provided above,\n@ .
  qq@</li>\n@ .
  qq@<li>\n@ .
  qq@uncheck the checkboxes next to any date ranges that you do not\n@ .
  qq@wish to use,\n@ .
  qq@</li>\n@ .
  qq@<li>\n@ .
  qq@click the <tt><b>Continue</b></tt> button to consolidate the ranges\n@ .
  qq@specified so far or to get a new blank menu with which you can add\n@ .
  qq@yet another date range,\n@ .
  qq@</li>\n@ .
  qq@<li>\n@ .
  qq@use the tool on the right to add (remove) periodic\n@ .
  qq@date ranges to (from) the list of ranges above,\n@ .
  qq@<img src="/icons/right_arrow_plain.gif" width="34" height="17">\n@ .
  qq@</li>\n@ .
  qq@<li>\n@ .
  qq@click the <tt><b>Finished</b></tt> button when you are ready\n@ .
  qq@to return to the main browser page. Your newly selected date\n@ .
  qq@ranges will now restrict your search for scenes.\n@ .
  qq@</li>\n@ .
  qq@</ul>\n@ .
  qq@</p>\n@ .
#  qq@<p>\n@ .
#  qq@This page lets you specify one or more custom date ranges\n@ .
#  qq@over which you would like to search for data.  Date ranges\n@ .
#  qq@are presented as <b><tt>day month year</tt></b> through\n@ .
#  qq@<b><tt>day month year</tt></b>, and all of the components\n@ .
#  qq@of the dates are pull down menus that you can use to change\n@ .
#  qq@the dates.\n@ .
#  qq@</p>\n@ .

#  qq@<p>\n@ .
#  qq@Additional date ranges can be entered by setting the six\n@ .
#  qq@blank fields below the other date ranges and clicking the\n@ .
#  qq@<b><tt>Continue</tt></b> button.  If you wish to remove\n@ .
#  qq@one of the date ranges that have already been specified,\n@ .
#  qq@then just uncheck the checkbox next to the date range in\n@ .
#  qq@question and click the <b><tt>Continue</tt></b> button.\n@ .
#  qq@</p>\n@ .

#  qq@<p>\n@ .
#  qq@You can also use the right-hand portion of this page to\n@ .
#  qq@quickly add (or remove) periodic date ranges. For instance,\n@ .
#  qq@to search for data collected in March and September of\n@ .
#  qq@every year between 2002 and 2004, first remove any date\n@ .
#  qq@ranges already listed above and then set the start year\n@ .
#  qq@to 2002, set the stop year to 2004, check <b><tt>Mar</tt></b>\n@ .
#  qq@and <b><tt>Sep</tt></b> in the <b><tt>Months</tt></b>\n@ .
#  qq@column, and click the <b><tt>Add</tt></b> button. For\n@ .
#  qq@this example, six date ranges would be displayed above.\n@ .
#  qq@</p>\n@ .

#  qq@<p>\n@ .
#  qq@Once you are satisfied with the date ranges you have\n@ .
#  qq@selected click on the <b><tt>Finished</tt></b> button\n@ .
#  qq@to return to the main browser page.\n@ .
#  qq@</p>\n@ .

  qq@</td></tr></table>\n@;
  ######################################################################

  ######################################################################
  my $py0 = year_selector("py0",$start_year,$end_year,$start_year);
  my $py1 = year_selector("py1",$start_year,$end_year,$end_year);

  my $pd0 =   day_selector("pd0",'');
  my $pd1 =   day_selector("pd1",'');
  my $pm0 = month_selector("pm0",'');
  my $pm1 = month_selector("pm1",'');

  my $periodic_ranges =
  qq@<h2>Add or remove periodic intervals</h2>\n@ .
  qq@<form action="$thisScript" method="post">\n@ .
  state_info(qw/form/) .
  qq@<table border>\n@ .
  qq@<tr>\n@ .
  qq@<td colspan="2">\n@ .
  qq@<table width="100%">\n@ .
  qq@<tr>\n@ .
  qq@<th align="left">\n@ .
  qq@Start year<br />\n@ .
  $py0 .
  qq@</th>\n@ .
  qq@<th align="right">\n@ .
  qq@Stop year<br />\n@ .
  $py1 .
  qq@</th>\n@ .
  qq@</tr>\n@ .
  qq@</table>\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@<tr>\n@ .
  qq@<td rowspan="2">\n@ .
  qq@<table>\n@ .
  qq@<tr>\n@ .
  qq@<th>Months</th>\n@ .
  qq@</tr>\n@;

  for(my $i=0; $i<@mon; $i++){
    $periodic_ranges .=
    qq@<tr>\n@ .
    qq@<td><input type="checkbox" name="psmd" value="$mon[$i]">\n@ .
    qq@$mon[$i]</td>\n@ .
    qq@</tr>\n@;
  }

  $periodic_ranges .=
  qq@</table>\n@ .
  qq@</td>\n@ .
  qq@<td>\n@ .
  qq@<table>\n@ .
  qq@<tr>\n@ .
  qq@<th>Boreal<br />seasons</th>\n@ .
  qq@</tr>\n@;

  foreach (qw/Spring Summer Autumn Winter/){
    $periodic_ranges .=
    qq@<tr>\n@ .
    qq@<td><input type="checkbox" name="psmd" value="$_">\n@ .
    qq@$_</td>\n@ .
    qq@</tr>\n@;
  }

  $periodic_ranges .=
  qq@</table>\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@<tr>\n@ .
  qq@<td>\n@ .
  qq@<table>\n@ .
  qq@<tr>\n@ .
  qq@<th>Days of<br />the week *</th>\n@ .
  qq@</tr>\n@;

  foreach (qw/Sun Mon Tue Wed Thu Fri Sat/){
    $periodic_ranges .=
    qq@<tr>\n@ .
    qq@<td><input type="checkbox" name="psmd" value="$_">\n@ .
    qq@$_</td>\n@ .
    qq@</tr>\n@;
  }

  $periodic_ranges .=
  qq@</table>\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@<tr>\n@ .
  qq@<th align="center" colspan="2">\n@ .
  qq@Choose your<br />own period<br />\n@ .
  qq@$pd0$pm0<br />\n@ .
  qq@<font size="-1">through</font><br />\n@ .
  qq@$pd1$pm1\n@ .
  qq@</th>\n@ .
  qq@</tr>\n@ .
  qq@<tr>\n@ .
  qq@<td colspan="3">\n@ .
  qq@<table width="100%">\n@ .
  qq@<tr>\n@ .
  qq@<td align="left">\n@ .
  qq@<input type="submit" name="pra" value="Add">\n@ .
  qq@</td>\n@ .
  qq@<td align="right">\n@ .
  qq@<input type="submit" name="pra" value="Remove">\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@</table>\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@</table>\n@ .
  qq@</form>\n@ .
  qq@<table><tr valign="top">\n@ .
  qq@<td><b>*</b></td>\n@ .
  qq@<td>\n@ .
  qq@<font size="-1">\n@ .
  qq@Note that adding or removing days of the week can generate\n@ .
  qq@numerous distinct time periods which will take longer to be loaded\n@ .
  qq@as option menus on the left side of this page.  If you use this\n@ .
  qq@&quot;Days of the week&quot; feature, please be patient.\n@ .
  qq@</font>\n@ .
  qq@</td>\n@ .
  qq@</tr></table>\n@;
  ######################################################################

  head("Define your own date ranges");

  print
  qq@<table width="100%"><tr>\n@,
  qq@<th>Define your own custom date ranges for data searches</th>\n@,
  qq@<td align="right"><a href="$comment_url"\n@,
  qq@target="_blank">$button{comment}</a></td>\n@,
  qq@<td align="right">@,
  qq@<a href="javascript:helpwin('date')">$button{help}</a>@,
  qq@<noscript><a href="$thisScript?sub=help&hlp=date" @,
  qq@target="OCBrowseHelp">@,
  qq@Help</a></noscript>@,
  qq@</td>\n@,
  qq@</tr></table>\n@,
  qq@<table width="100%"><tr valign="top" align="center">\n@,
  qq@<td bgcolor="#99ccff" width="75%">\n@,
  $ranges_table,
  qq@</td><td bgcolor="#66ccff" width="25%">\n@,
  $periodic_ranges,
  qq@</td>\n</tr></table>\n@;

  &tail;
}

sub help{
  my %page = (
  top    => '/browse_help/top_level',
  thumbs => '/browse_help/search_results',
  scene  => '/browse_help/individual_scene',
  order  => '/browse_help/order',
  date   => '/browse_help/custom_date_ranges',
  );

  defined $in{hlp} and $in{hlp} ne '' or $in{hlp} = 'top';

  my ($page,$anchor) = split /#/,$in{hlp};

  unless(defined $page{$page}){
    $page = 'top';
    $anchor = '';
  }

  if(defined $anchor){
    substr($anchor,0,0) = '#';
  }
  else{
    $anchor = '';
  }

  print "Location: $page{$page}$anchor\n\n";
}

sub thumbtable{
  my @list = @_;
  my $n = @list;

  my $state  = state_info(qw/url sub ndx file wid hgt/);

  my $up = $button{up_on};
  $up =~ s/alt=""/alt="Up to Search Page"/;

  $up =
  qq@<a href="$thisScript?sub=level3&@ .
  state_info(qw/url sub ndx file wid hgt id/) .
  qq@">$up</a>@;

  my $number;
  my $culled_complete;

  unless(defined $in{cul}){
    my $setno = int($in{ndx}/$in{set});
    my $setstart = $setno * $in{set} + 1;
    my $setend = $setstart + $in{set} - 1;
    if($setend > @list){
      $setend = @list;
    }
    $setstart = ordinal($setstart);
    $setend = ordinal($setend);
    $number =
    "$setstart through $setend of <b>" . scalar(@list) . "</b> swaths";
  }

  if(defined $in{id} and $in{id} ne "$invocation_time.$$"){

    # The list of scenes was generated during an earlier invocation
    # of this script, so it's possible that the user has already
    # manually selected some images.

    # The if statement above serves only to avoid the grep statement
    # below when it is known to be unnecessary. My thought is that
    # the grep could be a bit time consuming for longer lists of scenes,
    # so why do it if I don't need to?

    my @selected = grep defined $_->[6],@list;

    if(@selected){

      # Yep. One or more scenes have been manually singled out.

      if(defined $in{cul}){

        # The user wants to view only the scenes that she has already selected.
        # Reduce the list and offer a way back to the full list.

        my $pcc = @list;	# pre cull count
        @list = @selected;
        $n = @list;

        if($in{ndx} >= $n){
          for(my $i=0; $i<$n; $i++){
            $list[$i][6] > $in{ndx} and last;
          }
          $in{ndx} = $i > 0 ? $i - 1 : 0;
        }

        my $setno = int($in{ndx}/$in{set});
        my $setstart = $setno * $in{set} + 1;
        my $setend = $setstart + $in{set} - 1;
        if($setend > @list){
          $setend = @list;
        }
        $setstart = ordinal($setstart);
        $setend = ordinal($setend);

        if($setstart eq $setend){
          $number = $setstart;
        }
        else{
          $number = "$setstart through $setend";
        }

        $number =
        "$number of " .
        scalar(@list) .
        " swaths interactively selected from $pcc swaths";

        $culled_complete =
        qq@<a href="$thisScript?@ .
        state_info(qw/url cul/) .
        qq@">$button{eye_open}</a>@;
      }
      else{

        # The user wants to view the entire list of the search results.
        # Offer a link back to the reduced list.

        $culled_complete =
        qq@<a href="$thisScript?@ .
        state_info(qw/url cul/) .
        qq@&cul=1">$button{eye_closed}</a>@;
      }
    }
    else{
      # Even though this is a previously generated list of scenes,
      # none of them are currently selected by the user.
      $culled_complete = $button{eye_off};
    }
  }
  else{

    # The list of scenes was generated by the current instantiation of
    # this script, so the user has not yet had a chance to manually select
    # any of the scenes in the list.  I therefore inactivate the button that
    # requests that only manually selected scenes be displayed.

    $culled_complete = $button{eye_off};
  }

  my ($tc,$chl,$sst,$sst4) = tc_chl_sst($in{prm},$in{sen},$in{dnm});

  my $changeset =
  qq@<form method="post" action="$thisScript">\n@ .
  state_info(qw/form set/) .
  qq@Display@ .
  qq@<input type="text" name="set" value="$in{set}" size="3">@ .
  qq@at a time.\n@ .
  qq@</form>\n@;

  # Set up the ORDER DATA button.
  (my $thisScriptRestrict = $thisScript) =~ s"^/cgi"/restrict";
  my $filelist =
  qq@<a href="$thisScriptRestrict?@ .
  state_info(qw/url sub ord xn xs xw xe/) .
  qq@&sub=order">$button{order}</a>@;

  # Set up buttons that allow user to request list of filenames.
  my $l0_button = '';
  my $all_selected = not grep defined $_->[6], @list;
  my $modis_present = grep $_->[0]=~/^[TA]$/, @list;
  my $modis_selected = grep $_->[0]=~/^[TA]$/ && defined $_->[6], @list;
  if($all_selected and $modis_present or $modis_selected){
    # If there's at least one selected MODIS scene in the list...
    $l0_button =
    qq@<a href="$thisScript/filelist.$invocation_time.txt?@ .
    qq@sub=filenamelist&id=$in{id}&prm=L0">@ .
    qq@<img src="/icons/L0.png" @ .
    qq@alt="display list of level-0 filenames" @ .
    qq@width="22" height="16" border="0"></a>@;
  }
  my $filenamelist =
  qq@<a href="$thisScript/filelist.$invocation_time.txt?@ .
  qq@sub=filenamelist&id=$in{id}&prm=$in{prm}">@ .
  qq@<img src="/icons/List.png" @ .
  qq@alt="display list of filenames" width="37" height="18" border="0"></a>@ .
  $l0_button;

  my $prev = $button{left_off};
  my $next = $button{right_off};

  # Hyperlinks to different sets of scenes.
  my $setlist = '';
  my $setstart = 0;
  if($n > $in{set}){
    my $numsets = int(($n-1)/$in{set}) + 1;
    my $setno = int($in{ndx}/$in{set});
    $setstart = $setno * $in{set};
    my $setend = $setstart + $in{set} - 1;
    my $href = qq@$thisScript?sub=level1or2list&$state@;
    if($setstart > 0){
      my $ndx = $setstart - 1;
      $prev =~ s/inactive/active/;
      $prev =~ s/alt=""/alt="Previous set of thumbnails"/;
      $prev = qq@<a href="$href&ndx=$ndx">$prev</a>@;
    }
    if($setend >= $#list){
      $setend = $#list;
    }
    else{
      my $ndx = $setend + 1;
      $next =~ s/inactive/active/;
      $next =~ s/alt=""/alt="Next set of thumbnails"/;
      $next = qq@<a href="$href&ndx=$ndx">$next</a>@;
    }
    @list = @list[$setstart..$setend];
    $n = @list;
    $setlist = qq@<p>\n@;
    for(my $i=0; $i<$numsets; $i++){
      my $j = $i + 1;
      if($i == $setno){
        $setlist .= "$j\n";
      }
      else{
        my $ndx = $i * $in{set};
        $setlist .= qq@<a href="$href&ndx=$ndx">$j</a>\n@;
      }
    }
    $setlist .= qq@</p>\n@;
  }

  # First row of filename hyperlinks
  my $table = qq@<table border="1">\n<tr>\n@;
  for($i=0; $i<$n; $i+=4){
    my ($msn,$time) = gran2msntime($list[$i]);
    my $file = (split /\//,msntimetyp2name($msn,$time,$in{prm}))[-1];
    my $f = msntimetyp2name($msn,$time,$brs2parent{$in{prm}});

    my $ndx;
    if(defined $list[$i][6]){
      $ndx = $list[$i][6];
    }
    else{
      $ndx = $setstart + $i;
    }

    my $cs = 4 + ( ($i + 4) >= $n ? ($n - 1) % 4 : 0 );

    $table .=
    qq@<td colspan="$cs">@ .
    qq@<img src="/icons/blank.gif" width="20" height="1">@ .
    qq@<img src="/icons/left_down.gif" alt="" width="9" height="13">@ .
    qq@<a href="$thisScript?sub=level1or2&$state&file=$file&ndx=$ndx@ .
    qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$f</a></td>\n@;
  }
  $table .= qq@</tr>\n@;

  # Second row of filename hyperlinks
  if($n > 2){
    $table .= qq@<tr>\n<td colspan="2"></td>\n@;
    for($i=2; $i<$n; $i+=4){

      my ($msn,$time) = gran2msntime($list[$i]);
      my $file = (split /\//,msntimetyp2name($msn,$time,$in{prm}))[-1];
      my $f = msntimetyp2name($msn,$time,$brs2parent{$in{prm}});

      my $ndx;
      if(defined $list[$i][6]){
        $ndx = $list[$i][6];
      }
      else{
        $ndx = $setstart + $i;
      }

      my $cs = 4 + ( ($i + 4) >= $n ? ($n - 3) % 4 : 0 );

      $table .=
      qq@<td colspan="$cs">@ .
      qq@<img src="/icons/blank.gif" width="20" height="1">@ .
      qq@<img src="/icons/left_down.gif" alt="" width="9" height="13">@ .
      qq@<a href="$thisScript?sub=level1or2&$state&file=$file&ndx=$ndx@ .
      qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$f</td>\n@;
    }
    $table .= qq@</tr>\n@;
  }

  # Row of dates in more comprehensible form
  $table .= qq@<tr align="center">\n@;
  my $yyyyddd = substr $list[0][1],0,7;
  my $cs = 0;
  for($i=0; $i<$n; $i++){
    my $d = substr $list[$i][1],0,7;
    if($d ne $yyyyddd){
      my $date = yyyyddd2date($yyyyddd);
      $table .= qq@<td colspan="$cs">$date</td>\n@;
      $cs = 0;
      $yyyyddd = $d;
    }
    $cs++;
  }
  $cs += 3;
  my $date = yyyyddd2date($yyyyddd);
  $table .= qq@<td colspan="$cs">$date</td>\n@;
  $table .= qq@</tr>\n@;

  # Row of (Y/****) image-selection hyperlinks
  $table .= qq@<tr align="center">\n@;
  for($i=0; $i<$n; $i++){
    my ($yn,$bg);
    if(defined $list[$i][6]){
      $yn = 'Yes';
      $bg = ' bgcolor="#00ff00"';
    }
    else{
      $yn = '****';
      $bg = '';
    }

    my $ndx;
    if(defined $list[$i][6]){
      $ndx = $list[$i][6];
    }
    else{
      $ndx = $setstart + $i;
    }

    my $cs = ($i + 1 == $n) ? 4 : 1;
    $table .=
    qq@<td colspan="$cs"$bg><b>@ .
    qq@<a href="$thisScript?sub=level1or2list&$state&ndx=$ndx&tog=1">@ .
    qq@$yn</a></b>@ .
    qq@</td>\n@;
  }
  $table .= qq@</tr>\n@;

  # Row of thumbnail images
  $table .= qq@<tr>\n@;
  for($i=0; $i<$n; $i++){
    my ($msn,$time) = gran2msntime($list[$i]);
    my $file = (split /\//,msntimetyp2name($msn,$time,$in{prm}))[-1];
    my $f = msntimetyp2name($msn,$time,$brs2parent{$in{prm}});

    my $ndx;
    if(defined $list[$i][6]){
      $ndx = $list[$i][6];
    }
    else{
      $ndx = $setstart + $i;
    }

    my $w = 70;
    my $h = int($w*$list[$i][4]/$list[$i][3] + 0.5);
    my $cs = ($i + 1 == $n) ? 4 : 1;

    if ($list[$i][5] > 0 && $list[$i][0] !~ /(C|O|H)/){
		if (
			($in{prm} eq 'TC') &&
			($list[$i][5] == 1 || $list[$i][5] == 3)
			){
			$table .=
			qq@<td colspan="$cs">L1 Browse<br />not<br />@ .
			qq@available<br /></td>\n@;
			next;
		} elsif (
			($in{prm} =~ /(CHL|SST)/) &&
			($list[$i][5] == 2 || $list[$i][5] == 3)
			){
			$table .=
			qq@<td colspan="$cs">L2 Browse<br />not<br />@ .
			qq@available<br /></td>\n@;
			next;
		} elsif ($list[$i][5] > 3){
			$table .=
			qq@<td colspan="$cs">Browse<br />not<br />@ .
			qq@available<br /></td>\n@;
			next;
		}
    }
    if($list[$i][0] =~ /^S/ and $in{prm} eq 'SST'){
      $table .=
      qq@<td colspan="$cs">Sea<br />surface<br />@ .
      qq@temperature<br />not<br />available<br />from<br />@ .
      qq@SeaWiFS</td>\n@;
      next;
    }

	$in{hlt} ||= '';
    $table .= qq@<td colspan="$cs">@ .
    qq@<a href="$thisScript?sub=level1or2&$state&file=${file}_BRS&ndx=$ndx@ .
    qq@&wid=$list[$i][3]&hgt=$list[$i][4]"><img\n@ .
    qq@src="$thisScript/$file.png?@ .
    qq@sub=l12thumb&id=$in{id}&wid=$w&hgt=$h@ .
    qq@&file=$file&hlt=$in{hlt}"\n@ .
    qq@alt="$f" title="$f"\n@ .
    qq@width="$w" height="$h" border="0"></a></td>\n@;
  }
  $table .= qq@</tr>\n@;

  # First row of filename hyperlinks below thumbnails
  if($n > 1){
    $table .= qq@<tr>\n<td colspan="1">$filenamelist</td>\n@;
    for($i=1; $i<$n; $i+=4){

      my ($msn,$time) = gran2msntime($list[$i]);
      my $file = (split /\//,msntimetyp2name($msn,$time,$in{prm}))[-1];
      my $f = msntimetyp2name($msn,$time,$brs2parent{$in{prm}});

      my $ndx;
      if(defined $list[$i][6]){
        $ndx = $list[$i][6];
      }
      else{
        $ndx = $setstart + $i;
      }

      my $cs = 4 + ( ($i + 4) >= $n ? ($n - 2) % 4 : 0 );
      $table .=
      qq@<td colspan="$cs">@ .
      qq@<img src="/icons/blank.gif" width="20" height="1">@ .
      qq@<img src="/icons/left_up.gif" alt="" width="9" height="13">@ .
      qq@<a href="$thisScript?sub=level1or2&$state&file=$file&ndx=$ndx@ .
      qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$f</a></td>\n@;
    }
    $table .= qq@</tr>\n@;
  }

  # Second row of filename hyperlinks below thumbnails
  if($n > 3){
    $table .= qq@<tr>\n<td colspan="3"></td>\n@;
    for($i=3; $i<$n; $i+=4){

      my ($msn,$time) = gran2msntime($list[$i]);
      my $file = (split /\//,msntimetyp2name($msn,$time,$in{prm}))[-1];
      my $f = msntimetyp2name($msn,$time,$brs2parent{$in{prm}});

      my $ndx;
      if(defined $list[$i][6]){
        $ndx = $list[$i][6];
      }
      else{
        $ndx = $setstart + $i;
      }

      my $cs = 4 + ( ($i + 4) >= $n ? ($n - 4) % 4 : 0 );
      $table .=
      qq@<td colspan="$cs">@ .
      qq@<img src="/icons/blank.gif" width="20" height="1">@ .
      qq@<img src="/icons/left_up.gif" alt="" width="9" height="13">@ .
      qq@<a href="$thisScript?sub=level1or2&$state&file=$file&ndx=$ndx@ .
      qq@&wid=$list[$i][3]&hgt=$list[$i][4]">$f</a></td>\n@;
    }
    $table .= qq@</tr>\n@;
  }
  $table .= qq@</table>\n@;

  my $search_criteria = &present_criteria;

  my $highlight = highlight($in{hlt});

  qq@<table width="100%"><tr>\n@ .
  qq@<td align="left">\n@ .
  qq@<table border="0" cellpadding="0" cellspacing="0"><tr>\n@ .
  qq@<td>$up</td>\n@ .
  qq@<td>$prev</td>\n@ .
  qq@<td>$next</td>\n@ .
  qq@<td>$highlight</td>\n@ .
  qq@<td>$culled_complete</td>\n@ .
  qq@<td>$tc</td>\n@ .
  qq@<td>$chl</td>\n@ .
  qq@<td>$sst</td>\n@ .
  qq@<td>$sst4</td>\n@ .
  qq@<td><a href="/SeaWiFS/IMAGES/chl_sst_colorbar.png">@ .
  qq@$button{cbar}</a></td>\n@ .
  qq@</tr></table></td>\n@ .
  qq@<td>$changeset</td>\n@ .
  qq@<td>$filelist</td>\n@ .
  qq@<td align="right"><a href="$comment_url"\n@ .
  qq@target="_blank">$button{comment}</a></td>\n@ .
  qq@<td align="right">@ .
  qq@<a href="javascript:helpwin('thumbs')">$button{help}</a>@ .
  qq@<noscript><a href="$thisScript?sub=help&hlp=thumbs" @ .
  qq@target="OCBrowseHelp">@ .
  qq@Help</a></noscript>@ .
  qq@</td>\n@ .
  qq@</tr></table>\n@ .
  $table .
  qq@<table>\n@ .
  qq@<tr>\n@ .
  qq@<td>\n@ .
  $setlist .
  qq@</td>\n@ .
  qq@<td valign="top">\n@ .
  $search_criteria .
  qq@<p>\n@ .
  qq@<b>Number of swaths:</b> $number\n@ .
  qq@</p>\n@ .
  qq@</td>\n@ .
  qq@<tr>\n@ .
  qq@</table>\n@;
}

sub parent_table{
  my $f = shift;
#  write_debug("parent:$f");
  my @f = ();

  my ($msn,$time) = fname2msntime($f);

  # Only present a L0 filename if L1A file has an assoc. L0 parent available.
  my $f0 = msntimetyp2name($msn,$time,'L0');
  if ($f0) {
    my %lf = (pattern=>$f0, type=>'L0');
    my %l0exist = &locate_file(\%lf);
    if(keys %l0exist){
      my @l0 = keys %l0exist;
      push @f, $f0;
    }
  }

  # Always present an L1 filename.
  push @f ,msntimetyp2name($msn,$time,'L1');
#  write_debug(@f);

  # VIIRS also gets a GEO file.
  my $geo = msntimetyp2name($msn,$time,'GEO');
  if($geo){
    push @f ,$geo;
  }

  # Only present a standard L2 filename if a CHL browse file is available.
  if(-s msntimetyp2name($msn,$time,'CHL') ){
    foreach my $typ (qw(L2OC L2IOP)){
      my $l2 = msntimetyp2name($msn,$time,$typ);
      $l2 and push @f, $l2;
    }
  }

  # Only present L2 SST filenames if an SST browse file is available.
  if(-s msntimetyp2name($msn,$time,'SST') ){
    foreach my $typ (qw(L2SST L2SST3 L2SST4)){
      my $l2 = msntimetyp2name($msn,$time,$typ);
      $l2 and push @f, $l2;
    }
  }

  my $table = qq@<table>\n@;

  my $linkcount = 0;

  for(my $i=0; $i<@f; $i++){
    $table .= qq@<tr>\n@;
    my %urlsiz =();
    my %lf = (pattern=>"$f[$i]");
    
	 $lf{sensor} = $sen{$msn}[3];

    %urlsiz = &locate_file(\%lf);
#    write_debug(%urlsiz);

    my $url = (grep /$f[$i](?:\.(bz2|Z|gz|zip))?/, keys %urlsiz)[0];
    my $siz = $urlsiz{$url}{size};
    if ($f[$i] =~ /\.L0_/){$url =~ s/:$f[$i].*//}
    # Pop off the filename for getfile script SWB 20 Oct 2004
    my $webfile = (split /\//,$url)[-1];
    unless(			# Unless

    $url == -1			# file is not available,
    or
    $url eq ''			# file is not available,

    ){
      my $siz = add_commas($siz);

      my $pfx =  'ob';
      if ($webfile =~ /^S3(A|B)/){
         $pfx = 'sentinel';
      } elsif ($webfile =~ /^M/) {
         $pfx = 'echo';
      } elsif ($webfile =~ /^H/) {
         $pfx = 'hico';
      } elsif ($webfile =~ /^COMS_GOCI/) {
         $pfx = 'goci';
      }

      # Use getfile SWB 20 Oct 2004.
      $url = "https://oceandata.sci.gsfc.nasa.gov/$pfx/getfile/$webfile";
      if ($f[$i] =~ /L0/){
        $url = "https://oceandata.sci.gsfc.nasa.gov/$pfx/getfile/$f[$i].bz2";
      }

      $table .=
      qq@<th align="left">@ .
      qq@<a href="$url">@ .	# DO make the hyperlink.
      qq@$f[$i]</a>@ .
      qq@</th>\n@ .
      qq@<td align="right">@ .
      qq@<b>$siz bytes</b>@ .
      qq@</td>\n@;

      $linkcount++;
    }

    $table .= qq@</tr>\n@;
  }

  # Add some extra documentation to explain the hyperlinks if there are any.
  if($linkcount){
    my @plural = ('','','');
    if($linkcount > 1){
      $plural[0] = 's';
    }
    else{
      $plural[1] = 's';
      $plural[2] = ' a';
    }
    my $linkhelp =
    qq@<td colspan="2">\n@ .
    qq@(The above hyperlink$plural[0] point$plural[1] to$plural[2]\n@ .
    qq@<a href="javascript:helpwin('scene#download')">compressed\n@ .
    qq@file$plural[0]</a>@ .
    qq@<noscript><a href="$thisScript?sub=help&hlp=scene#download"\n@ .
    qq@target="OCBrowseHelp"> (click here if JavaScript not enabled)</a>@ .
    qq@</noscript>.\n@;
    if($f =~ /^[TAS]/){
      $linkhelp .=
      qq@<br />\n@ .
      qq@Documentation on these products can be found\n@ .
      qq@<a href="/products" target="OCBrowseHelp">HERE</a>.@;
    }
    $linkhelp .= qq@)\n</td>\n@;

    $table .= $linkhelp;
  }

  $table .= qq@</table>\n@;
}

sub swath_table{
  my ($f,$is_selected) = @_;

  my $titlerow = qq@<tr>\n@;
  my $imagerow = qq@<tr valign="top">\n@;

  my $bg = $is_selected ? ' bgcolor="#00ff00"' : '';

  my ($msn,$time) = fname2msntime($f);
  my $cs = 0;
  foreach my $prm (qw/TC CHL SST SST4/){
    my $file = msntimetyp2name($msn,$time,$prm);
    if(-s $file){

      $titlerow .= qq@<th$bg>$prm{$prm}[0]</th>\n@;

      my $ismap = '';
      $ismap = ' usemap="#insitu_map"' if $in{is} eq 'yes';

      $file = (split /\//,$file)[-1];

      $imagerow .=
      qq@<td><img src="$thisScript/$file.png?sub=l12image@ .
      qq@&id=$in{id}&file=$file&hlt=$in{hlt}&is=$in{is}&all=$in{all}"\n@ .
      qq@alt="$file" width="$in{wid}" height="$in{hgt}"\n@ .
      qq@border="0"$ismap></td>\n@;

      $cs++;
    }
  }

  $titlerow .= qq@</tr>\n@;
  $imagerow .= qq@</tr>\n@;

  my $sel_toggle = '';
  if(defined $in{id}){
    if($is_selected){
      $sel_toggle =
      qq@<td colspan="$cs" align="center"$bg>@ .
      qq@<a href="$thisScript?@ .
      state_info(qw/url/) .
      qq@&tog=1">@ .
      qq@Deselect this scene</a>@ .
      qq@</td>\n@
    } else{
      $sel_toggle =
      qq@<td colspan="$cs" align="center">@ .
      qq@<a href="$thisScript?@ .
      state_info(qw/url/) .
      qq@&tog=1">@ .
      qq@Select this scene</a>@ .
      qq@</td>\n@;
    }
    $sel_toggle = qq@<tr>\n$sel_toggle</tr>\n@;
  }

  qq@<table>\n@ .
  $sel_toggle .
  $titlerow .
  $imagerow .
  qq@</table>\n@;

}

sub brs2insitu{
  my $path = shift;
  -s $path or return ();
  my $id = shift;

  # Get all the in situ matchups for this scene.
  my ($msn,$time) = fname2msntime($path);

  my $tdelta = 120;
  my $stime = $time - $tdelta;
  my $etime = $time + $tdelta;

  my $q = join ' ',split ' ',<<"  --";
  SELECT distinct l.date_time,l.lat,l.lon,i.name,i.wavelength,i.value,
                  l.cruise,l.dataid,l.quad_id
  from insitu_loc l, insitu_data i, scene_info s, insitu_files f, sensors m
  where s.sen_id = m.sen_id
  and   s.sen_id = $sen{$msn}[3]
  and   s.start_time between $stime and $etime
  and   s.scene_id    = f.file_id
  and   f.dataid     = l.dataid
  and   l.dataid     = i.dataid
  --

  if($id){
    # If an ID is passed in, then further restrict in situ
    # matchups to those falling within the quadsphere bins
    # associated with the search ID.

    # Make a temporary table containing the
    # quadsphere bins in the user's search area.
    $dbh->do("call temp_qbin(-1)");
    $urtable = make_qbtable($id);

    $q =~ s/insitu_files f/insitu_files f, qbins r/;
    $q =~ s/i.dataid/i.dataid and l.quad_id = r.quad_id/;
  }
  my $sth = $dbh->prepare($q);
  #$sth->{"mysql_use_result"} = 0;
  $sth->execute;
  my ($result,@results);
  push @results,$result while($result = $sth->fetchrow_hashref);
  $sth->finish;


  @results or return ();

  # Convert longitude and latitude to x and y on the browse image.
  my @cmd = ($rnav,$path);
  my ($inp,$out,$err);
  foreach my $r (@results){
    foreach my $k ( keys %$r ){
      $r->{$k} = launder( $r->{$k}, "\$r->{$k}" );
    }
    $inp .= "$r->{lon} $r->{lat}\n";
  }
  run3 \@cmd,\$inp,\$out,\$err;# or die "Error running $rnav ($!) $err ";
  my ($w,$h,@i) = map launder($_), split ' ',$out;
  if(@i != @results){
    # Currently, $rnav does not pass
    # through any information associated with the input bins or
    # coordinates, so the only way to make the association is
    # by matching input and output records.  If the numbers of
    # records don't match (e.g. if some of the input coordinates
    # fall outside the scene), then I have no way of determining
    # which pixel index goes with which input coordinate.
    # To get around this problem I am looking into removing
    # the offending records from the database.  In the mean time,
    # I will just incur the expense of starting up extra processes
    # and rerun $rnav individually for each in situ record.
    foreach my $r (@results){
      my $out = `$rnav $path $r->{lon} $r->{lat}`;
      my ($w,$h,$i) = split ' ',$out;
      $r->{wid} = $w;
      $r->{hgt} = $h;

      ( $r->{x},      $r->{y} ) = defined $i && $i ne '' ?
      ( $i % $w, int($i / $w) ) :
      (    -999,        -999  );
    }
  }
  else{
    for(my $i=0; $i<@i; $i++){
      # Convert each pixel index returned by $rnav to an x/y coordinate.
      my ($x,$y) = ( $i[$i] % $w, int($i[$i] / $w) );
      # Add the x/y information to the record returned from the database.
      $results[$i]{x}   = $x;
      $results[$i]{y}   = $y;
      $results[$i]{wid} = $w;
      $results[$i]{hgt} = $h;
    }
  }
  @results;
}

sub qlpath{
  my ($sen,$prm,$type,$per,$day,$dn) = @_;
  my $dorn = '';
  if($prm =~ /^SST/){
    if($dn =~ /D/){ $dorn = '.D' }
    else          { $dorn = '.N' }
  }
  my $c = $sen{$sen}[1];	# 1st character of filename
  my $t = ".$type";
  my $path;
  if($per eq 'DAY'){
    my ($y,$d) = (gmtime 86400*$day)[5,7];
    $y += 1900;
    $d++;
    $path = $qdir{$sen}{$prm}{$type} . sprintf
    "/%04d/$c%04d%03d.$prm$t.$per$dorn.png", $y,$y,$d;
  }
  elsif($per eq 'CU' or $per eq 'CSTM'){
    # Every time I generate a new image for the entire mission it will have
    # a different end date as part of the filename.  Since I don't plan to
    # regenerate these images on a daily basis, and since the end date of
    # the mission does change every day, the only way to get the most
    # recently-generated mission (CU) filename is to do a directory search,
    # hence this block of code.
    #
    # I am also displaying the cumulative images when a custom time period
    # has been specified.
    if(opendir D,$qdir{$sen}{$prm}{$type}){
      my @yd = sort grep /^\d{4}$/, readdir D;
      closedir D;
      foreach my $y (@yd){
        $y = launder($y);
        my $d = "$qdir{$sen}{$prm}{$type}/$y";
        opendir D,$d or die qq@Could not open directory, "$d" ($!)@;
        my @f = sort grep /^$c\d{14}\.$prm$t\.CU$dorn\.png$/, readdir D;
        closedir D;
        @f or next;	# No CU files found; go to next year.
        my $f = launder( pop @f );
        $path = "$d/$f";
      }
      defined $path or warn qq@Mission length quicklook not available under directory, @, qq@"$qdir{$sen}{$prm}{$type}"@;
    } else{
      warn qq@Could not open directory, \$qdir{$sen}{$prm}{$type} = "$qdir{$sen}{$prm}{$type}" ($!)@;
    }
  }
  else{
    my ($start,$end) = period_bounds($per,$day);
    my ($y1,$d1) = (gmtime 86400*$start)[5,7];
    my ($y2,$d2) = (gmtime 86400*$end  )[5,7];
    $y1 += 1900;
    $y2 += 1900;
    $d1++;
    $d2++;
    $path = $qdir{$sen}{$prm}{$type} . sprintf
    "/%04d/$c%04d%03d%04d%03d.$prm$t.$per$dorn.png", $y1,$y1,$d1,$y2,$d2;
  }
  $path;
}

sub qlpaths{
  my ($sens,$prm,$per,$day,$dnm) = @_;
  
  # Find out which sensors and types were selected.
  # For SeaWiFS, the order that I arrange the types in (i.e. MLAC LAC GAC)
  # is significant because it controls which images get placed
  # on top of the stack when I composite them in the l3image
  # subroutine.  LAC tends to have the most minimal coverage,
  # so I want it on top.  GAC, on the other hand, tends to have
  # greater coverage than the other two types, so I want it
  # on the bottom of the stack when compositing.
  # On the other hand, MLAC being a "best of" combination of LAC and
  # HRPT and therefore having less noise should probably go on top
  # of LAC and HRPT.
  # The stacking order among sensors from top to bottom is SeaWiFS,
  # Terra-MODIS, Aqua-MODIS.  SeaWiFS goes on top because I am biased :-),
  # and the Terra/Aqua order is arbitrary.
  my @sens;
  if(defined $sens and $sens){
    # The following sort arranges the sensors in the image stack
    # with the lowest sort numbers at the top of the stack.
    @sens = sort {$sen{$a}[2] <=> $sen{$b}[2]} split /@/, $sens;
  }
  else{
    # No sensors selected is the same as all sensors selected.
    @sens = sort {$sen{$a}[2] <=> $sen{$b}[2]} keys %sen;
  }

  # Compose quicklook file paths for the selected parameter and period
  # for each of the selected sensors and types.
  my @paths;
  foreach my $s (@sens){
    my $t = $sen{$s}[4];
    # No SST for SeaWiFS and OCTS
    next if ($prm =~ /^SST/ and $s !~ /amod|tmod|vrsn/);

    if($s !~ /amod|tmod|vrsn/){
      my $path = qlpath($s,$prm,$t,$per,$day,$dnm);
      -s $path and push @paths, $path;
    } else{
      foreach my $dn (split /@/, $dnm){
        next if($dn eq 'M');	# No mixed quicklooks are currently produced.
        next if($dn eq 'N' and $prm !~ /^SST/);	# Only SST has
						# nighttime quicklooks.
        my $path = qlpath($s,$prm,$t,$per,$day,$dn);
        -s $path and push @paths, $path;
      }
    }
  }
  @paths;
}

sub qlstack{
  # Combine a series of 512x256 transparent PNG images into a single PNG.
  my @maps = @_;
  my $png;
  my @tmps;

  if($in{dnm} =~ /D/){
    # Any composites displayed when daytime data are selected should
    # be displayed as a map centered on the prime meridian.  That means
    # than any nighttime quicklooks in the stack (as there will be when
    # both daytime *and* nighttime are selected) must have their left
    # and right halves swapped since they are created with the 180-degree
    # meridian in the middle.

    foreach my $map (@maps){
      if($map =~ /\.N\.png$/){
        my $base = (split /\//,$map)[-1];
        $base =~ s/\.png$/.$invocation_time.$$/;

        # Split the western and eastern halves.
        my $c =
        "$pngtopnm $map " .
        "| $pamdice -outstem $tmpdir/$base -width $l3hgt -height $l3hgt";
        system($c) == 0 or die "Error executing $c ($!)";

        # Reassemble them in reverse order.
        $c =
        "$pnmcat -lr $tmpdir/${base}_0_1.ppm $tmpdir/${base}_0_0.ppm " .
        "| $pnmtopng -trans =black > $tmpdir/$base.png";
        system($c) == 0 or die "Error executing $c ($!)";

        unlink "$tmpdir/${base}_0_1.ppm","$tmpdir/${base}_0_0.ppm";
        push @tmps,"$tmpdir/$base.png";

        $map = "$tmpdir/$base.png";
      }
    }
    push @maps,$coast512x256_day;
  }
  elsif($in{dnm} =~ /N/){
    # If only nighttime data have been selected, then I need to
    # use the matching background coastline image.
    push @maps,$coast512x256_ngt;
  }

  if(@maps > 1){
    splice @maps,0,2,join ' ',@maps[0,1];
    my $c = "$composite " . join(" -|$composite - ",@maps) . " PNG:-";
    open P,"$c|" or die qq@Error opening pipe from command, "$c" ($!)@;
    local($/) = undef;
    $png = <P>;
    close P or die qq@Error executing command, "$c" ($!)@;
    unlink @tmps;
  }
  else{
    # No map images specified; just return the background image.
    open F,'<',$maps[0] or die qq@Could not open file, "$maps[0]" ($!)@;
    local($/) = undef;
    $png = <F>;
    close F;
  }
  $png;
}

sub mission_table{
# The default if no sensors are specified is that all sensors
  # specified in the globally visible hash, %sen, are selected.
  my @sen = keys %sen;

  my $sen = shift;
  if(defined $sen){
    my %s;
    grep $s{$_}++ && 0, split /@/,$sen;
    my @s = grep $s{$_}, @sen;
    @sen = @s if @s;
  }

  my @monabbr = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
  my $start_year = year($mission_start);
  my $end_year = year($mission_end);
  my $numyears = $end_year - $start_year + 1;

  my $start_first_mon = (period_bounds('MO',$mission_start))[0];
  my $end_last_mon    = (period_bounds('MO',$mission_end  ))[1];

  my @href = ('','');
  unless($in{per} eq 'CU'){
    my $state = state_info('url','per','day');
    @href =
    (qq@<a href="$thisScript?per=CU&day=$mission_start&$state">@,"</a>");
  }

  my $html =
  qq@<table border="1">\n@ .
  qq@<tr>\n@ .
  qq@<th rowspan="$numyears">\n@ .
  qq@$href[0]M<br />i<br />s<br />s<br />i<br />o<br />n<br />$href[1]\n@ .
  qq@</th>\n@;
  my $y;
  for($y=$start_year; $y<=$end_year; $y++){
    next if $y > 1986 && $y < 1996; # Break in coverage between CZCS and OCTS
    my $yday = timegm(0,0,0,1,0,$y-1900)/86400;
    my @href = ('','');
    unless($in{per} eq 'YR' and $in{day} == $yday){
      my $state = state_info('url','per','day');
      @href = (qq@<a href="$thisScript?per=YR&day=$yday&$state">@,"</a>");
    }
    $html .= qq@<tr>\n@ unless $y == $start_year;
    $html .=
    qq@<th>\n@ .
    qq@$href[0]$y$href[1]\n@ .
    qq@</th>\n@;
    my $m;
    for($m=0; $m<12; $m++){
      my $bg = '';
      my $mday = timegm(0,0,0,1,$m,$y-1900)/86400;
      my @mday = period_bounds('MO',$mday);

      # Account for the breaks in coverage between missions.
      my $mission_gap =
      $mday[0] > $czcs_end && $mday[1] < $octs_start ||
      $mday[0] > $octs_end && $mday[1] < $seawifs_start;

      if(
      monthinranges( $mday, perdaytoranges($in{per},$in{day}) )
      and !$mission_gap
      ){
        $bg = ' bgcolor="#ffff66"';
      }
      my @href = ('','');
      unless(
      $in{per} eq 'MO' and $in{day} == $mday or
      $mday < $start_first_mon or $mday > $end_last_mon or $mission_gap
      ){
        my $state = state_info('url','per','day');
        @href = (qq@<a href="$thisScript?per=MO&day=$mday&$state">@,"</a>")
      }
      $html .=
      qq@<td$bg>\n@ .
      qq@$href[0]<b><tt>$monabbr[$m]</tt></b>$href[1]\n@ .
      qq@</td>\n@;
    }
    $html .= qq@</tr>\n@;
  }
  $html .= qq@</table>\n@;
}

sub calendar{
  my $day = shift;
  my ($start,$end) = period_bounds('MO',$day);
  my $daysinmonth = $end - $start + 1;
  my $month = monstr($day);
  my $year = year($day);
  my $startdow = (gmtime(86400*$start))[6];

  # I add 7 days to each end of the period so I can
  # check any octaves that span a month boundary.
  my @gransperday = gransperday($in{sen},$start-7,$daysinmonth+14);

  my $html =
  qq@<table border="1" cellspacing="0" cellpadding="0">\n@ .
  qq@<tr>\n@ .
  qq@<td colspan="7">\n@ .
  qq@<table width="100%" border="0" cellspacing="0" cellpadding="0"><tr>\n@ .
  qq@<th>$month</th><th>$year</th>\n@ .
  qq@</tr>\n</table>\n@ .
  qq@</td>\n@ .
  qq@</tr>\n@ .
  qq@<tr align="center"><td>S</td><td>M</td><td>T</td><td>W</td>@ .
  qq@<td>T</td><td>F</td><td>S</td></tr>\n@ .
  qq@<tr align="center">\n@ .
  qq@<td></td>@ x $startdow;

  my @osym = qw/^^^ ooo *** xxx/;	# Octave symbol alternates to visually
					# distinguish adjacent 8-day periods.
  my $d;
  for($d=1; $d<=$daysinmonth; $d++){
    my $bg = '';
    my $day = $start + $d - 1;
    if(dayinranges($day, perdaytoranges($in{per},$in{day}) ) ){
      $bg = ' bgcolor="#99ff99"';
    }
    my $doy = (gmtime(86400*$day))[7];
    my $wkno = int($doy/8) + 1;
    my ($osday,$oeday) = period_bounds('8D',$day);
    my $ondx = $osday - $start + 7;
    my $ocnt = 0;
    grep { $ocnt += $_; 0 } @gransperday[$ondx .. $ondx+7];
    my @href = ('','');
    my @ohref = ('','');
    unless($in{per} eq 'DAY' and $in{day} == $day or $gransperday[$d+6] == 0){
      my $state = state_info('url','per','day');
      @href =
      (qq@<a href="$thisScript?per=DAY&day=$day&$state">@,"</a>");
    }
    unless($in{per} eq '8D' and $in{day} >= $osday and $in{day} <= $oeday
    or $ocnt == 0){
      my $state = state_info('url','per','day');
      @ohref =
      (qq@<a href="$thisScript?per=8D&day=$osday&$state">@,"</a>");
    }
    $html .=
    qq@<td$bg>$href[0]$d$href[1]<br />\n@ .
    qq@$ohref[0]$osym[$wkno%4]$ohref[1]</td>\n@;
    $html .= qq@</tr><tr align="center">\n@
             if(($d+$startdow)%7 == 0 and $d != $daysinmonth);
  }

  $html .= qq@<td></td>@ x (6 - ($startdow+$daysinmonth-1)%7);
  $html .= qq@</tr>\n</table>\n@;
}

sub process_ranges{
  my $r = '';
  for(my $i=1; $in{"rd.$i.0"}; $i++){
    defined $in{"rr.$i"} or next;
    my $start = ddmonyyyy($in{"rd.$i.0"},$in{"rm.$i.0"},$in{"ry.$i.0"});
    my $end   = ddmonyyyy($in{"rd.$i.1"},$in{"rm.$i.1"},$in{"ry.$i.1"});
    my $n = $end - $start + 1;
    $r .= ' ' if $i > 1;
    $r .= "$start $n";
  }
  my $start = ddmonyyyy($in{"rd.0.0"},$in{"rm.0.0"},$in{"ry.0.0"});
  my $end   = ddmonyyyy($in{"rd.0.1"},$in{"rm.0.1"},$in{"ry.0.1"});
  if($start >= 0 and $end >= 0){
    if($start > $end){
      my $tmp = $start;
      $start = $end;
      $end = $tmp;
    }
    my $n = $end - $start + 1;
    $r .= ' ' if $r != '';
    $r .= "$start $n";
  }
  $r;
}

sub process_periodics{
  my $r = '';
  my $sign = $in{pra} eq 'Remove' ? '-' : '';
  my @psmd = split /@/,$in{psmd};
  foreach my $p (@psmd){
    if   ($p =~ /^(Spring|Summer|Autumn|Winter)$/){

      my %mon = (
      Spring => ['Mar','Jun'],
      Summer => ['Jun','Sep'],
      Autumn => ['Sep','Dec'],
      Winter => ['Dec','Mar'],
      );

      my %yr = qw/Spring 0 Summer 0 Autumn 0 Winter 1/;

      foreach my $y ($in{py0}..$in{py1}){
        my $s = ddmonyyyy(21,$mon{$p}[0],$y-$yr{$p});
        my $e = ddmonyyyy(20,$mon{$p}[1],$y);
        my $n = $e - $s + 1;
        $r .= ' ' if $r != '';
        $r .= "$s $sign$n";
      }
    }
    elsif($p =~ /^(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)$/){

      my %mon = qw/Jan 31 Feb 28 Mar 31 Apr 30 May 31 Jun 30
                   Jul 31 Aug 31 Sep 30 Oct 31 Nov 30 Dec 31/;

      foreach my $y ($in{py0}..$in{py1}){
        my $ld = $p eq 'Feb' && isleap($y) ? 1 : 0;
        my $s = ddmonyyyy(1,$p,$y);
        my $e = ddmonyyyy($mon{$p} + $ld,$p,$y);
        my $n = $e - $s + 1;
        $r .= ' ' if $r != '';
        $r .= "$s $sign$n";
      }
    }
    elsif($p =~ /^(Sun|Mon|Tue|Wed|Thu|Fri|Sat)$/){
      my %wday = qw/Sun 0 Mon 1 Tue 2 Wed 3 Thu 4 Fri 5 Sat 6/;
      my $s = timegm(0,0,0, 1, 0,$in{py0}-1900)/86400;
      my $e = timegm(0,0,0,31,11,$in{py1}-1900)/86400;
      my $d1 = (gmtime 86400*$s)[6];    # Jan. 1 falls on this day of the week.
      if($wday{$p} > $d1){
        $s += $wday{$p} - $d1;
      }
      elsif($wday{$p} < $d1){
        $s += 7 - $d1 + $wday{$p};
      }
      for(my $d=$s; $d<=$e; $d+=7){
        $r .= ' ' if $r != '';
        $r .= "$d ${sign}1";
      }
    }
  }
  if($in{pd0} and $in{pm0} and $in{pd1} and $in{pm1}){
    my %mon = qw/Jan 0 Feb 1 Mar 2 Apr 3 May  4 Jun  5
                 Jul 6 Aug 7 Sep 8 Oct 9 Nov 10 Dec 11/;
    if($mon{$in{pm1}} > $mon{$in{pm0}}
    or      $in{pm1}  eq     $in{pm0}
    and     $in{pd1}  >=     $in{pd0}  ){
      # If the second date is the same as or later than the first date
      # in the same year, then I don't need to wrap around the year's end.
      foreach my $y ($in{py0}..$in{py1}){
        my $s = ddmonyyyy($in{pd0},$in{pm0},$y);
        my $e = ddmonyyyy($in{pd1},$in{pm1},$y);
        my $n = $e - $s + 1;
        $r .= ' ' if $r != '';
        $r .= "$s $sign$n";
      }
    }
    else{
      # Otherwise, start one year before the specified year range and
      # wrap each time range across the new year.
      foreach my $y ($in{py0}-1 .. $in{py1}){
        my $s = ddmonyyyy($in{pd0},$in{pm0},$y);
        my $e = ddmonyyyy($in{pd1},$in{pm1},$y+1);
        my $n = $e - $s + 1;
        $r .= ' ' if $r != '';
        $r .= "$s $sign$n";
      }
    }
  }
  $r;
}

sub ddmonyyyy{
  my ($dd,$mon,$yyyy) = @_;
  my %mon = qw/Jan 0 Feb 1 Mar 2 Apr 3 May  4 Jun  5
               Jul 6 Aug 7 Sep 8 Oct 9 Nov 10 Dec 11/;

  $dd >= 1 and $dd <= 31 or return -1;
  defined $mon{$mon}     or return -1;
  $yyyy =~ /^\d{4}$/     or return -1;

  timegm(0,0,0,$dd,$mon{$mon},$yyyy - 1900)/86400;
}

sub perdaytoranges{
  my ($per,$day) = @_;

  my %ranges = ();

  if   ($per eq 'CSTM'){
    %ranges = consolidate_ranges(split " ",$day);
  }
  elsif(defined $per{$per}){
    my ($start,$end) = period_bounds($per,$day);
    $ranges{$start} = $end - $start + 1;
  }
  %ranges;
}

sub rangestoperday{
  my %r = @_;
  my ($per,$day);
  my @s = sort {$a <=> $b} keys %r;
  if(@s == 1){
    # If there is only a single date range, then see
    # if it matches one of the predefined date ranges.
    foreach my $p (keys %per){
      next if $p eq 'CSTM';
      my ($s,$e) = period_bounds($p,$s[0]);
      if($s == $s[0]){
        my $end = $s[0] + $r{$s[0]} - 1;
        if($e == $end){
          $per = $p;
          $day = $s;
          return($per,$day);
        }
      }
    }
  }
  $per = 'CSTM';
  $day = '';
  foreach my $s (@s){
    $day .= ' ' if $day != '';
    $day .= "$s $r{$s}";
  }
  ($per,$day);
}

sub rangestostring{
  my %r = @_;
  my $str = '';
  foreach my $s (sort {$a <=> $b} keys %r){
    $str .= ' ' if $str != '';
    $str .= "$s $r{$s}";
  }
  $str;
}

# Time ranges are passed to the consolidate_ranges subroutine as
# start-day/number-of-days pairs.  The number-of-days in each pair
# can be either positive, meaning that the indicated time range
# should be added to the whole, or negative, meaning that the
# indicated time range should be removed from the whole.
sub consolidate_ranges{
  my @r;        # the array of time range pairs
  my @tmp;
  push @r,[@tmp] while(@tmp = splice @_,0,2);

  my %r;        # the hash returned by this subroutine

  my @ra = grep $_->[1] > 0, @r;        # time range pairs to be added

  @ra or return ();

  @ra = sort {$a->[0] <=> $b->[0]} @ra; # put in ascending order by start day

  foreach (@ra){
    push @$_, $_->[0] + $_->[1] - 1;    # add the end day to the range info
  }

  for(my $i=1; $i<@ra; $i++){
    if($ra[$i][0] <= $ra[$i-1][2] + 1){
      # If the start day of this range is less than or equal
      # to the day after the end day of the previous range ...
      if($ra[$i][2] > $ra[$i-1][2]){
        # If the end day of this range > the end day of the previous range
        $ra[$i-1][2] = $ra[$i][2];
        $ra[$i-1][1] = $ra[$i-1][2] - $ra[$i-1][0] + 1;
      }
      splice @ra,$i,1;
      $i--;
    }
  }

  my @rr = grep $_->[1] < 0, @r;        # time range pairs to be removed

  if(@rr){

    @rr = sort {$a->[0] <=> $b->[0]} @rr;# put in ascending order by start day

    foreach (@rr){
      push @$_, $_->[0] - $_->[1] - 1;  # add the end day to the range info
    }

    my $j = 0;
    REMOVALS:
    for($i=0; $i<@ra; $i++){
      while($rr[$j][2] < $ra[$i][0]){
        $j++;
        last REMOVALS if $j == @rr;
      }
      if($rr[$j][0] <= $ra[$i][2]){
        if($rr[$j][2] < $ra[$i][2]){
          my $tmp = [ $rr[$j][2] + 1, $ra[$i][2] - $rr[$j][2], $ra[$i][2] ];
          splice @ra, $i + 1, 0, $tmp;
        }
        $ra[$i][2] = $rr[$j][0] - 1;
        $ra[$i][1] = $ra[$i][2] - $ra[$i][0] + 1;
        $ra[$i][1] < 0 and $ra[$i][1] = 0;
      }
    }
  }

  for($i=0; $i<@ra; $i++){
    $r{$ra[$i][0]} = $ra[$i][1] if $ra[$i][1] > 0;
  }
  %r;
}

sub dayinranges{
  my ($day,%r) = @_;
  foreach my $s (keys %r){
    my $e = $s + $r{$s} - 1;
    if($day >= $s and $day <= $e){
      return 1;
    }
  }
  return 0;
}

sub monthinranges{
  my ($day,%r) = @_;
  my ($ms,$me) = period_bounds('MO',$day);
  foreach my $s (keys %r){
    my $e = $s + $r{$s} - 1;
    if($me >= $s and $ms <= $e){
      return 1;
    }
  }
  return 0;
}

sub day_selector{
  my ($name,$default) = @_;

  my $html = qq@<select name="$name" size="1">\n@;

  if(!defined $default or $default eq ''){
    $html .= qq@<option selected></option>\n@;
  }

  for(my $i=1; $i<=31; $i++){
    if($i == $default){
      $html .= qq@<option selected>$i</option>\n@;
    }
    else{
      $html .= qq@<option>$i</option>\n@;
    }
  }
  $html .= qq@</select>\n@;
}

sub month_selector{
  my ($name,$default) = @_;

  my $html = qq@<select name="$name" size="1">\n@;

  if(!defined $default or $default eq ''){
    $html .= qq@<option selected></option>\n@;
  }

  my $m;
  foreach $m (qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/){
    if($m eq $default){
      $html .= qq@<option selected>$m</option>\n@;
    }
    else{
      $html .= qq@<option>$m</option>\n@;
    }
  }
  $html .= qq@</select>\n@;
}

sub year_selector{
  my ($name,$start,$stop,$default) = @_;

  my $html = qq@<select name="$name" size="1">\n@;

  if(!defined $default or $default eq ''){
    $html .= qq@<option selected></option>\n@;
  }
  elsif($default < $start){
    $start = $default;
  }
  elsif($default > $stop){
    $stop = $default;
  }

  for(my $i=$start; $i<=$stop; $i++){
    if($i == $default){
      $html .= qq@<option selected>$i</option>\n@;
    }
    else{
      $html .= qq@<option>$i</option>\n@;
    }
  }
  $html .= qq@</select>\n@;
}

sub period_bounds{
  my ($per,$day) = @_;
  my ($start,$end);
  if($per eq 'CU'){
    ($start,$end) = ($mission_start,$mission_end);
  }
  elsif($per eq 'YR'){
    my ($yr,$doy) = (gmtime($day * 86400))[5,7];        # year, day of year
    $start = $day - $doy;
    $end = $start + (isleap($yr + 1900) ? 365 : 364);
  }
  elsif($per eq 'MO'){
    my ($mo,$yr,$doy) = (gmtime($day * 86400))[4,5,7];
    my (@firstdays,@lastdays);
    if(isleap($yr + 1900)){
      @firstdays = (0,31,60, 91,121,152,182,213,244,274,305,335);
      @lastdays = (30,59,90,120,151,181,212,243,273,304,334,365);
    }
    else{
      @firstdays = (0,31,59, 90,120,151,181,212,243,273,304,334);
      @lastdays = (30,58,89,119,150,180,211,242,272,303,333,364);
    }
    $start = $day - $doy + $firstdays[$mo];
    $end   = $day - $doy + $lastdays[$mo];
  }
  elsif($per eq '8D'){
    my ($yr,$doy) = (gmtime($day * 86400))[5,7];
    $start = $day - $doy % 8;
    $end = $start + ($doy < 360 ? 7 : (isleap($yr + 1900) ? 5 : 4));
  }
  elsif($per eq 'DAY'){
    $start = $day;
    $end   = $day;
  }
  else{
    ($start,$end) = (undef,undef);
  }
  ($start,$end);
}

sub isleap{
  my $y = shift;
  !($y%4) && $y%100 || !($y%400);
}

sub head{
  my $gtm1 = <<"  --";
<!-- Google Tag Manager -->
<script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push(
{'gtm.start': new Date().getTime(),event:'gtm.js'}
);var f=d.getElementsByTagName(s)[0],
j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
'https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
})(window,document,'script','dataLayer','GTM-WNP7MLF');</script>
<!-- End Google Tag Manager -->
  --
  my $gtm2 = <<"  --";
<!-- Google Tag Manager (noscript) -->
<noscript><iframe src="https://www.googletagmanager.com/ns.html?id=GTM-WNP7MLF"
height="0" width="0" style="display:none;visibility:hidden"></iframe></noscript>
<!-- End Google Tag Manager (noscript) -->
  --
  my $title = shift;
  my $title_stripped = $title;
  $title_stripped =~ s/<[^>]+>/ /g;
  print
  qq@Content-type: text/html\nExpires: Thu, 13 Dec 2007 20:50:00 GMT\n\n@,
  qq@<html lang="en">\n@,
  qq@<head>\n@,
  qq@<title>Ocean Color Browse : $title_stripped</title>\n@,
  $gtm1,
  qq@<meta http-equiv="Content-Style-Type" content="text/css">\n@,
  qq@<style type="text/css">\n@,
  qq@td     { line-height: 100%; font-size: 9pt; }\n@,
  qq@th     { line-height: 100%; font-size: 9pt; }\n@,
  qq@select { line-height: 100%; font-size: 8pt; }\n@,
  qq@#ticker { position: relative; }\n@,
  qq@#ticker_head { color: #080; font-size: 150%; font-weight: bold }\n@,
  qq@#ticker_body { display: none; position: absolute; top: -10px; left; -10px; }\n@,
  qq@#ticker:hover>#ticker_body { display: block; width: 400px; background-color: white; font-size: 120%; padding: 1em; border: 3px solid #080; line-height: 1.5; }\n@,
  qq@</style>\n@,
  qq@<script language="JavaScript1.1">\n@,
  qq@<!-- Begin\n@,
  qq@function helpwin(context) {\n@,
  qq@w=window.open('$thisScript?sub=help&hlp='+context,'OCBrowseHelp',@,
  qq@'scrollbars=yes,width=540,height=600');w.focus()}\n@,
  qq@// End -->\n@,
  qq@</script>\n@,
  qq@</head>\n@,
  qq@<body bgcolor="#ffffff">\n@,
  $gtm2,
  qq@<center>\n@;
}

sub tail{
  print
  qq@<hr />\n@,
  qq@<a href="/"><img src="/icons/oceancolor_icon.png"\n@,
  qq@width="371" height="50" border="0" alt="OceanColor Web"></a>\n@,
  qq@<hr />\n@,
  qq@<address>\n@,
  qq@<a href="/staff/gene/">gene carl feldman</a>\n@,
  qq#(<a href="mailto:gene.c.feldman\@nasa.gov">#,
  qq#gene.c.feldman\@nasa.gov</a>)\n#,
  qq@(301) 286-9428\n@,
  qq@</address>\n@,
  qq@</center>\n@,
  qq@</body>\n@,
  qq@</html>\n@;
}

sub daynumber{
  my $datestr = shift;
  my ($d,$m,$y) = $datestr =~ /(\d{1,2})([A-Za-z]{3})(\d{4})/;
  $m = lc $m;
  my %mon = qw/jan 0 feb 1 mar 2 apr 3 may  4 jun  5
               jul 6 aug 7 sep 8 oct 9 nov 10 dec 11/;
  defined $mon{$m} or return undef;
  timegm(0,0,0,$d,$mon{$m},$y-1900)/86400;
}

sub monstr{
  my $day = shift;
  my @m = qw/January February March April May June July
             August September October November December/;
  $m[(gmtime(86400*$day))[4]];
}

sub year{
  my $day = shift;
  (gmtime(86400*$day))[5] + 1900;
}

sub period2str{
  my ($per,$day,$wantlongstr) = @_;

  my @mon = qw/January February March April May June July
               August September October November December/;
  my @wday = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;

  my %ranges = perdaytoranges($per,$day);
  my @ranges = ();
  foreach my $s (sort {$a <=> $b} keys %ranges){
    $s = launder($s);
    my $e = $s + $ranges{$s} - 1;
    push @ranges,[$s,$e];
  }

  my ($start,$end) = @{$ranges[0]};
  my ($mday,$mon,$year,$wday) = (gmtime($start*86400))[3..6];
  $year += 1900;

  my $string;
  if($per eq 'CU'){
    if(defined $wantlongstr and $wantlongstr){
      my ($md,$mo,$ye,$wd) = (gmtime($end*86400))[3..6];
      $ye += 1900;
      $string = "$wday[$wday], $mday $mon[$mon] $year through " .
                "$wday[$wd], $md $mon[$mo] $ye";
    }
    else{
      $string = "Entire mission";
    }
  }
  elsif($per eq 'YR'){
    $string = "$year";
  }
  elsif($per eq 'MO'){
    if(defined $wantlongstr and $wantlongstr){
      $string = "$mon[$mon] $year";
    }
    else{
      $string = substr $mon[$mon],0,3;
    }
  }
  elsif($per eq '8D'){
    if(defined $wantlongstr and $wantlongstr){
      my $n = $end - $start + 1;
      $string = "$n-day period beginning $wday[$wday], $mday $mon[$mon] $year";
    }
    else{
      my ($md,$mo,$ye,$wd) = (gmtime($end*86400))[3..6];
      $string = sprintf "%s %d - %s %d",substr($mon[$mon],0,3),$mday,
                                        substr($mon[$mo] ,0,3),$md;
    }
  }
  elsif($per eq 'DAY'){
    if(defined $wantlongstr and $wantlongstr){
      $string = "$wday[$wday], $mday $mon[$mon] $year";
    }
    else{
      $string = sprintf "%s %d",substr($mon[$mon],0,3),$mday;
    }
  }
  elsif($per eq 'CSTM'){
    if(@ranges == 1){
      my ($md,$mo,$ye) = map launder($_),(gmtime($end*86400))[3..5];
      $ye += 1900;
      $string = sprintf
      "$mday %s $year through $md %s $ye",
      substr($mon[$mon],0,3),substr($mon[$mo],0,3);
    }
    elsif(@ranges > 1){
      my $e = $ranges[$#ranges][1];
      #my ($md,$mo,$ye) = (gmtime($e*86400))[3..5];
      my ($md,$mo,$ye) = map launder($_),(gmtime($end*86400))[3..5];
      $ye += 1900;
      $string = sprintf
      "Multiple date ranges starting $mday %s $year and ending $md %s $ye",
      substr($mon[$mon],0,3),substr($mon[$mo],0,3);
    }
    if(defined $wantlongstr and $wantlongstr){
      $string .= '<br />(mission composite displayed)';
    }
  }
  else{
    $string = undef;
  }
  $string;
}

sub prev_next_up{
  my ($per,$day,$mon) = @_;

  my $prev = $button{left_off};
  my $next = $button{right_off};
  my $up   = $button{up_off};

  # The Previous, Next, and Up buttons are non-functional
  # for Cumulative (CU) and Custom (CSTM) time periods.
  $per =~ /^C/ and return($prev,$next,$up);

  my $m = '';
  $per eq 'YR' and $m = "&mon=$mon";
  my $state = state_info('url','per','day','mon');
  my($start,$end) = period_bounds($per,$day);
  if($start > $mission_start){
    my $d = (period_bounds($per,$start - 1))[0];
    $prev = $button{left_on};
    $prev =~ s/alt=""/alt="Previous $per{$per}"/;
    $prev = qq@<a href="$thisScript?per=$per&day=$d$m&$state">$prev</a>@;
  }
  if($end < $mission_end){
    my $d = $end + 1;
    $next = $button{right_on};
    $next =~ s/alt=""/alt="Next $per{$per}"/;
    $next = qq@<a href="$thisScript?per=$per&day=$d$m&$state">$next</a>@;
  }

  # Sorting by the second character of the period identifiers
  # puts them in order -- shortest to longest.  It's a coincidence,
  # but it's one way of giving the proper period to the up arrow.
  my @per = sort { substr($a,1,1) cmp substr($b,1,1) } keys %per;
  until($per eq shift @per or @per == 0){}
  my $p = shift @per;
  if(defined $p){
    my $m = '';
    $p eq 'YR' and $m = "&mon=$mon";
    $up = $button{up_on};
    $up =~ s/alt=""/alt="Change to $per{$p} period"/;
    $up = qq@<a href="$thisScript?per=$p&day=$day$m&$state">$up</a>@;
  }

  ($prev,$next,$up);
}

sub highlight{
  my $hlt = shift;

  # If user's search area is the entire globe, highlighting is not an option.
  return($button{hlt_na}) if $in{all} == 1;

  my $state = state_info(qw/url hlt/);
  my $button;
  if($hlt){
    $button = qq@<a href="$thisScript?$state">$button{hlt_on}</a>@;
  }
  else{
    $button = qq@<a href="$thisScript?hlt=1&$state">$button{hlt_off}</a>@;
  }
  $button;
}

sub insitu_map{
  my $path = shift;
  my $wid  = shift;
  my $hgt  = shift;
  my @insitu = grep $_->{x} != -999, @_ or return '';

  # Only keep one in situ record for each dataid since all records
  # having the same dataid should also have the same coordinates.
  my %dataid;
  @insitu = grep $dataid{$_->{dataid}}++ == 0, @insitu;

  my $radius = 8;

  # Translate coordinates for images that have been rotated
  # or otherwise reoriented.
  my $tran;
  my $file = (split /\//,$path)[-1];
  if($file =~ /^C/){     # CZCS
    $tran = sub { ($_[0], $hgt - $_[1] - 1); };
  }
  else{
    my ($start_lat,$end_lat)
    = `$ncdump $path` =~
       / Start(?:\s|_)Center(?:\s|_)Latitude\s*=\s*([-0-9.]+).*
         End(?:\s|_)Center(?:\s|_)Latitude  \s*=\s*([-0-9.]+)  /sx;
    if($start_lat < $end_lat){
      $tran = sub { ($wid - $_[0] - 1, $hgt - $_[1] - 1); };
    }
    else{
      $tran = sub { @_ };
    }
  }

  my $map = <<"  --";
  <map name="insitu_map">
  --

  foreach my $i (@insitu){

    my ($x,$y) = &$tran($i->{x},$i->{y});

    $map .= <<"    --";
    <area shape="circle" coords="$x,$y,$radius"
    onMouseOver="document.getElementById('lat.$i->{dataid}').setAttribute('bgcolor', 'yellow'); document.getElementById('lon.$i->{dataid}').setAttribute('bgcolor', 'yellow')"
    onMouseOut="document.getElementById('lat.$i->{dataid}').setAttribute('bgcolor', '#ccccff'); document.getElementById('lon.$i->{dataid}').setAttribute('bgcolor', '#ccccff')" />
    --

  }

  $map .= <<"  --";
  </map>
  --
}

sub insitu_table{
  my @insitu = @_ or return '';

  my $wavelengths_present = grep defined $_->{wavelength}, @insitu;
  my $outside_the_scene   = grep $_->{x} == -999         , @insitu;
  my $chl                 = grep $_->{name} eq 'chlor_a' , @insitu;
  my $rrs                 = grep $_->{name} eq 'Rrs'     , @insitu;
  my $kd                  = grep $_->{name} eq 'Kd'      , @insitu;
  my $wt                  = grep $_->{name} eq 'wt'      , @insitu;
  my $sal                 = grep $_->{name} eq 'sal'     , @insitu;

  my $cs = $wavelengths_present ? 8 : 7;

  my $table = <<"  --";
  <table style="background: #ccccff">
  <tr><th colspan="$cs">In Situ Data Records</th></tr>
  <tr>
  <th>Date</th>
  <th>Time</th>
  <th>Latitude</th>
  <th>Longitude</th>
  <th>Cruise</th>
  --

  $table .= <<"  --" if $wavelengths_present;
  <th>Wavelength</th>
  --

  $table .= <<"  --";
  <th>Name</th>
  <th>Value</th>
  </tr>
  --

  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;

  @insitu = sort { $a->{date_time} <=> $b->{date_time} } @insitu;

  my @group;
  my $group_index = -1;
  my $prevdataid = -1;
  foreach my $i (@insitu){
    if($i->{dataid} != $prevdataid){
      $group_index++;
      $prevdataid = $i->{dataid};
    }
    push @{$group[$group_index]},$i;
  }

  foreach my $g (@group){
    my $rs = @$g;	# the number of rows in this group
    $rs = $rs > 1 ? qq@ rowspan="$rs"@ : '';
    my $first = 1;
    foreach my $i (@$g){
      my $flagcolor = '';
      my $aster = '';
      if($i->{x} == -999){
        $flagcolor = ' style="background: #ffcccc"';
        $aster = '* ';
      }
      my @t = gmtime $i->{date_time};
      my $date = "$t[3] $mon[$t[4]] @{[ $t[5] + 1900 ]}";
      my $time = sprintf "%02d:%02d:%02d UT", @t[2,1,0];

      $table .= <<"      --";
      <tr valign="top" align="center"$flagcolor>
      --

      $table .= <<"      --" if $first;
      <td$rs>$date</td>
      <td$rs>$time</td>
      <td id="lat.$i->{dataid}"$rs>$aster$i->{lat}</td>
      <td id="lon.$i->{dataid}"$rs>$aster$i->{lon}</td>
      <td$rs>$i->{cruise}</td>
      --

      $first = 0;

      $table .= <<"      --" if $wavelengths_present;
      <td>$i->{wavelength}</td>
      --

      $table .= <<"      --";
      <td>$i->{name}</td>
      <td>$i->{value}</td>
      </tr>
      --
    }
    $table .= <<"    --";
    <tr><td colspan="$cs"></td></tr>
    --
  }
  if($outside_the_scene){
    $table .= <<"    --";
    <tr style="background: #ffcccc">
    <td colspan="$cs">
    * These records are just outside of the scene.
    </td>
    </tr>
    --
  }

  $table .= <<"  --";
  <tr align="left">
  <th>Name</th><th colspan="@{[$cs-4]}">Meaning</th><th colspan="3">Units</th>
  </tr>
  --

  $table .= <<"  --" if $chl;
  <tr align="left">
  <th>chlor_a</th>
  <td colspan="@{[$cs-4]}">chlorophyll a concentration</td>
  <td colspan="3">mg / m^3</td>
  </tr>
  --
  $table .= <<"  --" if $rrs;
  <tr align="left">
  <th>Rrs</th>
  <td colspan="@{[$cs-4]}">remote sensing reflectance</td>
  <td colspan="3">1 / sr</td>
  </tr>
  --
  $table .= <<"  --" if $kd;
  <tr align="left">
  <th>kd</th>
  <td colspan="@{[$cs-4]}">diffuse attenuation coefficient</td>
  <td colspan="3">1/m</td>
  </tr>
  --
  $table .= <<"  --" if $wt;
  <tr align="left">
  <th>wt</th>
  <td colspan="@{[$cs-4]}">water temperature</td>
  <td colspan="3">degrees Celsius</td>
  </tr>
  --
  $table .= <<"  --" if $sal;
  <tr align="left">
  <th>sal</th>
  <td colspan="@{[$cs-4]}">salinity</td>
  <td colspan="3">practical salinity units</td>
  </tr>
  --

  $table .= <<"  --";
  </table>
  --
}

sub tc_chl_sst{
  my ($prm,$sen,$dn) = @_;

  my %alt = (
  TC  => 'quasi-true-color',
  CHL => 'chlorophyll',
  SST => 'sea surface temperature (11 micron)',
  SST4=> 'sea surface temperature (4 micron)',
  );

  my $state = state_info('url','prm');

  my %h;
  foreach my $p (qw/TC CHL SST SST4/){
    if($p =~ /^SST/ and defined $sen and $sen !~ /[at]m|v0/){
      # Make the SST buttons inactive if neither MODIS sensor
      # or VIIRS is selected (either explicitly or by default).
      $h{$p} = $button{"${p}_off"};
    }
    elsif($p =~ /^$prm$/i){
      # If the button's parameter is already displayed, then
      # make the button for this parameter inactive.
      $h{$p} = $button{"${p}_off"};
    }
    elsif($p =~ /^CHL/ || $p =~ /^TC/ and $dn !~ /D/){
      # There are no chlorophyll or true color
      # browse products for nighttime data.
      $h{$p} = $button{"${p}_off"};
    }
    elsif($p =~ /^SST4/ and $dn !~ /N/ || $sen =~ /^v0$/ ){
      # There are no 4 micron SST
      # browse products for daytime data nor for VIIRS data.
      $h{$p} = $button{"${p}_off"};
    }
    else{
      # Otherwise, present the active version of the button and
      # specify the hyperlink.
      $h{$p} =
      qq@<a href="$thisScript?$state&prm=$p">$button{"${p}_on"}</a>@;
    }
  }
  ($h{TC},$h{CHL},$h{SST},$h{SST4});
}

sub ticker{
  # This is no longer implemented as a scrolling ticker, but it used to be,
  # hence the name.
  my $ticker = '';
  if(!defined $in{sen} or $in{sen} =~ /me[fr]r/ or $in{sen} eq ''){
    $ticker = join ' ', split ' ', <<"    --";
    <div id="ticker">
      <div id="ticker_head">
        MERIS Reprocessing Information
      </div>
      <div id="ticker_body">
        MERIS data existing at the OB.DAAC are from the 3<sup>rd</sup>
        reprocessing of the instrument; however, a more current dataset is
        available. To obtain those data, please visit ESA’s
        <a href="https://meris-ds.eo.esa.int/oads/access/">Online
        Dissemination Service for MERIS</a>.
        To continue with data from the 3<sup>rd</sup> reprocessing,
        search below.
      </div>
    </div>
    --
  }
  $ticker;
}

sub region_select{
  opendir D,$qbinsetdir or die qq@Could not open directory, "$qbinsetdir". @,
                               qq@($!) Died@;
  my @rgn = sort grep s/\.txt$//, readdir D;
  closedir D;
  my %r;
  if(defined $in{rgn}){
    my @r = split /@/, $in{rgn};
    grep $r{$_}++ && 0, @r;
  }
  my $html = qq@<select name="rgn" size="11" multiple="true">\n@;
  my $r;
  if(defined $in{rgn}){
    # Put any selected regions at the top of the list
    # where the user is less likely to miss them.
    foreach $r (@rgn){
      if(defined $r{$r}){
        $html .= qq@<option selected>$r</option>\n@;
      }
    }
  }
  # Now print out the unselected regions in the list.
  foreach $r (@rgn){
    unless(defined $r{$r}){
      $html .= qq@<option>$r</option>\n@;
    }
  }
  $html .= qq@</select>\n@;
}

sub write_aoi_map{
  my ($id,@qbins) = @_;
  my $img   = "$tmpdir/browse_aoi.$id";
  my $qbinf = "$tmpdir/browse_qbins.$id";
  my ($n,$s,$w,$e,$clat,$clon) = qbins2latlonbounds(@qbins);

  # Save the qbin list for possible use later in highlighting
  # browse images.
  open(F,'>',$qbinf) or warn "Could not create $qbinf ($!)";
  foreach (sort {$a <=> $b} @qbins){
    print F "$_\n";
  }
  close F;

  # Make the circumscribed box boundaries visible to later parts
  # of the script. Only keep two decimal places.
  $in{cn} = sprintf "%.2f",$n;
  $in{cs} = sprintf "%.2f",$s;
  $in{cw} = sprintf "%.2f",$w;
  $in{ce} = sprintf "%.2f",$e;

  # Use nearest degree because pscoast sometimes pukes
  # when given too many decimal places.
  $clon   = sprintf "%.0f",$clon;

  $e += 360 if $w > $e;
  my $ww = $clon - 180;
  my $ee = $clon + 180;

  # I need to CD here so GMT is able to write out its .gmtcommands file.
  chdir $tmpdir or die "Could not chdir to $tmpdir ($!)";

  # Set the background color.
  my @c = ($gmt,'set','PS_PAGE_COLOR','255/255/255');
  system(@c) == 0 or die qq#Error executing command, "@c" ($!)#;

  # Choose a projection.
  my ($proj,$dimen);
  my $squish = '';
  if( $n > -20 && $n < 20 || $s > -20 && $s < 20 || $n > 20 && $s < -20 and $e - $w > 160){
    # Use a squished Mollweide projection.
    $proj = "-R$ww/$ee/-90/90 -JW$clon/4";
    $dimen = "400x200";
    $squish = " -resize 200x200!";
  } else{
    # Use an orthographic projection.
    $proj = "-R$ww/$ee/-90/90 -JG$clon/$clat/2";
    $dimen = "200x200";
  }

  # Make the background map in grayscale.
  my $c =
  "$gmt pscoast -P -X0 -Y0 $proj -G204 -S153 -Dc --PROJ_LENGTH_UNIT=i" .
  "|$gs -g$dimen - > $img.gray.ppm";
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  # Make it again in color.
  $c =
  "$gmt pscoast -P -X0 -Y0 $proj -G153/51/0 -S0/153/255 -Dc --PROJ_LENGTH_UNIT=i" .
  "|$gs -g$dimen - > $img.color.ppm";
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  # Make the area of interest mask.
  $c =
  "$qbins2outlines 2" .
  "|$gmt psxy $proj -P -X0 -Y0 -G0 -: -L --PROJ_LENGTH_UNIT=i" .
  "|$gs -g$dimen - > $img.mask.ppm";
  open P,"|$c" or die qq@Error opening pipe to command, "$c" ($!)@;
  foreach (@qbins){
    print P "$_\n";
  }
  close P or die qq@Error executing command, "$c" ($!)@;

  # Combine the color and grayscale image using the mask.
  $c = join ' ', split ' ', <<"  --";
  $convert $img.mask.ppm -negate $img.color.ppm +swap
  -compose CopyOpacity -composite $img.gray.ppm +swap
  -compose SrcOver -composite $squish -transparent white $img.png
  --
  system($c) == 0 or die qq@Error executing command, "$c" ($!)@;

  # Remove temporary files.
  unlink "$img.gray.ppm","$img.color.ppm","$img.mask.ppm";
}

sub qbins2latlonbounds{
  my @qbins = @_;

  my @cmd = ($qbins2outlines,1);

  local($") = "\n";
  my $input = "@qbins\n";
  my ($output,$err);

  run3 \@cmd,\$input,\$output,\$err or die qq@Error running $qbins2outlines ($!)@;

  my ($n,$s,$w,$e,$clat,$clon) = map launder($_), split " ",$output;
}

# The following function used to serve as a wrapper for the
# qbins2grans program; it is now an interface to a mysql database.
sub qbins2list{
    my ($sen,$per,$day,$frc,$rgn,$dnm,$is,$qid) = @_;
    my $dnflg = undef;
    my @dnfs = split /@/, $dnm;
    if ($#dnfs == 0){
	$dnflg = '1' if $dnfs[0] eq 'N';
	$dnflg = '0' if $dnfs[0] eq 'D';
    }
    $dnflg = 'NULL' if not defined $dnflg;

    my $quadbincnt = `wc -l <$tmpdir/browse_qbins.$qid`;
    chop $quadbincnt;

    $frc = 0. if !defined $frc || $frc < 0;
    $frc = 1 if $frc > 1;
    my $cnt = 0;
    if ($frc > 0){$cnt = 1}
    my @list;

    # Search metadata for selected sensors
    # or for all sensors if none are selected.
    my @sensors = split /@/, $sen;
    @sensors =  keys %sen if @sensors == 0;

    my %gran = ();

    my %ranges = perdaytoranges($per,$day);
    my @ranges = ();
    my $ndays = 0;
    foreach my $s (sort {$a <=> $b} keys %ranges){
	my $e = $s + $ranges{$s} - 1;
	push @ranges,[$s,$e];
	$ndays += ($e-$s);
    }

    foreach my $s (@sensors){
      my ($time,$sta,$wid,$hgt,$senchar,$fcnt,$hidevalue);
      my $sen_id = $sen{$s}[3];
      my $station = 'NULL';
      $station = "'" . $sen{$s}[4] . "'" if ($s =~ /^(sw|me|s3)/);
      my $insitu = 'NULL';
      my ($dataday,$sceneID);

      if($is && $is eq 'yes'){	# Only select scenes with in situ matchups.
        $insitu = 1;
      }
      foreach my $r (@ranges){
        my $sd = $r->[0];
        my $ed = 'NULL';
        if($r->[0] != $r->[1]){
		  $ed  = $r->[1];
        }
        my $sth = $dbh->prepare("call locate_scenes_new($sen_id,$station,$sd,$ed,$dnflg,$insitu,$cnt,1,\@s)");
        $sth->execute;
        $sth->bind_columns(\$time,\$sta,\$wid,\$hgt,\$senchar,\$fcnt,\$hidevalue);
        while ($sth->fetch){

          my $ntime = yyyydddhhmmss($time);
          my $filefrac = $fcnt/$quadbincnt;
          $senchar = "S3A" if ($sen_id == 29);
          $senchar = "S3B" if ($sen_id == 36);
          unless (exists $gran{"$senchar $time $sta"}){
              if ($frc > 0 && $filefrac >= $frc){
                  push @list,[$senchar,$ntime,$sta,$wid,$hgt,$hidevalue];
                  $gran{"$senchar $time $sta"}++;
              } elsif($frc == 0){
                  push @list,[$senchar,$ntime,$sta,$wid,$hgt,$hidevalue];
                  $gran{"$senchar $time $sta"}++;
              }
          }
         
        }
        $sth->finish;
      }
    }
    sort {   $b->[1] cmp $a->[1]
          or $b->[0] cmp $a->[0]
          or $b->[2] cmp $a->[2] } @list;
}

sub make_qbtable{
  # The argument to this subroutine ($qid) is expected to be the
  # same ID that is bandied about as $in{id} in other places.
  my $qid = shift;

  my $table_name = "qbins";

  open(QBINS,'<',"$tmpdir/browse_qbins.$qid");
  while(<QBINS>){
    chomp;
    my $qb = $_;
    $dbh->do("call temp_qbin($qb)");
  }

  $table_name;
}

# The following function used to serve as a wrapper for the
# datadays2grans program.  Now it is an interface to the
# MySQL database that replaced the original metadata files.
sub datadays2list{
  my ($sen,$per,$dnm,$is,$day) = @_;
  my $dnflg = undef;
  my @dnfs = split /@/, $dnm;
  if ($#dnfs == 0){
	$dnflg = '1' if $dnfs[0] eq 'N';
	$dnflg = '0' if $dnfs[0] eq 'D';
  }
  $dnflg = 'NULL' if not defined $dnflg;
  my %ranges = perdaytoranges($per,$day);

  # Search metadata for selected sensors
  # or for all sensors if none are selected.
  my @sensors = split /@/, $sen;
  @sensors =  keys %sen if @sensors == 0;
  
  my @ranges = ();
  foreach my $s (sort {$a <=> $b} keys %ranges){
    my $e = $s + $ranges{$s} - 1;
    push @ranges,[$s,$e];
  }
  my %gran;
  foreach my $s (@sensors){
    
    my $insitu = 'NULL';
    my $sen_id = $sen{$s}[3];
    my $station = "'" . $sen{$s}[4] . "'";
    my ($time,$dataday,$senchar,$sceneID,$sta,$wid,$hgt,$hidevalue);

    if($is eq 'yes'){	# Only select scenes with in situ matchups.
      $insitu = 1;
    }

    foreach my $r (@ranges){
      my $sd = $r->[0];
      my $ed = 'NULL';
      if($r->[0] != $r->[1]){
		$ed  = $r->[1];
      }
  
      #my $sqlcmd = "call locate_scenes_new($sen_id,$station,$sd,$ed,$dnflg,$insitu,0,0,\@s)";
      my $sth = $dbh->prepare("call locate_scenes_new($sen_id,$station,$sd,$ed,$dnflg,$insitu,0,0,\@s)");
      $sth->execute;
      $sth->bind_columns(\$time,\$sta,\$wid,\$hgt,\$senchar,\$sceneID,\$hidevalue);
      while ($sth->fetch){
        unless (defined $sta){$sta = 'LAC';}
		  $senchar = "S3A" if ($sen_id == 29 );
		  $senchar = "S3B" if ($sen_id == 36 );
        $gran{"$senchar $time $sta"} = [$wid,$hgt,$hidevalue];
      }
      $sth->finish;
    }
  }
  my @list;
  my ($k,$v);
  while(($k,$v) = each %gran){
    my ($sen,$time,$sta) = split " ",$k;
    $time = yyyydddhhmmss($time);
    my ($wid,$hgt,$hidevalue) = @$v;
    push @list,[$sen,$time,$sta,$wid,$hgt,$hidevalue];
  }
  sort {   $b->[1] cmp $a->[1]
        or $b->[0] cmp $a->[0]
        or $b->[2] cmp $a->[2] } @list;
}

sub write_list{
  my ($id,@list) = @_;
  my $file = "$tmpdir/browse_list.$id.txt";
  open F,">$file" or die qq@Could not create file, "$file". ($!) Died@;
  foreach (@list){
    print F "@$_\n";
  }
  close F;
}

sub read_list{
  my $id = shift;
  my $file = "$tmpdir/browse_list.$id.txt";
  my @list;
  unless(-e $file){
    head("Search results expired");
    print
    qq@</center>\n@,
    qq@<p>\n@,
    qq@The state information about your browse session has expired\n@,
    qq@from our system.  You will need to\n@,
    qq@<a href="$thisScript">re-enter</a> your search criteria\n@,
    qq@if you wish to continue browsing.\n@,
    qq@</p>\n@,
    qq@<center>\n@;
    &tail;
    exit;
  }
  open F,'<',$file or die qq@Could not open file, "$file". ($!) Died@;
  while(<F>){
    push @list,[split ' '];
  }
  close F;
  return @list;
}

sub limit_volume{
  my ($levels,$thresh,$list,
      $unavail,$allowed,$restricted,$denied,
      $allowed_vol,$restricted_vol,$denied_vol) = @_;

  $$allowed_vol    = 0;
  $$restricted_vol = 0;
  $$denied_vol     = 0;

  @$unavail        = ();
  @$allowed        = ();
  @$restricted     = ();
  @$denied         = ();

  my %fshash = ();

  my $volume = 0;

  foreach my $l (@$list){
    my ($msn,$time) = gran2msntime($l);

    foreach my $v (@$levels){

      $v eq 'L1A' and $v = 'L1';
      $v eq 'L2'  and $v = 'L2OC';

      my $f = msntimetyp2name($msn,$time,$v) or next;

      # Get the hide value for this scene.
      my $hide;
      my $hidecmd =
      "SELECT s.hide from scene_info s, sensors m " .
      "where s.sen_id = m.sen_id " .
      "and m.sen_id = $sen{$msn}->[3] "  .
      "and s.start_time = $time "  .
      "and s.station = '$l->[2]'";

      my $sth = $dbh->prepare($hidecmd);
      $sth->execute();

      while (my $fetchash = $sth->fetchrow_hashref){
	   $hide = $fetchash->{hide};
      }
      $sth->finish;
      my $new_f = $f;

      if($sen{$msn}->[3] == 19 && $new_f  =~ /.bz2/)
      {
         $new_f =~ s/\.bz2//g;
      }

      my $ordcmd = "select size from file_info where basename = '$new_f'";
      $sth = $dbh->prepare($ordcmd);
      $sth->execute();
      my $siz = 0;
      while (my $fetchash = $sth->fetchrow_hashref){
        next if($hide == 1 || $hide == 3 || $hide == 4 and $new_f =~ /L1(A|B)/
             or $hide  > 1                             and ($new_f =~ /L2/ && $new_f !~ /SST/)
             or $hide  > 3                             and $new_f =~ /L0/     );

        $siz = $fetchash->{size};
      }
      $sth->finish;
      # Put the scene in the appropriate category.

      if ($siz == 0){
        push @$unavail,$new_f;
        next;
      }
      $volume += $siz;
      if($volume <= $thresh){
        push @$allowed,$new_f;
        $$allowed_vol = $volume;
      }
      else{
        push @$denied,$new_f;
        $$denied_vol += $siz;
      }
    }
  }
}

sub list2names{
  my $list = shift;
  my %names;
  foreach my $l (@$list){
    $names{"$l->[0]$l->[1].L1A_$l->[2]"}++;
  }
  sort keys %names;
}

sub yyyydddhhmmss{
  my $t = shift;
  my ($s,$m,$h,$y,$d) = (gmtime $t)[0,1,2,5,7];
  sprintf "%04d%03d%02d%02d%02d",$y+1900,$d+1,$h,$m,$s;
}

sub t2isodate{
  my $t = shift;
  my $i = int $t;
  my $f = $t - $i;
  my ($s,$m,$h,$d,$o,$y) = (gmtime $i)[0..5];
  $s = $f > 0 ? sprintf("%06.3f",$s + $f) : sprintf("%02d",$s);
  sprintf "%04d-%02d-%02d %02d:%02d:%s",$y+1900,$o+1,$d,$h,$m,$s;
}

sub ymdhms2yd{
  my $yyyymmddhhmmss = shift;
  my ($y,$mo,$d,$h,$m,$s) = unpack "a4a2a2a2a2a2",$yyyymmddhhmmss;
  my $gmtime = timegm($s,$m,$h,$d,$mo-1,$y-1900) ;
  ($s,$m,$h,$y,$d) = (gmtime $gmtime)[0,1,2,5,7];
  sprintf "%04d%03d%02d%02d%02d",$y+1900,$d+1,$h,$m,$s;
}

sub ydhms2ymdhms{
  my $yyyydddhhmmss = shift;
  my ($y,$d,$h,$m,$s) = $yyyydddhhmmss =~ /^(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)/;
  my $gmtime = timegm($s,$m,$h,1,0,$y-1900) + 86400*($d - 1);
  my ($day,$mon) = (gmtime $gmtime)[3,4];
  sprintf "$y%02d%02d$h$m$s",$mon+1,$day;
}

sub ydhms2t{
  my $yyyydddhhmmss = shift;
  my ($y,$d,$h,$m,$s) = $yyyydddhhmmss =~ /^(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)/;
  my $t = timegm($s,$m,$h,1,0,$y-1900) + ($d - 1)*86400;
}

sub isodate2t{
  my $isodate = shift;
  my ($y,$mo,$d,$h,$m,$s);
  if ($isodate =~ /T/){
    ($y,$mo,$d,$h,$m,$s) = unpack "a4a2a2x1a2a2a2", $isodate;
  } elsif ($isodate =~ /-\d\d \d\d:/) {
    ($y,$mo,$d,$h,$m,$s) = unpack "a4x1a2x1a2x1a2x1a2x1a2", $isodate;
  } else {
    ($y,$mo,$d,$h,$m,$s) = unpack "a4a2a2a2a2a2", $isodate;
  }
  my $t = timegm($s,$m,$h,$d,$mo-1,$y-1900);
}

sub yyyyddd{
  my $day = shift;
  my ($y,$d) = (gmtime 86400*$day)[5,7];
  sprintf "%04d%03d",$y+1900,$d+1;
}

sub yyyyddd2date{
  my $yyyyddd = shift;
  my ($y,$d) = unpack "a4a3",$yyyyddd;
  my $t = timegm(0,0,0,1,0,$y-1900) + ($d - 1)*86400;
  my @mon = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
  my @t = gmtime($t);
  "$t[3]$mon[$t[4]]$y";
}

sub mission_dates{
  # Get the starting and ending dates for the combined selected sensors
  # by reading the appropriate DayIndex files.

  # The default if no sensors are specified is that all sensors
  # specified in the globally visible hash, %sen, are selected.

  my ($sen, @sen) = (shift);
  if(defined $sen and $sen){
    @sen = split /@/, $sen;
  } else {
	@sen = keys %sen;
  }

  my @uniqsen = uniq @sen;
  my ($start,$end) = (0,0);
  foreach my $s (@uniqsen){
    my $sen_id = $sen{$s}[3];

    my $mdselect = "select first_light, mission_end from sensors where sen_id = $sen_id limit 1";
    my $sth = $dbh->prepare($mdselect);
    $sth->execute;
    my ($firstlight,$missionend);
    $sth->bind_columns(\$firstlight,\$missionend);
    $sth->fetch;
    $sth->finish;

    my $fldt = &date2dataday($firstlight);
    my $medt;
    if ($missionend == 'NULL') {
      my @dt = gmtime $invocation_time;
      my $dt = sprintf "%04d-%03d", $dt[5]+1900,$dt[7]+1;
      $medt = &date2dataday($dt);
    } else {
      $medt = &date2dataday($missionend);
    }
    if ($start == 0){
      $start = $fldt;
    } else {
      $start = ($fldt > $start) ? $start : $fldt;
    }
    if ($end   == 0){
      $end   = $medt;
    } else {
      $end   = ($medt < $end  ) ? $end   : $medt;
    }
  }
  return ($start,$end);
}

sub gransperday{
  my ($sen,$sday,$num) = @_;
  my %gpd;
  my @gpd;
  my @sen;
  my @typ;
  if(defined $sen and $sen){
    @sen = map {$sen{$_}[3]} split /@/, $sen;
    @typ = map {$sen{$_}[4]} split /@/, $sen;
  }
  else{
    @sen = map {$sen{$_}[3]} keys %sen;
    @typ = map {$sen{$_}[4]} keys %sen;
  }
  
  my @uniqsen = uniq @sen;
  my @uniqtyp = uniq @typ;

  my $senstr = join("," , @uniqsen);

  my $gransperdaycmd= <<"--";
SELECT g.data_day,count(g.scene_id) from webstuff.granules g, webstuff.scene_info  s, webstuff.sensors i
where g.scene_id = s.scene_id
and s.sen_id = i.sen_id
and s.hide = 0
and g.data_day between $sday and @{[$sday+$num]}
and i.sen_id in ($senstr)
--

  my $typstr = "'" . join("','" , @uniqtyp) . "'";
  $gransperdaycmd .= " and s.station in ($typstr) ";

  $gransperdaycmd .= <<"--";
and g.daynight = 0
group by g.data_day
--

  my ($dataday,$count);
  my $sth = $dbh->prepare("$gransperdaycmd");
  $sth->execute;
  $sth->bind_columns(\$dataday,\$count);
  my $i = 0;
  while ($sth->fetch){
	$gpd{$dataday} += $count;
	$i++;
  }
  $sth->finish;
  for (my $day=$sday;$day<$sday+$num;$day++){
	 if (exists $gpd{$day}){push @gpd,$gpd{$day}}
	 else{push @gpd,0}
  }
  @gpd;
}

sub state_info{
  my ($style,@omit) = @_;

  my @state = qw/
  sub per day prm set ndx mon sen hlt
  typ rad frc rgn n s w e all file wid hgt id cul
  ord xn xs xw xe cn cs cw ce xtr dnm is
  /;

  my %omit;
  grep $omit{$_}++, @omit;
  @state = grep !defined $omit{$_}, @state;

  my $state;
  if($style eq 'form'){
    my $s;
    foreach $s (@state){
      if(defined $in{$s} and $in{$s} ne ''){
        $state .= qq@<input type="hidden" name="$s" value="$in{$s}">\n@;
      }
    }
  }
  elsif($style eq 'url'){
    my $s;
    foreach $s (@state){
      if(defined $in{$s} and $in{$s} ne ''){
        my $value = $in{$s};
        $value =~ s/\s+/+/g;	# Replace spaces with pluses
        $state .= '&' if defined $state;
        $state .= qq@$s=$value@;
      }
    }
  }
  else{
    die qq@The state_info subroutine must be called with a first argument @,
        qq@of either "form" or "url".  Died@;
  }
  $state;
}

sub present_criteria{
  my $period = period2str($in{per},$in{day},$in{per} !~ /^C/);
  my $dn = $in{dnm} =~ /D/ ? ($in{dnm} =~ /N/ ? "daytime and nighttime"
                                              : "daytime"              )
                                              : "nightime";
  my $sensor;
  my $type;
  my $insitu;
  my $region;
  my $fraction;
  if(defined $in{dnm}){
    if($in{dnm} =~ /D/){
      $dn = "daytime";
      if($in{dnm} =~ /N/){
        $dn .= " and nighttime";
      }
    }
    elsif($in{dnm} =~ /N/){
      $dn = "nighttime";
    }
    $dn = " ($dn)";
  }
  if(defined $in{sen}){
    $sensor = join ' and ', reverse sort
    map { $sen{$_}[0] } split /@/, $in{sen};
  }
  else{
    $sensor = join ' and ', reverse sort
    map { $sen{$_}[0] } keys %sen;
  }

  if($sensor =~ /SeaWiFS/){
    if(defined $in{typ}){
      $type = $in{typ};
      $type =~ s/@/ and /g;
    }
    else{
      $type = 'GAC and MLAC';
    }
    $type = qq@<b>SeaWiFS Data Types:</b> $type<br />\n@;
  }

  if($sensor =~ /MERIS/){
    if(defined $in{typ}){
      $type = $in{typ};
      $type =~ s/@/ and /g;
    }
    else{
      $type = 'RR and FRS';
    }
    $type = qq@<b>MERIS Data Types:</b> $type<br />\n@;
  }

  if($in{is} && $in{is} eq 'yes'){
    $insitu = qq@<b>Scenes must have coincident in situ data.</b><br />\n@;
  }

  if(defined $in{frc}){
    $fraction = 100 * $in{frc};
  }
  else{
    $fraction = "Any part";
  }
  if(defined $in{all} and $in{all} == 1){
    $region = "entire globe";
    $fraction = "Any part";
  }
  elsif( defined $in{'i.x'}
  and defined $in{'i.y'}
  or ($in{n} ne '' xor $in{s} ne '') and ($in{w} ne '' xor $in{e} ne '')){
    my ($lat,$lon) = &get_lat_lon;
    if(defined $in{rad}){
      $region = "Within $in{rad} km of ";
    }
    else{
      $region = "Within 72 km of ";
    }
    $region .= latstr($lat) . ',' . lonstr($lon);
  }
  elsif(defined $in{rgn}
  or    $in{n} ne '' and $in{s} ne ''
  and   $in{w} ne '' and $in{e} ne ''){
    if(defined $in{rgn}){
      $region = join ' and ',split /@/, $in{rgn};
    }
    if( $in{n} ne '' and $in{s} ne ''
    and $in{w} ne '' and $in{e} ne ''){
      if(defined $region){
        $region .= ' and ';
      }
      $region .= 'region bounded by ' .
      join ' and ',latstr($in{n}),latstr($in{s}),lonstr($in{w}),lonstr($in{e});
    }
  }

  my $map = '';
  if(defined $in{id} and !defined $in{all}){
    $map =
    qq@<img src="$thisScript?sub=aoimap&id=$in{id}"\n@ .
    qq@width="200" height="200" alt=""><br />\n@;
  }

  qq@<p>\n@ .
  qq@<b>Search Criteria</b><br />\n@ .
  qq@<b>Time Period:</b> $period$dn<br />\n@ .
  qq@<b>Sensors:</b> $sensor<br />\n@ .
  $type .
  $insitu .
  qq@<b>Area of Interest:</b> $region<br />\n@ .
  $map .
  qq@<b>Percentage of AOI that swaths must include:</b> $fraction<br />\n@ .
  qq@</p>\n@;
}

sub get_lat_lon{
  my ($lat,$lon);

  if(defined $in{'i.x'}){
    # The user clicked on the map to specify a search location.
    my ($x,$y) = ($in{'i.x'},$in{'i.y'});
    if($in{dnm} =~ /N/ and $in{dnm} !~ /D/){
      # Nighttime-only quicklooks are centered on the 180-degree meridian
      # so I shift the x coordinate by half the width of the map to  make
      # the longitude come out correctly.
      if($x < $l3hgt){ $x += $l3hgt; }
      else           { $x -= $l3hgt; }
    }
    my $inc = 360/$l3wid;
    $lat = 90 - $y*$inc - $inc/2;
    $lon = $x*$inc + $inc/2 - 180;

    # Make the lat and lon global so it can be displayed properly
    # in the list of search criteria.
    $in{n} = $lat;
    $in{w} = $lon;
    $in{e} = $in{s} = '';
  }
  else{
    # The user entered a latitude and a longitude in text form.
    $lat = $in{n} ne '' ? $in{n} : $in{s};
    $lon = $in{w} ne '' ? $in{w} : $in{e};
  }
  $lat >  90 and $lat =  90;
  $lat < -90 and $lat = -90;
  $lon += 360 while($lon < -180);
  $lon -= 360 while($lon >  180);
  ($lat,$lon);
}

sub ordinal{
  my $num = shift;
  $num = int $num;
  if($num  =~ /1[1-9]$/ or $num =~ /[04-9]$/){
    $num .= "th";
  }
  elsif($num =~ /1$/){
    $num .= "st";
  }
  elsif($num =~ /2$/){
    $num .= "nd";
  }
  elsif($num =~ /3$/){
    $num .= "rd";
  }
  $num;
}

sub latstr{
  my $lat = shift;
  sprintf "%.1f%s",($lat<0 ? (-$lat,'S') : ($lat,'N'));
}

sub lonstr{
  my $lon = shift;
  sprintf "%.1f%s",($lon<0 ? (-$lon,'W') : ($lon,'E'));
}

sub deltalon{
  my ($west,$east) = @_;
  $west += 360 while $west < -180;
  $west -= 360 while $west >  180;
  $east += 360 while $east < -180;
  $east -= 360 while $east >  180;
  $west > $east ? 360 + $east - $west : $east - $west;
}

sub orderid{
  # Construct a unique order ID from the system time and the process ID.
  my @a = split /\B/, sprintf "%08lx", $invocation_time;
  my @b = split /\B/, sprintf "%08lx", $$;
  my @o;
  push(@o,pop(@a),shift(@b)) while(@a);
  join "",@o;
}

sub add_commas{
  # Add commas to a number string (e.g. 1357924683 => 1,357,924,683)
  my $num = reverse shift;
  $num =~ s/(...)/$1,/g;
  chop $num if $num =~ /,$/;
  reverse $num;
}

# This subroutine scans a list of interactively selected scenes to
# find the scene that is nearest to the specified position in the
# complete list of search results ($ndx).  The index into the @selected
# array is returned.
sub find_nearest{
  my ($ndx,$selected) = @_;

  if($ndx < $selected->[0][6]){
    return 0;
  }
  elsif($ndx > $selected->[@$selected - 1][6]){
    return @$selected - 1;
  }
  else{
    # I assume that in the majority of cases the list of interactively
    # selected scenes will not be that long, so a linear search of the
    # array should be fast enough.
    for(my $i=0; $i<@$selected; $i++){
      if($ndx == $selected->[$i][6]){
        return $i;
      }
      elsif($ndx < $selected->[$i][6]){
        my $mid = ($selected->[$i][6] + $selected->[$i-1][6])/2;
        if($ndx > $mid){
          return $i;
        }
        else{
          return $i - 1;
        }
      }
    }
  }
}

sub tabulate{

  my ($items,$cols,$order,$size) = @_;

  # Arrange the strings in the array pointed to by $items into a table.
  # Specify how many columns the table should have with $cols.
  # Specify whether the items should be arranged across the rows ("row")
  # or down the columns ("col") with $order.
  # Control the size of the font with $size.

  my $numitems = @$items;

  my ($f1,$f2) = ('','');
  if(defined $size){
    $f1 = qq@<font size="$size">@;
    $f2 = qq@</font>@;
  }

  my $table =
  qq@<table border="1" cellspacing="0" cellpadding="0">\n@;

  if($order eq 'row'){

    my $c = 0;

    foreach my $item (@$items){

      $table .= qq@<tr>\n@ if $c % $cols == 0;

      $table .= qq@<td>$f1$item$f2</td>\n@;

      $c++;

      $table .= qq@</tr>\n@ if $c % $cols == 0;

    }

    my $remainder = $c % $cols;
    if($remainder != 0){
      $table .= qq@<td></td>@ x ($cols - $remainder) . qq@</tr>\n@;
    }
  }
  else{

    my $rows = int(($numitems - 1)/$cols) + 1;
    my $remainder = $numitems % $cols;

    my $r;
    for($r=0; $r<$rows; $r++){

      $table .= qq@<tr>\n@;

      my $c;
      for($c=0; $c<$cols; $c++){

        if($remainder == 0 or $r < $rows - 1 or $c < $remainder){
          my $i = $c*$rows + $r
                - ($remainder>0 && $c>$remainder ? $c-$remainder : 0);
          $table .= qq#<td>$f1@$items[$i]$f2</td>\n#;
        }
        else{
          $table .= qq@<td></td>\n@;
        }

      }

      $table .= qq@</tr>\n@;
    }
  }

  $table .= qq@</table>\n@;
}

# Generate the markup for a menu button.
sub menu_button{

  my $f = shift;

  my $dr = '/data1/www-data/oceancolor.gsfc.nasa.gov/static';

  my ($w,$h) = pngdim("$dr$f");

  qq@<img src="$f" alt="" width="$w" height="$h" border="0">@;
}

# Read the dimensions of a PNG image.
sub pngdim{

  my $f = shift;

  my $pnghdr = "\211PNG\r\n\032\n\0\0\0\015IHDR";

  open F,'<',$f or die qq@Could not open file, "$f" ($!)@;

  my ($hdr,$wh);

  read(F,$hdr,16) == 16
  and $hdr eq $pnghdr
  or die "$f is not a proper PNG image.\n";

  unless(read(F,$wh,8) == 8){
    warn "Error reading $f ($!)";
    close F;
    return ('','');
  }

  close F;

  unpack "NN",$wh;
}

sub ord_checked{
  my ($v,@o) = @_;
  my $checked = '';
  $checked = ' checked' if grep /^$v$/,@o;
  $checked;
}

sub xtr_checked{
  my $v = shift;
  my $checked = '';
  $checked = ' checked' if(defined $in{xtr} and $in{xtr} eq $v);
  $checked;
}

sub msntimetyp2name {
  my ($mission, $time, $type) = @_;
  # Unpack the time.
  my ($ss,$mm,$hh,$dd,$mo,$yyyy,$doy) = (gmtime $time)[0..5,7];
  $yyyy += 1900;
  $doy = sprintf "%03d", $doy + 1;
  ($ss,$mm,$hh,$dd,$mo) = map sprintf("%02d",$_), $ss,$mm,$hh,$dd,$mo+1;

  $dtidstr = join ',',@{$dtids{$mission}};
  my $tdelta = 120;
  my $stime = $time - $tdelta;
  my $etime = $time + $tdelta;
  my $sts = t2isodate($stime);
  my $ets = t2isodate($etime);
  my $sqlcmd =
      "SELECT basename from datastore.files " .
      "where sensor_id = $sen{$mission}[3] ".
      "and start_time between '$sts' and '$ets' " .
      "and data_type_id in ($dtidstr) and (status_flag & 1912) = 0 order by create_date DESC";

  my @rows;
  my $basename;
  if ($mission =~ /swga|swml/){
     my $basenamesearch =  "S${yyyy}${doy}${hh}${mm}${ss}%";
     $sqlcmd = 
      "SELECT basename from datastore.files " .
      "where sensor_id = $sen{$mission}[3] ".
      "and basename like \"$basenamesearch\" ".
      "and data_type_id in ($dtidstr) and (status_flag & 1912) = 0";

  } 
  my $sth = $dbh->prepare($sqlcmd);
  $sth->execute();
  $sth->bind_columns(\$basename);
  while ($sth->fetch){push @rows,$basename}
  $sth->finish;
  
  my $name, @limitrow;

  ########################### (Aqua|Terra)/MODIS #############################
  if ($mission =~ /mod/){
    my $msndir = 'MODISA';
    $msndir = 'MODIST' if ($mission eq 'tmod'); 
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/$msndir/L1BRS/LAC/$yyyy/$doy/" . $limitrow[0] . "_BRS";
      $name =~ s/L1A/L1B/;
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/$msndir/L2BRS/OC/LAC/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'SST') {
      @limitrow = grep /A_MODIS.*SST\./, @rows;
       $name = "/web/data/$msndir/L2BRS/SST/LAC/$yyyy/$mo$dd/" . $limitrow[0] . "_SST_BRS";
      unless (-e $name) {
       @limitrow = grep /SST\./, @rows;
       $name = "/web/data/$msndir/L2BRS/SST/LAC/$yyyy/$doy/" . $limitrow[0] . "_SST_BRS";
      }
    }
    elsif($type eq 'SST4') {
      @limitrow = grep /A_MODIS.*SST4/, @rows;
      $name = "/web/data/$msndir/L2BRS/SST4/LAC/$yyyy/$mo$dd/" . $limitrow[0] . "_SST4_BRS";
      unless (-e $name) {
        @limitrow = grep /SST4/, @rows;
        $name = "/web/data/$msndir/L2BRS/SST4/LAC/$yyyy/$doy/" . $limitrow[0] . "_SST4_BRS";
      }
    }
    elsif($type eq 'L0') {
      @limitrow = grep /PDS/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /IOP/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2SST') {
      @limitrow = grep /A_MODIS.*SST\./, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2SST4') {
      @limitrow = grep /A_MODIS.*SST4/, @rows;
      $name = $limitrow[0]; 
    }
  }
  ############################### CZCS #######################################
  elsif($mission eq 'czcs'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/CZCS/L1BRS/MLAC/$yyyy/$doy/" . $limitrow[0] . "_BRS";
      $name =~ s/_LAC/_MLAC/;
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/CZCS/L2BRS/OC/MLAC/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0] . ".bz2";
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0];
    }
  }
  ############################### GOCI #######################################
  elsif($mission eq 'goci'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/GOCI/L1BRS/$yyyy/$mo$dd/" . $limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /L2/, @rows;
      $name = "/web/data/GOCI/L2BRS/OC/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0] . ".gz";
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /L2/, @rows;
      $name = $limitrow[0]; 
    }
  }
  ############################### HICO #######################################
  elsif($mission eq 'hico'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/HICO/L1BRS/$yyyy/$doy/" . $limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/HICO/L2BRS/OC/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0];
    }
  }
  ############################# HAWKEYE ######################################
  elsif($mission eq 'hawk'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/SE1/L1BRS/L1ARNAV/$yyyy/$mo$dd/" . $limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/SE1/L2BRS/OC/L1ARNAV/$yyyy/$mo$dd/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0];
    }
  }
  ############################### MERIS ######################################
  elsif($mission =~ /mefr|merr/){

    my $etyp = "RR";
    $etyp = "FRS" if ($mission eq "mefr");

    if   ($type eq 'TC') {
      @limitrow = grep /MER_FRS_1|MER_RR__1/, @rows;
      if ($limitrow[0] =~ /^EN1/){
        my $basename = substr($limitrow[0],20,15);
        $name = "/web/data/MERIS/L1BRS/$etyp/$yyyy/$mo$dd/EN1${basename}.L1B_${etyp}_BRS";
      } else {
        $name = "/web/data/MERIS/L1BRS/$etyp/$yyyy/$mo$dd/" . $limitrow[0] . "_BRS";
      }
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/MERIS/L2BRS/OC/FRS/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
      $name =~ s/FRS/RR/ if ($mission eq 'merr');
    }
    elsif($type eq 'L1') {
      @limitrow = grep /MER_FRS_1|MER_RR__1/, @rows;
      if ($limitrow[0] =~ /^EN1/){ 
        $name = $limitrow[0];
      }
      else {
        $name = $limitrow[0] . ".bz2";
      }
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0] . ".bz2";
    }
  }
  ############################### OCTS #######################################
  elsif($mission eq 'octs'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/OCTS/L1BRS/GAC/$yyyy/$doy/" . $limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/OCTS/L2BRS/OC/GAC/$yyyy/$doy/" . $limitrow[0] . "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0]; #. ".Z";
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /IOP/, @rows;
      $name = $limitrow[0]; 
    }
  }
  ############################### S3(AB)/OLCI #################################
  elsif($mission =~ /s3ar|s3af|s3br|s3bf/){
    my $msndir = "S3OLCI";
    my $prefix = "S3A";
    my $etyp = "ERR";
    if ($mission =~ /s3bf|s3br/) {
      $msndir = "S3BOLCI";
      $prefix = "S3B";
    } 
    $etyp = "EFR" if ($mission =~ /s3af|s3bf/); 

    if ($type eq 'TC') {
      @limitrow = grep /OL_1_E/, @rows;
      my $basename = substr($limitrow[0],16,15);
      $name = "/web/data/$msndir/L1BRS/$etyp/$yyyy/$mo$dd/${prefix}$basename.L1B_${etyp}_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /OL_1_E/, @rows;
      $name = $limitrow[0]; 

    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0]; 
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /IOP/, @rows;
      $name = $limitrow[0]; 
    }
  }
  ############################### SeaWiFS GAC ################################
  elsif($mission eq 'swga'){
    my @gacrows = grep /GAC/, @rows;
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @gacrows;
      $name = "/web/data/SeaWiFS/L1BRS/GAC/$yyyy/$doy/" .$limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /L2_GAC_OC/, @gacrows;
      $name = "/web/data/SeaWiFS/L2BRS/OC/GAC/$yyyy/$doy/" . $limitrow[0]. "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @gacrows;
      $name = $limitrow[0];# . ".Z";
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /L2_GAC_OC/, @gacrows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /L2_GAC_IOP/, @gacrows;
      $name = $limitrow[0];
    }
  }
  ############################### SeaWiFS MLAC ###############################
  elsif($mission eq 'swml'){
    my @mlacrows = grep /MLAC/, @rows;
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @mlacrows;
      $name = "/web/data/SeaWiFS/L1BRS/MLAC/$yyyy/$doy/" .$limitrow[0] . "_BRS";
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /L2_MLAC_OC/, @mlacrows;
      $name = "/web/data/SeaWiFS/L2BRS/OC/MLAC/$yyyy/$doy/" . $limitrow[0]. "_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @mlacrows;
      $name = $limitrow[0];# . ".bz2";
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /L2_MLAC_OC/, @mlacrows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /L2_MLAC_IOP/, @mlacrows;
      $name = $limitrow[0];
    }
  }
  ############################### NOAA-20/VIIRS ##############################
  elsif($mission eq 'vrj1'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      $name = "/web/data/VIIRSJ1/L1BRS/TLAC6/$yyyy/$doy/". $limitrow[0] ."_BRS";
      $name =~ s/L1A/L1B/;
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      $name = "/web/data/VIIRSJ1/L2BRS/OC/TLAC6/$yyyy/$doy/" . $limitrow[0] ."_CHLOR_A_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'GEO') {
      @limitrow = grep /GEO/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /IOP/, @rows;
      $name = $limitrow[0];
    }
  }
  ############################### Suomi-NPP/VIIRS ############################
  elsif($mission eq 'vrsn'){
    if   ($type eq 'TC') {
      @limitrow = grep /L1/, @rows;
      #$name = "/web/data/VIIRS/L1BRS/TLAC6/$yyyy/$doy/V$yyyy$doy$hh$mm$ss.L1B_SNPP.nc_BRS";
      $name = "/web/data/VIIRS/L1BRS/TLAC6/$yyyy/$doy/". $limitrow[0] ."_BRS";
      $name =~ s/L1A/L1B/;
    }
    elsif($type eq 'CHL') {
      @limitrow = grep /OC/, @rows;
      #$name = "/web/data/VIIRS/L2BRS/OC/TLAC6/$yyyy/$doy/V$yyyy$doy$hh$mm$ss.L2_SNPP_OC.nc_CHLOR_A_BRS";
      $name = "/web/data/VIIRS/L2BRS/OC/TLAC6/$yyyy/$doy/" . $limitrow[0] ."_CHLOR_A_BRS";
    }
    elsif($type eq 'SST') {
      @limitrow = grep /SNPP_VIIRS.*SST\./, @rows;
      #$name = "/web/data/VIIRS/L2BRS/SST/TLAC6/$yyyy/$mo$dd/SNPP_VIIRS.$yyyy$mo${dd}T$hh$mm$ss.L2.SST.NRT.nc_SST_BRS";
      $name = "/web/data/VIIRS/L2BRS/SST/TLAC6/$yyyy/$mo$dd/" . $limitrow[0] ."_SST_BRS";
    }
    elsif($type eq 'L1') {
      @limitrow = grep /L1/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'GEO') {
      @limitrow = grep /GEO/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2OC') {
      @limitrow = grep /OC/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2IOP') {
      @limitrow = grep /IOP/, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2SST') {
      @limitrow = grep /SNPP_VIIRS.*SST\./, @rows;
      $name = $limitrow[0];
    }
    elsif($type eq 'L2SST3') {
      @limitrow = grep /SNPP_VIIRS.*SST3/, @rows;
      $name = $limitrow[0];
    }
  }
  $name;

}

sub fname2type {
  my $name = (split /\//,shift)[-1];

  my @type_patterns = (
  [ qr/^MOD00|L0/                  , 'L0' ],
  [ qr/^S3[AB]_OL_1|^MER_....1|EN1_............1|L1/ , 'L1' ],
  [ qr/GEO/                        , 'GEO' ],
  [ qr/SST4/                       , 'L2SST4' ],
  [ qr/SST3/                       , 'L2SST3' ],
  [ qr/SST/                        , 'L2SST' ],
  [ qr/IOP/                        , 'L2IOP' ],
  [ qr/L2/                         , 'L2OC' ]
  );

  my $type;
  foreach my $tp (@type_patterns){
    if($name =~ $tp->[0]) {
      $type = $tp->[1];
      last;
    }
  }

  # If a browse file name is passed in, return a browse type.
  if($name =~ /BRS$/){
    my %t = (
    L1     => 'TC',
    L2OC   => 'CHL',
    L2SST  => 'SST',
    L2SST4 => 'SST4'
    );
    $type = $t{$type} || '';
  }

#  write_debug("fname2type: $name $type");
  return $type;
}

sub fname2msntime {
  my $name = (split /\//,shift)[-1];
  
  my @mission_patterns = (
  [ qr/^A(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'amod' ],
  [ qr/^AQUA_MODIS\.(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\./        , 'amod' ],
  [ qr/^MOD00\.P(\d{4})(\d{3})\.(\d\d)(\d\d)_/                       , 'amod' ],
  [ qr/^T(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'tmod' ],
  [ qr/^TERRA_MODIS\.(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\./       , 'tmod' ],
  [ qr/^MOD00\.A(\d{4})(\d{3})\.(\d\d)(\d\d)_/                       , 'tmod' ],
  [ qr/^S(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.L(?:1A|2)_GAC/            , 'swga' ],
  [ qr/^S(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.L(?:1A|2)_MLAC/           , 'swml' ],
  [ qr/^V(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.(?:L1[AB]|L2|GEO-M)_SNPP/ , 'vrsn' ],
  [ qr/^SNPP_VIIRS\.(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\./        , 'vrsn' ],
  [ qr/^V(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.(?:L1[AB]|L2|GEO-M)_JPSS1/, 'vrj1' ],
  [ qr/^C(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'czcs' ],
  [ qr/^O(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'octs' ],
  [ qr/^H(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'hico' ],
  [ qr/^MER_RR__[012B][PC][A-Z]{4}(\d{4})(\d\d)(\d\d)_(\d\d)(\d\d)(\d\d)_/, 'merr' ],
  [ qr/^MER_FRS_[012B][PC][A-Z]{4}(\d{4})(\d\d)(\d\d)_(\d\d)(\d\d)(\d\d)_/, 'mefr' ],
  [ qr/^M(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.L2_RR_/                   , 'merr' ],
  [ qr/^M(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\.L2_FRS_/                  , 'mefr' ],
  [ qr/^EN1_MDSI_MER_RR__[12]P_(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/, 'merr' ],
  [ qr/^EN1_MDSI_MER_FRS_[12]P_(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/, 'mefr' ],
  [ qr/^EN1(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_RR/           , 'merr' ],
  [ qr/^EN1(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_FRS/          , 'mefr' ],
  [ qr/^S3A_OL_[012_]_ERR___(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/ , 's3ar' ],
  [ qr/^S3A_OL_[012_]_EFR___(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/ , 's3af' ],
  [ qr/^S3B_OL_[012_]_ERR___(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/ , 's3br' ],
  [ qr/^S3B_OL_[012_]_EFR___(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)_/ , 's3bf' ],
  [ qr/^S3A(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_ERR/          , 's3ar' ],
  [ qr/^S3A(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_EFR/          , 's3af' ],
  [ qr/^S3B(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_ERR/          , 's3br' ],
  [ qr/^S3B(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\.L1B_EFR/          , 's3bf' ],
  [ qr/^COMS_GOCI_L1B_GA_(\d{4})(\d\d)(\d\d)_?(\d\d)(\d\d)(\d\d)\./  , 'goci' ],
  [ qr/^G(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)\./                         , 'goci' ],
  [ qr/^SEAHAWK1_HAWKEYE\.(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)\./  , 'hawk' ]
  );

  my ($mission, $time);
  foreach my $mp (@mission_patterns){
    if($name =~ $mp->[0]) {
      $mission = $mp->[1];
      $time = strings2time($1, $2, $3, $4, $5, $6);
      last;
    }
  }
  return ($mission, $time);
}

#### SWB
sub fname2msntimeDB {
  my $name = (split /\//,shift)[-1];
  $name =~ s/nc_.*/nc/;
  $name =~ s/_CHLOR_A_BRS//;
  $name =~ s/_BRS//;
  my $sqlcmd =
      "SELECT start_time, data_type_id from datastore.files " .
      "where basename = $name";

  my $sth = $dbh->prepare($sqlcmd);
	  #$sth->{mysql_use_result} = 1;
  $sth->execute();
  my ($dataTypeID, $mission, $isotime, $time);
  while (my $fetchash = $sth->fetchrow_hashref){
    $time = $fetchash->{start_time};
    $dataTypeID= $fetchash->{data_type_id};
  }
  $sth->finish;

  $time = isodate2t($isotime);
  foreach my $mp (@mission_patterns){
    if($name =~ $mp->[0]) {
      $mission = $mp->[1];
      $time = strings2time($1, $2, $3, $4, $5, $6);
      last;
    }
  }
  return ($mission, $time);
}

sub gran2msntime {
  # The input granule record ($gran) points to an array that contains:
  #    a sensor/mission identifier,
  #    a start time stored as yyyydddhhmmss, yyyymoddhhmmss, or yyyymoddThhmmss,
  #    a data-type/station identifier,
  #    a browse image width,
  #    a browse image height, and
  #    a hide value.
  my $gran = shift;
  
  # I'm expecting the time to be stored as either a 13- or 14- or 15-digit
  # string (see the above granule record description).
  my $timestr = $gran->[1];
  $timestr =~ s/T//;
  $timestr = ymdhms2yd($timestr) if length($timestr) == 14;
  my $time = ydhms2t($timestr);

  my %mission = (
    A   => 'amod',
    T   => 'tmod',
    S   => 'swga',
    V   => 'vrsn',
    VJ1 => 'vrj1',
    C   => 'czcs',
    O   => 'octs',
    H   => 'hico',
    M   => 'merr',
    S3A => 's3ar',
    S3B => 's3br',
    G   => 'goci',
    K   => 'hawk'
  );
  my $msn = $mission{$gran->[0]};
  if   ($msn eq 'swga' and $gran->[2] eq 'MLAC'){ $msn = 'swml';   }
  elsif($msn eq 'merr' and $gran->[2] eq 'FRS' ){ $msn = 'mefr';   }
  elsif($msn =~ /^s3/  and $gran->[2] eq 'EFR' ){ $msn =~ s/r$/f/; }
  return ($msn, $time);
}


sub strings2time {
  my $year = shift;
  my ($doy,$mon,$day);
  if(length($_[0]) == 3) {
    $doy = shift;
  }
  else {
    $mon = shift;
    $day = shift;
  }
  my $hour = shift;
  my $min  = shift;
  my $sec = shift || 0;
  if(defined $doy) {
    return timegm($sec,$min,$hour,1,0,$year-1900) + 86400*($doy - 1);
  }
  else {
    return timegm($sec,$min,$hour,$day,$mon-1,$year-1900);
  }
}

sub write_debug {
  my (@list) = @_;
  my $file = "$logdir/browse.pl.debug.log";

 # open(my $fh, '>>', $file) or die qq@Could not create file, "$file". ($!) Died@; 
  open F,">>$file" or die qq@Could not create file, "$file". ($!) Died@;
  foreach my $line (@list){
    print F "$line\n";
  }
  close F;
}
